@@ -1,20 +1,19 @@
 /* FFNet.cpp
  *
- * Copyright (C) 1997-2011, 2015 David Weenink
+ * Copyright (C) 1997-2011, 2015-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -37,7 +36,7 @@
 #include "FFNet_Matrix.h"
 #include "Matrix_extensions.h"
 #include "TableOfReal_extensions.h"
-#include "Pattern.h"
+#include "PatternList.h"
 #include "Collection.h"
 #include "Categories.h"
 
@@ -67,7 +66,7 @@ Thing_implement (FFNet, Daata, 0);
 static void FFNet_checkLayerNumber (FFNet me, long layer) {
 	if (layer < 1 || layer > my nLayers) {
 		if (layer == 0) {
-			Melder_throw (U"A Layer number of 0 is not allowed.");
+			Melder_throw (U"A layer number of 0 is not allowed.");
 		} else if (layer < 0) {
 			Melder_throw (U"A negative layer number is not allowed.");
 		} else if (layer > my nLayers) {
@@ -144,7 +143,7 @@ static double minimumCrossEntropy (FFNet me, const double target[]) {
 
 /* *********************************************************************** */
 
-void bookkeeping (FFNet me) {
+static void bookkeeping (FFNet me) {
 	long nWeights = 0;
 	my nNodes = my nUnitsInLayer[0];
 	for (long i = 1; i <= my nLayers; i++) {
@@ -157,7 +156,7 @@ void bookkeeping (FFNet me) {
 	my nWeights = nWeights;
 
 	// The following test is essential because when an FFNet is read from file the w array already exists
-	if (my w == 0) {
+	if (! my w) {
 		my w = NUMvector<double> (1, my nWeights);
 	}
 	my activity = NUMvector<double> (1, my nNodes);
@@ -197,21 +196,21 @@ void bookkeeping (FFNet me) {
 }
 
 void structFFNet :: v_info () {
-	structDaata :: v_info ();
-	MelderInfo_writeLine (U"Number of layers: ", nLayers);
+	our structDaata :: v_info ();
+	MelderInfo_writeLine (U"Number of layers: ", our nLayers);
 	MelderInfo_writeLine (U"Total number of units: ", FFNet_getNumberOfUnits (this));
-	MelderInfo_writeLine (U"   Number of units in layer ", nLayers, U" (output): ", nUnitsInLayer[nLayers]);
-	for (long i = nLayers - 1; i >= 1; i--) {
-		MelderInfo_writeLine (U"   Number of units in layer ", i, U" (hidden): ", nUnitsInLayer[i]);
+	MelderInfo_writeLine (U"   Number of units in layer ", our nLayers, U" (output): ", our nUnitsInLayer [nLayers]);
+	for (long i = our nLayers - 1; i >= 1; i --) {
+		MelderInfo_writeLine (U"   Number of units in layer ", i, U" (hidden): ", our nUnitsInLayer [i]);
 	}
-	MelderInfo_writeLine (U"   Number of units in layer 0 (input): ", nUnitsInLayer[0]);
-	MelderInfo_writeLine (U"Outputs are linear: ", Melder_boolean (outputsAreLinear));
-	MelderInfo_writeLine (U"Number of weights: ", nWeights, U" (",
+	MelderInfo_writeLine (U"   Number of units in layer 0 (input): ", our nUnitsInLayer [0]);
+	MelderInfo_writeLine (U"Outputs are linear: ", Melder_boolean (our outputsAreLinear));
+	MelderInfo_writeLine (U"Number of weights: ", our nWeights, U" (",
 	                       FFNet_dimensionOfSearchSpace (this), U" selected)");
-	MelderInfo_writeLine (U"Number of nodes: ", nNodes);
+	MelderInfo_writeLine (U"Number of nodes: ", our nNodes);
 }
 
-void FFNet_init (FFNet me, long numberOfInputs, long nodesInLayer1, long nodesInLayer2, long numberOfOutputs, int outputsAreLinear) {
+void FFNet_init (FFNet me, long numberOfInputs, long nodesInLayer1, long nodesInLayer2, long numberOfOutputs, bool outputsAreLinear) {
 	long numberOfLayers = 3;
 
 	if (numberOfInputs < 1) {
@@ -221,22 +220,22 @@ void FFNet_init (FFNet me, long numberOfInputs, long nodesInLayer1, long nodesIn
 		Melder_throw (U"Number of outputs must be a natural number.");
 	}
 	if (nodesInLayer1 < 1) {
-		numberOfLayers--;
+		numberOfLayers --;
 	}
 	if (nodesInLayer2 < 1) {
-		numberOfLayers--;
+		numberOfLayers --;
 	}
 	my nLayers = numberOfLayers;
 	my nUnitsInLayer = NUMvector<long> (0, numberOfLayers);
 
-	my nUnitsInLayer[numberOfLayers--] = numberOfOutputs;
+	my nUnitsInLayer [numberOfLayers --] = numberOfOutputs;
 	if (nodesInLayer2 > 0) {
-		my nUnitsInLayer[numberOfLayers--] = nodesInLayer2;
+		my nUnitsInLayer [numberOfLayers --] = nodesInLayer2;
 	}
 	if (nodesInLayer1 > 0) {
-		my nUnitsInLayer[numberOfLayers--] = nodesInLayer1;
+		my nUnitsInLayer [numberOfLayers --] = nodesInLayer1;
 	}
-	my nUnitsInLayer[numberOfLayers] = numberOfInputs;
+	my nUnitsInLayer [numberOfLayers] = numberOfInputs;
 	Melder_assert (numberOfLayers == 0);
 	my outputsAreLinear = outputsAreLinear;
 
@@ -248,16 +247,16 @@ void FFNet_init (FFNet me, long numberOfInputs, long nodesInLayer1, long nodesIn
 }
 
 void FFNet_setOutputCategories (FFNet me, Categories thee) {
-	autoCategories uniq = Categories_selectUniqueItems (thee, 1);
-	if (uniq -> size == thy size) {
-		my outputCategories = uniq.transfer();
+	autoCategories uniq = Categories_selectUniqueItems (thee);
+	if (uniq->size == thy size) {
+		my outputCategories = uniq.move();
 	}
 }
 
-autoFFNet FFNet_create (long numberOfInputs, long numberInLayer1, long numberInLayer2, long numberOfOutputs, int outputsAreLinear) {
+autoFFNet FFNet_create (long numberOfInputs, long numberInLayer1, long numberInLayer2, long numberOfOutputs, bool outputsAreLinear) {
 	try {
 		autoFFNet me = Thing_new (FFNet);
-		FFNet_init (me.peek(), numberOfInputs, numberInLayer1, numberInLayer2, numberOfOutputs, outputsAreLinear);
+		FFNet_init (me.get(), numberOfInputs, numberInLayer1, numberInLayer2, numberOfOutputs, outputsAreLinear);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"FFNet not created.");
@@ -298,7 +297,7 @@ void FFNet_setBias (FFNet me, long layer, long unit, double value) {
 	if (node < 1) {
 		Melder_throw (U"Not a valid unit / layer combination.");
 	}
-	long bias_unit = my wLast[node]; // ??? +1
+	long bias_unit = my wLast [node]; // ??? +1
 	my w[bias_unit] = value;
 }
 
@@ -311,7 +310,7 @@ void FFNet_setWeight (FFNet me, long layer, long unit, long unit_from, double va
 	if (nodef < 1) {
 		Melder_throw (U"Not a valid unit / layer combination.");
 	}
-	long w_unit = my wFirst[node] + unit_from - 1;
+	long w_unit = my wFirst [node] + unit_from - 1;
 	my w[w_unit] = value;
 }
 
@@ -328,17 +327,17 @@ double FFNet_getWeight (FFNet me, long layer, long unit, long unit_from) {
 	return my w[w_unit];
 }
 
-void FFNet_reset (FFNet me, double wrange) {
-	for (long i = 1; i <= my nWeights; i++) {
-		if (my wSelected[i]) {
-			my w[i] = NUMrandomUniform (-wrange, wrange);
+void FFNet_reset (FFNet me, double weightRange) {
+	for (long i = 1; i <= my nWeights; i ++) {
+		if (my wSelected [i]) {
+			my w [i] = NUMrandomUniform (- weightRange, weightRange);
 		}
 	}
-	for (long i = 1; i <= my nNodes; i++) {
-		my activity[i] = (my isbias[i] ? 1.0 : 0.0);
+	for (long i = 1; i <= my nNodes; i ++) {
+		my activity [i] = ( my isbias [i] ? 1.0 : 0.0 );
 	}
 	my accumulatedCost = 0.0;
-	forget (my minimizer);
+	my minimizer.reset();
 }
 
 
@@ -500,9 +499,15 @@ void FFNet_weightConnectsUnits (FFNet me, long index, long *fromUnit, long *toUn
 	if (i > 1) {
 		index -= nw - np;
 	}
-	*fromUnit = index % (my nUnitsInLayer[i - 1] + 1);
-	*toUnit = (index - 1) / (my nUnitsInLayer[i - 1] + 1) + 1;
-	*layer = i;
+	if (fromUnit) {
+		*fromUnit = index % (my nUnitsInLayer[i - 1] + 1);
+	}
+	if (toUnit) {
+		*toUnit = (index - 1) / (my nUnitsInLayer[i - 1] + 1) + 1;
+	}
+	if (layer) {
+		*layer = i;
+	}
 }
 
 long FFNet_getNodeNumberFromUnitNumber (FFNet me, long unit, long layer) {
@@ -555,7 +560,7 @@ long FFNet_getNumberOfUnitsInLayer (FFNet me, int layer) {
 }
 
 double FFNet_getMinimum (FFNet me) {
-	return my minimizer ? Minimizer_getMinimum (my minimizer) : NUMundefined;
+	return my minimizer ? Minimizer_getMinimum (my minimizer.get()) : NUMundefined;
 }
 
 void FFNet_drawTopology (FFNet me, Graphics g) {
@@ -625,7 +630,7 @@ void FFNet_drawTopology (FFNet me, Graphics g) {
 			for (long j = 1; j <= my nOutputs; j++) {
 				Graphics_arrow (g, x2WC, y2WC + radius, x2WC, y2WC + radius + dy / 4.0);
 				if (my outputCategories) {
-					Categories_drawItem (my outputCategories.peek(), g, j, x2WC, y2WC + radius + dy / 4.0);
+					Categories_drawItem (my outputCategories.get(), g, j, x2WC, y2WC + radius + dy / 4.0);
 				}
 				x2WC += dx2;
 			}
@@ -677,9 +682,9 @@ void FFNet_drawWeightsToLayer (FFNet me, Graphics g, int layer, int scaling, int
 	if (layer < 1 || layer > my nLayers) {
 		Melder_throw (U"Layer must be in [1,", my nLayers, U"].");
 	}
-	autoMatrix weights = FFNet_weightsToMatrix (me, layer, 0);
-	Matrix_scale (weights.peek(), scaling);
-	Matrix_drawAsSquares (weights.peek(), g, 0.0, 0.0, 0.0, 0.0, 0);
+	autoMatrix weights = FFNet_weightsToMatrix (me, layer, false);
+	Matrix_scale (weights.get(), scaling);
+	Matrix_drawAsSquares (weights.get(), g, 0.0, 0.0, 0.0, 0.0, 0);
 	if (garnish) {
 		double x1WC, x2WC, y1WC, y2WC;
 		Graphics_inqWindow (g, & x1WC, & x2WC, & y1WC, & y2WC);
@@ -699,12 +704,12 @@ void FFNet_drawWeightsToLayer (FFNet me, Graphics g, int layer, int scaling, int
 
 void FFNet_drawWeights (FFNet me, Graphics g, long layer, int garnish) {
 	autoTableOfReal thee = FFNet_extractWeights (me, layer);
-	TableOfReal_drawAsSquares (thee.peek(), g, 1, thy numberOfRows, 1, thy numberOfColumns, garnish);
+	TableOfReal_drawAsSquares (thee.get(), g, 1, thy numberOfRows, 1, thy numberOfColumns, garnish);
 }
 
 void FFNet_drawCostHistory (FFNet me, Graphics g, long iFrom, long iTo, double costMin, double costMax, int garnish) {
 	if (my minimizer) {
-		Minimizer_drawHistory (my minimizer, g, iFrom, iTo, costMin, costMax, 0);
+		Minimizer_drawHistory (my minimizer.get(), g, iFrom, iTo, costMin, costMax, 0);
 	}
 	if (garnish) {
 		Graphics_drawInnerBox (g);
@@ -718,32 +723,32 @@ void FFNet_drawCostHistory (FFNet me, Graphics g, long iFrom, long iTo, double c
 
 autoCollection FFNet_createIrisExample (long numberOfHidden1, long numberOfHidden2) {
 	try {
-		autoCollection c = Collection_create (classDaata, 3);
-		autoCategories uniq = Categories_sequentialNumbers (3);
-		autoFFNet me = FFNet_create (4, numberOfHidden1, numberOfHidden2, 3, 0);
-		FFNet_setOutputCategories (me.peek(), uniq.peek());
-		autostring32 name = FFNet_createNameFromTopology (me.peek());
-		Thing_setName (me.peek(), name.peek());
-		Collection_addItem (c.peek(), me.transfer());
+		autoCollection collection = Collection_create ();
+		autoCategories uniq = Categories_createWithSequentialNumbers (3);
+		autoFFNet me = FFNet_create (4, numberOfHidden1, numberOfHidden2, 3, false);
+		FFNet_setOutputCategories (me.get(), uniq.get());
+		autostring32 name = FFNet_createNameFromTopology (me.get());
+		Thing_setName (me.get(), name.peek());
+		collection -> addItem_move (me.move());
 		autoTableOfReal iris = TableOfReal_createIrisDataset ();
 
-		// Scale data to interval [0-1]
-
-		for (long i = 1; i <= 150; i++) {
-			for (long j = 1; j <= 4; j++) {
-				iris -> data[i][j] /= 10.0;
+		/*
+		 * Scale data to interval [0-1]
+		 */
+		for (long i = 1; i <= 150; i ++) {
+			for (long j = 1; j <= 4; j ++) {
+				iris -> data [i] [j] /= 10.0;
 			}
 		}
 
-		Pattern thee = nullptr;
-		Categories him = nullptr;
-		TableOfReal_to_Pattern_and_Categories (iris.peek(), 0, 0, 0, 0, &thee, &him);
-		autoPattern ap = thee; autoCategories ac = him;
-		Thing_setName (ap.peek(), U"iris");
-		Thing_setName (ac.peek(), U"iris");
-		Collection_addItem (c.peek(), ap.transfer());
-		Collection_addItem (c.peek(), ac.transfer());
-		return c;
+		autoPatternList ap;
+		autoCategories ac;
+		TableOfReal_to_PatternList_and_Categories (iris.get(), 0, 0, 0, 0, & ap, & ac);
+		Thing_setName (ap.get(), U"iris");
+		Thing_setName (ac.get(), U"iris");
+		collection -> addItem_move (ap.move());
+		collection -> addItem_move (ac.move());
+		return collection;
 	} catch (MelderError) {
 		Melder_throw (U"Iris example not created.");
 	}
@@ -753,29 +758,29 @@ autoTableOfReal FFNet_extractWeights (FFNet me, long layer) {
 	try {
 		FFNet_checkLayerNumber (me, layer);
 
-		long numberOfUnitsFrom = my nUnitsInLayer[layer - 1] + 1;
-		long numberOfUnitsTo = my nUnitsInLayer[layer];
+		long numberOfUnitsFrom = my nUnitsInLayer [layer - 1] + 1;
+		long numberOfUnitsTo = my nUnitsInLayer [layer];
 		autoTableOfReal thee = TableOfReal_create (numberOfUnitsFrom, numberOfUnitsTo);
 
-		char32 label[40];
-		for (long i = 1; i <= numberOfUnitsFrom - 1; i++) {
+		char32 label [40];
+		for (long i = 1; i <= numberOfUnitsFrom - 1; i ++) {
 			Melder_sprint (label,40, U"L", layer - 1, U"-", i);
-			TableOfReal_setRowLabel (thee.peek(), i, label);
+			TableOfReal_setRowLabel (thee.get(), i, label);
 		}
-		TableOfReal_setRowLabel (thee.peek(), numberOfUnitsFrom, U"Bias");
+		TableOfReal_setRowLabel (thee.get(), numberOfUnitsFrom, U"Bias");
 		for (long i = 1; i <= numberOfUnitsTo; i++) {
 			Melder_sprint (label,40, U"L", layer, U"-", i);
-			TableOfReal_setColumnLabel (thee.peek(), i, label);
+			TableOfReal_setColumnLabel (thee.get(), i, label);
 		}
 
 		long node = 1;
-		for (long i = 0; i < layer; i++) {
+		for (long i = 0; i < layer; i ++) {
 			node += my nUnitsInLayer[i] + 1;
 		}
-		for (long i = 1; i <= numberOfUnitsTo; i++, node++) {
+		for (long i = 1; i <= numberOfUnitsTo; i ++, node ++) {
 			long k = 1;
-			for (long j = my wFirst[node]; j <= my wLast[node]; j++) {
-				thy data[k++][i] = my w[j];
+			for (long j = my wFirst [node]; j <= my wLast [node]; j ++) {
+				thy data [k ++] [i] = my w [j];
 			}
 		}
 		return thee;
@@ -787,8 +792,7 @@ autoTableOfReal FFNet_extractWeights (FFNet me, long layer) {
 autoFFNet FFNet_and_TabelOfReal_to_FFNet (FFNet me, TableOfReal him, long layer) {
 	try {
 		FFNet_checkLayerNumber (me, layer);
-		if ((my nUnitsInLayer[layer] != his numberOfColumns) ||
-		        (my nUnitsInLayer[layer] == his numberOfColumns && my nUnitsInLayer[layer - 1] + 1 == his numberOfRows)) {
+		if (my nUnitsInLayer [layer] != his numberOfColumns || my nUnitsInLayer [layer - 1] + 1 == his numberOfRows) {
 			long trys[1+3], rows[1+3], cols[1+3], ntry = my nLayers > 3 ? 3 : my nLayers, ok = 0;
 			if (my nLayers > 3) {
 				Melder_throw (U"Dimensions don't fit.");
