@@ -1,20 +1,19 @@
 /* praat_FFNet_init.cpp
  *
- * Copyright (C) 1994-2011 David Weenink
+ * Copyright (C) 1994-2011, 2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -37,10 +36,11 @@
 #include "Minimizers.h"
 #include "FFNet_Eigen.h"
 #include "FFNet_Matrix.h"
-#include "FFNet_Pattern.h"
-#include "FFNet_Activation_Categories.h"
-#include "FFNet_Pattern_Activation.h"
-#include "FFNet_Pattern_Categories.h"
+#include "FFNet_PatternList.h"
+#include "FFNet_ActivationList_Categories.h"
+#include "FFNet_PatternList_ActivationList.h"
+#include "FFNet_PatternList_Categories.h"
+#include "RBM_extensions.h"
 
 /* Routines to be removed sometime in the future:
 20040422, 2.4.04: FFNet_drawWeightsToLayer  use FFNet_drawWeights
@@ -98,8 +98,8 @@ FORM (FFNet_create, U"Create FFNet", U"Create FFNet...")
 DO
 	long numberOfInputs, numberOfOutputs, numberOfHidden1, numberOfHidden2;
 	FFNet_create_checkCommonFields (dia, &numberOfInputs, &numberOfOutputs, &numberOfHidden1, &numberOfHidden2);
-	autoFFNet thee = FFNet_create (numberOfInputs, numberOfHidden1, numberOfHidden2, numberOfOutputs, 0);
-	praat_new (thee.transfer(), GET_STRING (U"Name"));
+	autoFFNet thee = FFNet_create (numberOfInputs, numberOfHidden1, numberOfHidden2, numberOfOutputs, false);
+	praat_new (thee.move(), GET_STRING (U"Name"));
 END
 
 FORM (FFNet_create_linearOutputs, U"Create FFNet", U"Create FFNet (linear outputs)...")
@@ -109,8 +109,8 @@ FORM (FFNet_create_linearOutputs, U"Create FFNet", U"Create FFNet (linear output
 DO
 	long numberOfInputs, numberOfOutputs, numberOfHidden1, numberOfHidden2;
 	FFNet_create_checkCommonFields (dia, &numberOfInputs, &numberOfOutputs, &numberOfHidden1, &numberOfHidden2);
-	autoFFNet thee = FFNet_create (numberOfInputs, numberOfHidden1, numberOfHidden2, numberOfOutputs, 1);
-	praat_new (thee.transfer(), GET_STRING (U"Name"));
+	autoFFNet thee = FFNet_create (numberOfInputs, numberOfHidden1, numberOfHidden2, numberOfOutputs, true);
+	praat_new (thee.move(), GET_STRING (U"Name"));
 END
 
 FORM (FFNet_createIrisExample, U"Create iris example", U"Create iris example...")
@@ -121,7 +121,7 @@ DO
 	long numberOfHidden1, numberOfHidden2;
 	FFNet_create_checkCommonFields_hidden (dia, &numberOfHidden1, &numberOfHidden2);
 	autoCollection thee = FFNet_createIrisExample (numberOfHidden1, numberOfHidden2);
-	praat_new (thee.transfer());
+	praat_new (thee.move());
 END
 
 DIRECT (FFNet_getNumberOfInputs)
@@ -153,22 +153,22 @@ DO
 	}
 END
 
-FORM (FFNet_getCategoryOfOutputUnit, U"FFNet: Get category of output unit", U"")
+FORM (FFNet_getCategoryOfOutputUnit, U"FFNet: Get category of output unit", nullptr)
 	NATURAL (U"Output unit", U"1")
 	OK
-	DO
+DO
 	LOOP {
 		iam (FFNet);
 		long unit = GET_INTEGER (U"Output unit");
-		if (unit > my outputCategories -> size) {
-			Melder_throw (U"Output unit cannot be larger than ", my outputCategories -> size, U".");
+		if (unit > my outputCategories->size) {
+			Melder_throw (U"Output unit cannot be larger than ", my outputCategories->size, U".");
 		}
-		SimpleString ss = (SimpleString) my outputCategories -> item[unit];
+		SimpleString ss = my outputCategories->at [unit];
 		Melder_information (ss -> string);
 	}
 END
 
-FORM (FFNet_getOutputUnitOfCategory, U"FFNet: Get output unit of category", U"")
+FORM (FFNet_getOutputUnitOfCategory, U"FFNet: Get output unit of category", nullptr)
 	SENTENCE (U"Category", U"u")
 	OK
 DO
@@ -176,8 +176,8 @@ DO
 		iam (FFNet);
 		char32 *category = GET_STRING (U"Category");
 		long index = 0;
-		for (long i = 1; i <= my outputCategories -> size; i++) {
-			SimpleString s = (SimpleString) my outputCategories -> item[i];
+		for (long i = 1; i <= my outputCategories->size; i ++) {
+			SimpleString s = my outputCategories->at [i];
 			if (Melder_equ (s -> string, category)) {
 				index = i;
 				break;
@@ -187,7 +187,7 @@ DO
 	}
 END
 
-FORM (FFNet_getBias, U"FFNet: Get bias", 0)
+FORM (FFNet_getBias, U"FFNet: Get bias", nullptr)
 	NATURAL (U"Layer", U"1")
 	NATURAL (U"Unit", U"1")
 	OK
@@ -201,7 +201,7 @@ DO
 	}
 END
 
-FORM (FFNet_setBias, U"FFNet: Set bias", 0)
+FORM (FFNet_setBias, U"FFNet: Set bias", nullptr)
 	NATURAL (U"Layer", U"1")
 	NATURAL (U"Unit", U"1")
 	REAL (U"Value", U"0.0")
@@ -213,7 +213,7 @@ DO
 	}
 END
 
-FORM (FFNet_getWeight, U"FFNet: Get weight", 0)
+FORM (FFNet_getWeight, U"FFNet: Get weight", nullptr)
 	NATURAL (U"Layer", U"1")
 	NATURAL (U"Unit", U"1")
 	NATURAL (U"Unit from", U"1")
@@ -231,7 +231,7 @@ DO
 	}
 END
 
-FORM (FFNet_setWeight, U"FFNet: Set weight", 0)
+FORM (FFNet_setWeight, U"FFNet: Set weight", nullptr)
 	NATURAL (U"Layer", U"1")
 	NATURAL (U"Unit", U"1")
 	NATURAL (U"Unit (from)", U"1")
@@ -270,26 +270,26 @@ DIRECT (FFNet_getNumberOfOutputWeights)
 	}
 END
 
-/**************** New Pattern ***************************/
+/**************** New PatternList ***************************/
 
-FORM (Pattern_create, U"Create Pattern", 0)
+FORM (PatternList_create, U"Create PatternList", nullptr)
 	WORD (U"Name", U"1x1")
 	NATURAL (U"Dimension of a pattern", U"1")
 	NATURAL (U"Number of patterns", U"1")
 	OK
 DO
-	praat_new (Pattern_create (GET_INTEGER (U"Number of patterns"),
+	praat_new (PatternList_create (GET_INTEGER (U"Number of patterns"),
 		GET_INTEGER (U"Dimension of a pattern")), GET_STRING (U"Name"));
 END
 
 /**************** New Categories ***************************/
 
-FORM (Categories_create, U"Create Categories", U"")
+FORM (Categories_create, U"Create Categories", nullptr)
 	WORD (U"Name", U"empty")
 	OK
 DO
 	autoCategories thee = Categories_create ();
-	praat_new (thee.transfer(), GET_STRING (U"Name"));
+	praat_new (thee.move(), GET_STRING (U"Name"));
 END
 
 DIRECT (FFNet_help) Melder_help (U"Feedforward neural networks"); END
@@ -319,7 +319,7 @@ FORM (FFNet_selectBiasesInLayer, U"FFNet: Select biases", U"FFNet: Select biases
 	LABEL (U"", U"during a following learning phase.")
 	NATURAL (U"Layer number", U"1")
 	OK
-	DO
+DO
 	LOOP {
 		iam (FFNet);
 		FFNet_selectBiasesInLayer (me, GET_INTEGER (U"Layer number"));
@@ -343,14 +343,14 @@ DIRECT (FFNet_drawTopology)
 	}
 END
 
-FORM (FFNet_drawWeightsToLayer, U"FFNet: Draw weights to layer", 0)
+FORM (FFNet_drawWeightsToLayer, U"FFNet: Draw weights to layer", nullptr)
 	LABEL (U"", U"Warning: Disapproved. Use \"Draw weights..\" instead.")
 	NATURAL (U"Layer number", U"1")
 	RADIO (U"Scale", 1)
-	RADIOBUTTON (U"by maximum of all weights to layer")
-	RADIOBUTTON (U"by maximum weight from 'from-unit'")
-	RADIOBUTTON (U"by maximum weight to 'to-unit'")
-	BOOLEAN (U"Garnish", 1)
+		RADIOBUTTON (U"by maximum of all weights to layer")
+		RADIOBUTTON (U"by maximum weight from 'from-unit'")
+		RADIOBUTTON (U"by maximum weight to 'to-unit'")
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -363,7 +363,7 @@ END
 
 FORM (FFNet_drawWeights, U"FFNet: Draw weights", U"FFNet: Draw weights...")
 	NATURAL (U"Layer number", U"1")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -378,7 +378,7 @@ FORM (FFNet_drawCostHistory, U"FFNet: Draw cost history", U"FFNet: Draw cost his
 	INTEGER (U"right Iteration_range", U"0")
 	REAL (U"left Cost_range", U"0.0")
 	REAL (U"right Cost_range", U"0.0")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -393,38 +393,38 @@ END
 FORM (FFNet_extractWeights, U"FFNet: Extract weights", U"FFNet: Extract weights...")
 	NATURAL (U"Layer number", U"1")
 	OK
-	DO
+DO
 	LOOP {
 		iam (FFNet);
 		autoTableOfReal thee = FFNet_extractWeights (me, GET_INTEGER (U"Layer number"));
-		praat_new (thee.transfer());
+		praat_new (thee.move());
 	}
 END
 
-FORM (FFNet_weightsToMatrix, U"FFNet: Weights to Matrix ", 0)
+FORM (FFNet_weightsToMatrix, U"FFNet: Weights to Matrix ", nullptr)
 	LABEL (U"", U"Warning: Use \"Extract weights..\" instead.")
 	NATURAL (U"Layer number", U"1")
 	OK
 DO
 	LOOP {
 		iam (FFNet);
-		autoMatrix thee = FFNet_weightsToMatrix (me, GET_INTEGER (U"Layer number"), 0);
-		praat_new (thee.transfer(), my name);
+		autoMatrix thee = FFNet_weightsToMatrix (me, GET_INTEGER (U"Layer number"), false);
+		praat_new (thee.move(), my name);
 	}
 END
 
-/******************* FFNet && Activation *************************************/
+/******************* FFNet && ActivationList *************************************/
 
-FORM (FFNet_Activation_to_Categories, U"FFNet & Activation: To Categories", 0)
+FORM (FFNet_ActivationList_to_Categories, U"FFNet & ActivationList: To Categories", 0)
 	RADIO (U"Kind of labeling", 1)
 	RADIOBUTTON (U"Winner-takes-all")
 	RADIOBUTTON (U"Stochastic")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Activation thee = FIRST (Activation);
-	autoCategories him = FFNet_Activation_to_Categories (me, thee, GET_INTEGER (U"Kind of labeling"));
-	praat_new (him.transfer(), my name, U"_", thy name);
+	ActivationList thee = FIRST (ActivationList);
+	autoCategories him = FFNet_ActivationList_to_Categories (me, thee, GET_INTEGER (U"Kind of labeling"));
+	praat_new (him.move(), my name, U"_", thy name);
 END
 
 /******************* FFNet && Eigen ******************************************/
@@ -472,11 +472,11 @@ END
 
 /************************* FFNet && Categories **********************************/
 
-DIRECT (FFNet_Categories_to_Activation)
+DIRECT (FFNet_Categories_to_ActivationList)
 	FFNet me = FIRST (FFNet);
 	Categories thee = FIRST (Categories);
-	autoActivation him = FFNet_Categories_to_Activation (me, thee);
-	praat_new (him.transfer(), my name);
+	autoActivationList him = FFNet_Categories_to_ActivationList (me, thee);
+	praat_new (him.move(), my name);
 END
 
 /************************* FFNet && Matrix **********************************/
@@ -488,150 +488,146 @@ DO
 	FFNet me = FIRST (FFNet);
 	Matrix thee = FIRST (Matrix);
 	autoFFNet him = FFNet_weightsFromMatrix (me, thee, GET_INTEGER (U"Layer"));
-	praat_new (him.transfer(), my name);
+	praat_new (him.move(), my name);
 END
 
-/************************* FFNet && Pattern **********************************/
+/************************* FFNet && PatternList **********************************/
 
-FORM (FFNet_Pattern_drawActivation, U"Draw an activation", 0)
-	NATURAL (U"Pattern (row) number", U"1");
+FORM (FFNet_PatternList_drawActivation, U"Draw an activation", 0)
+	NATURAL (U"PatternList (row) number", U"1");
 	OK
 DO
 	autoPraatPicture picture;
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	FFNet_Pattern_drawActivation (me, thee, GRAPHICS, GET_INTEGER (U"Pattern"));
+	PatternList thee = FIRST (PatternList);
+	FFNet_PatternList_drawActivation (me, thee, GRAPHICS, GET_INTEGER (U"PatternList"));
 END
 
-FORM (FFNet_Pattern_to_Activation, U"To activations in layer", 0)
+FORM (FFNet_PatternList_to_ActivationList, U"To activations in layer", 0)
 	NATURAL (U"Layer", U"1")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	autoActivation him = FFNet_Pattern_to_Activation (me, thee, GET_INTEGER (U"Layer"));
-	praat_new (him.transfer(), my name, U"_", thy name);
+	PatternList thee = FIRST (PatternList);
+	autoActivationList him = FFNet_PatternList_to_ActivationList (me, thee, GET_INTEGER (U"Layer"));
+	praat_new (him.move(), my name, U"_", thy name);
 END
 
-DIRECT (hint_FFNet_and_Pattern_classify)
+DIRECT (hint_FFNet_and_PatternList_classify)
 	Melder_information (U"You can use the FFNet as a classifier by selecting a\n"
-		"FFNet and a Pattern together and choosing \"To Categories...\".");
+		"FFNet and a PatternList together and choosing \"To Categories...\".");
 END
 
-DIRECT (hint_FFNet_and_Pattern_and_Categories_learn)
+DIRECT (hint_FFNet_and_PatternList_and_Categories_learn)
 	Melder_information (U"You can teach a FFNet to classify by selecting a\n"
-		"FFNet, a Pattern and a Categories together and choosing \"Learn...\".");
+		"FFNet, a PatternList and a Categories together and choosing \"Learn...\".");
 END
 
-FORM (FFNet_Pattern_to_Categories, U"FFNet & Pattern: To Categories", U"FFNet & Pattern: To Categories...")
+FORM (FFNet_PatternList_to_Categories, U"FFNet & PatternList: To Categories", U"FFNet & PatternList: To Categories...")
 	RADIO (U"Determine output category as", 1)
-	RADIOBUTTON (U"Winner-takes-all")
-	RADIOBUTTON (U"Stochastic")
+		RADIOBUTTON (U"Winner-takes-all")
+		RADIOBUTTON (U"Stochastic")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	autoCategories him = FFNet_Pattern_to_Categories (me, thee, GET_INTEGER (U"Determine output category as"));
-	praat_new (him.transfer(), my name, U"_", thy name);
+	PatternList thee = FIRST (PatternList);
+	autoCategories him = FFNet_PatternList_to_Categories (me, thee, GET_INTEGER (U"Determine output category as"));
+	praat_new (him.move(), my name, U"_", thy name);
 END
 
-/*********** FFNet Pattern Activation **********************************/
+/*********** FFNet PatternList ActivationList **********************************/
 
-FORM (FFNet_Pattern_Activation_getCosts_total, U"FFNet & Pattern & Activation: Get total costs", U"FFNet & Pattern & Activation: Get total costs...")
+FORM (FFNet_PatternList_ActivationList_getCosts_total, U"FFNet & PatternList & ActivationList: Get total costs", U"FFNet & PatternList & ActivationList: Get total costs...")
 	RADIO (U"Cost function", 1)
 	RADIOBUTTON (U"Minimum-squared-error")
 	RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	Activation him = FIRST (Activation);
-	Melder_information (FFNet_Pattern_Activation_getCosts_total (me, thee, him, GET_INTEGER (U"Cost function")));
+	PatternList thee = FIRST (PatternList);
+	ActivationList him = FIRST (ActivationList);
+	Melder_information (FFNet_PatternList_ActivationList_getCosts_total (me, thee, him, GET_INTEGER (U"Cost function")));
 END
 
-FORM (FFNet_Pattern_Activation_getCosts_average, U"FFNet & Pattern & Activation: Get average costs", U"FFNet & Pattern & Activation: Get average costs...")
+FORM (FFNet_PatternList_ActivationList_getCosts_average, U"FFNet & PatternList & ActivationList: Get average costs", U"FFNet & PatternList & ActivationList: Get average costs...")
 	RADIO (U"Cost function", 1)
 	RADIOBUTTON (U"Minimum-squared-error")
 	RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	Activation him = FIRST (Activation);
-	Melder_information (FFNet_Pattern_Activation_getCosts_average (me, thee, him, GET_INTEGER (U"Cost function")));
+	PatternList thee = FIRST (PatternList);
+	ActivationList him = FIRST (ActivationList);
+	Melder_information (FFNet_PatternList_ActivationList_getCosts_average (me, thee, him, GET_INTEGER (U"Cost function")));
 END
 
-FORM (FFNet_Pattern_Activation_learnSD, U"FFNet & Pattern & Activation: Learn slow", 0)
+FORM (FFNet_PatternList_ActivationList_learnSD, U"FFNet & PatternList & ActivationList: Learn slow", 0)
 	NATURAL (U"Maximum number of epochs", U"100")
 	POSITIVE (U"Tolerance of minimizer", U"1e-7")
 	LABEL (U"Specifics", U"Specific for this minimization")
 	POSITIVE (U"Learning rate", U"0.1")
 	REAL (U"Momentum", U"0.9")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	Activation him = FIRST (Activation);
-	struct structSteepestDescentMinimizer_parameters p;
-	p.eta = GET_REAL (U"Learning rate");
-	p.momentum = GET_REAL (U"Momentum");
-	return FFNet_Pattern_Activation_learnSD (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
-			GET_REAL (U"Tolerance of minimizer"), & p, GET_INTEGER (U"Cost function"));
+	PatternList thee = FIRST (PatternList);
+	ActivationList him = FIRST (ActivationList);
+	return FFNet_PatternList_ActivationList_learnSD (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
+			GET_REAL (U"Tolerance of minimizer"), GET_REAL (U"Learning rate"), GET_REAL (U"Momentum"), GET_INTEGER (U"Cost function"));
 END
 
-FORM (FFNet_Pattern_Activation_learnSM, U"FFNet & Pattern & Activation: Learn", 0)
+FORM (FFNet_PatternList_ActivationList_learnSM, U"FFNet & PatternList & ActivationList: Learn", 0)
 	NATURAL (U"Maximum number of epochs", U"100")
 	POSITIVE (U"Tolerance of minimizer", U"1e-7")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
-	Activation him = FIRST (Activation);
-	return FFNet_Pattern_Activation_learnSM (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
-		GET_REAL (U"Tolerance of minimizer"), NULL,
-		GET_INTEGER (U"Cost function"));
+	PatternList thee = FIRST (PatternList);
+	ActivationList him = FIRST (ActivationList);
+	return FFNet_PatternList_ActivationList_learnSM (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
+		GET_REAL (U"Tolerance of minimizer"), GET_INTEGER (U"Cost function"));
 END
 
-/*********** FFNet Pattern Categories **********************************/
+/*********** FFNet PatternList Categories **********************************/
 
-FORM (FFNet_Pattern_Categories_getCosts_total, U"FFNet & Pattern & Categories: Get total costs", U"FFNet & Pattern & Categories: Get total costs...")
+FORM (FFNet_PatternList_Categories_getCosts_total, U"FFNet & PatternList & Categories: Get total costs", U"FFNet & PatternList & Categories: Get total costs...")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
+	PatternList thee = FIRST (PatternList);
 	Categories him = FIRST (Categories);
-	Melder_information (FFNet_Pattern_Categories_getCosts_total (me, thee, him,
+	Melder_information (FFNet_PatternList_Categories_getCosts_total (me, thee, him,
 		GET_INTEGER (U"Cost function")));
 END
 
-FORM (FFNet_Pattern_Categories_getCosts_average, U"FFNet & Pattern & Categories: Get average costs", U"FFNet & Pattern & Categories: Get average costs...")
+FORM (FFNet_PatternList_Categories_getCosts_average, U"FFNet & PatternList & Categories: Get average costs", U"FFNet & PatternList & Categories: Get average costs...")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
+	PatternList thee = FIRST (PatternList);
 	Categories him = FIRST (Categories);
-	Melder_information (FFNet_Pattern_Categories_getCosts_average (me, thee, him,
+	Melder_information (FFNet_PatternList_Categories_getCosts_average (me, thee, him,
 		GET_INTEGER (U"Cost function")));
 END
 
-FORM (Pattern_Categories_to_FFNet, U"Pattern & Categories: To FFNet", U"Pattern & Categories: To FFNet...")
+FORM (PatternList_Categories_to_FFNet, U"PatternList & Categories: To FFNet", U"PatternList & Categories: To FFNet...")
 	INTEGER (U"Number of units in hidden layer 1", U"0")
 	INTEGER (U"Number of units in hidden layer 2", U"0")
 	OK
 DO
-	Pattern me = FIRST (Pattern);
+	PatternList me = FIRST (PatternList);
 	Categories thee = FIRST (Categories);
 	long nHidden1 = GET_INTEGER (U"Number of units in hidden layer 1");
 	long nHidden2 = GET_INTEGER (U"Number of units in hidden layer 2");
@@ -642,64 +638,68 @@ DO
 	if (nHidden2 < 1) {
 		nHidden2 = 0;
 	}
-	autoCategories uniq = Categories_selectUniqueItems (thee, 1);
-	long numberOfOutputs = uniq -> size;
+	autoCategories uniq = Categories_selectUniqueItems (thee);
+	long numberOfOutputs = uniq->size;
 	if (numberOfOutputs < 1) Melder_throw (U"There are not enough categories in the Categories.\n"
 		U"Please try again with more categories in the Categories.");
 
-	autoFFNet ffnet = FFNet_create (my nx, nHidden1, nHidden2, numberOfOutputs, 0);
-	FFNet_setOutputCategories (ffnet.peek(), uniq.peek());
-	autostring32 ffnetName = FFNet_createNameFromTopology (ffnet.peek());
-	praat_new (ffnet.transfer(), ffnetName.peek());
+	autoFFNet ffnet = FFNet_create (my nx, nHidden1, nHidden2, numberOfOutputs, false);
+	FFNet_setOutputCategories (ffnet.get(), uniq.get());
+	autostring32 ffnetName = FFNet_createNameFromTopology (ffnet.get());
+	praat_new (ffnet.move(), ffnetName.peek());
 END
 
-FORM (FFNet_Pattern_Categories_learnSM, U"FFNet & Pattern & Categories: Learn", U"FFNet & Pattern & Categories: Learn...")
+FORM (FFNet_PatternList_Categories_learnSM, U"FFNet & PatternList & Categories: Learn", U"FFNet & PatternList & Categories: Learn...")
 	NATURAL (U"Maximum number of epochs", U"100")
 	POSITIVE (U"Tolerance of minimizer", U"1e-7")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
+	PatternList thee = FIRST (PatternList);
 	Categories him = FIRST (Categories);
-	FFNet_Pattern_Categories_learnSM (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
-		GET_REAL (U"Tolerance of minimizer"), NULL, GET_INTEGER (U"Cost function"));
+	FFNet_PatternList_Categories_learnSM (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
+		GET_REAL (U"Tolerance of minimizer"), GET_INTEGER (U"Cost function"));
 END
 
-FORM (FFNet_Pattern_Categories_learnSD, U"FFNet & Pattern & Categories: Learn slow", U"FFNet & Pattern & Categories: Learn slow...")
+FORM (FFNet_PatternList_Categories_learnSD, U"FFNet & PatternList & Categories: Learn slow", U"FFNet & PatternList & Categories: Learn slow...")
 	NATURAL (U"Maximum number of epochs", U"100")
 	POSITIVE (U"Tolerance of minimizer", U"1e-7")
 	LABEL (U"Specifics", U"Specific for this minimization")
 	POSITIVE (U"Learning rate", U"0.1")
 	REAL (U"Momentum", U"0.9")
 	RADIO (U"Cost function", 1)
-	RADIOBUTTON (U"Minimum-squared-error")
-	RADIOBUTTON (U"Minimum-cross-entropy")
+		RADIOBUTTON (U"Minimum-squared-error")
+		RADIOBUTTON (U"Minimum-cross-entropy")
 	OK
 DO
 	FFNet me = FIRST (FFNet);
-	Pattern thee = FIRST (Pattern);
+	PatternList thee = FIRST (PatternList);
 	Categories him = FIRST (Categories);
-	struct structSteepestDescentMinimizer_parameters p;
-	p.eta = GET_REAL (U"Learning rate");
-	p.momentum = GET_REAL (U"Momentum");
-	FFNet_Pattern_Categories_learnSD (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
-		GET_REAL (U"Tolerance of minimizer"), &p, GET_INTEGER (U"Cost function"));
+	FFNet_PatternList_Categories_learnSD (me, thee, him, GET_INTEGER (U"Maximum number of epochs"),
+		GET_REAL (U"Tolerance of minimizer"), GET_REAL (U"Learning rate"), GET_REAL (U"Momentum"), GET_INTEGER (U"Cost function"));
+END
+
+DIRECT (RBM_PatternList_to_ActivationList)
+	iam_ONLY (RBM);
+	thouart_ONLY (PatternList);
+	autoActivationList him = RBM_PatternList_to_ActivationList (me, thee);
+	praat_new (him.move(), my name, U"_", thy name);
 END
 
 void praat_uvafon_FFNet_init ();
 void praat_uvafon_FFNet_init () {
 	Thing_recognizeClassesByName (classFFNet, NULL);
 
-	praat_addMenuCommand (U"Objects", U"New", U"Neural nets", 0, 0, 0);
+	praat_addMenuCommand (U"Objects", U"New", U"Feedforward neural networks", 0, 0, 0);
 	praat_addMenuCommand (U"Objects", U"New", U"Feedforward neural networks", 0, 1, DO_FFNet_help);
 	praat_addMenuCommand (U"Objects", U"New", U"-- FFNet --", 0, 1, 0);
 	praat_addMenuCommand (U"Objects", U"New", U"Create iris example...", 0, 1, DO_FFNet_createIrisExample);
 	praat_addMenuCommand (U"Objects", U"New", U"Create FFNet...", 0, 1, DO_FFNet_create);
 	praat_addMenuCommand (U"Objects", U"New", U"Advanced", 0, 1, 0);
-	praat_addMenuCommand (U"Objects", U"New", U"Create Pattern...", 0, 2, DO_Pattern_create);
+	praat_addMenuCommand (U"Objects", U"New", U"Create PatternList...", 0, 2, DO_PatternList_create);
 	praat_addMenuCommand (U"Objects", U"New", U"Create Categories...", 0, 2, DO_Categories_create);
 	praat_addMenuCommand (U"Objects", U"New", U"Create FFNet (linear outputs)...", 0, 2, DO_FFNet_create_linearOutputs);
 
@@ -731,42 +731,44 @@ void praat_uvafon_FFNet_init () {
 	praat_addAction1 (classFFNet, 0, EXTRACT_BUTTON, 0, 0, 0);
 	praat_addAction1 (classFFNet, 0, U"Extract weights...", 0, 1, DO_FFNet_extractWeights);
 	praat_addAction1 (classFFNet, 0, U"Weights to Matrix...", 0, praat_DEPTH_1 | praat_HIDDEN, DO_FFNet_weightsToMatrix);
-	praat_addAction1 (classFFNet, 0, U"& Pattern: Classify?", 0, 0, DO_hint_FFNet_and_Pattern_classify);
-	praat_addAction1 (classFFNet, 0, U"& Pattern & Categories: Learn?", 0, 0, DO_hint_FFNet_and_Pattern_and_Categories_learn);
+	praat_addAction1 (classFFNet, 0, U"& PatternList: Classify?", 0, 0, DO_hint_FFNet_and_PatternList_classify);
+	praat_addAction1 (classFFNet, 0, U"& PatternList & Categories: Learn?", 0, 0, DO_hint_FFNet_and_PatternList_and_Categories_learn);
 
-	praat_addAction2 (classFFNet, 1, classActivation, 1, U"Analyse", 0, 0, 0);
-	praat_addAction2 (classFFNet, 1, classActivation, 1, U"To Categories...", 0, 0, DO_FFNet_Activation_to_Categories);
+	praat_addAction2 (classFFNet, 1, classActivationList, 1, U"Analyse", 0, 0, 0);
+	praat_addAction2 (classFFNet, 1, classActivationList, 1, U"To Categories...", 0, 0, DO_FFNet_ActivationList_to_Categories);
 
 	praat_addAction2 (classFFNet, 1, classEigen, 1, U"Draw", 0, 0, 0);
 	praat_addAction2 (classFFNet, 1, classEigen, 1, U"Draw hyperplane intersections", 0, 0, DO_FFNet_Eigen_drawIntersection);
 
 	praat_addAction2 (classFFNet, 1, classCategories, 1, U"Analyse", 0, 0, 0);
-	praat_addAction2 (classFFNet, 1, classCategories, 1, U"To Activation", 0, 0, DO_FFNet_Categories_to_Activation);
+	praat_addAction2 (classFFNet, 1, classCategories, 1, U"To ActivationList", 0, 0, DO_FFNet_Categories_to_ActivationList);
 
 	praat_addAction2 (classFFNet, 1, classMatrix, 1, U"Modify", 0, 0, 0);
 	praat_addAction2 (classFFNet, 1, classMatrix, 1, U"Weights from Matrix...", 0, 0, DO_FFNet_weightsFromMatrix);
 
-	praat_addAction2 (classFFNet, 1, classPattern, 1, U"Draw", 0, 0, 0);
-	praat_addAction2 (classFFNet, 1, classPattern, 1, U"Draw activation...", 0, 0, DO_FFNet_Pattern_drawActivation);
-	praat_addAction2 (classFFNet, 1, classPattern, 1, U"Analyse", 0, 0, 0);
-	praat_addAction2 (classFFNet, 1, classPattern, 1, U"To Categories...", 0, 0, DO_FFNet_Pattern_to_Categories);
-	praat_addAction2 (classFFNet, 1, classPattern, 1, U"To Activation...", 0, 0, DO_FFNet_Pattern_to_Activation);
+	praat_addAction2 (classFFNet, 1, classPatternList, 1, U"Draw", 0, 0, 0);
+	praat_addAction2 (classFFNet, 1, classPatternList, 1, U"Draw activation...", 0, 0, DO_FFNet_PatternList_drawActivation);
+	praat_addAction2 (classFFNet, 1, classPatternList, 1, U"Analyse", 0, 0, 0);
+	praat_addAction2 (classFFNet, 1, classPatternList, 1, U"To Categories...", 0, 0, DO_FFNet_PatternList_to_Categories);
+	praat_addAction2 (classFFNet, 1, classPatternList, 1, U"To ActivationList...", 0, 0, DO_FFNet_PatternList_to_ActivationList);
 
 	praat_addAction2 (classFFNet, 1, classPCA, 1, U"Draw decision plane...", 0, 0, DO_FFNet_PCA_drawDecisionPlaneInEigenspace);
+	
+	praat_addAction2 (classRBM, 1, classPatternList, 1, U"To ActivationList", 0, 0, DO_RBM_PatternList_to_ActivationList);
 
-	praat_addAction2 (classPattern, 1, classCategories, 1, U"To FFNet...", 0, 0, DO_Pattern_Categories_to_FFNet);
+	praat_addAction2 (classPatternList, 1, classCategories, 1, U"To FFNet...", 0, 0, DO_PatternList_Categories_to_FFNet);
 
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classActivation, 1, U"Get total costs...", 0, 0, DO_FFNet_Pattern_Activation_getCosts_total);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classActivation, 1, U"Get average costs...", 0, 0, DO_FFNet_Pattern_Activation_getCosts_average);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classActivation, 1, U"Learn", 0, 0, 0);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classActivation, 1, U"Learn...", 0, 0, DO_FFNet_Pattern_Activation_learnSM);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classActivation, 1, U"Learn slow...", 0, 0, DO_FFNet_Pattern_Activation_learnSD);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classActivationList, 1, U"Get total costs...", 0, 0, DO_FFNet_PatternList_ActivationList_getCosts_total);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classActivationList, 1, U"Get average costs...", 0, 0, DO_FFNet_PatternList_ActivationList_getCosts_average);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classActivationList, 1, U"Learn", 0, 0, 0);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classActivationList, 1, U"Learn...", 0, 0, DO_FFNet_PatternList_ActivationList_learnSM);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classActivationList, 1, U"Learn slow...", 0, 0, DO_FFNet_PatternList_ActivationList_learnSD);
 
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classCategories, 1, U"Get total costs...", 0, 0, DO_FFNet_Pattern_Categories_getCosts_total);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classCategories, 1, U"Get average costs...", 0, 0, DO_FFNet_Pattern_Categories_getCosts_average);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classCategories, 1, U"Learn", 0, 0, 0);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classCategories, 1, U"Learn...", 0, 0, DO_FFNet_Pattern_Categories_learnSM);
-	praat_addAction3 (classFFNet, 1, classPattern, 1, classCategories, 1, U"Learn slow...", 0, 0, DO_FFNet_Pattern_Categories_learnSD);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classCategories, 1, U"Get total costs...", 0, 0, DO_FFNet_PatternList_Categories_getCosts_total);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classCategories, 1, U"Get average costs...", 0, 0, DO_FFNet_PatternList_Categories_getCosts_average);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classCategories, 1, U"Learn", 0, 0, 0);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classCategories, 1, U"Learn...", 0, 0, DO_FFNet_PatternList_Categories_learnSM);
+	praat_addAction3 (classFFNet, 1, classPatternList, 1, classCategories, 1, U"Learn slow...", 0, 0, DO_FFNet_PatternList_Categories_learnSD);
 
 	INCLUDE_MANPAGES (manual_FFNet_init)
 }
