@@ -1,20 +1,19 @@
 /* Cepstrum.cpp
  *
- * Copyright (C) 1994-2015 David Weenink
+ * Copyright (C) 1994-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -28,53 +27,6 @@
 #include "NUM2.h"
 #include "Vector.h"
 
-static void NUMvector_gaussianBlur (double sigma, long filterLength, double *filter) {
-	if (filterLength <= 1) {
-		filter[1] = 1;
-		return;
-	}
-	double sum = 0, mid = (filterLength + 1) / 2;
-	for (long i = 1; i <= filterLength; i++) {
-		double val = (mid - i) / sigma;
-		filter[i] = exp (- 0.5 * val * val);
-		sum += filter[i];
-	}
-	for (long i = 1; i <= filterLength; i++) {
-		filter[i] /= sum;
-	}
-}
-
-// filter must be normalised: sum(i=1, nfilters, filter[i]) == 1
-static void NUMvector_filter (double *input, long numberOfDataPoints, double *filter, long numberOfFilterCoefficients, double *output, int edgeTreatment) {
-	long nleft = (numberOfFilterCoefficients - 1) / 2;
-	if (edgeTreatment == 0) { // outside values are zero
-		for (long i = 1; i <= numberOfDataPoints; i++) {
-			long ifrom = i - nleft, ito = i + nleft;
-			ito = numberOfFilterCoefficients % 2 == 0 ? ito - 1 : ito;
-			long jfrom = ifrom < 1 ? 1 : ifrom;
-			long jto = ito > numberOfDataPoints ? numberOfDataPoints : ito;
-			long index = ifrom < 1 ? 2 - ifrom : 1;
-			double out = 0.0, sum = 0.0;
-			for (long j = jfrom; j <= jto; j++, index++) {
-				out += filter[index] * input[j];
-				sum += filter[index];
-			}
-			output[i] = out / sum;
-		}
-	} else if (edgeTreatment == 1) { // wrap-around
-		for (long i = 1; i <= numberOfDataPoints; i++) {
-			double out = 0;
-			for (long j = 1; j <= numberOfFilterCoefficients; j++) {
-				long index = (i - nleft + j - 2) % numberOfDataPoints + 1;
-				out += filter[j] * input[index];
-			}
-			output[i] = out;
-		}
-	}
-	
-	
-}
-
 Thing_implement (Cepstrum, Matrix, 2);
 Thing_implement (PowerCepstrum, Cepstrum, 2); // derives from Matrix therefore also version 2
 
@@ -105,7 +57,7 @@ autoCepstrum Cepstrum_create (double qmax, long nq) {
 		autoCepstrum me = Thing_new (Cepstrum);
 		double dq = qmax / (nq - 1);
 
-		Matrix_init (me.peek(), 0.0, qmax, nq, dq, 0.0, 1.0, 1.0, 1, 1, 1.0);
+		Matrix_init (me.get(), 0.0, qmax, nq, dq, 0.0, 1.0, 1.0, 1, 1, 1.0);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Cepstrum not created.");
@@ -129,7 +81,7 @@ autoPowerCepstrum PowerCepstrum_create (double qmax, long nq) {
 		autoPowerCepstrum me = Thing_new (PowerCepstrum);
 		double dq = qmax / (nq - 1);
 
-		Matrix_init (me.peek(), 0.0, qmax, nq, dq, 0.0, 1.0, 1.0, 1, 1, 1.0);
+		Matrix_init (me.get(), 0.0, qmax, nq, dq, 0.0, 1.0, 1.0, 1, 1, 1.0);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"PowerCepstrum not created.");
@@ -314,6 +266,7 @@ void PowerCepstrum_fitTiltLine (PowerCepstrum me, double qmin, double qmax, doub
 	}
 }
 
+#if 0
 // Hillenbrand subtracts dB values and if the result is negative it is made zero
 static void PowerCepstrum_subtractTiltLine_inline2 (PowerCepstrum me, double slope, double intercept, int lineType) {
 	for (long j = 1; j <= my nx; j++) {
@@ -326,6 +279,7 @@ static void PowerCepstrum_subtractTiltLine_inline2 (PowerCepstrum me, double slo
 		my z[1][j] = diff;
 	}
 }
+#endif
 
 // clip with tilt line
 static void PowerCepstrum_subtractTiltLine_inline (PowerCepstrum me, double slope, double intercept, int lineType) {
@@ -353,49 +307,30 @@ void PowerCepstrum_subtractTilt_inline (PowerCepstrum me, double qstartFit, doub
 autoPowerCepstrum PowerCepstrum_subtractTilt (PowerCepstrum me, double qstartFit, double qendFit, int lineType, int fitMethod) {
 	try {
 		autoPowerCepstrum thee = Data_copy (me);
-		PowerCepstrum_subtractTilt_inline (thee.peek(), qstartFit,  qendFit, lineType, fitMethod);
+		PowerCepstrum_subtractTilt_inline (thee.get(), qstartFit,  qendFit, lineType, fitMethod);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": couldn't subtract tilt line.");
 	}
 }
 
-static void PowerCepstrum_smooth_inline2 (PowerCepstrum me, double quefrencyAveragingWindow) {
-	try {
-		long numberOfQuefrencyBins = (long) floor (quefrencyAveragingWindow / my dx);
-		if (numberOfQuefrencyBins > 1) {
-			autoNUMvector<double> qin (1, my nx);
-			autoNUMvector<double> qout (1, my nx);
-			for (long iq = 1; iq <= my nx; iq++) {
-				qin[iq] = my z[1][iq];
-			}
-			NUMvector_smoothByMovingAverage (qin.peek(), my nx, numberOfQuefrencyBins, qout.peek());
-			for (long iq = 1; iq <= my nx; iq++) {
-				my z[1][iq] = qout[iq];
-			}
-		}
-	} catch (MelderError) {
-		Melder_throw (me, U": not smoothed.");
-	}
-}
-
 void PowerCepstrum_smooth_inline (PowerCepstrum me, double quefrencyAveragingWindow, long numberOfIterations) {
 	try {
 		long numberOfQuefrencyBins = (long) floor (quefrencyAveragingWindow / my dx);
 		if (numberOfQuefrencyBins > 1) {
 			autoNUMvector<double> qin (1, my nx);
 			autoNUMvector<double> qout (1, my nx);
-			for (long iq = 1; iq <= my nx; iq++) {
-				qin[iq] = my z[1][iq];
+			for (long iq = 1; iq <= my nx; iq ++) {
+				qin [iq] = my z [1] [iq];
 			}
 			double *xin, *xout;
-			for (long k = 1; k <= numberOfIterations; k++) {
+			for (long k = 1; k <= numberOfIterations; k ++) {
 				xin  = k % 2 == 1 ? qin.peek() : qout.peek ();
 				xout = k % 2 == 1 ? qout.peek () : qin.peek();
 				NUMvector_smoothByMovingAverage (xin, my nx, numberOfQuefrencyBins, xout);
 			}
 			for (long iq = 1; iq <= my nx; iq++) {
-				my z[1][iq] = xout[iq];
+				my z [1] [iq] = xout [iq];
 			}
 		}
 	} catch (MelderError) {
@@ -405,69 +340,44 @@ void PowerCepstrum_smooth_inline (PowerCepstrum me, double quefrencyAveragingWin
 
 autoPowerCepstrum PowerCepstrum_smooth (PowerCepstrum me, double quefrencyAveragingWindow, long numberOfIterations) {
 	autoPowerCepstrum thee = Data_copy (me);
-	PowerCepstrum_smooth_inline (thee.peek(), quefrencyAveragingWindow, numberOfIterations);
+	PowerCepstrum_smooth_inline (thee.get(), quefrencyAveragingWindow, numberOfIterations);
 	return thee;
 }
 
 void PowerCepstrum_getMaximumAndQuefrency (PowerCepstrum me, double pitchFloor, double pitchCeiling, int interpolation, double *peakdB, double *quefrency) {
-	*peakdB = *quefrency = NUMundefined;
 	autoPowerCepstrum thee = Data_copy (me);
-	double lowestQuefrency = 1 / pitchCeiling, highestQuefrency = 1 / pitchFloor;
-	for (long i = 1; i <= my nx; i++) {
+	double lowestQuefrency = 1.0 / pitchCeiling, highestQuefrency = 1.0 / pitchFloor;
+	for (long i = 1; i <= my nx; i ++) {
 		thy z[1][i] = my v_getValueAtSample (i, 1, 0); // 10 log val^2
 	}
-	Vector_getMaximumAndX ((Vector) thee.peek(), lowestQuefrency, highestQuefrency, 1, interpolation, peakdB, quefrency);
-}
-
-static void Cepstrum_getZ2 (Cepstrum me, long imin, long imax, double peakdB, long margin, long keep, double *z) {
-	*z = NUMundefined;
-	long npeaks = 0, n = (imax - imin) / 2 + keep;
-	autoNUMvector<double> ymax (1, n);
-	autoNUMvector<long> index (1, n);
-	for (long i = imin + 1; i < imax; i++) {
-		if (my z[1][i] > my z[1][i-1] && my z[1][i] > my z[1][i+1]) {
-			ymax[++npeaks] = my z[1][i];
-			index[npeaks] = i;
-		}
-	}
-	NUMsort2<double, long> (npeaks, ymax.peek(), index.peek());
-	long i = npeaks - 1, ipeak = 0;
-	while (i > 0 && ipeak < keep) {
-		if (labs(index[i] - index[npeaks]) > margin) {
-			ipeak++;
-			ymax[npeaks +ipeak] = ymax[i];
-		}
-		i--;
-	}
-	double mean, variance;
-	NUMvector_avevar (&ymax[npeaks], ipeak, &mean, &variance);
-	double sigma = sqrt (variance / (ipeak - 1));
-	double peak = exp (peakdB * NUMln10 / 10.0) - 1e-30;
-	*z = sigma <= 0.0 ? NUMundefined : peak / sigma;
+	Vector_getMaximumAndX ((Vector) thee.get(), lowestQuefrency, highestQuefrency, 1, interpolation, peakdB, quefrency);   // FIXME cast
 }
 
+#if 0
 static void Cepstrum_getZ (Cepstrum me, long imin, long imax, double peakdB, double slope, double intercept, int lineType, double *z) {
 	long ndata = imax - imin + 1;
 	autoNUMvector<double> dabs (1, ndata);
-	for (long i = imin; i <= imax; i++) {
+	for (long i = imin; i <= imax; i ++) {
 		double q = my x1 + (i - 1) * my dx;
-		q = i == 1 ? 0.5 * my dx : q; // approximation
-		double xq = lineType == 2 ? log(q) : q;
+		q = ( i == 1 ? 0.5 * my dx : q );   // approximation
+		double xq = ( lineType == 2 ? log (q) : q );
 		double db_background = slope * xq + intercept;
 		double db_cepstrum = my v_getValueAtSample (i, 1, 0);
 		double diff = exp ((db_cepstrum - db_background) * NUMln10 / 10.0) - 1e-30;
 		//double diff = fabs (db_cepstrum - db_background);
-		dabs[i - imin + 1] = diff;
+		dabs [i - imin + 1] = diff;
 	}
 	double q50 = NUMquantile (ndata, dabs.peek(), 0.5);
 	double peak = exp (peakdB * NUMln10 / 10.0) - 1e-30;
-	//*z = peakdB / q50;
-	*z = peak / q50;
+	if (z) {
+		*z = peak / q50;
+	}
 }
+#endif
 
 double PowerCepstrum_getRNR (PowerCepstrum me, double pitchFloor, double pitchCeiling, double f0fractionalWidth) {
 	double rnr = NUMundefined;
-	double qmin = 1 / pitchCeiling, qmax = 1 / pitchFloor, peakdB, qpeak;
+	double qmin = 1.0 / pitchCeiling, qmax = 1.0 / pitchFloor, peakdB, qpeak;
 	PowerCepstrum_getMaximumAndQuefrency (me, pitchFloor, pitchCeiling, 2, &peakdB, &qpeak);
 	long imin, imax;
 	if (! Matrix_getWindowSamplesX (me, qmin, qmax, & imin, & imax)) {
@@ -488,14 +398,15 @@ double PowerCepstrum_getRNR (PowerCepstrum me, double pitchFloor, double pitchCe
 		double qx = my x1 + (i - 1) * my dx;
 		sum += val;
 		// is qx within an interval around a multiple of the peak's q ?
-		for (long j = 1; j <= npeaks; j++) {
-			double f0c = 1 / (j * qpeak);
-			double f0clow = f0c * (1 - f0fractionalWidth);
-			double f0chigh = f0c * (1 + f0fractionalWidth);
-			double qclow =  1 / f0chigh;
-			double qchigh = f0fractionalWidth >= 1 ? qmax : 1 / f0clow;
-			if (qx >= qclow && qx <= qchigh) { // yes in rhamonic interval
-				sumr += val; break;
+		for (long j = 1; j <= npeaks; j ++) {
+			double f0c = 1.0 / (j * qpeak);
+			double f0clow = f0c * (1.0 - f0fractionalWidth);
+			double f0chigh = f0c * (1.0 + f0fractionalWidth);
+			double qclow =  1.0 / f0chigh;
+			double qchigh = ( f0fractionalWidth >= 1 ? qmax : 1.0 / f0clow );
+			if (qx >= qclow && qx <= qchigh) {   // yes in rahmonic interval
+				sumr += val;
+				break;
 			}
 		}
 	}
@@ -507,23 +418,23 @@ double PowerCepstrum_getPeakProminence_hillenbrand (PowerCepstrum me, double pit
 	double slope, intercept, quefrency, peakdB;
 	PowerCepstrum_fitTiltLine (me, 0.001, 0, &slope, &intercept, 1, 1);
 	autoPowerCepstrum thee = Data_copy (me);
-	PowerCepstrum_subtractTiltLine_inline (thee.peek(), slope, intercept, 1);
-	PowerCepstrum_getMaximumAndQuefrency (thee.peek(), pitchFloor, pitchCeiling, 0, &peakdB, &quefrency);
+	PowerCepstrum_subtractTiltLine_inline (thee.get(), slope, intercept, 1);
+	PowerCepstrum_getMaximumAndQuefrency (thee.get(), pitchFloor, pitchCeiling, 0, & peakdB, & quefrency);
 	if (qpeak) {
 		*qpeak = quefrency;
 	}
 	return peakdB;
 }
 
-double PowerCepstrum_getPeakProminence (PowerCepstrum me, double pitchFloor, double pitchCeiling, int interpolation, double qstartFit, double qendFit, int lineType, int fitMethod, double *qpeak) {
-	double slope, intercept, quefrency, peakdB;
+double PowerCepstrum_getPeakProminence (PowerCepstrum me, double pitchFloor, double pitchCeiling, int interpolation, double qstartFit, double qendFit, int lineType, int fitMethod, double *p_qpeak) {
+	double slope, intercept, qpeak, peakdB;
 	PowerCepstrum_fitTiltLine (me, qstartFit, qendFit, &slope, &intercept, lineType, fitMethod);
-	PowerCepstrum_getMaximumAndQuefrency (me, pitchFloor, pitchCeiling, interpolation, &peakdB, &quefrency);
-	double xq = lineType == 2 ? log(quefrency) : quefrency;
+	PowerCepstrum_getMaximumAndQuefrency (me, pitchFloor, pitchCeiling, interpolation, & peakdB, & qpeak);
+	double xq = lineType == 2 ? log(qpeak) : qpeak;
 	double db_background = slope * xq + intercept;
 	double cpp = peakdB - db_background;
-	if (qpeak != nullptr) {
-		*qpeak = quefrency;
+	if (p_qpeak) {
+		*p_qpeak = qpeak;
 	}
 	return cpp;
 }
@@ -531,7 +442,7 @@ double PowerCepstrum_getPeakProminence (PowerCepstrum me, double pitchFloor, dou
 autoMatrix PowerCepstrum_to_Matrix (PowerCepstrum me) {
 	try {
 		autoMatrix thee = Thing_new (Matrix);
-		my structMatrix :: v_copy (thee.peek());
+		my structMatrix :: v_copy (thee.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no Matrix created.");
@@ -544,7 +455,7 @@ autoPowerCepstrum Matrix_to_PowerCepstrum (Matrix me) {
 			Melder_throw (U"Matrix should have exactly 1 row.");
 		}
 		autoPowerCepstrum thee = Thing_new (PowerCepstrum);
-		my structMatrix :: v_copy (thee.peek());
+		my structMatrix :: v_copy (thee.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to PowerCepstrum.");
