@@ -1,20 +1,19 @@
-/* manual_LPC.c
+/* manual_LPC.cpp
  *
- * Copyright (C) 1994-2014 David Weenink
+ * Copyright (C) 1994-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -475,10 +474,10 @@ TAG (U"##Use filter at time (s)")
 DEFINITION (U"determines which LPC frame will be chosen to inverse filter the sound. ")
 MAN_END
 
-MAN_BEGIN (U"MFCC", U"djmw", 20141022)
+MAN_BEGIN (U"MFCC", U"djmw", 20160225)
 INTRO (U"One of the @@types of objects@ in P\\s{RAAT}.")
 NORMAL (U"An object of type MFCC represents mel frequency cepstral coefficients "
-	"as a function of time. The coefficients are represented in frames "
+	"as a function of time. The coefficients are represented in a number of frames centred at equally spaced times. "
 	"at constant sampling period.")
 ENTRY (U"MFCC commands")
 NORMAL (U"Creation:")
@@ -558,7 +557,7 @@ MAN_END
 
 #define Sound_to_LPC_COMMON_HELP(method) \
 INTRO (U"With this command you create a new @LPC from every selected @Sound, " \
-	"using " #method " method.") \
+	"using " method " method.") \
 ENTRY (U"Warning") \
 NORMAL (U"You are advised not to use this command for formant analysis. " \
 	"For formant analysis, instead use @@Sound: To Formant (burg)...@, " \
@@ -665,12 +664,16 @@ NORMAL (U"J.D. Markel & A.H. Gray, Jr. (1976): %%Linear Prediction of Speech.% "
 	"Springer Verlag, Berlin.")
 MAN_END
 
-
 MAN_BEGIN (U"Marple (1980)", U"djmw", 19980114)
 NORMAL (U"L. Marple (1980): \"A new autoregressive spectrum analysis algorithm.\" "
 	"%%IEEE Trans. on ASSP% #28, 441\\--454.")
 MAN_END
 
+MAN_BEGIN (U"Rothweiler (1999)", U"djmw", 20160507)
+NORMAL (U"J. Rothweiler (1999): \"On Polynomial Reduction in the Computation of LSP Frequencies.\" "
+	"%%IEEE Trans. on ASSP% #7, 592\\--594.")
+MAN_END
+
 MAN_BEGIN (U"Theil (1950)", U"djmw", 20121118)
 NORMAL (U"H. Theil (1950): \"A rank-invariant method of linear and polynomial regression analysis\", "
 	"%%Proceedings of Koninklijke Nederlandse Akademie van Wetenschappen% ##A.53#: 1397\\--1412.")
@@ -690,4 +693,4 @@ MAN_END
  19980322 djmw Sound_into_LPC_Frame_auto did not return (nCoefficients == 0) when (r[1] == 0).
 */
 
-/* End of file manual_LPC.c */
+/* End of file manual_LPC.cpp */
