@@ -1,20 +1,19 @@
 /* praat_contrib_Ola_KNN.cpp
  *
- * Copyright (C) 2007-2009 Ola Söder, 2010-2011,2015 Paul Boersma
+ * Copyright (C) 2007-2009 Ola Söder, 2010-2011,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -55,7 +54,7 @@ FORM (KNN_Pattern_Categories_to_KNN, U"Create kNN classifier", U"kNN classifiers
 	RADIOBUTTON (U"Sequential")
 	OK2
 DO
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	thouart_ONLY (Categories);
 	int ordering = GET_INTEGER (U"Ordering");
 	autoKNN knn = KNN_create ();
@@ -66,12 +65,12 @@ DO
 		case 2:
 			ordering = kOla_SEQUENTIAL;
 	}
-	int result = KNN_learn (knn.peek(), me, thee, kOla_REPLACE, ordering);
+	int result = KNN_learn (knn.get(), me, thee, kOla_REPLACE, ordering);
 	switch (result) {
 		case kOla_PATTERN_CATEGORIES_MISMATCH:
-			Melder_throw (U"The number of Categories should be equal to the number of rows in Pattern.");
+			Melder_throw (U"The number of Categories should be equal to the number of rows in PatternList.");
 		case kOla_DIMENSIONALITY_MISMATCH:
-			Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+			Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 		default:
 			praat_new (knn.move(), GET_STRING(U"Name"));
 	}
@@ -114,7 +113,7 @@ DO
 	}
 	autoFeatureWeights fws = FeatureWeights_create ((my input) -> nx);
 	int dist;
-	KNN_modelSearch (me, fws.peek(), &k, &dist, mode, lrate, nseeds);
+	KNN_modelSearch (me, fws.get(), &k, &dist, mode, lrate, nseeds);
 	switch (dist) {
 		case kOla_SQUARED_DISTANCE_WEIGHTED_VOTING:
 			Melder_information (U"Vote weighting: Inversed squared distance\n", U"k: ", k);
@@ -167,9 +166,9 @@ DO
 			break;
 	}
 	autoFeatureWeights fws = FeatureWeights_create (my input -> nx);
-	double result = KNN_evaluate (me, fws.peek(), k, vt, mode);
+	double result = KNN_evaluate (me, fws.get(), k, vt, mode);
 	if (lround (result) == kOla_FWEIGHTS_MISMATCH)
-		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the Pattern.");
+		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the PatternList.");
 	Melder_information (100 * result, U" percent of the instances correctly classified.");   // BUG: use Melder_percent
 END2 }
 
@@ -214,7 +213,7 @@ DO
 	}
 	double result = KNN_evaluate (me, thee, k, vt, mode);
 	if (lround (result) == kOla_FWEIGHTS_MISMATCH)
-		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the Pattern.");
+		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the PatternList.");
 	Melder_information (100 * result, U" percent of the instances correctly classified.");
 END2 }
 
@@ -222,7 +221,7 @@ DIRECT2 (KNN_extractInputPatterns) {
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	autoPattern input = Data_copy (my input);
+	autoPatternList input = Data_copy (my input.get());
 	praat_new (input.move(), U"Input Patterns");
 END2 }
 
@@ -230,7 +229,7 @@ DIRECT2 (KNN_extractOutputCategories) {
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	autoCategories output = Data_copy (my output);
+	autoCategories output = Data_copy (my output.get());
 	praat_new (output.move(), U"Output Categories");
 END2 }
 
@@ -239,8 +238,8 @@ FORM (KNN_reset, U"Reset", U"KNN: Reset...") {
     OK2
 DO
 	iam_ONLY (KNN);
-	forget (my input);
-	forget (my output);
+	my input.reset();
+	my output.reset();
 	my nInstances = 0;
 	praat_dataChanged (me);   // BUG: this should be inserted much more often
 END2 }
@@ -287,7 +286,7 @@ FORM (KNN_learn, U"Learning", U"kNN classifiers 1. What is a kNN classifier?") {
 	OK2
 DO
 	iam_ONLY (KNN);
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	heis_ONLY (Categories);
 	int ordering = GET_INTEGER (U"Ordering");
 	switch (ordering) {
@@ -309,9 +308,9 @@ DO
 	}
 	switch (result) {
 		case kOla_PATTERN_CATEGORIES_MISMATCH:  
-			Melder_throw (U"The number of Categories should be equal to the number of rows in Pattern.");
+			Melder_throw (U"The number of Categories should be equal to the number of rows in PatternList.");
 		case kOla_DIMENSIONALITY_MISMATCH:
-			Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+			Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 	}
 END2 }
 
@@ -319,7 +318,7 @@ END2 }
 // Evaluation                                                                          //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (KNN_evaluateWithTestSet, U"Evaluation", U"KNN & Pattern & Categories: Evaluate...") {
+FORM (KNN_evaluateWithTestSet, U"Evaluation", U"KNN & PatternList & Categories: Evaluate...") {
 	INTEGER (U"k neighbours", U"1")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")
@@ -330,7 +329,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	heis_ONLY (Categories);
 	long k = GET_INTEGER (U"k neighbours");
 	if (k < 1 || k > my nInstances)
@@ -348,15 +347,15 @@ DO
 			break;
 	}
 	if (thy ny != his size)
-		Melder_throw (U"The number of Categories should be equal to the number of rows in Pattern.");
+		Melder_throw (U"The number of Categories should be equal to the number of rows in PatternList.");
 	if (thy nx != (my input)->nx)
-		Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+		Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 	autoFeatureWeights fws = FeatureWeights_create (thy nx);
-	double result = KNN_evaluateWithTestSet (me, thee, him, fws.peek(), k, vt);
+	double result = KNN_evaluateWithTestSet (me, thee, him, fws.get(), k, vt);
 	Melder_information (100 * result, U" percent of the instances correctly classified.");
 END2 }
 
-FORM (KNN_evaluateWithTestSetAndFeatureWeights, U"Evaluation", U"KNN & Pattern & Categories & FeatureWeights: Evaluate...") {
+FORM (KNN_evaluateWithTestSetAndFeatureWeights, U"Evaluation", U"KNN & PatternList & Categories & FeatureWeights: Evaluate...") {
 	INTEGER (U"k neighbours", U"1")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")
@@ -367,7 +366,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty");
-	Pattern p = (Pattern) ONLY (classPattern);
+	PatternList p = (PatternList) ONLY (classPatternList);
 	Categories c = (Categories) ONLY (classCategories);
 	FeatureWeights fws = (FeatureWeights) ONLY (classFeatureWeights);
 	long k = GET_INTEGER (U"k neighbours");
@@ -385,12 +384,12 @@ DO
 			vt = kOla_FLAT_VOTING;
 			break;
 	}
-	if (p -> ny != c -> size)
-		Melder_throw (U"The number of Categories should be equal to the number of rows in Pattern.");
+	if (p -> ny != c->size)
+		Melder_throw (U"The number of Categories should be equal to the number of rows in PatternList.");
 	if (p -> nx != my input -> nx)
-		Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+		Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 	if (p->nx != fws -> fweights -> numberOfColumns)
-		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the Pattern.");
+		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the PatternList.");
 	double result = KNN_evaluateWithTestSet (me, p, c, fws, k, vt);
 	Melder_information (100 * result, U" percent of the instances correctly classified.");
 END2 }
@@ -399,7 +398,7 @@ END2 }
 // Classification                                                                      //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (KNN_toCategories, U"Classification", U"KNN & Pattern: To Categories...") {
+FORM (KNN_toCategories, U"Classification", U"KNN & PatternList: To Categories...") {
 	INTEGER (U"k neighbours", U"1")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")
@@ -410,7 +409,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	long k = GET_INTEGER (U"k neighbours");
 	if (k < 1 || k > my nInstances)
 		Melder_throw (U"Please select a value of k such that 0 < k < ", my nInstances + 1, U".");
@@ -427,13 +426,13 @@ DO
 			break;
 	}
 	if (thy nx != my input -> nx)
-		Melder_throw (U"The dimensionality of Pattern should match that of the instance base.");
+		Melder_throw (U"The dimensionality of PatternList should match that of the instance base.");
 	autoFeatureWeights fws = FeatureWeights_create (thy nx);
-	autoCategories result = KNN_classifyToCategories (me, thee, fws.peek(), k, vt);
+	autoCategories result = KNN_classifyToCategories (me, thee, fws.get(), k, vt);
 	praat_new (result.move(), U"Output");
 END2 }
 
-FORM (KNN_toTableOfReal, U"Classification", U"KNN & Pattern: To TabelOfReal...") {
+FORM (KNN_toTableOfReal, U"Classification", U"KNN & PatternList: To TabelOfReal...") {
 	INTEGER (U"k neighbours", U"1")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")
@@ -444,7 +443,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	long k = GET_INTEGER (U"k neighbours");
 	if (k < 1 || k > my nInstances)
 		Melder_throw (U"Please select a value of k such that 0 < k < ", my nInstances + 1, U".");
@@ -462,13 +461,13 @@ DO
 			break;
 	}
 	if (thy nx != my input -> nx)
-		Melder_throw (U"The dimensionality of Pattern should match that of the instance base.");
-	autoTableOfReal result = KNN_classifyToTableOfReal (me, thee, fws.peek(), k, vt);
+		Melder_throw (U"The dimensionality of PatternList should match that of the instance base.");
+	autoTableOfReal result = KNN_classifyToTableOfReal (me, thee, fws.get(), k, vt);
 	praat_new (result.move(), U"Output");
 END2 }
 
-FORM (KNN_toCategoriesWithFeatureWeights, U"Classification", U"KNN & Pattern & FeatureWeights: To Categories...") {
-	INTEGER (U"k neighbours", U"KNN & Pattern & FeatureWeights: To Categories...")
+FORM (KNN_toCategoriesWithFeatureWeights, U"Classification", U"KNN & PatternList & FeatureWeights: To Categories...") {
+	INTEGER (U"k neighbours", U"KNN & PatternList & FeatureWeights: To Categories...")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")  
 	RADIOBUTTON (U"Inversed distance")
@@ -478,7 +477,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	heis_ONLY (FeatureWeights);
 	int vt = GET_INTEGER (U"Vote weighting");
 	switch (vt) {
@@ -496,14 +495,14 @@ DO
 	if (k < 1 || k > my nInstances)
 		Melder_throw (U"Please select a value of k such that 0 < k < ", my nInstances + 1, U".");
 	if (thy nx != (my input)->nx)
-		Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+		Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 	if (thy nx != his fweights -> numberOfColumns)
-		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the Pattern.");
+		Melder_throw (U"The number of feature weights should be equal to the dimensionality of the PatternList.");
 	autoCategories result = KNN_classifyToCategories (me, thee, him, k, vt);
 	praat_new (result.move(), U"Output");
 END2 }
 
-FORM (KNN_toTableOfRealWithFeatureWeights, U"Classification", U"KNN & Pattern & FeatureWeights: To TableOfReal...") {
+FORM (KNN_toTableOfRealWithFeatureWeights, U"Classification", U"KNN & PatternList & FeatureWeights: To TableOfReal...") {
 	INTEGER (U"k neighbours", U"1")
 	RADIO (U"Vote weighting", 1)
 	RADIOBUTTON (U"Inversed squared distance")
@@ -514,7 +513,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty.");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	heis_ONLY (FeatureWeights);
 	long k = GET_INTEGER (U"k neighbours");
 	int vt = GET_INTEGER (U"Vote weighting");
@@ -541,13 +540,13 @@ END2 }
 // Clustering                                                                          //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (Pattern_to_Categories_cluster, U"k-means clustering", U"Pattern: To Categories...") {
+FORM (Pattern_to_Categories_cluster, U"k-means clustering", U"PatternList: To Categories...") {
 	INTEGER (U"k clusters", U"1")
 	POSITIVE (U"Cluster size ratio constraint", U"0.0000001");
 	INTEGER (U"Maximum number of reseeds", U"1000")
 	OK2
 DO
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	if (my nx > 0 && my ny > 0) {
 		long k = GET_INTEGER (U"k clusters");
 		if (k < 1 || k > my ny)
@@ -559,20 +558,20 @@ DO
 		if (rc > 1 || rc <= 0)
 			Melder_throw (U"Please select a value of the cluster size ratio constraint c such that 0 < c <= 1.");
 		autoFeatureWeights fws = FeatureWeights_create (my nx);
-		autoCategories result = Pattern_to_Categories_cluster (me, fws.peek(), k, rc, rs);
+		autoCategories result = PatternList_to_Categories_cluster (me, fws.get(), k, rc, rs);
 		praat_new (result.move(), U"Output");
 	} else {
-		Melder_throw (U"Pattern is empty.");
+		Melder_throw (U"PatternList is empty.");
 	}
 END2 }
 
-FORM (Pattern_to_Categories_clusterWithFeatureWeights, U"k-means clustering", U"Pattern & FeatureWeights: To Categories...") {
+FORM (Pattern_to_Categories_clusterWithFeatureWeights, U"k-means clustering", U"PatternList & FeatureWeights: To Categories...") {
 	INTEGER (U"k clusters", U"1")
 	POSITIVE (U"Cluster size ratio constraint", U"0.0000001");
 	INTEGER (U"Maximum number of reseeds", U"1000")
 	OK2
 DO
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	if (my nx > 0 && my ny > 0) {
 		thouart_ONLY (FeatureWeights); 
 		if (my nx != thy fweights -> numberOfColumns)
@@ -586,10 +585,10 @@ DO
 		double rc =  GET_REAL(U"Cluster size ratio constraint");
 		if (rc > 1 || rc <= 0)
 			Melder_throw (U"Please select a value of the cluster size ratio constraint c such that 0 < c <= 1.");
-		autoCategories result = Pattern_to_Categories_cluster (me, thee, k, rc, rs);
+		autoCategories result = PatternList_to_Categories_cluster (me, thee, k, rc, rs);
 		praat_new (result.move(), U"Output");
 	} else {
-		Melder_throw (U"Pattern is empty.");
+		Melder_throw (U"PatternList is empty.");
 	}
 END2 }
 
@@ -598,14 +597,14 @@ END2 }
 /////////////////////////////////////////////////////////////////////////////////////////
 
 DIRECT2 (KNN_patternToDissimilarity) {
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	autoFeatureWeights fws = FeatureWeights_create (my nx);
-	autoDissimilarity result = KNN_patternToDissimilarity (me, fws.peek());
+	autoDissimilarity result = KNN_patternToDissimilarity (me, fws.get());
 	praat_new (result.move(), U"Output");
 END2 }
 
 DIRECT2 (KNN_patternToDissimilarityWithFeatureWeights) {
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	thouart_ONLY (FeatureWeights);  
 	if (my nx != thy fweights -> numberOfColumns)
 		Melder_throw (U"The number of features and the number of feature weights should be equal.");
@@ -617,7 +616,7 @@ END2 }
 // Computation of permutation                                                          //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (KNN_SA_computePermutation, U"To Permutation...", U"Pattern & Categories: To FeatureWeights...") {
+FORM (KNN_SA_computePermutation, U"To Permutation...", U"PatternList & Categories: To FeatureWeights...") {
 	NATURAL (U"Tries per step", U"200")
 	NATURAL (U"Iterations", U"10")
 	POSITIVE (U"Step size", U"10")
@@ -643,21 +642,21 @@ END2 }
 // Computation of feature weights                                                      //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (FeatureWeights_computeRELIEF, U"Feature weights", U"Pattern & Categories: To FeatureWeights...") {
+FORM (FeatureWeights_computeRELIEF, U"Feature weights", U"PatternList & Categories: To FeatureWeights...") {
 	INTEGER (U"Number of neighbours", U"1")
 	OK2
 DO
-	iam_ONLY (Pattern);
+	iam_ONLY (PatternList);
 	thouart_ONLY (Categories);
 	if (my ny < 2)
-		Melder_throw (U"The Pattern object should contain at least 2 rows.");
+		Melder_throw (U"The PatternList object should contain at least two rows.");
 	if (my ny != thy size)
-		Melder_throw (U"The number of rows in the Pattern object should equal the number of categories in the Categories object.");
+		Melder_throw (U"The number of rows in the PatternList object should equal the number of categories in the Categories object.");
 	autoFeatureWeights result = FeatureWeights_compute (me, thee, GET_INTEGER (U"Number of neighbours"));
 	praat_new (result.move(), U"Output");
 END2 }
 
-FORM (FeatureWeights_computeWrapperExt, U"Feature weights", U"KNN & Pattern & Categories: To FeatureWeights..") {
+FORM (FeatureWeights_computeWrapperExt, U"Feature weights", U"KNN & PatternList & Categories: To FeatureWeights..") {
 	POSITIVE (U"Learning rate", U"0.02")
 	NATURAL (U"Number of seeds", U"20")
 	POSITIVE (U"Stop at", U"1")
@@ -674,7 +673,7 @@ DO
 	iam_ONLY (KNN);
 	if (my nInstances <= 0)
 		Melder_throw (U"Instance base is empty");
-	thouart_ONLY (Pattern);
+	thouart_ONLY (PatternList);
 	heis_ONLY (Categories);
 	int mode = GET_INTEGER (U"Vote weighting");
 	switch (mode) {
@@ -692,7 +691,7 @@ DO
 	if (k < 1 || k > my nInstances)
 		Melder_throw (U"Please select a value of k such that 0 < k < ", my nInstances + 1, U".");
 	if (thy nx != my input -> nx)
-		Melder_throw (U"The dimensionality of Pattern should be equal to that of the instance base.");
+		Melder_throw (U"The dimensionality of PatternList should be equal to that of the instance base.");
 	autoFeatureWeights result = FeatureWeights_computeWrapperExt (me, thee, him, k, mode, GET_INTEGER (U"Number of seeds"),
 		GET_REAL (U"Learning rate"), GET_REAL (U"Stop at"), (int) GET_INTEGER (U"Optimization"));
 	praat_new (result.move(), U"Output");
@@ -751,13 +750,13 @@ END2 }
 // Creation and processing of auxiliary datatypes                                      //
 /////////////////////////////////////////////////////////////////////////////////////////
 
-FORM (Pattern_create, U"Create Pattern", 0) {
+FORM (Pattern_create, U"Create PatternList", 0) {
 	WORD (U"Name", U"1x1")
 	NATURAL (U"Dimension of a pattern", U"1")
 	NATURAL (U"Number of patterns", U"1")
 	OK2
 DO
-	autoPattern result = Pattern_create (GET_INTEGER (U"Number of patterns"), GET_INTEGER (U"Dimension of a pattern"));
+	autoPatternList result = PatternList_create (GET_INTEGER (U"Number of patterns"), GET_INTEGER (U"Dimension of a pattern"));
 	praat_new (result.move(), GET_STRING (U"Name"));
 END2 }
 
@@ -787,8 +786,8 @@ END2 }
 #ifdef _DEBUG
 
 DIRECT (KNN_debug_KNN_SA_partition)
-    Pattern p = ONLY (classPattern);
-    autoPattern output = Pattern_create (p->ny, p->nx);
+    PatternList p = ONLY (classPatternList);
+    autoPatternList output = PatternList_create (p->ny, p->nx);
     autoNUMvector <long> result (0, p->ny);
     KNN_SA_partition (p, 1, p->ny, result);
 
@@ -829,31 +828,31 @@ DIRECT2 (hint_KNN_and_FeatureWeights_evaluate) {
 END2 }
 
 DIRECT2 (hint_KNN_and_Pattern_classify) {
-	Melder_information (U"You can use the KNN as a classifier by selecting a KNN and a Pattern and choosing \"To Categories...\" or \"To TableOfReal...\".");
+	Melder_information (U"You can use the KNN as a classifier by selecting a KNN and a PatternList and choosing \"To Categories...\" or \"To TableOfReal...\".");
 END2 }
 
 DIRECT2 (hint_KNN_and_Pattern_and_FeatureWeights_classify) {
-	Melder_information (U"You can use the KNN as a classifier by selecting a KNN, a Pattern and an FeatureWeights object and choosing \"To Categories...\" or \"To TableOfReal...\".");
+	Melder_information (U"You can use the KNN as a classifier by selecting a KNN, a PatternList and an FeatureWeights object and choosing \"To Categories...\" or \"To TableOfReal...\".");
 END2 }
 
 DIRECT2 (hint_KNN_and_Pattern_and_Categories_learn) {
-	Melder_information (U"You can train a KNN by selecting a KNN, a Pattern and a Categories object together and choosing \"Learn...\".");
+	Melder_information (U"You can train a KNN by selecting a KNN, a PatternList and a Categories object together and choosing \"Learn...\".");
 END2 }
 
 DIRECT2 (hint_KNN_and_Pattern_and_Categories_evaluate) {
-	Melder_information (U"The accuracy of a KNN can be estimated by selecting a KNN, a test Pattern and the corresponding Categories object and choosing \"Evaluate...\".");
+	Melder_information (U"The accuracy of a KNN can be estimated by selecting a KNN, a test PatternList and the corresponding Categories object and choosing \"Evaluate...\".");
 END2 }
 
 DIRECT2 (hint_KNN_and_Pattern_and_Categories_and_FeatureWeights_evaluate) {
-	Melder_information (U"The accuracy of a KNN can be estimated by selecting a KNN, a test Pattern, an FeatureWeights object, and the corresponding Categories object and choosing \"Evaluate...\".");
+	Melder_information (U"The accuracy of a KNN can be estimated by selecting a KNN, a test PatternList, an FeatureWeights object, and the corresponding Categories object and choosing \"Evaluate...\".");
 END2 }
 
 DIRECT2 (hint_Pattern_and_FeatureWeights_to_Categories) {
-	Melder_information (U"A Pattern object and a FeatureWeights object can be used to compute a fixed number of clusters using the k-means clustering clustering algorithm.");
+	Melder_information (U"A PatternList object and a FeatureWeights object can be used to compute a fixed number of clusters using the k-means clustering clustering algorithm.");
 END2 }
 
 DIRECT2 (hint_Pattern_and_FeatureWeights_to_Dissimilarity) {
-	Melder_information (U"A Dissimilarity matrix can be generated from a Pattern and a FeatureWeights object.");
+	Melder_information (U"A Dissimilarity matrix can be generated from a PatternList and a FeatureWeights object.");
 END2 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -863,66 +862,65 @@ END2 }
 void praat_contrib_Ola_KNN_init ();
 void praat_contrib_Ola_KNN_init ()
 {
-    Thing_recognizeClassesByName (classKNN, NULL);
-    Thing_recognizeClassesByName (classFeatureWeights, NULL);
+    Thing_recognizeClassesByName (classKNN, classFeatureWeights, nullptr);
 
 //////////
 // Menu //
 //////////
 
-    praat_addMenuCommand (U"Objects", U"New", U"kNN classifiers", 0, 0, 0);
+    praat_addMenuCommand (U"Objects", U"New", U"kNN classifiers", nullptr, 0, nullptr);
 
-    praat_addMenuCommand (U"Objects", U"New", U"kNN classifiers", 0, 1, DO_KNN_help);
-    praat_addMenuCommand (U"Objects", U"New", U"-- KNN --", 0, 1, 0);
+    praat_addMenuCommand (U"Objects", U"New", U"kNN classifiers", nullptr, 1, DO_KNN_help);
+    praat_addMenuCommand (U"Objects", U"New", U"-- KNN --", nullptr, 1, nullptr);
 
-    praat_addMenuCommand (U"Objects", U"New", U"Create kNN classifier...", 0, 1, DO_KNN_create);
+    praat_addMenuCommand (U"Objects", U"New", U"Create kNN classifier...", nullptr, 1, DO_KNN_create);
 
-    praat_addMenuCommand (U"Objects", U"New", U"Advanced", 0, 1, 0);
-    praat_addMenuCommand (U"Objects", U"New", U"Create Pattern...", 0, 2, DO_Pattern_create);
-    praat_addMenuCommand (U"Objects", U"New", U"Create Categories...", 0, 2, DO_Categories_create);
-    praat_addMenuCommand (U"Objects", U"New", U"Create FeatureWeights...", 0, 2, DO_FeatureWeights_create);
+    praat_addMenuCommand (U"Objects", U"New", U"Advanced", nullptr, 1, nullptr);
+    praat_addMenuCommand (U"Objects", U"New", U"Create Pattern...", nullptr, 2, DO_Pattern_create);
+    praat_addMenuCommand (U"Objects", U"New", U"Create Categories...", nullptr, 2, DO_Categories_create);
+    praat_addMenuCommand (U"Objects", U"New", U"Create FeatureWeights...", nullptr, 2, DO_FeatureWeights_create);
 
 /////////////
 // Actions //
 /////////////
 
-    praat_addAction1 (classKNN, 0, U"kNN help", 0, 0, DO_KNN_help);
-    praat_addAction1 (classKNN, 0, QUERY_BUTTON, 0, 0, 0);
-    praat_addAction1 (classKNN, 1, U"Get optimized parameters...", 0, 2, DO_KNN_getOptimumModel);
-    praat_addAction1 (classKNN, 1, U"Get accuracy estimate...", 0, 2, DO_KNN_evaluate);
-    praat_addAction1 (classKNN, 1, U"Get size of instancebase", 0, 2, DO_KNN_getNumberOfInstances);
+    praat_addAction1 (classKNN, 0, U"kNN help", nullptr, 0, DO_KNN_help);
+    praat_addAction1 (classKNN, 0, QUERY_BUTTON, nullptr, 0, nullptr);
+    praat_addAction1 (classKNN, 1, U"Get optimized parameters...", nullptr, 2, DO_KNN_getOptimumModel);
+    praat_addAction1 (classKNN, 1, U"Get accuracy estimate...", nullptr, 2, DO_KNN_evaluate);
+    praat_addAction1 (classKNN, 1, U"Get size of instancebase", nullptr, 2, DO_KNN_getNumberOfInstances);
 
-    praat_addAction1 (classKNN, 0, MODIFY_BUTTON, 0, 0, 0);
-    praat_addAction1 (classKNN, 1, U"Shuffle", 0, 1, DO_KNN_shuffle);
-    praat_addAction1 (classKNN, 1, U"Prune...", 0, 1, DO_KNN_prune);
-    praat_addAction1 (classKNN, 1, U"Reset...", 0, 1, DO_KNN_reset);
-    praat_addAction1 (classKNN, 0, EXTRACT_BUTTON, 0, 0, 0);
-    praat_addAction1 (classKNN, 0, U"Extract input Patterns", 0, 1, DO_KNN_extractInputPatterns);
-    praat_addAction1 (classKNN, 0, U"Extract output Categories", 0, 1, DO_KNN_extractOutputCategories);
+    praat_addAction1 (classKNN, 0, MODIFY_BUTTON, nullptr, 0, nullptr);
+    praat_addAction1 (classKNN, 1, U"Shuffle", nullptr, 1, DO_KNN_shuffle);
+    praat_addAction1 (classKNN, 1, U"Prune...", nullptr, 1, DO_KNN_prune);
+    praat_addAction1 (classKNN, 1, U"Reset...", nullptr, 1, DO_KNN_reset);
+    praat_addAction1 (classKNN, 0, EXTRACT_BUTTON, nullptr, 0, nullptr);
+    praat_addAction1 (classKNN, 0, U"Extract input Patterns", nullptr, 1, DO_KNN_extractInputPatterns);
+    praat_addAction1 (classKNN, 0, U"Extract output Categories", nullptr, 1, DO_KNN_extractOutputCategories);
 
-    praat_addAction1 (classKNN, 0, U"To FeatureWeights...", 0, 0, DO_FeatureWeights_computeWrapperInt);
+    praat_addAction1 (classKNN, 0, U"To FeatureWeights...", nullptr, 0, DO_FeatureWeights_computeWrapperInt);
 
- // praat_addAction1 (classKNN, 0, U"To Permutation...", 0, 0, DO_KNN_SA_computePermutation);
- // praat_addAction2 (classKNN, 1, classFeatureWeights, 1, U"To Permutation...", 0, 0, DO_KNN_evaluateWithFeatureWeights);
+ // praat_addAction1 (classKNN, 0, U"To Permutation...", nullptr, 0, DO_KNN_SA_computePermutation);
+ // praat_addAction2 (classKNN, 1, classFeatureWeights, 1, U"To Permutation...", nullptr, 0, DO_KNN_evaluateWithFeatureWeights);
 
-    praat_addAction (classKNN, 1, classPattern, 1, classCategories, 1, U"Learn...", 0, 0, DO_KNN_learn);
-    praat_addAction2 (classKNN, 1, classFeatureWeights, 1, U"Evaluate...", 0, 0, DO_KNN_evaluateWithFeatureWeights);
-    praat_addAction (classKNN, 1, classPattern, 1, classCategories, 1, U"Evaluate...", 0, 0, DO_KNN_evaluateWithTestSet);
-    praat_addAction4 (classKNN, 1, classPattern, 1, classCategories, 1, classFeatureWeights, 1, U"Evaluate...", 0, 0, DO_KNN_evaluateWithTestSetAndFeatureWeights);
-    praat_addAction (classKNN, 1, classPattern, 1, classCategories, 1, U"To FeatureWeights...", 0, 0, DO_FeatureWeights_computeWrapperExt);
-    praat_addAction2 (classKNN, 1, classPattern, 1, U"To Categories...", 0, 0, DO_KNN_toCategories);
-    praat_addAction2 (classKNN, 1, classPattern, 1, U"To TableOfReal...", 0, 0, DO_KNN_toTableOfReal);
+    praat_addAction (classKNN, 1, classPatternList, 1, classCategories, 1, U"Learn...", nullptr, 0, DO_KNN_learn);
+    praat_addAction2 (classKNN, 1, classFeatureWeights, 1, U"Evaluate...", nullptr, 0, DO_KNN_evaluateWithFeatureWeights);
+    praat_addAction (classKNN, 1, classPatternList, 1, classCategories, 1, U"Evaluate...", nullptr, 0, DO_KNN_evaluateWithTestSet);
+    praat_addAction4 (classKNN, 1, classPatternList, 1, classCategories, 1, classFeatureWeights, 1, U"Evaluate...", nullptr, 0, DO_KNN_evaluateWithTestSetAndFeatureWeights);
+    praat_addAction (classKNN, 1, classPatternList, 1, classCategories, 1, U"To FeatureWeights...", nullptr, 0, DO_FeatureWeights_computeWrapperExt);
+    praat_addAction2 (classKNN, 1, classPatternList, 1, U"To Categories...", nullptr, 0, DO_KNN_toCategories);
+    praat_addAction2 (classKNN, 1, classPatternList, 1, U"To TableOfReal...", nullptr, 0, DO_KNN_toTableOfReal);
 
-    praat_addAction (classKNN, 1, classPattern, 1, classFeatureWeights, 1, U"To Categories...", 0, 0, DO_KNN_toCategoriesWithFeatureWeights);
-    praat_addAction (classKNN, 1, classPattern, 1, classFeatureWeights, 1, U"To TableOfReal...", 0, 0, DO_KNN_toTableOfRealWithFeatureWeights);
+    praat_addAction (classKNN, 1, classPatternList, 1, classFeatureWeights, 1, U"To Categories...", nullptr, 0, DO_KNN_toCategoriesWithFeatureWeights);
+    praat_addAction (classKNN, 1, classPatternList, 1, classFeatureWeights, 1, U"To TableOfReal...", nullptr, 0, DO_KNN_toTableOfRealWithFeatureWeights);
 
-    praat_addAction1 (classPattern, 1, U"To Dissimilarity", 0, 1, DO_KNN_patternToDissimilarity);
-    praat_addAction1 (classPattern, 1, U"To Categories...", 0, 1, DO_Pattern_to_Categories_cluster);
-    praat_addAction2 (classPattern, 1, classFeatureWeights, 1, U"To Dissimilarity", 0, 0, DO_KNN_patternToDissimilarityWithFeatureWeights);
-    praat_addAction2 (classPattern, 1, classFeatureWeights, 1, U"To Categories...", 0, 0, DO_Pattern_to_Categories_clusterWithFeatureWeights);
+    praat_addAction1 (classPatternList, 1, U"To Dissimilarity", nullptr, 1, DO_KNN_patternToDissimilarity);
+    praat_addAction1 (classPatternList, 1, U"To Categories...", nullptr, 1, DO_Pattern_to_Categories_cluster);
+    praat_addAction2 (classPatternList, 1, classFeatureWeights, 1, U"To Dissimilarity", nullptr, 0, DO_KNN_patternToDissimilarityWithFeatureWeights);
+    praat_addAction2 (classPatternList, 1, classFeatureWeights, 1, U"To Categories...", nullptr, 0, DO_Pattern_to_Categories_clusterWithFeatureWeights);
 
-    praat_addAction2 (classPattern, 1, classCategories, 1, U"To FeatureWeights...", 0, 0, DO_FeatureWeights_computeRELIEF);
-    praat_addAction2 (classPattern, 1, classCategories, 1, U"To KNN Classifier...", 0, 0, DO_KNN_Pattern_Categories_to_KNN);
+    praat_addAction2 (classPatternList, 1, classCategories, 1, U"To FeatureWeights...", nullptr, 0, DO_FeatureWeights_computeRELIEF);
+    praat_addAction2 (classPatternList, 1, classCategories, 1, U"To KNN Classifier...", nullptr, 0, DO_KNN_Pattern_Categories_to_KNN);
 
 ///////////
 // DEBUG //
@@ -942,16 +940,16 @@ void praat_contrib_Ola_KNN_init ()
 // Hints //
 ///////////
 
-    praat_addAction1 (classPattern, 0, U"& FeatureWeights: To Categories?", 0, 0, DO_hint_Pattern_and_FeatureWeights_to_Categories);
-    praat_addAction1 (classPattern, 0, U"& FeatureWeights: To Dissimilarity?", 0, 0, DO_hint_Pattern_and_FeatureWeights_to_Dissimilarity);
+    praat_addAction1 (classPatternList, 0, U"& FeatureWeights: To Categories?", nullptr, 0, DO_hint_Pattern_and_FeatureWeights_to_Categories);
+    praat_addAction1 (classPatternList, 0, U"& FeatureWeights: To Dissimilarity?", nullptr, 0, DO_hint_Pattern_and_FeatureWeights_to_Dissimilarity);
 
-    praat_addAction1 (classKNN, 0, U"& FeatureWeights: Evaluate?", 0, 0, DO_hint_KNN_and_FeatureWeights_evaluate);
-//  praat_addAction1 (classKNN, 0, U"& FeatureWeights: To Permutation?", 0, 0, DO_hint_Pattern_and_FeatureWeights_to_Dissimilarity);
-    praat_addAction1 (classKNN, 0, U"& Pattern: Classify?", 0, 0, DO_hint_KNN_and_Pattern_classify);
-    praat_addAction1 (classKNN, 0, U"& Pattern & FeatureWeights: Classify?", 0, 0, DO_hint_KNN_and_Pattern_and_FeatureWeights_classify);
-    praat_addAction1 (classKNN, 0, U"& Pattern & Categories: Learn?", 0, 0, DO_hint_KNN_and_Pattern_and_Categories_learn);
-    praat_addAction1 (classKNN, 0, U"& Pattern & Categories: Evaluate?", 0, 0, DO_hint_KNN_and_Pattern_and_Categories_evaluate);
-    praat_addAction1 (classKNN, 0, U"& Pattern & Categories & FeatureWeights: Evaluate?", 0, 0, DO_hint_KNN_and_Pattern_and_Categories_and_FeatureWeights_evaluate);
+    praat_addAction1 (classKNN, 0, U"& FeatureWeights: Evaluate?", nullptr, 0, DO_hint_KNN_and_FeatureWeights_evaluate);
+//  praat_addAction1 (classKNN, 0, U"& FeatureWeights: To Permutation?", nullptr, 0, DO_hint_Pattern_and_FeatureWeights_to_Dissimilarity);
+    praat_addAction1 (classKNN, 0, U"& Pattern: Classify?", nullptr, 0, DO_hint_KNN_and_Pattern_classify);
+    praat_addAction1 (classKNN, 0, U"& Pattern & FeatureWeights: Classify?", nullptr, 0, DO_hint_KNN_and_Pattern_and_FeatureWeights_classify);
+    praat_addAction1 (classKNN, 0, U"& Pattern & Categories: Learn?", nullptr, 0, DO_hint_KNN_and_Pattern_and_Categories_learn);
+    praat_addAction1 (classKNN, 0, U"& Pattern & Categories: Evaluate?", nullptr, 0, DO_hint_KNN_and_Pattern_and_Categories_evaluate);
+    praat_addAction1 (classKNN, 0, U"& Pattern & Categories & FeatureWeights: Evaluate?", nullptr, 0, DO_hint_KNN_and_Pattern_and_Categories_and_FeatureWeights_evaluate);
 
     INCLUDE_MANPAGES (manual_KNN_init)
 }
