@@ -1,20 +1,19 @@
 /* Eigen.cpp
  *
- * Copyright (C) 1993-2015 David Weenink
+ * Copyright (C) 1993-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -101,8 +100,7 @@ static void Graphics_ticks (Graphics g, double min, double max, bool hasNumber,
 	}
 }
 
-void Eigen_init (I, long numberOfEigenvalues, long dimension) {
-	iam (Eigen);
+void Eigen_init (Eigen me, long numberOfEigenvalues, long dimension) {
 	my numberOfEigenvalues = numberOfEigenvalues;
 	my dimension = dimension;
 	my eigenvalues = NUMvector<double> (1, numberOfEigenvalues);
@@ -116,8 +114,7 @@ void Eigen_init (I, long numberOfEigenvalues, long dimension) {
 	Eigenvectors: the columns of the matrix V
 	Eigenvalues: D_i^2
 */
-void Eigen_initFromSquareRoot (I, double **a, long numberOfRows, long numberOfColumns) {
-	iam (Eigen);
+void Eigen_initFromSquareRoot (Eigen me, double **a, long numberOfRows, long numberOfColumns) {
 	long numberOfZeroed, numberOfEigenvalues;
 	long nsv = MIN (numberOfRows, numberOfColumns);
 
@@ -133,7 +130,7 @@ void Eigen_initFromSquareRoot (I, double **a, long numberOfRows, long numberOfCo
 		basis.)
 	*/
 
-	numberOfZeroed = SVD_zeroSmallSingularValues (svd.peek(), 0.0);
+	numberOfZeroed = SVD_zeroSmallSingularValues (svd.get(), 0.0);
 
 	numberOfEigenvalues = nsv - numberOfZeroed;
 
@@ -151,9 +148,7 @@ void Eigen_initFromSquareRoot (I, double **a, long numberOfRows, long numberOfCo
 	Eigen_sort (me);
 }
 
-void Eigen_initFromSquareRootPair (I, double **a, long numberOfRows,
-                                   long numberOfColumns, double **b, long numberOfRows_b) {
-	iam (Eigen);
+void Eigen_initFromSquareRootPair (Eigen me, double **a, long numberOfRows, long numberOfColumns, double **b, long numberOfRows_b) {
 	double *u = nullptr, *v = nullptr, maxsv2 = -10.0;
 	char jobu = 'N', jobv = 'N', jobq = 'Q';
 	long k, ll, m = numberOfRows, n = numberOfColumns, p = numberOfRows_b;
@@ -223,20 +218,7 @@ void Eigen_initFromSquareRootPair (I, double **a, long numberOfRows,
 	NUMnormalizeRows (my eigenvectors, my numberOfEigenvalues, numberOfColumns, 1);
 }
 
-void Eigen_initFromSymmetricMatrix_f (I, float **a, long n) {
-	iam (Eigen);
-
-	autoNUMmatrix<double> m (1, n, 1, n);
-	for (long i = 1; i <= n; i++) {
-		for (long j = 1; j <= n; j++) {
-			m[i][j] = a[i][j];
-		}
-	}
-	Eigen_initFromSymmetricMatrix (me, m.peek(), n);
-}
-
-void Eigen_initFromSymmetricMatrix (I, double **a, long n) {
-	iam (Eigen);
+void Eigen_initFromSymmetricMatrix (Eigen me, double **a, long n) {
 	double wt[1], temp;
 	char jobz = 'V', uplo = 'U';
 	long lwork = -1, info;
@@ -277,37 +259,32 @@ void Eigen_initFromSymmetricMatrix (I, double **a, long n) {
 	}
 }
 
-Eigen Eigen_create (long numberOfEigenvalues, long dimension) {
+autoEigen Eigen_create (long numberOfEigenvalues, long dimension) {
 	try {
 		autoEigen me = Thing_new (Eigen);
-		Eigen_init (me.peek(), numberOfEigenvalues, dimension);
-		return me.transfer();
+		Eigen_init (me.get(), numberOfEigenvalues, dimension);
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Eigen not created.");
 	}
 }
 
-long Eigen_getNumberOfEigenvectors (I) {
-	iam (Eigen);
+long Eigen_getNumberOfEigenvectors (Eigen me) {
 	return my numberOfEigenvalues;
 }
 
-double Eigen_getEigenvectorElement (I, long ivec, long element) {
-	iam (Eigen);
+double Eigen_getEigenvectorElement (Eigen me, long ivec, long element) {
 	if (ivec > my numberOfEigenvalues || element < 1 || element > my dimension) {
 		return NUMundefined;
 	}
 	return my eigenvectors[ivec][element];
 }
 
-long Eigen_getDimensionOfComponents (I) {
-	iam (Eigen);
+long Eigen_getDimensionOfComponents (Eigen me) {
 	return my dimension;
 }
 
-double Eigen_getSumOfEigenvalues (I, long from, long to) {
-	iam (Eigen);
-
+double Eigen_getSumOfEigenvalues (Eigen me, long from, long to) {
 	if (from < 1) {
 		from = 1;
 	}
@@ -324,8 +301,7 @@ double Eigen_getSumOfEigenvalues (I, long from, long to) {
 	return sum;
 }
 
-double Eigen_getCumulativeContributionOfComponents (I, long from, long to) {
-	iam (Eigen);
+double Eigen_getCumulativeContributionOfComponents (Eigen me, long from, long to) {
 	double partial = 0.0, sum = 0.0;
 
 	if (to == 0) {
@@ -343,8 +319,7 @@ double Eigen_getCumulativeContributionOfComponents (I, long from, long to) {
 
 }
 
-long Eigen_getDimensionOfFraction (I, double fraction) {
-	iam (Eigen);
+long Eigen_getDimensionOfFraction (Eigen me, double fraction) {
 	double sum = Eigen_getSumOfEigenvalues (me, 0, 0);
 
 	if (sum == 0.0) {
@@ -359,8 +334,7 @@ long Eigen_getDimensionOfFraction (I, double fraction) {
 	return n;
 }
 
-void Eigen_sort (I) {
-	iam (Eigen);
+void Eigen_sort (Eigen me) {
 	double temp, *e = my eigenvalues, **v = my eigenvectors;
 
 	for (long i = 1; i < my numberOfEigenvalues; i++) {
@@ -383,8 +357,7 @@ void Eigen_sort (I) {
 	}
 }
 
-void Eigen_invertEigenvector (I, long ivec) {
-	iam (Eigen);
+void Eigen_invertEigenvector (Eigen me, long ivec) {
 
 	if (ivec < 1 || ivec > my numberOfEigenvalues) {
 		return;
@@ -395,10 +368,8 @@ void Eigen_invertEigenvector (I, long ivec) {
 	}
 }
 
-void Eigen_drawEigenvalues (I, Graphics g, long first, long last, double ymin, double ymax, int fractionOfTotal, int cumulative, double size_mm, const char32 *mark, int garnish) {
-	iam (Eigen);
+void Eigen_drawEigenvalues (Eigen me, Graphics g, long first, long last, double ymin, double ymax, bool fractionOfTotal, bool cumulative, double size_mm, const char32 *mark, bool garnish) {
 	double xmin = first, xmax = last, scale = 1.0, sumOfEigenvalues = 0.0;
-	long i;
 
 	if (first < 1) {
 		first = 1;
@@ -418,31 +389,33 @@ void Eigen_drawEigenvalues (I, Graphics g, long first, long last, double ymin, d
 		scale = sumOfEigenvalues;
 	}
 	if (ymax <= ymin) {
-		ymax = Eigen_getSumOfEigenvalues (me, (cumulative ? 1 : first), first) / scale;
-		ymin = Eigen_getSumOfEigenvalues (me, (cumulative ? 1 : last), last) / scale;
+		ymax = Eigen_getSumOfEigenvalues (me, ( cumulative ? 1 : first ), first) / scale;
+		ymin = Eigen_getSumOfEigenvalues (me, ( cumulative ? 1 : last ), last) / scale;
 		if (ymin > ymax) {
 			double tmp = ymin; ymin = ymax; ymax = tmp;
 		}
+		if (ymin == ymax) { // only one eigenvalue
+			ymin -= 0.1 * ymin; ymax = ymax += 0.1 * ymax;
+		}
 	}
 	Graphics_setInner (g);
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
-	for (i = first; i <= last; i++) {
-		double accu = Eigen_getSumOfEigenvalues (me, (cumulative ? 1 : i), i);
+	for (long i = first; i <= last; i ++) {
+		double accu = Eigen_getSumOfEigenvalues (me, ( cumulative ? 1 : i ), i);
 		Graphics_mark (g, i, accu / scale, size_mm, mark);
 	}
 	Graphics_unsetInner (g);
 	if (garnish) {
 		Graphics_drawInnerBox (g);
-		Graphics_textLeft (g, true, fractionOfTotal ? (cumulative ? U"Cumulative fractional eigenvalue" : U"Fractional eigenvalue") :
-			                   (cumulative ? U"Cumulative eigenvalue" : U"Eigenvalue"));
+		Graphics_textLeft (g, true, ( fractionOfTotal ? ( cumulative ? U"Cumulative fractional eigenvalue" : U"Fractional eigenvalue" ) :
+			                   ( cumulative ? U"Cumulative eigenvalue" : U"Eigenvalue" ) ));
 		Graphics_ticks (g, first, last, true, true, false, true);
 		Graphics_marksLeft (g, 2, true, true, false);
 		Graphics_textBottom (g, true, U"Index");
 	}
 }
 
-void Eigen_drawEigenvector (I, Graphics g, long ivec, long first, long last, double ymin, double ymax, int weigh, double size_mm, const char32 *mark, int connect, char32 **rowLabels, int garnish) {
-	iam (Eigen);
+void Eigen_drawEigenvector (Eigen me, Graphics g, long ivec, long first, long last, double ymin, double ymax, bool weigh, double size_mm, const char32 *mark, bool connect, char32 **rowLabels, bool garnish) {
 	double xmin = first, xmax = last;
 
 	if (ivec < 1 || ivec > my numberOfEigenvalues) {
@@ -486,18 +459,18 @@ void Eigen_drawEigenvector (I, Graphics g, long ivec, long first, long last, dou
 	}
 }
 
-void Eigens_alignEigenvectors (Collection me) {
+void Eigens_alignEigenvectors (OrderedOf<structEigen>* me) {
 	if (my size < 2) {
 		return;
 	}
 
-	Eigen e1 = (Eigen) my item[1];
+	Eigen e1 = my at [1];
 	double **evec1 = e1 -> eigenvectors;
 	long nev1 = e1 -> numberOfEigenvalues;
 	long dimension = e1 -> dimension;
 
-	for (long i = 2; i <= my size; i++) {
-		Eigen e2 = (Eigen) my item[i];
+	for (long i = 2; i <= my size; i ++) {
+		Eigen e2 = my at [i];
 		if (e2 -> dimension != dimension) {
 			Melder_throw (U"The dimension of the eigenvectors must be equal (offending object is ",  i, U").");
 		}
@@ -508,26 +481,25 @@ void Eigens_alignEigenvectors (Collection me) {
 		If r < 0 then mirror the eigenvector.
 	*/
 
-	for (long i = 2; i <= my size; i++) {
-		Eigen e2 = (Eigen) my item[i];
+	for (long i = 2; i <= my size; i ++) {
+		Eigen e2 = my at [i];
 		double **evec2 = e2 -> eigenvectors;
 
-		for (long j = 1; j <= MIN (nev1, e2 -> numberOfEigenvalues); j++) {
+		for (long j = 1; j <= MIN (nev1, e2 -> numberOfEigenvalues); j ++) {
 			double ip = 0.0;
-			for (long k = 1; k <= dimension; k++) {
-				ip += evec1[j][k] * evec2[j][k];
+			for (long k = 1; k <= dimension; k ++) {
+				ip += evec1 [j] [k] * evec2 [j] [k];
 			}
 			if (ip < 0.0) {
-				for (long k = 1; k <= dimension; k++) {
-					evec2[j][k] = - evec2[j][k];
+				for (long k = 1; k <= dimension; k ++) {
+					evec2 [j] [k] = - evec2 [j] [k];
 				}
 			}
 		}
 	}
 }
 
-static void Eigens_getAnglesBetweenSubspaces (I, thou, long ivec_from, long ivec_to, double *angles_degrees) {
-	iam (Eigen); thouart (Eigen);
+static void Eigens_getAnglesBetweenSubspaces (Eigen me, Eigen thee, long ivec_from, long ivec_to, double *angles_degrees) {
 	long nvectors = ivec_to - ivec_from + 1;
 	for (long i = 1; i <= nvectors; i++) {
 		angles_degrees[i] = NUMundefined;
@@ -563,14 +535,42 @@ static void Eigens_getAnglesBetweenSubspaces (I, thou, long ivec_from, long ivec
 	}
 }
 
-double Eigens_getAngleBetweenEigenplanes_degrees (I, thou) {
-	iam (Eigen); thouart (Eigen);
+double Eigens_getAngleBetweenEigenplanes_degrees (Eigen me, Eigen thee) {
 	double angles_degrees[3];
 
 	Eigens_getAnglesBetweenSubspaces (me, thee, 1, 2, angles_degrees);
 	return angles_degrees[2];
 }
 
+/* Very low level */
+
+void Eigen_and_matrix_into_matrix_principalComponents (Eigen me, double **from, long numberOfRows, long from_colbegin, double **to, long numberOfDimensionsToKeep, long to_colbegin) {
+	/*
+	 * Preconditions:
+	 * 
+	 * 	from[numberOfRows, from_colbegin - 1 + my dimension] exists
+	 * 	to [numberOfRows, to_colbegin - 1 + numberOfDimensionsToKeep] exists
+	 * 
+	 * Project/rotate the vectors in matrix 'from' along the 'numberOfDimensionsToKeep' eigenvectors into the matrix 'to'.
+	 */
+	from_colbegin = from_colbegin <= 0 ? 1 : from_colbegin;
+	to_colbegin = to_colbegin <= 0 ? 1 : to_colbegin;
+	if (numberOfDimensionsToKeep > my numberOfEigenvalues) {
+		Melder_throw (U"The number of dimensions to keep must not be larger than the number of eigenvalues.");
+	}
+	for (long irow = 1; irow <= numberOfRows; irow ++) {
+		for (long icol = 1; icol <= numberOfDimensionsToKeep; icol ++) {
+			double r = 0.0;
+			for (long k = 1; k <= my dimension; k ++) {
+				// eigenvector[icol] is in row[icol] of my eigenvectors
+				r += my eigenvectors  [icol] [k] * from [irow] [from_colbegin + k - 1];
+			}
+			to [irow] [to_colbegin + icol - 1] = r;
+		}
+	}
+}
+
+
 #undef MAX
 #undef MIN
 #undef SWAP
