@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1993-2011, 2014, 2015 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -66,11 +65,11 @@ autoConfusion Confusion_createFromStringses (Strings me, Strings thee) {
 		autoConfusion him = Confusion_create (my numberOfStrings, thy numberOfStrings);
 		for (long irow = 1; irow <= my numberOfStrings; irow++) {
 			const char32 *label = my strings[irow];
-			TableOfReal_setRowLabel (him.peek(), irow, label);
+			TableOfReal_setRowLabel (him.get(), irow, label);
 		}
 		for (long icol = 1; icol <= thy numberOfStrings; icol++) {
 			const char32 *label = thy strings[icol];
-			TableOfReal_setColumnLabel (him.peek(), icol, label);
+			TableOfReal_setColumnLabel (him.get(), icol, label);
 		}
 		return him;
 	} catch (MelderError) {
@@ -81,7 +80,7 @@ autoConfusion Confusion_createFromStringses (Strings me, Strings thee) {
 autoConfusion Confusion_create (long numberOfStimuli, long numberOfResponses) {
 	try {
 		autoConfusion me = Thing_new (Confusion);
-		TableOfReal_init (me.peek(), numberOfStimuli, numberOfResponses);
+		TableOfReal_init (me.get(), numberOfStimuli, numberOfResponses);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Confusion not created.");
@@ -102,8 +101,8 @@ autoConfusion Confusion_createSimple (const char32 *labels) {
 					Melder_throw (U"Label ", i, U"and ", ilabel, U"may not be equal.");
 				}
 			}
-			TableOfReal_setRowLabel (me.peek(), ilabel, token);
-			TableOfReal_setColumnLabel (me.peek(), ilabel, token);
+			TableOfReal_setRowLabel (me.get(), ilabel, token);
+			TableOfReal_setColumnLabel (me.get(), ilabel, token);
 			ilabel++;
 		}
 		return me;
@@ -118,21 +117,21 @@ autoConfusion Categories_to_Confusion (Categories me, Categories thee) {
 			Melder_throw (U"Categories_to_Confusion: dimensions do not agree.");
 		}
 
-		autoCategories ul1 = Categories_selectUniqueItems (me, 1);
-		autoCategories ul2 = Categories_selectUniqueItems (thee, 1);
-		autoConfusion him = Confusion_create (ul1 -> size, ul2 -> size);
+		autoCategories ul1 = Categories_selectUniqueItems (me);
+		autoCategories ul2 = Categories_selectUniqueItems (thee);
+		autoConfusion him = Confusion_create (ul1->size, ul2->size);
 
-		for (long i = 1; i <= ul1 -> size; i++) {
-			SimpleString s = (SimpleString) ul1 -> item[i];
-			TableOfReal_setRowLabel (him.peek(), i, s -> string);
+		for (long i = 1; i <= ul1->size; i ++) {
+			SimpleString s = ul1->at [i];
+			TableOfReal_setRowLabel (him.get(), i, s -> string);
 		}
-		for (long i = 1; i <= ul2 -> size; i++) {
-			SimpleString s = (SimpleString) ul2 -> item[i];
-			TableOfReal_setColumnLabel (him.peek(), i, s -> string);
+		for (long i = 1; i <= ul2->size; i ++) {
+			SimpleString s = ul2->at [i];
+			TableOfReal_setColumnLabel (him.get(), i, s -> string);
 		}
-		for (long i = 1; i <= my size; i++) {
-			SimpleString myi = (SimpleString) my item[i], thyi = (SimpleString) thy item[i];
-			Confusion_increase (him.peek(), SimpleString_c (myi), SimpleString_c (thyi));
+		for (long i = 1; i <= my size; i ++) {
+			SimpleString myi = my at [i], thyi = thy at [i];
+			Confusion_increase (him.get(), SimpleString_c (myi), SimpleString_c (thyi));
 		}
 		return him;
 	} catch (MelderError) {
@@ -142,8 +141,8 @@ autoConfusion Categories_to_Confusion (Categories me, Categories thee) {
 
 #define TINY 1.0e-30
 
-void Confusion_getEntropies (Confusion me, double *h, double *hx, double *hy, double *hygx, double *hxgy, double *uygx, double *uxgy, double *uxy) {
-	*h = *hx = *hy = *hxgy = *hygx = *uygx = *uxgy = *uxy = 0.0;
+void Confusion_getEntropies (Confusion me, double *p_h, double *p_hx, double *p_hy, double *p_hygx, double *p_hxgy, double *p_uygx, double *p_uxgy, double *p_uxy) {
+	double h = 0.0, hx = 0.0, hy = 0.0, hxgy = 0.0, hygx = 0.0, uygx = 0.0, uxgy = 0.0, uxy = 0.0;
 
 	autoNUMvector<double> rowSum (1, my numberOfRows);
 	autoNUMvector<double> colSum (1, my numberOfColumns);
@@ -158,27 +157,51 @@ void Confusion_getEntropies (Confusion me, double *h, double *hx, double *hy, do
 	}
 	for (long i = 1; i <= my numberOfRows; i++) {
 		if (rowSum[i] > 0.0) {
-			*hy -= rowSum[i] / sum * NUMlog2 (rowSum[i] / sum);
+			hy -= rowSum[i] / sum * NUMlog2 (rowSum[i] / sum);
 		}
 	}
 	for (long j = 1; j <= my numberOfColumns; j++) {
 		if (colSum[j] > 0.0) {
-			*hx -= colSum[j] / sum * NUMlog2 (colSum[j] / sum);
+			hx -= colSum[j] / sum * NUMlog2 (colSum[j] / sum);
 		}
 	}
 	for (long i = 1; i <= my numberOfRows; i++) {
 		for (long j = 1; j <= my numberOfColumns; j++) {
 			if (my data[i][j] > 0.0) {
-				*h -= my data[i][j] / sum * NUMlog2 (my data[i][j] / sum);
+				h -= my data[i][j] / sum * NUMlog2 (my data[i][j] / sum);
 			}
 		}
 	}
 
-	*hygx = *h - *hx;
-	*hxgy = *h - *hy;
-	*uygx = (*hy - *hygx) / (*hy + TINY);
-	*uxgy = (*hx - *hxgy) / (*hx + TINY);
-	*uxy = 2.0 * (*hx + *hy - *h) / (*hx + *hy + TINY);
+	hygx = h - hx;
+	hxgy = h - hy;
+	uygx = (hy - hygx) / (hy + TINY);
+	uxgy = (hx - hxgy) / (hx + TINY);
+	uxy = 2.0 * (hx + hy - h) / (hx + hy + TINY);
+	if (p_h) {
+		*p_h  = h;
+	}
+	if (p_hx) {
+		*p_hx  = hx;
+	}
+	if (p_hy) {
+		*p_hy  = hy;
+	}
+	if (p_hygx) {
+		*p_hygx  = hygx;
+	}
+	if (p_hxgy) {
+		*p_hxgy  = hxgy;
+	}
+	if (p_uygx) {
+		*p_uygx  = uygx;
+	}
+	if (p_uxgy) {
+		*p_uxgy  = uxgy;
+	}
+	if (p_uxy) {
+		*p_uxy  = uxy;
+	}
 }
 
 void Confusion_increase (Confusion me, const char32 *stim, const char32 *resp) {
@@ -329,11 +352,11 @@ void Confusion_Matrix_draw (Confusion me, Matrix thee, Graphics g, long index, d
 				xs = 0.0;
 			}
 			double ys = perc * rmax / 100.0;
-			Polygon_scale (p.peek(), xs, ys);
-			Polygon_translate (p.peek(), x1, y1 - ys / 2);
-			Polygon_rotate (p.peek(), alpha, x1, y1);
-			Polygon_translate (p.peek(), 1.1 * r * cos (alpha), 1.1 * r * sin (alpha));
-			Polygon_drawInside (p.peek(), g);
+			Polygon_scale (p.get(), xs, ys);
+			Polygon_translate (p.get(), x1, y1 - ys / 2);
+			Polygon_rotate (p.get(), alpha, x1, y1);
+			Polygon_translate (p.get(), 1.1 * r * cos (alpha), 1.1 * r * sin (alpha));
+			Polygon_drawInside (p.get(), g);
 		}
 	}
 
@@ -417,10 +440,10 @@ autoConfusion Confusion_condense (Confusion me, const char32 *search, const char
 		scol -> strings = columnLabels.transfer();
 
 		/* Find dimension of new Confusion */
-		autoDistributions dcol = Strings_to_Distributions (scol.peek());
+		autoDistributions dcol = Strings_to_Distributions (scol.get());
 		long nresp = dcol -> numberOfRows;
 
-		autoDistributions drow = Strings_to_Distributions (srow.peek());
+		autoDistributions drow = Strings_to_Distributions (srow.get());
 		long nstim = drow -> numberOfRows;
 
 		autoConfusion thee = Confusion_create (nstim, nresp);
@@ -450,7 +473,7 @@ autoConfusion TableOfReal_to_Confusion (TableOfReal me) {
 			Melder_throw (U"Elements may not be less than zero.");
 		}
 		autoConfusion thee = Thing_new (Confusion);
-		my structTableOfReal :: v_copy (thee.peek());
+		my structTableOfReal :: v_copy (thee.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Confusion.");
@@ -460,7 +483,7 @@ autoConfusion TableOfReal_to_Confusion (TableOfReal me) {
 autoConfusion Confusion_group (Confusion me, const char32 *labels, const char32 *newLabel, long newpos) {
 	try {
 		autoConfusion stim = Confusion_groupStimuli (me, labels, newLabel, newpos);
-		autoConfusion thee = Confusion_groupResponses (stim.peek(), labels, newLabel, newpos);
+		autoConfusion thee = Confusion_groupResponses (stim.get(), labels, newLabel, newpos);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not grouped.");
@@ -506,7 +529,7 @@ autoConfusion Confusion_groupStimuli (Confusion me, const char32 *labels, const
 		autoConfusion thee = Confusion_create (newnstim, my numberOfColumns);
 		NUMstrings_copyElements (my columnLabels, thy columnLabels, 1, my numberOfColumns);
 
-		TableOfReal_setRowLabel (thee.peek(), newpos, newLabel);
+		TableOfReal_setRowLabel (thee.get(), newpos, newLabel);
 		long inewrow = 1;
 		for (long i = 1; i <= my numberOfRows; i++) {
 			long rowpos = newpos;
@@ -516,7 +539,7 @@ autoConfusion Confusion_groupStimuli (Confusion me, const char32 *labels, const
 				}
 				rowpos = inewrow;
 				inewrow++;
-				TableOfReal_setRowLabel (thee.peek(), rowpos, my rowLabels[i]);
+				TableOfReal_setRowLabel (thee.get(), rowpos, my rowLabels[i]);
 			}
 			for (long j = 1; j <= my numberOfColumns; j++) {
 				thy data[rowpos][j] += my data[i][j];
@@ -566,7 +589,7 @@ autoConfusion Confusion_groupResponses (Confusion me, const char32 *labels, cons
 		}
 		autoConfusion thee = Confusion_create (my numberOfRows, newnresp);
 		NUMstrings_copyElements (my rowLabels, thy rowLabels, 1, my numberOfRows);
-		TableOfReal_setColumnLabel (thee.peek(), newpos, newLabel);
+		TableOfReal_setColumnLabel (thee.get(), newpos, newLabel);
 		long inewcol = 1;
 		for (long i = 1; i <= my numberOfColumns; i++) {
 			long colpos = newpos;
@@ -576,7 +599,7 @@ autoConfusion Confusion_groupResponses (Confusion me, const char32 *labels, cons
 				}
 				colpos = inewcol;
 				inewcol++;
-				TableOfReal_setColumnLabel (thee.peek(), colpos, my columnLabels[i]);
+				TableOfReal_setColumnLabel (thee.get(), colpos, my columnLabels[i]);
 			}
 			for (long j = 1; j <= my numberOfRows; j++) {
 				thy data[j][colpos] += my data[j][i];
@@ -626,7 +649,7 @@ void Confusion_drawAsNumbers (Confusion me, Graphics g, int marginals, int iform
 	autoTableOfReal athee;
 	if (marginals) {
 		athee = Confusion_to_TableOfReal_marginals (me);
-		thee = athee.peek();
+		thee = athee.get();
 	}
 	TableOfReal_drawAsNumbers (thee, g, 1, thy numberOfRows, iformat, precision);
 }
