@@ -3,21 +3,20 @@
 
 /* Eigen_and_Matrix.h
  *
- * Copyright (C) 1993-2011, 2015 David Weenink
+ * Copyright (C) 1993-2011,2015-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -29,17 +28,15 @@
 #include "Eigen.h"
 #include "Matrix.h"
 
-autoMatrix Eigen_and_Matrix_project (Eigen me, Matrix thee, long numberOfComponents);
+autoMatrix Eigen_and_Matrix_to_Matrix_projectRows (Eigen me, Matrix thee, long numberOfDimensionsToKeep);
 /*
-	Purpose: project the columns of the matrix (thou) on the 
-	eigenspace (me). 
+	Purpose: project the rows of the matrix (thee) on the eigenspace (me). 
 */
 
-void Eigen_and_Matrix_project_into (Eigen me, Matrix thee, autoMatrix *p_him);
+autoMatrix Eigen_and_Matrix_to_Matrix_projectColumns (Eigen me, Matrix thee, long numberOfDimensionsToKeep);
 /*
-	Purpose: project the columns of the Matrix (thou) on the 
-	eigenspace (me). Result in existing Matrix (him). 
-
+	Purpose: project the columnss of the matrix (thee) on the eigenspace (me). 
 */
 
+
 #endif /* _Eigen_and_Matrix_h_ */
