@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1993-2013, 2015 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -49,8 +48,7 @@
 /*
   Precondition: size (my buffer) >= nbuf
 */
-static void _LongSound_to_multichannel_buffer (LongSound me, short *buffer, long nbuf,
-        int nchannels, int ichannel, long ibuf) {
+static void _LongSound_to_multichannel_buffer (LongSound me, short *buffer, long nbuf, int nchannels, int ichannel, long ibuf) {
 	long numberOfReads = (my nx - 1) / nbuf + 1;
 	long n_to_read = 0;
 
@@ -130,8 +128,7 @@ static void MelderFile_truncate (MelderFile me, long size) {
 
 	MelderFile_close (me);
 
-	hFile = CreateFileW (Melder_peek32toW (my path), fdwAccess, fdwShareMode, lpsa, fdwCreate,
-	                     FILE_ATTRIBUTE_NORMAL, nullptr);
+	hFile = CreateFileW (Melder_peek32toW (my path), fdwAccess, fdwShareMode, lpsa, fdwCreate, FILE_ATTRIBUTE_NORMAL, nullptr);
 	if (hFile == INVALID_HANDLE_VALUE) {
 		Melder_throw (U"Can't open file ", me, U".");
 	}
@@ -170,18 +167,17 @@ static void writePartToOpenFile16 (LongSound me, int audioFileType, long imin, l
 			long numberOfSamplesToCopy = ibuffer < numberOfBuffers ? my nmax : numberOfSamplesInLastBuffer;
 			LongSound_readAudioToShort (me, my buffer, offset, numberOfSamplesToCopy);
 			offset += numberOfSamplesToCopy;
-			MelderFile_writeShortToAudio (file, my numberOfChannels,
-			    Melder_defaultAudioFileEncoding (audioFileType, numberOfBitsPerSamplePoint), my buffer, numberOfSamplesToCopy);
+			MelderFile_writeShortToAudio (file, my numberOfChannels, Melder_defaultAudioFileEncoding (audioFileType, numberOfBitsPerSamplePoint), my buffer, numberOfSamplesToCopy);
 		}
 	}
-	/*
-	 * We "have" no samples any longer.
-	 */
+
+	//  We "have" no samples any longer.
+	
 	my imin = 1;
 	my imax = 0;
 }
 
-void LongSounds_appendToExistingSoundFile (Collection me, MelderFile file) {
+void LongSounds_appendToExistingSoundFile (OrderedOf<structSampled>* me, MelderFile file) {
 	long pre_append_endpos = 0, numberOfBitsPerSamplePoint = 16;
 	try {
 		if (my size < 1) {
@@ -204,8 +200,7 @@ void LongSounds_appendToExistingSoundFile (Collection me, MelderFile file) {
 		long startOfData;
 		int32 numberOfSamples;
 		int numberOfChannels, encoding;
-		int audioFileType = MelderFile_checkSoundFile (file, &numberOfChannels,
-		                    &encoding, &sampleRate_d, &startOfData, &numberOfSamples);
+		int audioFileType = MelderFile_checkSoundFile (file, &numberOfChannels, &encoding, &sampleRate_d, &startOfData, &numberOfSamples);
 
 		if (audioFileType == 0) {
 			Melder_throw (U"Not a sound file.");
@@ -214,9 +209,9 @@ void LongSounds_appendToExistingSoundFile (Collection me, MelderFile file) {
 		// Check whether all the sample rates and channels match.
 
 		long sampleRate = (long) floor (sampleRate_d);
-		for (long i = 1; i <= my size; i++) {
-			int sampleRatesMatch, numbersOfChannelsMatch;
-			Sampled data = (Sampled) my item [i];
+		for (long i = 1; i <= my size; i ++) {
+			bool sampleRatesMatch, numbersOfChannelsMatch;
+			Sampled data = my at [i];
 			if (data -> classInfo == classSound) {
 				Sound sound = (Sound) data;
 				sampleRatesMatch = floor (1.0 / sound -> dx + 0.5) == sampleRate;
@@ -243,8 +238,8 @@ void LongSounds_appendToExistingSoundFile (Collection me, MelderFile file) {
 		pre_append_endpos = MelderFile_tell (file);
 
 		errno = 0;
-		for (long i = 1; i <= my size; i++) {
-			Sampled data = (Sampled) my item [i];
+		for (long i = 1; i <= my size; i ++) {
+			Sampled data = my at [i];
 			if (data -> classInfo == classSound) {
 				Sound sound = (Sound) data;
 				MelderFile_writeFloatToAudio (file, sound -> ny, Melder_defaultAudioFileEncoding
