@@ -2,21 +2,20 @@
  *
  * Mel Frequency Cepstral Coefficients class.
  *
- * Copyright (C) 1993-2014 David Weenink
+ * Copyright (C) 1993-2015 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -31,12 +30,11 @@
 
 Thing_implement (MFCC, CC, 1);
 
-MFCC MFCC_create (double tmin, double tmax, long nt, double dt, double t1,
-	  long maximumNumberOfCoefficients, double fmin_mel, double fmax_mel) {
+autoMFCC MFCC_create (double tmin, double tmax, long nt, double dt, double t1, long maximumNumberOfCoefficients, double fmin_mel, double fmax_mel) {
 	try {
 		autoMFCC me = Thing_new (MFCC);
-		CC_init (me.peek(), tmin, tmax, nt, dt, t1, maximumNumberOfCoefficients, fmin_mel, fmax_mel);
-		return me.transfer();
+		CC_init (me.get(), tmin, tmax, nt, dt, t1, maximumNumberOfCoefficients, fmin_mel, fmax_mel);
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"MFCC not created.");
 	}
@@ -61,12 +59,12 @@ void MFCC_lifter (MFCC me, long lifter) {
 	}
 }
 
-TableOfReal MFCC_to_TableOfReal (MFCC me, bool includeC0) {
+autoTableOfReal MFCC_to_TableOfReal (MFCC me, bool includeC0) {
 	try {
 		long numberOfColumns = my maximumNumberOfCoefficients + (includeC0 ? 1 : 0);
 		autoTableOfReal thee = TableOfReal_create (my nx, numberOfColumns);
 		for (long i = 1; i <= numberOfColumns; i++) {
-			TableOfReal_setColumnLabel (thee.peek(), i, Melder_cat (U"c", includeC0 ? i - 1 : i));
+			TableOfReal_setColumnLabel (thee.get(), i, Melder_cat (U"c", includeC0 ? i - 1 : i));
 		}
 		long offset = includeC0 ? 1 : 0;
 		for (long iframe = 1; iframe <= my nx; iframe++) {
@@ -78,14 +76,14 @@ TableOfReal MFCC_to_TableOfReal (MFCC me, bool includeC0) {
 				thy data[iframe][1] = cf -> c0;
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to TabelOfReal.");
 	}
 }
 
 // as_Sound not to_Sound
-Sound MFCC_to_Sound (MFCC me) {
+autoSound MFCC_to_Sound (MFCC me) {
 	try {
 		autoSound thee = Sound_create (my maximumNumberOfCoefficients, my xmin, my xmax, my nx, my dx, my x1);
 		for (long iframe = 1; iframe <= my nx; iframe++) {
@@ -94,13 +92,13 @@ Sound MFCC_to_Sound (MFCC me) {
 				thy z[j][iframe] = cf -> c[j];
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not represented as Sound.");
 	}
 }
 
-Sound MFCCs_crossCorrelate (MFCC me, MFCC thee, enum kSounds_convolve_scaling scaling, enum kSounds_convolve_signalOutsideTimeDomain signalOutsideTimeDomain) {
+autoSound MFCCs_crossCorrelate (MFCC me, MFCC thee, enum kSounds_convolve_scaling scaling, enum kSounds_convolve_signalOutsideTimeDomain signalOutsideTimeDomain) {
 	try {
 		if (my dx != thy dx) {
 			Melder_throw (U"The samplings of the two MFCC's have to be equal.");
@@ -110,14 +108,14 @@ Sound MFCCs_crossCorrelate (MFCC me, MFCC thee, enum kSounds_convolve_scaling sc
 		}
 		autoSound target = MFCC_to_Sound (me);
 		autoSound source = MFCC_to_Sound (thee);
-		autoSound cc = Sounds_crossCorrelate (target.peek(), source.peek(), scaling, signalOutsideTimeDomain);
-		return cc.transfer();
+		autoSound cc = Sounds_crossCorrelate (target.get(), source.get(), scaling, signalOutsideTimeDomain);
+		return cc;
 	} catch (MelderError) {
 		Melder_throw (U"No cross-correlation between ", me, U" and ", thee, U" calculated.");
 	}
 }
 
-Sound MFCCs_convolve (MFCC me, MFCC thee, enum kSounds_convolve_scaling scaling, enum kSounds_convolve_signalOutsideTimeDomain signalOutsideTimeDomain) {
+autoSound MFCCs_convolve (MFCC me, MFCC thee, enum kSounds_convolve_scaling scaling, enum kSounds_convolve_signalOutsideTimeDomain signalOutsideTimeDomain) {
 	try {
 		if (my dx != thy dx) {
 			Melder_throw (U"The samplings of the two MFCC's have to be equal.");
@@ -127,8 +125,8 @@ Sound MFCCs_convolve (MFCC me, MFCC thee, enum kSounds_convolve_scaling scaling,
 		}
 		autoSound target = MFCC_to_Sound (me);
 		autoSound source = MFCC_to_Sound (thee);
-		autoSound cc = Sounds_convolve (target.peek(), source.peek(), scaling, signalOutsideTimeDomain);
-		return cc.transfer();
+		autoSound cc = Sounds_convolve (target.get(), source.get(), scaling, signalOutsideTimeDomain);
+		return cc;
 	} catch (MelderError) {
 		Melder_throw (me, U" and ", thee, U" not convolved.");
 	}
@@ -152,7 +150,7 @@ static double CC_Frames_distance (CC_Frame me, CC_Frame thee, bool includeEnergy
  * 3: spectral center of gravity (gs)
  * 4: stable internal duration
  */
-Matrix MFCC_to_Matrix_features (MFCC me, double windowLength, bool includeEnergy) {
+autoMatrix MFCC_to_Matrix_features (MFCC me, double windowLength, bool includeEnergy) {
 	try {
 		long nw = (long) floor (windowLength / my dx / 2);
 		autoMelSpectrogram him = MFCC_to_MelSpectrogram (me, 0, 0, 1);
@@ -209,7 +207,7 @@ Matrix MFCC_to_Matrix_features (MFCC me, double windowLength, bool includeEnergy
 			
 			// 4: stable internal duration
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no features calculated.");
 	}
