@@ -4,19 +4,18 @@
  *
  * Copyright (C) 1993-2012, 2014, 2015 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -95,7 +94,10 @@ autoPolygon Sounds_to_Polygon_enclosed (Sound me, Sound thee, int channel, doubl
 
 autoPolygon Polygons_union (Polygon me, Polygon thee);
 
-autoCollection Polygons_clip (Polygon me, Polygon thee);
+Collection_define (PolygonBag, CollectionOf, Polygon) {
+};
+
+autoPolygonBag Polygons_clip (Polygon me, Polygon thee);
 
 autoPolygon Polygon_convexHull (Polygon me);
 
