@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1993-2012, 2014, 2015 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -71,17 +70,18 @@
 #define Graphics_TWOWAYARROW 2
 #define Graphics_LINE 3
 
-static TableOfReal TableOfReal_and_TableOfReal_columnCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize);
-static TableOfReal TableOfReal_and_TableOfReal_rowCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize);
+static autoTableOfReal TableOfReal_and_TableOfReal_columnCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize);
+static autoTableOfReal TableOfReal_and_TableOfReal_rowCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize);
 
 long TableOfReal_getColumnIndexAtMaximumInRow (TableOfReal me, long rowNumber) {
 	long columnNumber = 0;
 	if (rowNumber > 0 && rowNumber <= my numberOfRows) {
-		double max = my data[rowNumber][1];
+		double max = my data [rowNumber] [1];
 		columnNumber = 1;
-		for (long icol = 2; icol <= my numberOfColumns; icol++) {
-			if (my data[rowNumber][icol] > max) {
-				max = my data[rowNumber][icol]; columnNumber = icol;
+		for (long icol = 2; icol <= my numberOfColumns; icol ++) {
+			if (my data [rowNumber] [icol] > max) {
+				max = my data [rowNumber] [icol];
+				columnNumber = icol;
 			}
 		}
 	}
@@ -93,42 +93,6 @@ const char32 *TableOfReal_getColumnLabelAtMaximumInRow (TableOfReal me, long row
 	return my v_getColStr (columnNumber);
 }
 
-
-int TableOfReal_areAllCellsDefined (TableOfReal me, long rb, long re, long cb, long ce) {
-	if (re <= rb || rb < 1 || re > my numberOfRows) {
-		rb = 1; re = my numberOfRows;
-	}
-	if (ce <= cb || cb < 1 || ce > my numberOfColumns) {
-		cb = 1; ce = my numberOfColumns;
-	}
-
-	autoNUMvector<long> invalid_columns (1, my numberOfColumns);
-
-	long numberOfInvalidRows = 0, numberOfInvalidColumns = 0;
-	for (long i = rb; i <= re; i++) {
-		for (long j = cb; j <= ce; j++) {
-			long rowcount = 0;
-			if (my data[i][j] == NUMundefined) {
-				invalid_columns[j]++;
-				rowcount++;
-			}
-			if (rowcount > 0) {
-				numberOfInvalidRows++;
-			}
-		}
-	}
-	if (numberOfInvalidRows != 0) {
-		for (long j = 1; j <= my numberOfColumns; j++) {
-			if (invalid_columns[j] > 0) {
-				numberOfInvalidColumns++;
-			}
-		}
-		Melder_throw (numberOfInvalidRows == 1 ? U"One row contains invalid data." : (numberOfInvalidColumns == 1 ?  U"One column contains invalid data." :
-		              U"Several rows and columns contain invalid data."));
-	}
-	return numberOfInvalidRows == 0 ? 1 : 0;
-}
-
 void TableOfReal_copyOneRowWithLabel (TableOfReal me, TableOfReal thee, long myrow, long thyrow) {
 	try {
 		if (me == thee && myrow == thyrow) {
@@ -174,7 +138,7 @@ int TableOfReal_hasColumnLabels (TableOfReal me) {
 	return 1;
 }
 
-TableOfReal TableOfReal_createIrisDataset () {
+autoTableOfReal TableOfReal_createIrisDataset () {
 	double iris[150][4] = {
 		{5.1, 3.5, 1.4, 0.2}, {4.9, 3.0, 1.4, 0.2}, {4.7, 3.2, 1.3, 0.2}, {4.6, 3.1, 1.5, 0.2}, {5.0, 3.6, 1.4, 0.2},
 		{5.4, 3.9, 1.7, 0.4}, {4.6, 3.4, 1.4, 0.3}, {5.0, 3.4, 1.5, 0.2}, {4.4, 2.9, 1.4, 0.2}, {4.9, 3.1, 1.5, 0.1},
@@ -211,26 +175,26 @@ TableOfReal TableOfReal_createIrisDataset () {
 	try {
 		autoTableOfReal me = TableOfReal_create (150, 4);
 
-		TableOfReal_setColumnLabel (me.peek(), 1, U"sl");
-		TableOfReal_setColumnLabel (me.peek(), 2, U"sw");
-		TableOfReal_setColumnLabel (me.peek(), 3, U"pl");
-		TableOfReal_setColumnLabel (me.peek(), 4, U"pw");
+		TableOfReal_setColumnLabel (me.get(), 1, U"sl");
+		TableOfReal_setColumnLabel (me.get(), 2, U"sw");
+		TableOfReal_setColumnLabel (me.get(), 3, U"pl");
+		TableOfReal_setColumnLabel (me.get(), 4, U"pw");
 		for (long i = 1; i <= 150; i++) {
 			int kind = (i - 1) / 50 + 1;
 			char32 const *label = kind == 1 ? U"1" : kind == 2 ? U"2" : U"3";
 			for (long j = 1; j <= 4; j++) {
 				my data[i][j] = iris[i - 1][j - 1];
 			}
-			TableOfReal_setRowLabel (me.peek(), i, label);
+			TableOfReal_setRowLabel (me.get(), i, label);
 		}
-		Thing_setName (me.peek(), U"iris");
-		return me.transfer();
+		Thing_setName (me.get(), U"iris");
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal from iris data not created.");
 	}
 }
 
-Strings TableOfReal_extractRowLabels (TableOfReal me) {
+autoStrings TableOfReal_extractRowLabels (TableOfReal me) {
 	try {
 		autoStrings thee = Thing_new (Strings);
 
@@ -244,13 +208,13 @@ Strings TableOfReal_extractRowLabels (TableOfReal me) {
 				thy strings[i] = Melder_dup (label);
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": row labels not extracted.");
 	}
 }
 
-Strings TableOfReal_extractColumnLabels (TableOfReal me) {
+autoStrings TableOfReal_extractColumnLabels (TableOfReal me) {
 	try {
 		autoStrings thee = Thing_new (Strings);
 
@@ -263,13 +227,13 @@ Strings TableOfReal_extractColumnLabels (TableOfReal me) {
 				thy strings[i] = Melder_dup (label);
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": column labels not extracted.");
 	}
 }
 
-TableOfReal TableOfReal_transpose (TableOfReal me) {
+autoTableOfReal TableOfReal_transpose (TableOfReal me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfColumns, my numberOfRows);
 
@@ -280,14 +244,15 @@ TableOfReal TableOfReal_transpose (TableOfReal me) {
 		}
 		NUMstrings_copyElements (my rowLabels, thy columnLabels, 1, my numberOfRows);
 		NUMstrings_copyElements (my columnLabels, thy rowLabels, 1, my numberOfColumns);
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not transposed.");
 	}
 }
 
-int TableOfReal_to_Pattern_and_Categories (TableOfReal me, long fromrow, long torow, long fromcol, long tocol, Pattern *p, Categories *c) {
-	*p = nullptr; *c = nullptr;
+void TableOfReal_to_PatternList_and_Categories (TableOfReal me, long fromrow, long torow, long fromcol, long tocol,
+	autoPatternList *p, autoCategories *c)
+{
 	try {
 		long ncol = my numberOfColumns, nrow = my numberOfRows;
 
@@ -308,44 +273,50 @@ int TableOfReal_to_Pattern_and_Categories (TableOfReal me, long fromrow, long to
 
 		nrow = torow - fromrow + 1;
 		ncol = tocol - fromcol + 1;
-		autoPattern ap = Pattern_create (nrow, ncol);
+		autoPatternList ap = PatternList_create (nrow, ncol);
 		autoCategories ac = Categories_create ();
 
 		long row = 1;
-		for (long i = fromrow; i <= torow; i++, row++) {
-			char32 const *s = my rowLabels[i] ? my rowLabels[i] : U"?";
+		for (long i = fromrow; i <= torow; i ++, row ++) {
+			char32 const *s = my rowLabels [i] ? my rowLabels [i] : U"?";
 			autoSimpleString item = SimpleString_create (s);
-			Collection_addItem (ac.peek(), item.transfer());
+			ac -> addItem_move (item.move());
 			long col = 1;
-			for (long j = fromcol; j <= tocol; j++, col++) {
-				ap -> z[row][col] = my data[i][j];
+			for (long j = fromcol; j <= tocol; j ++, col ++) {
+				ap -> z [row] [col] = my data [i] [j];
 			}
 		}
-		*p = ap.transfer();
-		*c = ac.transfer();
-		return 1;
+		*p = ap.move();
+		*c = ac.move();
 	} catch (MelderError) {
-		Melder_throw (U"Pattern and Categories not created from TableOfReal.");
+		Melder_throw (U"PatternList and Categories not created from TableOfReal.");
 	}
 }
 
-void TableOfReal_getColumnExtrema (TableOfReal me, long col, double *min, double *max) {
-	*min = NUMundefined; *max = NUMundefined;
+void TableOfReal_getColumnExtrema (TableOfReal me, long col, double *p_min, double *p_max) {
+	double min = NUMundefined, max = NUMundefined;
 	if (col < 1 || col > my numberOfColumns) {
 		Melder_throw (U"Invalid column number.");
 	}
-	*min = *max = my data[1][col];
+	min = max = my data[1][col];
 	for (long i = 2; i <= my numberOfRows; i++) {
-		if (my data[i][col] > *max) {
-			*max = my data[i][col];
-		} else if (my data[i][col] < *min) {
-			*min = my data[i][col];
+		if (my data[i][col] > max) {
+			max = my data[i][col];
+		} else if (my data[i][col] < min) {
+			min = my data[i][col];
 		}
 	}
+	if (p_min) {
+		*p_min = min;
+	}
+	if (p_max) {
+		*p_max = max;
+	}
 }
 
 void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, const char32 *rows, long colb, long cole, double ymin,
-	double ymax, double xoffsetFraction, double interbarFraction, double interbarsFraction, const char32 *greys, int garnish) {
+	double ymax, double xoffsetFraction, double interbarFraction, double interbarsFraction, const char32 *greys, bool garnish)
+{
 	if (colb >= cole) {
 		colb = 1; cole = my numberOfColumns;
 	}
@@ -400,7 +371,7 @@ void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, const char32 *
 				}
 				Graphics_setGrey (g, grey);
 				Graphics_fillRectangle (g, x1, x2, y1, y2);
-				Graphics_setGrey (g, 0); /* Black */
+				Graphics_setGrey (g, 0.0);   // black
 				Graphics_rectangle (g, x1, x2, y1, y2);
 			}
 			x1 += dx;
@@ -411,9 +382,9 @@ void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, const char32 *
 
 	if (garnish) {
 		double xb = (xoffsetFraction + 0.5 * (nrows + (nrows - 1) * interbarFraction)) * bar_width;
-		for (long j = colb; j <= cole; j++) {
-			if (my columnLabels[j]) {
-				Graphics_markBottom (g, xb, false, false, false, my columnLabels[j]);
+		for (long j = colb; j <= cole; j ++) {
+			if (my columnLabels [j]) {
+				Graphics_markBottom (g, xb, false, false, false, my columnLabels [j]);
 			}
 			xb += dx;
 		}
@@ -422,7 +393,7 @@ void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, const char32 *
 	}
 }
 
-void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xmax, double ymin, double ymax, double sv_splitfactor, int labelsize, int garnish) {
+void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xmax, double ymin, double ymax, double sv_splitfactor, int labelsize, bool garnish) {
 	long nr = my numberOfRows, nc = my numberOfColumns, nPoints = nr + nc;
 	int fontsize = Graphics_inqFontSize (g);
 
@@ -431,8 +402,8 @@ void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xma
 	NUMmatrix_copyElements (my data, svd -> u, 1, nr, 1, nc);
 	NUMcentreColumns (svd -> u, 1, nr, 1, nc, nullptr);
 
-	SVD_compute (svd.peek());
-	long numberOfZeroed = SVD_zeroSmallSingularValues (svd.peek(), 0.0);
+	SVD_compute (svd.get());
+	long numberOfZeroed = SVD_zeroSmallSingularValues (svd.get(), 0.0);
 
 	long nmin = MIN (nr, nc) - numberOfZeroed;
 	if (nmin < 2) {
@@ -475,15 +446,15 @@ void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xma
 	}
 	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
 
-	for (long i = 1; i <= nPoints; i++) {
+	for (long i = 1; i <= nPoints; i ++) {
 		char32 const *label;
 		if (i <= nr) {
-			label = my rowLabels[i];
+			label = my rowLabels [i];
 			if (! label) {
 				label = U"?__r_";
 			}
 		} else {
-			label = my columnLabels[i - nr];
+			label = my columnLabels [i - nr];
 			if (! label) {
 				label = U"?__c_";
 			}
@@ -504,7 +475,7 @@ void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xma
 	}
 }
 
-void TableOfReal_drawBoxPlots (TableOfReal me, Graphics g, long rowmin, long rowmax, long colmin, long colmax, double ymin, double ymax, int garnish) {
+void TableOfReal_drawBoxPlots (TableOfReal me, Graphics g, long rowmin, long rowmax, long colmin, long colmax, double ymin, double ymax, bool garnish) {
 	if (rowmax < rowmin || rowmax < 1) {
 		rowmin = 1; rowmax = my numberOfRows;
 	}
@@ -615,15 +586,15 @@ void TableOfReal_labelsFromCollectionItemNames (TableOfReal me, Collection thee,
 	try {
 		if (row) {
 			Melder_assert (my numberOfRows == thy size);
-			for (long i = 1; i <= my numberOfRows; i++) {
-				const char32 *name = Thing_getName ( (Thing) thy item[i]);
+			for (long i = 1; i <= my numberOfRows; i ++) {
+				const char32 *name = Thing_getName (thy at [i]);
 				TableOfReal_setRowLabel (me, i, name);
 			}
 		}
 		if (column) {
 			Melder_assert (my numberOfColumns == thy size);
-			for (long i = 1; i <= my numberOfColumns; i++) {
-				const char32 *name = Thing_getName ( (Thing) thy item[i]);
+			for (long i = 1; i <= my numberOfColumns; i ++) {
+				const char32 *name = Thing_getName (thy at [i]);
 				TableOfReal_setColumnLabel (me, i, name);
 			}
 		}
@@ -648,13 +619,16 @@ void TableOfReal_and_Categories_setRowLabels (TableOfReal me, Categories thee) {
 				and then delete the newly created categories.
 		*/
 
-		autoCategories c = Data_copy (thee);
+		autoCategories categories_copy = Data_copy (thee);
 
-		for (long i = 1; i <= my numberOfRows; i++) {
-			SimpleString s = (SimpleString) c -> item[i];
-			char32 *t = s -> string;
-			s -> string = my rowLabels[i];
-			my rowLabels[i] = t;
+		for (long i = 1; i <= my numberOfRows; i ++) {
+			/*
+				The modification of my rowLabels is implemented as a swap
+				with the contents of the SimpleStrings in the copied Categories.
+			*/
+			char32 *tmp = categories_copy->at [i] -> string;
+			categories_copy->at [i] -> string = my rowLabels [i];
+			my rowLabels [i] = tmp;
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": row labels not set from categories.");
@@ -662,14 +636,15 @@ void TableOfReal_and_Categories_setRowLabels (TableOfReal me, Categories thee) {
 }
 
 void TableOfReal_centreColumns_byRowLabel (TableOfReal me) {
-	char32 *label = my rowLabels[1];
+	char32 *label = my rowLabels [1];
 	long index = 1;
 
-	for (long i = 2; i <= my numberOfRows; i++) {
-		char32 *li = my rowLabels[i];
-		if (Melder_cmp (li, label) != 0) {
+	for (long i = 2; i <= my numberOfRows; i ++) {
+		char32 *li = my rowLabels [i];
+		if (! Melder_equ (li, label)) {
 			NUMcentreColumns (my data, index, i - 1, 1, my numberOfColumns, 0);
-			label = li; index = i;
+			label = li;
+			index = i;
 		}
 	}
 	NUMcentreColumns (my data, index, my numberOfRows, 1, my numberOfColumns, nullptr);
@@ -681,8 +656,8 @@ double TableOfReal_getRowSum (TableOfReal me, long index) {
 	}
 
 	double sum = 0.0;
-	for (long j = 1; j <= my numberOfColumns; j++) {
-		sum += my data[index][j];
+	for (long j = 1; j <= my numberOfColumns; j ++) {
+		sum += my data [index] [j];
 	}
 	return sum;
 }
@@ -709,17 +684,17 @@ double TableOfReal_getColumnSum (TableOfReal me, long index) {
 	}
 
 	double sum = 0.0;
-	for (long i = 1; i <= my numberOfRows; i++) {
-		sum += my data[i][index];
+	for (long i = 1; i <= my numberOfRows; i ++) {
+		sum += my data [i] [index];
 	}
 	return sum;
 }
 
 double TableOfReal_getGrandSum (TableOfReal me) {
 	double sum = 0.0;
-	for (long i = 1; i <= my numberOfRows; i++) {
-		for (long j = 1; j <= my numberOfColumns; j++) {
-			sum += my data[i][j];
+	for (long i = 1; i <= my numberOfRows; i ++) {
+		for (long j = 1; j <= my numberOfColumns; j ++) {
+			sum += my data [i] [j];
 		}
 	}
 	return sum;
@@ -860,7 +835,7 @@ void TableOfReal_drawScatterPlotMatrix (TableOfReal me, Graphics g, long colb, l
 	Graphics_unsetInner (g);
 }
 
-void TableOfReal_drawAsSquares_area (TableOfReal me, Graphics g, double zmin, double zmax, double cellSizeFactor, int randomFillOrder, int garnish) {
+void TableOfReal_drawAsSquares_area (TableOfReal me, Graphics g, double zmin, double zmax, double cellSizeFactor, int randomFillOrder, bool garnish) {
 	try {
 		cellSizeFactor = cellSizeFactor <= 0.0 ? 1.0 : cellSizeFactor;
 		if (zmin == 0 && zmax == 0) {
@@ -881,7 +856,7 @@ void TableOfReal_drawAsSquares_area (TableOfReal me, Graphics g, double zmin, do
 	}
 }
 
-void TableOfReal_drawScatterPlot (TableOfReal me, Graphics g, long icx, long icy, long rowb, long rowe, double xmin, double xmax, double ymin, double ymax, int labelSize, int useRowLabels, const char32 *label, int garnish) {
+void TableOfReal_drawScatterPlot (TableOfReal me, Graphics g, long icx, long icy, long rowb, long rowe, double xmin, double xmax, double ymin, double ymax, int labelSize, bool useRowLabels, const char32 *label, bool garnish) {
 	double m = my numberOfRows, n = my numberOfColumns;
 	int fontSize = Graphics_inqFontSize (g);
 
@@ -963,61 +938,45 @@ void TableOfReal_drawScatterPlot (TableOfReal me, Graphics g, long icx, long icy
 	}
 }
 
-/****************  TABLESOFREAL **************************************/
 
-Thing_implement (TablesOfReal, Ordered, 0);
+#pragma mark - class TableOfRealList
 
-void TablesOfReal_init (TablesOfReal me, ClassInfo klas) {
-	Ordered_init (me, klas, 10);
-}
-
-TablesOfReal TablesOfReal_create () {
-	try {
-		autoTablesOfReal me = Thing_new (TablesOfReal);
-		TablesOfReal_init (me.peek(), classTableOfReal);
-		return me.transfer();
-	} catch (MelderError) {
-		Melder_throw (U"TablesOfReal not created.");
-	}
-}
-
-TableOfReal TablesOfReal_sum (TablesOfReal me) {
+autoTableOfReal TableOfRealList_sum (TableOfRealList me) {
 	try {
 		if (my size <= 0) {
-			return nullptr;
+			return autoTableOfReal();
 		}
-		autoTableOfReal thee = Data_copy ((TableOfReal) my item[1]);
+		autoTableOfReal thee = Data_copy (my at [1]);
 
-		for (long i = 2; i <= my size; i++) {
-			TableOfReal him = (TableOfReal) my item[i];
-			if (thy numberOfRows != his numberOfRows || thy numberOfColumns != his numberOfColumns || ! TableOfReal_equalLabels (thee.peek(), him, 1, 1)) {
+		for (long i = 2; i <= my size; i ++) {
+			TableOfReal him = my at [i];
+			if (thy numberOfRows != his numberOfRows || thy numberOfColumns != his numberOfColumns || ! TableOfReal_equalLabels (thee.get(), him, 1, 1)) {
 				Melder_throw (U"Dimensions or labels differ for table 1 and ", i, U".");
 			}
-			for (long j = 1; j <= thy numberOfRows; j++) {
-				for (long k = 1; k <= thy numberOfColumns; k++) {
-					thy data[j][k] += his data[j][k];
+			for (long j = 1; j <= thy numberOfRows; j ++) {
+				for (long k = 1; k <= thy numberOfColumns; k ++) {
+					thy data [j] [k] += his data [j] [k];
 				}
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": sum not created.");
 	}
 }
 
-int TablesOfReal_checkDimensions (TablesOfReal me) {
+bool TableOfRealList_haveIdenticalDimensions (TableOfRealList me) {
 	if (my size < 2) {
-		return 1;
+		return true;
 	}
-	TableOfReal t1 = (TableOfReal) my item[1];
-	for (long i = 2; i <= my size; i++) {
-		TableOfReal t = (TableOfReal) my item[i];
-		if (t -> numberOfColumns != t1 -> numberOfColumns ||
-		        t -> numberOfRows != t1 -> numberOfRows) {
-			return 0;
+	TableOfReal t1 = my at [1];
+	for (long i = 2; i <= my size; i ++) {
+		TableOfReal t = my at [i];
+		if (t -> numberOfColumns != t1 -> numberOfColumns || t -> numberOfRows != t1 -> numberOfRows) {
+			return false;
 		}
 	}
-	return 1;
+	return true;
 }
 
 double TableOfReal_getColumnQuantile (TableOfReal me, long col, double quantile) {
@@ -1039,7 +998,7 @@ double TableOfReal_getColumnQuantile (TableOfReal me, long col, double quantile)
 	}
 }
 
-static TableOfReal TableOfReal_createPolsVanNieropData (int choice, int include_levels) {
+static autoTableOfReal TableOfReal_createPolsVanNieropData (int choice, bool include_levels) {
 	try {
 		autoTable table = Table_createFromPolsVanNieropData ();
 
@@ -1056,78 +1015,78 @@ static TableOfReal TableOfReal_createPolsVanNieropData (int choice, int include_
 
 		autoTableOfReal thee = TableOfReal_create (nrows, ncols);
 
-		for (long i = 1; i <= nrows; i++) {
-			TableRow row = (TableRow) table -> rows -> item[ib + i - 1];
-			TableOfReal_setRowLabel (thee.peek(), i, row -> cells[4].string);
+		for (long i = 1; i <= nrows; i ++) {
+			TableRow row = table -> rows.at [ib + i - 1];
+			TableOfReal_setRowLabel (thee.get(), i, row -> cells [4]. string);
 			for (long j = 1; j <= 3; j++) {
-				thy data[i][j] = Melder_atof (row -> cells[4 + j].string);
+				thy data [i] [j] = Melder_atof (row -> cells [4 + j]. string);
 				if (include_levels) {
-					thy data[i][3 + j] = Melder_atof (row -> cells[7 + j].string);
+					thy data [i] [3 + j] = Melder_atof (row -> cells [7 + j]. string);
 				}
 			}
 		}
-		for (long j = 1; j <= 3; j++) {
-			const char32 *label = table -> columnHeaders[4 + j].label;
-			TableOfReal_setColumnLabel (thee.peek(), j, label);
+		for (long j = 1; j <= 3; j ++) {
+			const char32 *label = table -> columnHeaders [4 + j]. label;
+			TableOfReal_setColumnLabel (thee.get(), j, label);
 			if (include_levels) {
 				label = table -> columnHeaders[7 + j].label;
-				TableOfReal_setColumnLabel (thee.peek(), 3 + j, label);
+				TableOfReal_setColumnLabel (thee.get(), 3 + j, label);
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal from Pols & Van Nierop data not created.");
 	}
 }
 
-TableOfReal TableOfReal_createFromPolsData_50males (int include_levels) {
+autoTableOfReal TableOfReal_createFromPolsData_50males (bool include_levels) {
 	return TableOfReal_createPolsVanNieropData (1, include_levels);
 }
 
-TableOfReal TableOfReal_createFromVanNieropData_25females (int include_levels) {
+autoTableOfReal TableOfReal_createFromVanNieropData_25females (bool include_levels) {
 	return TableOfReal_createPolsVanNieropData (2, include_levels);
 }
 
-TableOfReal TableOfReal_createFromWeeninkData (int option) {
+autoTableOfReal TableOfReal_createFromWeeninkData (int option) {
 	try {
 		long nvowels = 12, ncols = 3, nrows = 10 * nvowels;
 
 		autoTable table = Table_createFromWeeninkData ();
 
-		long ib = option == 1 ? 1 : option == 2 ? 11 : 21; /* m f c*/
+		long ib = ( option == 1 ? 1 : option == 2 ? 11 : 21 ); /* m f c*/
 		ib = (ib - 1) * nvowels + 1;
 
 		autoTableOfReal thee = TableOfReal_create (nrows, ncols);
 
-		for (long i = 1; i <= nrows; i++) {
-			TableRow row = (TableRow) table -> rows -> item[ib + i - 1];
-			TableOfReal_setRowLabel (thee.peek(), i, row -> cells[5].string);
-			for (long j = 1; j <= 3; j++) {
-				thy data[i][j] = Melder_atof (row -> cells[6 + j].string); /* Skip F0 */
+		for (long i = 1; i <= nrows; i ++) {
+			TableRow row = table -> rows.at [ib + i - 1];
+			TableOfReal_setRowLabel (thee.get(), i, row -> cells [5]. string);
+			for (long j = 1; j <= 3; j ++) {
+				thy data [i] [j] = Melder_atof (row -> cells [6 + j]. string); /* Skip F0 */
 			}
 		}
 		for (long j = 1; j <= 3; j++)  {
-			const char32 *label = table -> columnHeaders[6 + j].label;
-			TableOfReal_setColumnLabel (thee.peek(), j, label);
+			const char32 *label = table -> columnHeaders [6 + j]. label;
+			TableOfReal_setColumnLabel (thee.get(), j, label);
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal from Weenink data not created.");
 	}
 }
 
-TableOfReal TableOfReal_randomizeRows (TableOfReal me) {
+autoTableOfReal TableOfReal_randomizeRows (TableOfReal me) {
 	try {
 		autoPermutation p = Permutation_create (my numberOfRows);
-		Permutation_permuteRandomly_inline (p.peek(), 0, 0);
-		autoTableOfReal thee = TableOfReal_and_Permutation_permuteRows (me, p.peek());
-		return thee.transfer();
+		Permutation_permuteRandomly_inline (p.get(), 0, 0);
+		autoTableOfReal thee = TableOfReal_and_Permutation_permuteRows (me, p.get());
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": randomized rows not created");
 	}
 }
 
-TableOfReal TableOfReal_bootstrap (TableOfReal me) {
+autoTableOfReal TableOfReal_bootstrap (TableOfReal me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfRows, my numberOfColumns);
 
@@ -1135,7 +1094,7 @@ TableOfReal TableOfReal_bootstrap (TableOfReal me) {
 
 		for (long i = 1; i <= my numberOfColumns; i++) {
 			if (my columnLabels[i]) {
-				TableOfReal_setColumnLabel (thee.peek(), i, my columnLabels[i]);
+				TableOfReal_setColumnLabel (thee.get(), i, my columnLabels[i]);
 			}
 		}
 
@@ -1149,10 +1108,10 @@ TableOfReal TableOfReal_bootstrap (TableOfReal me) {
 			long p = NUMrandomInteger (1, my numberOfRows);
 			NUMvector_copyElements (my data[p], thy data[i], 1, my numberOfColumns);
 			if (my rowLabels[p]) {
-				TableOfReal_setRowLabel (thee.peek(), i, my rowLabels[p]);
+				TableOfReal_setRowLabel (thee.get(), i, my rowLabels[p]);
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": bootstrapped data not created.");
 	}
@@ -1211,7 +1170,7 @@ long TableOfReal_getNumberOfLabelMatches (TableOfReal me, const char32 *search,
 	return nmatches;
 }
 
-void TableOfReal_drawVectors (TableOfReal me, Graphics g, long colx1, long coly1, long colx2, long coly2, double xmin, double xmax, double ymin, double ymax, int vectype, int labelsize, int garnish) {
+void TableOfReal_drawVectors (TableOfReal me, Graphics g, long colx1, long coly1, long colx2, long coly2, double xmin, double xmax, double ymin, double ymax, int vectype, int labelsize, bool garnish) {
 	long nx = my numberOfColumns, ny = my numberOfRows;
 	int fontsize = Graphics_inqFontSize (g);
 
@@ -1293,18 +1252,18 @@ void TableOfReal_drawVectors (TableOfReal me, Graphics g, long colx1, long coly1
 	}
 }
 
-void TableOfReal_drawColumnAsDistribution (TableOfReal me, Graphics g, int column, double minimum, double maximum, long nBins, double freqMin, double freqMax, int cumulative, int garnish) {
+void TableOfReal_drawColumnAsDistribution (TableOfReal me, Graphics g, int column, double minimum, double maximum, long nBins, double freqMin, double freqMax, bool cumulative, bool garnish) {
 	if (column < 1 || column > my numberOfColumns) {
 		return;
 	}
 	autoMatrix thee = TableOfReal_to_Matrix (me);
-	Matrix_drawDistribution (thee.peek(), g, column - 0.5, column + 0.5, 0.0, 0.0, minimum, maximum, nBins, freqMin, freqMax, cumulative, garnish);
+	Matrix_drawDistribution (thee.get(), g, column - 0.5, column + 0.5, 0.0, 0.0, minimum, maximum, nBins, freqMin, freqMax, cumulative, garnish);
 	if (garnish && my columnLabels[column] != 0) {
 		Graphics_textBottom (g, true, my columnLabels[column]);
 	}
 }
 
-TableOfReal TableOfReal_sortRowsByIndex (TableOfReal me, long *index, int reverse) {
+autoTableOfReal TableOfReal_sortRowsByIndex (TableOfReal me, long index[], int reverse) {
 	try {
 		if (my rowLabels == 0) {
 			Melder_throw (U"No labels to sort");
@@ -1338,7 +1297,7 @@ TableOfReal TableOfReal_sortRowsByIndex (TableOfReal me, long *index, int revers
 		// Copy column labels.
 
 		NUMstrings_copyElements (my columnLabels, thy columnLabels, 1, my numberOfColumns);
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not sorted by row index.");
 	}
@@ -1354,11 +1313,11 @@ long *TableOfReal_getSortedIndexFromRowLabels (TableOfReal me) {
 	}
 }
 
-TableOfReal TableOfReal_sortOnlyByRowLabels (TableOfReal me) {
+autoTableOfReal TableOfReal_sortOnlyByRowLabels (TableOfReal me) {
 	try {
 		autoPermutation index = TableOfReal_to_Permutation_sortRowLabels (me);
-		autoTableOfReal thee = TableOfReal_and_Permutation_permuteRows (me, index.peek());
-		return thee.transfer();
+		autoTableOfReal thee = TableOfReal_and_Permutation_permuteRows (me, index.get());
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not sorted by row labels.");
 	}
@@ -1392,7 +1351,7 @@ static void NUMstatsColumns (double **a, long rb, long re, long cb, long ce, int
 	}
 }
 
-TableOfReal TableOfReal_meansByRowLabels (TableOfReal me, int expand, int stats) {
+autoTableOfReal TableOfReal_meansByRowLabels (TableOfReal me, int expand, int stats) {
 	try {
 		autoTableOfReal thee;
 		autoNUMvector<long> index (TableOfReal_getSortedIndexFromRowLabels (me), 1);
@@ -1407,7 +1366,7 @@ TableOfReal TableOfReal_meansByRowLabels (TableOfReal me, int expand, int stats)
 
 				if (expand == 0) {
 					indexr++;
-					TableOfReal_copyOneRowWithLabel (sorted.peek(), sorted.peek(), indexi, indexr);
+					TableOfReal_copyOneRowWithLabel (sorted.get(), sorted.get(), indexi, indexr);
 				}
 				label = li; indexi = i;
 			}
@@ -1419,28 +1378,28 @@ TableOfReal TableOfReal_meansByRowLabels (TableOfReal me, int expand, int stats)
 			// Now invert the table.
 
 			char32 **tmp = sorted -> rowLabels; sorted -> rowLabels = my rowLabels;
-			thee.reset (TableOfReal_sortRowsByIndex (sorted.peek(), index.peek(), 1));
+			thee = TableOfReal_sortRowsByIndex (sorted.get(), index.peek(), 1);
 			sorted -> rowLabels = tmp;
 		} else {
 			indexr++;
-			TableOfReal_copyOneRowWithLabel (sorted.peek(), sorted.peek(), indexi, indexr);
-			thee.reset (TableOfReal_create (indexr, my numberOfColumns));
+			TableOfReal_copyOneRowWithLabel (sorted.get(), sorted.get(), indexi, indexr);
+			thee = TableOfReal_create (indexr, my numberOfColumns);
 			for (long i = 1; i <= indexr; i++) {
-				TableOfReal_copyOneRowWithLabel (sorted.peek(), thee.peek(), i, i);
+				TableOfReal_copyOneRowWithLabel (sorted.get(), thee.get(), i, i);
 			}
 			NUMstrings_copyElements (sorted -> columnLabels, thy columnLabels, 1, my numberOfColumns);
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": means by row labels not created.");
 	}
 }
 
-TableOfReal TableOfReal_rankColumns (TableOfReal me) {
+autoTableOfReal TableOfReal_rankColumns (TableOfReal me) {
 	try {
 		autoTableOfReal thee = Data_copy (me);
 		NUMrankColumns (thy data, 1, thy numberOfRows, 1, thy numberOfColumns);
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": column ranks not created.");
 	}
@@ -1473,18 +1432,18 @@ void TableOfReal_setSequentialRowLabels (TableOfReal me, long from, long to, con
 }
 
 /* For the inheritors */
-TableOfReal TableOfReal_to_TableOfReal (TableOfReal me) {
+autoTableOfReal TableOfReal_to_TableOfReal (TableOfReal me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfRows, my numberOfColumns);
 		NUMmatrix_copyElements (my data, thy data, 1, my numberOfRows, 1, my numberOfColumns);
-		TableOfReal_copyLabels (me, thee.peek(), 1, 1);
-		return thee.transfer();
+		TableOfReal_copyLabels (me, thee.get(), 1, 1);
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not copied.");
 	}
 }
 
-TableOfReal TableOfReal_choleskyDecomposition (TableOfReal me, int upper, int inverse) {
+autoTableOfReal TableOfReal_choleskyDecomposition (TableOfReal me, int upper, int inverse) {
 	try {
 		char diag = 'N';
 		long n = my numberOfColumns, lda = my numberOfRows, info;
@@ -1515,13 +1474,13 @@ TableOfReal TableOfReal_choleskyDecomposition (TableOfReal me, int upper, int in
 				Melder_throw (U"dtrtri fails");
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": Cholesky decomposition not performed.");
 	}
 }
 
-TableOfReal TableOfReal_appendColumns (TableOfReal me, TableOfReal thee) {
+autoTableOfReal TableOfReal_appendColumns (TableOfReal me, TableOfReal thee) {
 	try {
 		long ncols = my numberOfColumns + thy numberOfColumns;
 		long labeldiffs = 0;
@@ -1550,22 +1509,22 @@ TableOfReal TableOfReal_appendColumns (TableOfReal me, TableOfReal thee) {
 		if (labeldiffs > 0) {
 			Melder_warning (labeldiffs, U" row labels differed.");
 		}
-		return him.transfer();
+		return him;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal with appended columns not created.");
 	}
 }
 
-TableOfReal TableOfReal_appendColumnsMany (Collection me) {
+autoTableOfReal TableOfRealList_appendColumnsMany (TableOfRealList me) {
 	try {
 		if (my size == 0) {
 			Melder_throw (U"No tables selected.");
 		}
-		TableOfReal thee = (TableOfReal) my item [1];
+		TableOfReal thee = my at [1];
 		long nrow = thy numberOfRows;
 		long ncol = thy numberOfColumns;
-		for (long itab = 2; itab <= my size; itab++) {
-			thee = (TableOfReal) my item [itab];
+		for (long itab = 2; itab <= my size; itab ++) {
+			thee = my at [itab];
 			ncol += thy numberOfColumns;
 			if (thy numberOfRows != nrow) {
 				Melder_throw (U"Numbers of rows in item ", itab, U" differs from previous.");
@@ -1573,29 +1532,33 @@ TableOfReal TableOfReal_appendColumnsMany (Collection me) {
 		}
 		autoTableOfReal him = TableOfReal_create (nrow, ncol);
 		/* Unsafe: new attributes not initialized. */
-		for (long irow = 1; irow <= nrow; irow++) {
-			TableOfReal_setRowLabel (him.peek(), irow, thy rowLabels [irow]);
+		for (long irow = 1; irow <= nrow; irow ++) {
+			TableOfReal_setRowLabel (him.get(), irow, thy rowLabels [irow]);
 		}
 		ncol = 0;
-		for (long itab = 1; itab <= my size; itab++) {
-			thee = (TableOfReal) my item [itab];
-			for (long icol = 1; icol <= thy numberOfColumns; icol++) {
+		for (long itab = 1; itab <= my size; itab ++) {
+			thee = my at [itab];
+			for (long icol = 1; icol <= thy numberOfColumns; icol ++) {
 				ncol++;
-				TableOfReal_setColumnLabel (him.peek(), ncol, thy columnLabels [icol]);
-				for (long irow = 1; irow <= nrow; irow++) {
-					his data[irow][ncol] = thy data[irow][icol];
+				TableOfReal_setColumnLabel (him.get(), ncol, thy columnLabels [icol]);
+				for (long irow = 1; irow <= nrow; irow ++) {
+					his data [irow] [ncol] = thy data [irow] [icol];
 				}
 			}
 		}
 		Melder_assert (ncol == his numberOfColumns);
-		return him.transfer();
+		return him;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal with appended columns not created.");
 	}
 }
 
-double TableOfReal_normalityTest_BHEP (TableOfReal me, double *h, double *tnb, double *lnmu, double *lnvar) {
+double TableOfReal_normalityTest_BHEP (TableOfReal me, double *h, double *p_tnb, double *p_lnmu, double *p_lnvar) {
 	try {
+		/* Henze & Wagner (1997), A new approach to the BHEP tests for multivariate normality, 
+		 *  Journal of Multivariate Analysis 62, 1-23.
+		 */
+
 		long n = my numberOfRows, p = my numberOfColumns;
 		double beta = *h > 0 ? NUMsqrt1_2 / *h : NUMsqrt1_2 * pow ( (1.0 + 2 * p) / 4, 1.0 / (p + 4)) * pow (n, 1.0 / (p + 4));
 		double p2 = p / 2.0;
@@ -1608,7 +1571,7 @@ double TableOfReal_normalityTest_BHEP (TableOfReal me, double *h, double *tnb, d
 			*h = NUMsqrt1_2 / beta;
 		}
 
-		*tnb = *lnmu = *lnvar = NUMundefined;
+		double tnb = NUMundefined, lnmu = NUMundefined, lnvar = NUMundefined;
 
 		if (n < 2 || p < 1) {
 			return NUMundefined;
@@ -1616,9 +1579,9 @@ double TableOfReal_normalityTest_BHEP (TableOfReal me, double *h, double *tnb, d
 
 		autoCovariance thee = TableOfReal_to_Covariance (me);
 		try {
-			SSCP_expandLowerCholesky (thee.peek());
+			SSCP_expandLowerCholesky (thee.get());
 		} catch (MelderError) {
-			*tnb = 4 * n;
+			tnb = 4.0 * n;
 		}
 		{
 			double djk, djj, sumjk = 0.0, sumj = 0.0;
@@ -1636,28 +1599,37 @@ double TableOfReal_normalityTest_BHEP (TableOfReal me, double *h, double *tnb, d
 				djj = NUMmahalanobisDistance_chi (thy lowerCholesky, my data[j], thy centroid, p, p);
 				sumj += exp (-b2 * djj);
 			}
-			*tnb = (1.0 / n) * sumjk - 2.0 * pow (1.0 + beta2, - p2) * sumj + n * pow (gamma, - p2); // n *
+			tnb = (1.0 / n) * sumjk - 2.0 * pow (1.0 + beta2, - p2) * sumj + n * pow (gamma, - p2); // n *
 		}
 		double mu = 1.0 - pow (gamma, -p2) * (1.0 + p * beta2 / gamma + p * (p + 2) * beta4 / (2.0 * gamma2));
 		double var = 2.0 * pow (1 + 4 * beta2, -p2)
-		             + 2.0 * pow (gamma,  -p) * (1.0 + 2 * p * beta4 / gamma2  + 3 * p * (p + 2) * beta8 / (4.0 * gamma4))
-		             - 4.0 * pow (delta, -p2) * (1.0 + 3 * p * beta4 / (2 * delta) + p * (p + 2) * beta8 / (2.0 * delta2));
+			+ 2.0 * pow (gamma,  -p) * (1.0 + 2 * p * beta4 / gamma2  + 3 * p * (p + 2) * beta8 / (4.0 * gamma4))
+			- 4.0 * pow (delta, -p2) * (1.0 + 3 * p * beta4 / (2 * delta) + p * (p + 2) * beta8 / (2.0 * delta2));
 		double mu2 = mu * mu;
-		*lnmu = 0.5 * log (mu2 * mu2 / (mu2 + var)); //log (sqrt (mu2 * mu2 /(mu2 + var)));
-		*lnvar = sqrt (log ( (mu2 + var) / mu2));
-		prob = NUMlogNormalQ (*tnb, *lnmu, *lnvar);
+		lnmu = 0.5 * log (mu2 * mu2 / (mu2 + var)); //log (sqrt (mu2 * mu2 /(mu2 + var)));
+		lnvar = sqrt (log ( (mu2 + var) / mu2));
+		prob = NUMlogNormalQ (tnb, lnmu, lnvar);
+		if (p_tnb) {
+			*p_tnb = tnb;
+		}
+		if (p_lnmu) {
+			*p_lnmu = lnmu;
+		}
+		if (p_lnvar) {
+			*p_lnvar = lnvar;
+		}
 		return prob;
 	} catch (MelderError) {
 		Melder_throw (me, U": cannot determine normality.");
 	}
 }
 
-TableOfReal TableOfReal_and_TableOfReal_crossCorrelations (TableOfReal me, TableOfReal thee, int by_columns, int center, int normalize) {
+autoTableOfReal TableOfReal_and_TableOfReal_crossCorrelations (TableOfReal me, TableOfReal thee, int by_columns, int center, int normalize) {
 	return by_columns ? TableOfReal_and_TableOfReal_columnCorrelations (me, thee, center, normalize) :
 	       TableOfReal_and_TableOfReal_rowCorrelations (me, thee, center, normalize);
 }
 
-TableOfReal TableOfReal_and_TableOfReal_rowCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize) {
+autoTableOfReal TableOfReal_and_TableOfReal_rowCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize) {
 	try {
 		if (my numberOfColumns != thy numberOfColumns) {
 			Melder_throw (U"Both tables must have the same number of columns.");
@@ -1685,13 +1657,13 @@ TableOfReal TableOfReal_and_TableOfReal_rowCorrelations (TableOfReal me, TableOf
 				his data[i][k] = ctmp;
 			}
 		}
-		return him.transfer();
+		return him;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal with row correlations not created.");
 	}
 }
 
-TableOfReal TableOfReal_and_TableOfReal_columnCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize) {
+autoTableOfReal TableOfReal_and_TableOfReal_columnCorrelations (TableOfReal me, TableOfReal thee, int center, int normalize) {
 	try {
 		if (my numberOfRows != thy numberOfRows) {
 			Melder_throw (U"Both tables must have the same number of rows.");
@@ -1720,7 +1692,7 @@ TableOfReal TableOfReal_and_TableOfReal_columnCorrelations (TableOfReal me, Tabl
 				his data[j][k] = ctmp;
 			}
 		}
-		return him.transfer();
+		return him;
 	} catch (MelderError) {
 		Melder_throw (U"TableOfReal with column correlations not created.");
 	}
