@@ -1,20 +1,19 @@
 /* TextGrid_extensions.cpp
  *
- * Copyright (C) 1993-2015 David Weenink
+ * Copyright (C) 1993-2016 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -146,28 +145,26 @@ static int isTimitWord (const char label[]) {
 	return 1;
 }
 
-Any TextGrid_TIMITLabelFileRecognizer (int nread, const char *header, MelderFile file) {
+autoDaata TextGrid_TIMITLabelFileRecognizer (int nread, const char *header, MelderFile file) {
 	char hkruis[3] = "h#", label1[512], label2[512];
 	int length, phnFile = 0;
 	long it[5]; 
-	if (nread < 12 ||
-	        sscanf (header, "%ld%ld%s%n\n", &it[1], &it[2], label1, &length) != 3 ||
-	        it[1] < 0 || it[2] <= it[1] ||
-	        sscanf (&header[length], "%ld%ld%s\n", &it[3], &it[4], label2) != 3 ||
-	        // 20120512 djmw removed the extra "it[3] < it[2]" check, because otherwise train/dr7/mdlm0/si1864.wrd cannot be read
-	        it[4] <= it[3]) {
-		return 0;
+	if (nread < 12 || sscanf (header, "%ld%ld%s%n\n", &it[1], &it[2], label1, &length) != 3 ||
+		it[1] < 0 || it[2] <= it[1] || sscanf (&header[length], "%ld%ld%s\n", &it[3], &it[4], label2) != 3 || it[4] <= it[3]) {
+		// 20120512 djmw removed the extra "it[3] < it[2]" check, because otherwise train/dr7/mdlm0/si1864.wrd cannot be read
+		return autoDaata ();
 	}
 	if (! strcmp (label1, hkruis)) {
 		if (isTimitPhoneticLabel (label2)) {
 			phnFile = 1;
 		} else if (! isTimitWord (label2)) {
-			return 0;
+			return autoDaata ();
 		}
 	} else if (! isTimitWord (label1) || ! isTimitWord (label2)) {
-		return 0;
+		return autoDaata ();
 	}
-	return TextGrid_readFromTIMITLabelFile (file, phnFile);
+	autoTextGrid thee = TextGrid_readFromTIMITLabelFile (file, phnFile);
+	return thee.move();
 }
 
 static void IntervalTier_add (IntervalTier me, double xmin, double xmax, const char32 *label) {
@@ -177,7 +174,7 @@ static void IntervalTier_add (IntervalTier me, double xmin, double xmax, const c
 	}
 
 	autoTextInterval newti = TextInterval_create (xmin, xmax, label);
-	TextInterval interval = (TextInterval) my intervals -> item[i];
+	TextInterval interval = my intervals.at [i];
 	double xmaxi = interval -> xmax;
 	if (xmax > xmaxi) {
 		Melder_throw (U"Don't know what to do");    // Don't know what to do
@@ -189,19 +186,19 @@ static void IntervalTier_add (IntervalTier me, double xmin, double xmax, const c
 		}
 		// split interval
 		interval -> xmin = xmax;
-		Collection_addItem (my intervals, newti.transfer());
+		my intervals. addItem_move (newti.move());
 		return;
 	}
 	interval -> xmax = xmin;
-	Collection_addItem (my intervals, newti.transfer());
+	my intervals. addItem_move (newti.move());
 	// extra interval when xmax's are not the same
 	if (xmax < xmaxi) {
 		autoTextInterval newti2 = TextInterval_create (xmax, xmaxi, interval -> text);
-		Collection_addItem (my intervals, newti2.transfer());
+		my intervals. addItem_move (newti2.move());
 	}
 }
 
-TextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
+autoTextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
 	try {
 		double dt = 1.0 / 16000; /* 1 / (TIMIT samplingFrequency) */
 		double xmax = dt;
@@ -211,7 +208,7 @@ TextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
 		// We start with a sufficiently long duration (one hour) and correct this later.
 
 		autoTextGrid me = TextGrid_create (0.0, 3600.0, U"wrd", 0);
-		IntervalTier timit = (IntervalTier) my tiers -> item[1];
+		IntervalTier timit = (IntervalTier) my tiers->at [1];
 		long linesRead = 0;
 		char line[200], label[200];
 		while (fgets (line, 199, f)) {
@@ -225,7 +222,7 @@ TextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
 			}
 			xmax = it2 * dt;
 			double xmin = it1 * dt;
-			long ni = timit -> intervals -> size - 1;
+			long ni = timit -> intervals.size - 1;
 			if (ni < 1) {
 				ni = 1;
 				// Some files do not start with a first line "0 <number2> h#".
@@ -236,7 +233,7 @@ TextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
 					xmin = 0.0;
 				}
 			}
-			TextInterval interval = (TextInterval) timit -> intervals -> item[ni];
+			TextInterval interval = timit -> intervals.at [ni];
 			if (xmin < interval -> xmax && linesRead > 1) {
 				xmin = interval -> xmax;
 				Melder_warning (U"File \"", MelderFile_messageName (file), U"\": Start time set to previous end "
@@ -250,34 +247,34 @@ TextGrid TextGrid_readFromTIMITLabelFile (MelderFile file, int phnFile) {
 		// Now correct the end times, based on last read interval.
 		// (end time was set to large value!)
 
-		if (timit -> intervals -> size < 2) {
+		if (timit -> intervals.size < 2) {
 			Melder_throw (U"Empty TextGrid");
 		}
-		Collection_removeItem (timit -> intervals, timit -> intervals -> size);
-		TextInterval interval = (TextInterval) timit -> intervals -> item[timit -> intervals -> size];
+		timit -> intervals. removeItem (timit -> intervals.size);
+		TextInterval interval = timit -> intervals.at [timit -> intervals.size];
 		timit -> xmax = interval -> xmax;
 		my xmax = xmax;
 		if (phnFile) { // Create tier 2 with IPA symbols
 			autoIntervalTier ipa = Data_copy (timit);
-			Thing_setName (ipa.peek(), U"ipa");
+			Thing_setName (ipa.get(), U"ipa");
 			// First change the data in ipa
-			for (long i = 1; i <= ipa -> intervals -> size; i++) {
-				interval = (TextInterval) timit -> intervals -> item[i];
+			for (long i = 1; i <= ipa -> intervals.size; i ++) {
+				interval = timit -> intervals.at [i];
 
-				TextInterval_setText ((TextInterval) ipa -> intervals -> item[i],
+				TextInterval_setText (ipa -> intervals.at [i],
 					Melder_peek8to32 (timitLabelToIpaLabel (Melder_peek32to8 (interval -> text))));
 			}
-			Collection_addItem (my tiers, ipa.transfer()); // Then: add to collection
+			my tiers -> addItem_move (ipa.move()); // Then: add to collection
 			Thing_setName (timit, U"phn");  // rename wrd
 		}
 		f.close (file);
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"TextGrid not read from file ", file, U".");
 	}
 }
 
-TextGrid TextGrids_merge (TextGrid me, TextGrid thee) {
+autoTextGrid TextGrids_merge (TextGrid me, TextGrid thee) {
 	try {
 		int at_end = 0, at_start = 1;
 
@@ -291,23 +288,23 @@ TextGrid TextGrids_merge (TextGrid me, TextGrid thee) {
 		double extra_time_start = fabs (g2 -> xmin - g1 -> xmin);
 
 		if (g1 -> xmin > g2 -> xmin) {
-			TextGrid_extendTime (g1.peek(), extra_time_start, at_start);
+			TextGrid_extendTime (g1.get(), extra_time_start, at_start);
 		}
 		if (g1 -> xmax < g2 -> xmax) {
-			TextGrid_extendTime (g1.peek(), extra_time_end, at_end);
+			TextGrid_extendTime (g1.get(), extra_time_end, at_end);
 		}
 		if (g2 -> xmin > g1 -> xmin) {
-			TextGrid_extendTime (g2.peek(), extra_time_start, at_start);
+			TextGrid_extendTime (g2.get(), extra_time_start, at_start);
 		}
 		if (g2 -> xmax < g1 -> xmax) {
-			TextGrid_extendTime (g2.peek(), extra_time_end, at_end);
+			TextGrid_extendTime (g2.get(), extra_time_end, at_end);
 		}
 
-		for (long i = 1; i <= g2 -> tiers -> size; i++) {
-			autoFunction tier = Data_copy ( (Function) g2 -> tiers -> item [i]);
-			Collection_addItem (g1 -> tiers, tier.transfer());
+		for (long i = 1; i <= g2 -> tiers->size; i ++) {
+			autoFunction tier = Data_copy (g2 -> tiers->at [i]);
+			g1 -> tiers -> addItem_move (tier.move());
 		}
-		return g1.transfer();
+		return g1;
 	} catch (MelderError) {
 		Melder_throw (me, U" & ", thee, U": not merged.");
 	}
@@ -316,11 +313,12 @@ TextGrid TextGrids_merge (TextGrid me, TextGrid thee) {
 void IntervalTier_setLaterEndTime (IntervalTier me, double xmax, const char32 *mark) {
 	try {
 		if (xmax <= my xmax) return; // nothing to be done
-		TextInterval ti = (TextInterval) my intervals -> item[my intervals -> size];
+		Melder_assert (my intervals.size > 0);
+		TextInterval ti = my intervals.at [my intervals.size];
 		Melder_assert (xmax > ti -> xmax);
 		if (mark) {
 			autoTextInterval interval = TextInterval_create (ti -> xmax, xmax, mark);
-			Collection_addItem (my intervals, interval.transfer());
+			my intervals. addItem_move (interval.move());
 		} else {
 			// extend last interval
 			ti -> xmax = xmax;
@@ -333,12 +331,15 @@ void IntervalTier_setLaterEndTime (IntervalTier me, double xmax, const char32 *m
 
 void IntervalTier_setEarlierStartTime (IntervalTier me, double xmin, const char32 *mark) {
 	try {
-		if (xmin >= my xmin) return; // nothing to be done
-		TextInterval ti = (TextInterval) my intervals -> item[1];
+		if (xmin >= my xmin) {
+			return;
+		}
+		Melder_assert (my intervals.size > 0);
+		TextInterval ti = my intervals.at [1];
 		Melder_assert (xmin < ti -> xmin);
 		if (mark) {
 			autoTextInterval interval = TextInterval_create (xmin, ti -> xmin, mark);
-			Collection_addItem (my intervals, interval.transfer());
+			my intervals. addItem_move (interval.move());
 		} else {
 			// extend first interval
 			ti -> xmin = xmin;
@@ -351,21 +352,21 @@ void IntervalTier_setEarlierStartTime (IntervalTier me, double xmin, const char3
 
 void IntervalTier_moveBoundary (IntervalTier me, long iint, bool atStart, double newTime) {
     try {
-        if (iint < 1 or iint > my intervals -> size) {
+        if (iint < 1 or iint > my intervals.size) {
             Melder_throw (U"Interval out of range.");
         }
-        if ((iint == 1 && atStart) or ((iint == my intervals -> size && not atStart))) {
+        if ((iint == 1 && atStart) or (iint == my intervals.size && ! atStart)) {
             Melder_throw (U"Cannot change the domain.");
         }
-        TextInterval interval = (TextInterval) my intervals -> item[iint];
+        TextInterval interval = my intervals.at [iint];
         if (atStart) {
-            TextInterval pinterval = (TextInterval) my intervals -> item[iint-1];
+            TextInterval pinterval = my intervals.at [iint-1];
             if (newTime <= pinterval -> xmin) {
                 Melder_throw (U"Cannot move past the start of previous interval.");
             }
             pinterval -> xmax = interval -> xmin = newTime;
         } else {
-            TextInterval ninterval = (TextInterval) my intervals -> item[iint+1];
+            TextInterval ninterval = my intervals.at [iint+1];
             if (newTime >= ninterval -> xmax) {
                 Melder_throw (U"Cannot move past the end of next interval.");
             }
@@ -379,10 +380,12 @@ void IntervalTier_moveBoundary (IntervalTier me, long iint, bool atStart, double
 
 void TextTier_setLaterEndTime (TextTier me, double xmax, const char32 *mark) {
 	try {
-		if (xmax <= my xmax) return; // nothing to be done
+		if (xmax <= my xmax) {
+			return;
+		}
 		if (mark) {
 			autoTextPoint textpoint = TextPoint_create (my xmax, mark);
-			Collection_addItem (my points, textpoint.transfer());
+			my points. addItem_move (textpoint.move());
 		}
 		my xmax = xmax;
 	} catch (MelderError) {
@@ -392,10 +395,12 @@ void TextTier_setLaterEndTime (TextTier me, double xmax, const char32 *mark) {
 
 void TextTier_setEarlierStartTime (TextTier me, double xmin, const char32 *mark) {
 	try {
-		if (xmin >= my xmin) return; // nothing to be done
+		if (xmin >= my xmin) {
+			return;
+		}
 		if (mark) {
 			autoTextPoint textpoint = TextPoint_create (my xmin, mark);
-			Collection_addItem (my points, textpoint.transfer());
+			my points. addItem_move (textpoint.move());
 		}
 		my xmin = xmin;
 	} catch (MelderError) {
@@ -408,8 +413,8 @@ void TextGrid_setEarlierStartTime (TextGrid me, double xmin, const char32 *imark
 		if (xmin >= my xmin) {
 			return;
 		}
-		for (long tierNumber = 1 ; tierNumber <= my tiers -> size; tierNumber++) {
-			Function tier = (Function) my tiers -> item [tierNumber];
+		for (long tierNumber = 1 ; tierNumber <= my tiers->size; tierNumber ++) {
+			Function tier = my tiers->at [tierNumber];
 			if (tier -> classInfo == classIntervalTier) {
 				IntervalTier_setEarlierStartTime ((IntervalTier) tier, xmin, imark);
 			} else {
@@ -424,12 +429,13 @@ void TextGrid_setEarlierStartTime (TextGrid me, double xmin, const char32 *imark
 
 void TextGrid_setLaterEndTime (TextGrid me, double xmax, const char32 *imark, const char32 *pmark) {
 	try {
-		if (xmax <= my xmax) return;
-		for (long tierNumber =1 ; tierNumber <= my tiers -> size; tierNumber++) {
-			Function tier = (Function) my tiers -> item [tierNumber];
+		if (xmax <= my xmax) {
+			return;
+		}
+		for (long tierNumber =1 ; tierNumber <= my tiers->size; tierNumber ++) {
+			Function tier = my tiers->at [tierNumber];
 			if (tier -> classInfo == classIntervalTier) {
 				IntervalTier_setLaterEndTime ((IntervalTier) tier, xmax, imark);
-
 			} else {
 				TextTier_setLaterEndTime ((TextTier) tier, xmax, pmark);
 			}
@@ -441,7 +447,7 @@ void TextGrid_setLaterEndTime (TextGrid me, double xmax, const char32 *imark, co
 }
 
 void TextGrid_extendTime (TextGrid me, double extra_time, int position) {
-	autoTextGrid thee = 0;
+	autoTextGrid thee;
 	try {
 		double xmax = my xmax, xmin = my xmin;
 		int at_end = position == 0;
@@ -450,7 +456,7 @@ void TextGrid_extendTime (TextGrid me, double extra_time, int position) {
 			return;
 		}
 		extra_time = fabs (extra_time); // Just in case...
-		thee.reset (Data_copy (me));
+		thee = Data_copy (me);
 
 		if (at_end) {
 			xmax += extra_time;
@@ -458,8 +464,8 @@ void TextGrid_extendTime (TextGrid me, double extra_time, int position) {
 			xmin -= extra_time;
 		}
 
-		for (long i = 1; i <= my tiers -> size; i++) {
-			Function anyTier = (Function) my tiers -> item [i];
+		for (long i = 1; i <= my tiers->size; i ++) {
+			Function anyTier = my tiers->at [i];
 			double tmin = anyTier -> xmin, tmax = anyTier -> xmax;
 
 			if (at_end) {
@@ -474,7 +480,7 @@ void TextGrid_extendTime (TextGrid me, double extra_time, int position) {
 			if (anyTier -> classInfo == classIntervalTier) {
 				IntervalTier tier = (IntervalTier) anyTier;
 				autoTextInterval interval = TextInterval_create (tmin, tmax, U"");
-				Collection_addItem (tier -> intervals, interval.transfer());
+				tier -> intervals. addItem_move (interval.move());
 			}
 		}
 		my xmin = xmin;
@@ -486,40 +492,44 @@ void TextGrid_extendTime (TextGrid me, double extra_time, int position) {
 
 void TextGrid_setTierName (TextGrid me, long itier, const char32 *newName) {
 	try {
-		long ntiers = my tiers -> size;
-
-		if (itier < 1 || itier > ntiers) Melder_throw (U"Tier number (", itier, U") should not be "
-			        U"larger than the number of tiers (", ntiers, U").");
-		Thing_setName ( (Thing) my tiers -> item [itier], newName);
+		long ntiers = my tiers->size;
+		if (itier < 1 || itier > ntiers) {
+			Melder_throw (U"Tier number (", itier, U") should not be larger than the number of tiers (", ntiers, U").");
+		}
+		Thing_setName (my tiers->at [itier], newName);
 	} catch (MelderError) {
 		Melder_throw (me, U": tier name not set.");
 	}
 }
 
 static void IntervalTier_cutInterval (IntervalTier me, long index, int extend_option) {
-	long size_pre = my intervals -> size;
+	long size_pre = my intervals.size;
 
-	/* There always must be at least one interval */
+	// There always must be at least one interval
 	if (size_pre == 1 || index > size_pre || index < 1) {
 		return;
 	}
 
-	TextInterval ti = (TextInterval) my intervals -> item[index];
+	TextInterval ti = my intervals.at [index];
 	double xmin = ti -> xmin;
 	double xmax = ti -> xmax;
-	Collection_removeItem (my intervals, index);
-	if (index == 1) { // Change xmin of the new first interval.
-		ti = (TextInterval) my intervals -> item[index];
+	my intervals. removeItem (index);
+	if (index == 1) { 
+		// Change xmin of the new first interval.
+		ti = my intervals.at [index];
 		ti -> xmin = xmin;
-	} else if (index == size_pre) { // Change xmax of the new last interval.
-		ti = (TextInterval) my intervals -> item[my intervals -> size];
+	} else if (index == size_pre) { 
+		// Change xmax of the new last interval.
+		ti = my intervals.at [my intervals.size];
 		ti -> xmax = xmax;
 	} else {
-		if (extend_option == 0) { // extend earlier interval to the right
-			ti = (TextInterval) my intervals -> item[index - 1];
+		if (extend_option == 0) { 
+			// extend earlier interval to the right
+			ti = my intervals.at [index - 1];
 			ti -> xmax = xmax;
-		} else { // extend next interval to the left
-			ti = (TextInterval) my intervals -> item[index];
+		} else {
+			// extend next interval to the left
+			ti = my intervals.at [index];
 			ti -> xmin = xmin;
 		}
 	}
@@ -527,59 +537,59 @@ static void IntervalTier_cutInterval (IntervalTier me, long index, int extend_op
 
 void IntervalTier_removeBoundariesBetweenIdenticallyLabeledIntervals (IntervalTier me, const char32 *label) {
     try {
-        for (long iint = my intervals -> size; iint > 1; iint--) {
-            TextInterval ti = (TextInterval) my intervals -> item[iint];
-            if (Melder_equ (ti -> text, label)) {
-                TextInterval tim1 = (TextInterval) my intervals -> item[iint - 1];
-                if (Melder_equ (tim1 -> text, label)) {
-                    Melder_free (tim1 -> text);
-                    IntervalTier_removeLeftBoundary (me, iint);
-                }
-            }
-        }
-    } catch (MelderError) {
-        Melder_throw (me, U": boundaries not removed.");
-    }
+		for (long iinterval = my intervals.size; iinterval > 1; iinterval --) {
+			TextInterval thisInterval = my intervals.at [iinterval];
+			if (Melder_equ (thisInterval -> text, label)) {
+				TextInterval previousInterval = my intervals.at [iinterval - 1];
+				if (Melder_equ (previousInterval -> text, label)) {
+					Melder_free (previousInterval -> text);
+					IntervalTier_removeLeftBoundary (me, iinterval);
+				}
+			}
+		}
+	} catch (MelderError) {
+		Melder_throw (me, U": boundaries not removed.");
+	}
 }
 
 void IntervalTier_cutIntervals_minimumDuration (IntervalTier me, const char32 *label, double minimumDuration) {
-	long i = 1;
-	while (i <= my intervals -> size) {
-		TextInterval ti = (TextInterval) my intervals -> item[i];
-		if ( (! label || (ti -> text && str32equ (ti -> text, label))) &&
-		        ti -> xmax - ti -> xmin < minimumDuration) {
-			IntervalTier_cutInterval (me, i, 0);
+	long iinterval = 1;
+	while (iinterval <= my intervals.size) {
+		TextInterval interval = my intervals.at [iinterval];
+		if ((! label || (interval -> text && str32equ (interval -> text, label))) &&
+			interval -> xmax - interval -> xmin < minimumDuration)
+		{
+			IntervalTier_cutInterval (me, iinterval, 0);
 		} else {
-			i++;
+			iinterval ++;
 		}
 	}
 }
 
 void IntervalTier_cutIntervalsOnLabelMatch (IntervalTier me, const char32 *label) {
-	long i = 1;
-	while (i < my intervals -> size) {
-		TextInterval ti = (TextInterval) my intervals -> item[i];
-		TextInterval tip1 = (TextInterval) my intervals -> item[i + 1];
-		if ( (! label || (ti -> text && str32equ (ti -> text, label))) &&
-		        (Melder_cmp (ti -> text, tip1 -> text) == 0)) {
-
-			IntervalTier_cutInterval (me, i, 1);
+	long iinterval = 1;
+	while (iinterval < my intervals.size) {
+		TextInterval thisInterval = my intervals.at [iinterval];
+		TextInterval nextInterval = my intervals.at [iinterval + 1];
+		if ( (! label || (thisInterval -> text && str32equ (thisInterval -> text, label))) &&
+			Melder_equ (thisInterval -> text, nextInterval -> text))
+		{
+			IntervalTier_cutInterval (me, iinterval, 1);
 		} else {
-			i++;
+			iinterval ++;
 		}
 	}
 }
 
-void IntervalTier_changeLabels (I, long from, long to, const char32 *search, const char32 *replace, int use_regexp, long *nmatches, long *nstringmatches) {
-	iam (IntervalTier);
+void IntervalTier_changeLabels (IntervalTier me, long from, long to, const char32 *search, const char32 *replace, int use_regexp, long *nmatches, long *nstringmatches) {
 	try {
 		if (from == 0) {
 			from = 1;
 		}
 		if (to == 0) {
-			to = my intervals -> size;
+			to = my intervals.size;
 		}
-		if (from > to || from < 1 || to > my intervals -> size) {
+		if (from > to || from < 1 || to > my intervals.size) {
 			Melder_throw (U"Incorrect specification of where to act.");
 		}
 		if (use_regexp && str32len (search) == 0) {
@@ -589,52 +599,51 @@ void IntervalTier_changeLabels (I, long from, long to, const char32 *search, con
 		long nlabels = to - from + 1;
 		autoNUMvector<char32 *> labels (1, nlabels);
 
-		for (long i = from; i <= to; i++) {
-			TextInterval interval = (TextInterval) my intervals -> item[i];
+		for (long i = from; i <= to; i ++) {
+			TextInterval interval = my intervals.at [i];
 			labels[i - from + 1] = interval -> text;   // Shallow copy.
 		}
 		autostring32vector newlabels (strs_replace (labels.peek(), 1, nlabels, search, replace, 0, nmatches, nstringmatches, use_regexp), 1, nlabels);
 
-		for (long i = from; i <= to; i++) {
-			TextInterval interval = (TextInterval) my intervals -> item[i];
+		for (long i = from; i <= to; i ++) {
+			TextInterval interval = my intervals.at [i];
 			Melder_free (interval -> text);
-			interval -> text = newlabels[i - from + 1];   // Transfer of ownership.
-			newlabels[i - from + 1] = nullptr;
+			interval -> text = newlabels [i - from + 1];   // Transfer of ownership.
+			newlabels [i - from + 1] = nullptr;
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": labels not changed.");
 	}
 }
 
-void TextTier_changeLabels (I, long from, long to, const char32 *search, const char32 *replace, int use_regexp, long *nmatches, long *nstringmatches) {
-	iam (TextTier);
+void TextTier_changeLabels (TextTier me, long from, long to, const char32 *search, const char32 *replace, int use_regexp, long *nmatches, long *nstringmatches) {
 	try {
 		if (from == 0) {
 			from = 1;
 		}
 		if (to == 0) {
-			to = my points -> size;
+			to = my points.size;
 		}
-		if (from > to || from < 1 || to > my points -> size) {
+		if (from > to || from < 1 || to > my points.size) {
 			Melder_throw (U"Incorrect specification of where to act.");
 		}
 		if (use_regexp && str32len (search) == 0) {
 			Melder_throw (U"The regex search string cannot be empty.\nYou may search for an empty string with the expression \"^$\"");
 		}
 		long nmarks = to - from + 1;
-		autoNUMvector<char32 *> marks (1, nmarks);
+		autoNUMvector<char32 *> marks (1, nmarks);   // a non-owning vector of strings
 
-		for (long i = from; i <= to; i++) {
-			TextPoint point = (TextPoint) my points -> item[i];
-			marks[i - from + 1] = point -> mark;   // Shallow copy.
+		for (long i = from; i <= to; i ++) {
+			TextPoint point = my points.at [i];
+			marks [i - from + 1] = point -> mark;   // reference copy
 		}
-		autostring32vector newmarks (strs_replace (marks.peek(), 1, nmarks, search, replace, 0, nmatches, nstringmatches, use_regexp), 1, nmarks);
+		autostring32vector newMarks (strs_replace (marks.peek(), 1, nmarks, search, replace, 0, nmatches, nstringmatches, use_regexp), 1, nmarks);
 
-		for (long i = from; i <= to; i++) {
-			TextPoint point = (TextPoint) my points -> item[i];
-			Melder_free (point -> mark);
-			point -> mark = newmarks[i - from + 1];   // Transfer of ownership.
-			newmarks[i - from + 1] = 0;
+		for (long i = from; i <= to; i ++) {
+			TextPoint point = my points.at [i];
+			Melder_free (point -> mark);   // this discards the original mark, and dangles its reference copy in `marks`, which will not be used
+			point -> mark = newMarks [i - from + 1];   // move the new mark; this consists of a copy from A to B...
+			newMarks [i - from + 1] = nullptr;   // ...followed by zeroing A
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": no labels changed.");
@@ -643,19 +652,18 @@ void TextTier_changeLabels (I, long from, long to, const char32 *search, const c
 
 void TextGrid_changeLabels (TextGrid me, int tier, long from, long to, const char32 *search, const char32 *replace, int use_regexp, long *nmatches, long *nstringmatches) {
 	try {
-		long ntiers = my tiers -> size;
-
+		long ntiers = my tiers->size;
 		if (tier < 1 || tier > ntiers) {
 			Melder_throw (U"The tier number (", tier, U") should not be larger than the number of tiers (", ntiers, U").");
 		}
 		if (use_regexp && str32len (search) == 0) {
 			Melder_throw (U"The regex search string cannot be empty.\nYou may search for an empty string with the expression \"^$\"");
 		}
-		Daata anyTier = (Daata) my tiers -> item [tier];
+		Function anyTier = my tiers->at [tier];
 		if (anyTier -> classInfo == classIntervalTier) {
-			IntervalTier_changeLabels (anyTier, from, to, search, replace, use_regexp, nmatches, nstringmatches);
+			IntervalTier_changeLabels ((IntervalTier) anyTier, from, to, search, replace, use_regexp, nmatches, nstringmatches);
 		} else {
-			TextTier_changeLabels (anyTier, from, to, search, replace, use_regexp, nmatches, nstringmatches);
+			TextTier_changeLabels ((TextTier) anyTier, from, to, search, replace, use_regexp, nmatches, nstringmatches);
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": labels not changed.");
@@ -663,11 +671,12 @@ void TextGrid_changeLabels (TextGrid me, int tier, long from, long to, const cha
 }
 
 static void IntervalTier_checkStartAndEndTime (IntervalTier me) {
-	TextInterval ti = (TextInterval) my intervals -> item[1];
+	Melder_assert (my intervals.size > 0);
+	TextInterval ti = my intervals.at [1];
 	if (my xmin != ti -> xmin) {
 		Melder_throw (me, U": start time of first interval doesn't match start time of the tier.");
 	}
-	ti = (TextInterval) my intervals -> item[my intervals -> size];
+	ti = my intervals.at [my intervals.size];
 	if (my xmax != ti -> xmax) {
 		Melder_throw (me, U": end time of last interval doesn't match end time of the tier.");
 	}
@@ -683,12 +692,12 @@ void IntervalTiers_append_inline (IntervalTier me, IntervalTier thee, bool prese
 		if (preserveTimes && my xmax < thy xmin) {
 			autoTextInterval connection = TextInterval_create (my xmax, thy xmin, U"");
             xmax_previous = thy xmin;
-			Collection_addItem (my intervals, connection.transfer());
+			my intervals. addItem_move (connection.move());
 		}
-		for (long iint = 1; iint <= thy intervals -> size; iint++) {
-			autoTextInterval ti = (TextInterval) Data_copy ((Daata) thy intervals -> item[iint]);
+		for (long iint = 1; iint <= thy intervals.size; iint ++) {
+			autoTextInterval ti = Data_copy (thy intervals.at [iint]);
 			if (preserveTimes) {
-				Collection_addItem (my intervals, ti.transfer());
+				my intervals. addItem_move (ti.move());
 			} else {
 				/* the interval could be so short that if we test ti -> xmin < ti->xmax it might be true
 				 * but after assigning ti->xmin = xmax_previous and ti->xmax += time_shift the test
@@ -700,7 +709,7 @@ void IntervalTiers_append_inline (IntervalTier me, IntervalTier thee, bool prese
 				volatile double xmax = ti -> xmax + time_shift;
 				if (xmin < xmax) {
 					ti -> xmin = xmin; ti -> xmax = xmax;
-					Collection_addItem (my intervals, ti.transfer());
+					my intervals. addItem_move (ti.move());
 					xmax_previous = xmax;
 				}
 				// else don't include interval
@@ -715,12 +724,12 @@ void IntervalTiers_append_inline (IntervalTier me, IntervalTier thee, bool prese
 // Precondition: if (preserveTimes) { my xmax <= thy xmin }
 void TextTiers_append_inline (TextTier me, TextTier thee, bool preserveTimes) {
 	try {
-		for (long iint = 1; iint <= thy points -> size; iint++) {
-			autoTextPoint tp = (TextPoint) Data_copy ((Daata) thy points -> item[iint]);
+		for (long iint = 1; iint <= thy points.size; iint ++) {
+			autoTextPoint tp = Data_copy (thy points.at [iint]);
 			if (not preserveTimes) {
 				tp -> number += my xmax - thy xmin;
 			}
-			Collection_addItem (my points, tp.transfer());
+			my points. addItem_move (tp.move());
 		}
 		my xmax = preserveTimes ? thy xmax : my xmax + (thy xmax - thy xmin);
 	} catch (MelderError) {
@@ -729,8 +738,8 @@ void TextTiers_append_inline (TextTier me, TextTier thee, bool preserveTimes) {
 }
 
 static void TextGrid_checkStartAndEndTimesOfTiers (TextGrid me) {
-	for (long itier = 1; itier <= my tiers -> size; itier++) {
-		Function tier = (Function) my tiers -> item[itier];
+	for (long itier = 1; itier <= my tiers->size; itier ++) {
+		Function tier = my tiers->at [itier];
 		if (tier -> xmin != my xmin) {
 			Melder_throw (me, U": the start time of tier ", itier, U" does not match the start time of its TextGrid.");
 		} else if (tier -> xmax != my xmax) {
@@ -742,8 +751,8 @@ static void TextGrid_checkStartAndEndTimesOfTiers (TextGrid me) {
 void TextGrids_append_inline (TextGrid me, TextGrid thee, bool preserveTimes)
 {
 	try {
-		if (my tiers -> size != thy tiers -> size) {
-			Melder_throw (U"The number of tiers must be equal.");
+		if (my tiers->size != thy tiers->size) {
+			Melder_throw (U"The numbers of tiers must be equal.");
 		}
 		if (preserveTimes && thy xmin < my xmax) {
 			Melder_throw (U"The start time of the second TextGrid can't be earlier than the end time of the first one if you want to preserve times.");
@@ -753,14 +762,14 @@ void TextGrids_append_inline (TextGrid me, TextGrid thee, bool preserveTimes)
 		TextGrid_checkStartAndEndTimesOfTiers (thee);
 		// last intervals must have the same end time
 		double xmax = preserveTimes ? thy xmax : my xmax + (thy xmax - thy xmin);
-		for (long itier = 1; itier <= my tiers -> size; itier++) {
-			Function myTier = my tier (itier), thyTier = thy tier (itier);
+		for (long itier = 1; itier <= my tiers->size; itier ++) {
+			Function myTier = my tiers->at [itier], thyTier = thy tiers->at [itier];
 			if (myTier -> classInfo == classIntervalTier && thyTier -> classInfo == classIntervalTier) {
 				IntervalTier  myIntervalTier = static_cast <IntervalTier>  (myTier);
 				IntervalTier thyIntervalTier = static_cast <IntervalTier> (thyTier);
 				IntervalTiers_append_inline (myIntervalTier, thyIntervalTier, preserveTimes);
                 // make sure last interval has correct end time
-                TextInterval lastInterval = myIntervalTier -> interval (myIntervalTier -> numberOfIntervals());
+                TextInterval lastInterval = myIntervalTier -> intervals.at [myIntervalTier -> intervals.size];
                 lastInterval -> xmax = xmax;
                 Melder_assert (lastInterval -> xmax > lastInterval -> xmin);
 			} else if (myTier -> classInfo == classTextTier && thyTier -> classInfo == classTextTier) {
@@ -779,16 +788,17 @@ void TextGrids_append_inline (TextGrid me, TextGrid thee, bool preserveTimes)
 	}
 }
 
-TextGrid TextGrids_to_TextGrid_appendContinuous (Collection me, bool preserveTimes) {
+autoTextGrid TextGrids_to_TextGrid_appendContinuous (OrderedOf<structTextGrid>* me, bool preserveTimes) {
 	try {
-		autoTextGrid thee = Data_copy ((TextGrid) my item[1]);
-		for (long igrid = 2; igrid <= my size; igrid++) {
-			TextGrids_append_inline (thee.peek(), (TextGrid) my item[igrid], preserveTimes);
+		Melder_assert (my size > 0);
+		autoTextGrid thee = Data_copy (my at [1]);
+		for (long igrid = 2; igrid <= my size; igrid ++) {
+			TextGrids_append_inline (thee.get(), my at [igrid], preserveTimes);
 		}
 		if (! preserveTimes) {
-			Function_shiftXBy (thee.peek(), -thy xmin);
+			Function_shiftXBy (thee.get(), -thy xmin);
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (U"No aligned TextGrid created from Collection.");
 	}
