@@ -2,19 +2,18 @@
  *
  * Copyright (C) 2014 David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "praatP.h"
@@ -436,10 +435,14 @@ FORM (Formants_extractSmoothestPart, U"Formants: Extract smoothest part", U"Form
 	REAL (U"Parameter variance power", U"1.5")
 	OK
 DO
-	autoCollection set = praat_getSelectedObjects ();
+	OrderedOf<structFormant> formants;
+	LOOP {
+		iam (Formant);
+		formants. addItem_ref (me);
+	}
 	double tmin = GET_REAL (U"left Time range"), tmax = GET_REAL (U"right Time range");
-	long index = Formants_getSmoothestInInterval (set.peek(), tmin, tmax, GET_INTEGER (U"Number of formant tracks"), GET_INTEGER (U"Order of polynomials") + 1,
-		GET_INTEGER (U"Weigh data") - 1, 0, GET_REAL (U"Number of sigmas"), GET_REAL (U"Parameter variance power"), 1, 1, 1, 1, 1);
+	long index = Formants_getSmoothestInInterval (& formants, tmin, tmax, GET_INTEGER (U"Number of formant tracks"), GET_INTEGER (U"Order of polynomials") + 1,
+		GET_INTEGER (U"Weigh data") - 1, 0, GET_REAL (U"Number of sigmas"), GET_REAL (U"Parameter variance power"), 1.0, 1.0, 1.0, 1.0, 1.0);
 	// next code is necessary to get the Formant at postion index selected and to get its name
 	long iselected = 0;
 	Formant him = nullptr;
@@ -454,7 +457,6 @@ DO
 	praat_new (thee.move(), his name, U"_part");
 END
 
-
 FORM (Formants_extractSmoothestPart_constrained, U"Formants: Extract smoothest part (constrained)", U"Formants: Extract smoothest part (constrained)...")
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
@@ -477,9 +479,13 @@ FORM (Formants_extractSmoothestPart_constrained, U"Formants: Extract smoothest p
 	POSITIVE (U"Minimum F3 (Hz)", U"1500.0")
 	OK
 DO
-	autoCollection set = praat_getSelectedObjects ();
+	OrderedOf<structFormant> formants;
+	LOOP {
+		iam (Formant);
+		formants. addItem_ref (me);
+	}
 	double tmin = GET_REAL (U"left Time range"), tmax = GET_REAL (U"right Time range");
-	long index = Formants_getSmoothestInInterval (set.peek(), tmin, tmax, GET_INTEGER (U"Number of formant tracks"), GET_INTEGER (U"Order of polynomials") + 1,
+	long index = Formants_getSmoothestInInterval (& formants, tmin, tmax, GET_INTEGER (U"Number of formant tracks"), GET_INTEGER (U"Order of polynomials") + 1,
 		GET_INTEGER (U"Weigh data") - 1, 1, GET_REAL (U"Number of sigmas"), GET_REAL (U"Parameter variance power"),
 		GET_REAL (U"Minimum F1"), GET_REAL (U"Maximum F1"), GET_REAL (U"Minimum F2"), GET_REAL (U"Maximum F2"), GET_REAL (U"Minimum F3"));
 	// next code is necessary to get the Formant at postion index selected and to get its name
@@ -498,7 +504,7 @@ END
 
 /********************** FormantModeler ******************************/
 
-FORM (FormantModeler_drawEstimatedTracks, U"FormantModeler: Draw estimated tracks", 0)
+FORM (FormantModeler_drawEstimatedTracks, U"FormantModeler: Draw estimated tracks", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"Maximum frequency (Hz)", U"5500.0")
@@ -506,7 +512,7 @@ FORM (FormantModeler_drawEstimatedTracks, U"FormantModeler: Draw estimated track
 	NATURAL (U"right Formant range", U"3")
 	INTEGER (U"Order of polynomials for estimation", U"3")
 	REAL (U"Horizontal offset (mm)", U"0.0")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -520,14 +526,14 @@ DO
 	}
 END
 
-FORM (FormantModeler_drawTracks, U"FormantModeler: Draw tracks", 0)
+FORM (FormantModeler_drawTracks, U"FormantModeler: Draw tracks", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"Maximum frequency (Hz)", U"5500.0")
 	NATURAL (U"left Formant range", U"1")
 	NATURAL (U"right Formant range", U"3")
 	REAL (U"Horizontal offset (mm)", U"0.0")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -540,16 +546,16 @@ DO
 	}
 END
 
-FORM (FormantModeler_speckle, U"FormantModeler: Speckle", 0)
+FORM (FormantModeler_speckle, U"FormantModeler: Speckle", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"Maximum frequency (Hz)", U"5500.0")
 	NATURAL (U"left Formant range", U"1")
 	NATURAL (U"right Formant range", U"3")
-	BOOLEAN (U"Draw error bars", 1)
+	BOOLEAN (U"Draw error bars", true)
 	REAL (U"Bar width (mm)", U"1.0")
 	REAL (U"Horizontal offset (mm)", U"0.0")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -562,7 +568,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_drawOutliersMarked, U"FormantModeler: Draw outliers marked", 0)
+FORM (FormantModeler_drawOutliersMarked, U"FormantModeler: Draw outliers marked", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"Maximum frequency (Hz)", U"5500.0")
@@ -577,7 +583,7 @@ FORM (FormantModeler_drawOutliersMarked, U"FormantModeler: Draw outliers marked"
 	WORD (U"Mark", U"o")
 	NATURAL (U"Mark font size", U"12")
 	REAL (U"Horizontal offset (mm)", U"0.0")
-	BOOLEAN (U"Garnish", 0)
+	BOOLEAN (U"Garnish", false)
 	OK
 DO
 	autoPraatPicture picture;
@@ -590,7 +596,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_drawVariancesOfShiftedTracks, U"FormantModeler: Draw variances of shifted tracks", 0)
+FORM (FormantModeler_drawVariancesOfShiftedTracks, U"FormantModeler: Draw variances of shifted tracks", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range", U"0.0")
 	REAL (U"left Variance range", U"0.0")
@@ -601,7 +607,7 @@ FORM (FormantModeler_drawVariancesOfShiftedTracks, U"FormantModeler: Draw varian
 		OPTION (U"Down")
 	NATURAL (U"left Formant range", U"1")
 	NATURAL (U"right Formant range", U"4")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -613,7 +619,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_drawCumulativeChiScores, U"FormantModeler: Draw cumulative chi scores", 0)
+FORM (FormantModeler_drawCumulativeChiScores, U"FormantModeler: Draw cumulative chi scores", nullptr)
 	REAL (U"left Time range", U"0.0")
 	REAL (U"right Time range", U"0.0")
 	REAL (U"left Chisq range", U"0.0")
@@ -623,7 +629,7 @@ FORM (FormantModeler_drawCumulativeChiScores, U"FormantModeler: Draw cumulative
 		OPTION (U"Bandwidth")
 		OPTION (U"Bandwidth / frequency")
 		OPTION (U"Sqrt bandwidth")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -635,7 +641,7 @@ DO
 END
 
 
-FORM (FormantModeler_normalProbabilityPlot, U"FormantModeler: Normal probability plot", 0)
+FORM (FormantModeler_normalProbabilityPlot, U"FormantModeler: Normal probability plot", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OPTIONMENU (U"Weigh data", 2)
 		OPTION (U"Equally")
@@ -646,7 +652,7 @@ FORM (FormantModeler_normalProbabilityPlot, U"FormantModeler: Normal probability
 	REAL (U"Number of sigmas", U"0.0")
 	NATURAL (U"Label size", U"12")
 	WORD (U"Label", U"+")
-	BOOLEAN (U"Garnish", 1);
+	BOOLEAN (U"Garnish", true);
 	OK
 DO
 	autoPraatPicture picture;
@@ -658,7 +664,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_drawBasisFunction, U"FormantModeler: Draw basis function", 0)
+FORM (FormantModeler_drawBasisFunction, U"FormantModeler: Draw basis function", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"left Amplitude range (Hz)", U"0.0")
@@ -666,9 +672,9 @@ FORM (FormantModeler_drawBasisFunction, U"FormantModeler: Draw basis function",
 	//long iterm, bool scaled, long numberOfPoints, int garnish
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Basis function", U"2")
-	BOOLEAN (U"Scale function with parameter value", 0)
+	BOOLEAN (U"Scale function with parameter value", false)
 	NATURAL (U"Number of points", U"200")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -681,7 +687,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getModelValueAtTime, U"", 0)
+FORM (FormantModeler_getModelValueAtTime, U"", nullptr)
 	NATURAL (U"Formant number", U"1")
 	REAL (U"Time (s)", U"0.1")
 	OK
@@ -693,7 +699,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getDataPointValue, U"FormantModeler: Get data point value", 0)
+FORM (FormantModeler_getDataPointValue, U"FormantModeler: Get data point value", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Index", U"1")
 	OK
@@ -707,7 +713,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getDataPointSigma, U"FormantModeler: Get data point sigma", 0)
+FORM (FormantModeler_getDataPointSigma, U"FormantModeler: Get data point sigma", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Index", U"1")
 	OK
@@ -721,7 +727,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getDataPointStatus, U"FormantModeler: Get data point status", 0)
+FORM (FormantModeler_getDataPointStatus, U"FormantModeler: Get data point status", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Index", U"1")
 	OK
@@ -743,7 +749,7 @@ DIRECT (FormantModeler_getNumberOfTracks)
 	}
 END
 
-FORM (FormantModeler_getNumberOfParameters, U"FormantModeler: Get number of parameters", 0)
+FORM (FormantModeler_getNumberOfParameters, U"FormantModeler: Get number of parameters", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -755,7 +761,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getNumberOfFixedParameters, U"FormantModeler: Get number of fixed parameters", 0)
+FORM (FormantModeler_getNumberOfFixedParameters, U"FormantModeler: Get number of fixed parameters", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -775,7 +781,7 @@ DIRECT (FormantModeler_getNumberOfDataPoints)
 	}
 END
 
-FORM (FormantModeler_getNumberOfInvalidDataPoints, U"FormantModeler: Get number of invalid data points", 0)
+FORM (FormantModeler_getNumberOfInvalidDataPoints, U"FormantModeler: Get number of invalid data points", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -787,7 +793,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getParameterValue, U"FormantModeler: Get parameter value", 0)
+FORM (FormantModeler_getParameterValue, U"FormantModeler: Get parameter value", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Parameter number", U"1")
 	OK
@@ -800,7 +806,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getParameterStatus, U"FormantModeler: Get parameter status", 0)
+FORM (FormantModeler_getParameterStatus, U"FormantModeler: Get parameter status", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Parameter number", U"1")
 	OK
@@ -814,7 +820,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getParameterStandardDeviation, U"FormantModeler: Get parameter standard deviatio", 0)
+FORM (FormantModeler_getParameterStandardDeviation, U"FormantModeler: Get parameter standard deviatio", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Parameter number", U"1")
 	OK
@@ -827,7 +833,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getVarianceOfParameters, U"FormantModeler: Get variance of parameters", 0)
+FORM (FormantModeler_getVarianceOfParameters, U"FormantModeler: Get variance of parameters", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	INTEGER (U"left Parameter range", U"0")
@@ -843,7 +849,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getCoefficientOfDetermination, U"FormantModeler: Get coefficient of determination", 0)
+FORM (FormantModeler_getCoefficientOfDetermination, U"FormantModeler: Get coefficient of determination", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	OK
@@ -868,7 +874,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getStandardDeviation, U"FormantModeler: Get formant standard deviation", 0)
+FORM (FormantModeler_getStandardDeviation, U"FormantModeler: Get formant standard deviation", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -880,7 +886,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_reportChiSquared, U"FormantModeler: Report chi squared", 0)
+FORM (FormantModeler_reportChiSquared, U"FormantModeler: Report chi squared", nullptr)
 	OPTIONMENU (U"Weigh data", 2)
 		OPTION (U"Equally")
 		OPTION (U"Bandwidth")
@@ -890,7 +896,7 @@ FORM (FormantModeler_reportChiSquared, U"FormantModeler: Report chi squared", 0)
 DO
 	LOOP {
 		iam (FormantModeler);
-		long numberOfFormants = my trackmodelers -> size;
+		long numberOfFormants = my trackmodelers.size;
 		int useSigmaY = GET_INTEGER (U"Weigh data") - 1;
 		double chisq = 0, ndf = 0, probability;
 		MelderInfo_open ();
@@ -899,14 +905,14 @@ DO
 			useSigmaY == DataModeler_DATA_WEIGH_SIGMA ? U"\tBandwidths are used as estimate for local standard deviations." : 
 			useSigmaY == DataModeler_DATA_WEIGH_RELATIVE ? U"\t1/Q's are used as estimate for local standard deviations." :
 			U"\tSqrt bandwidths are used as estimate for local standard deviations.");
-		for (long iformant = 1; iformant <= numberOfFormants; iformant++) {
+		for (long iformant = 1; iformant <= numberOfFormants; iformant ++) {
 			chisq = FormantModeler_getChiSquaredQ (me, iformant, iformant, useSigmaY, &probability, &ndf);
 			MelderInfo_writeLine (U"Formant track ", iformant, U":");
 			MelderInfo_writeLine (U"\tChi squared (F", iformant, U") = ", chisq);
 			MelderInfo_writeLine (U"\tProbability (F", iformant, U") = ", probability);
 			MelderInfo_writeLine (U"\tNumber of degrees of freedom (F", iformant, U") = ", ndf);
 		}
-		chisq = FormantModeler_getChiSquaredQ (me, 1, numberOfFormants, useSigmaY, &probability, &ndf);
+		chisq = FormantModeler_getChiSquaredQ (me, 1, numberOfFormants, useSigmaY, & probability, & ndf);
 		MelderInfo_writeLine (U"Chi squared test for the complete model with ", numberOfFormants, U" formants:");
 		MelderInfo_writeLine (U"\tChi squared = ", chisq);
 		MelderInfo_writeLine (U"\tProbability = ", probability);
@@ -915,7 +921,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getDegreesOfFreedom, U"FormantModeler: Get degrees of freedom", 0)
+FORM (FormantModeler_getDegreesOfFreedom, U"FormantModeler: Get degrees of freedom", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -927,7 +933,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getSmoothnessValue, U"FormantModeler: Get smoothness value", 0)
+FORM (FormantModeler_getSmoothnessValue, U"FormantModeler: Get smoothness value", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	INTEGER (U"Order of polynomials", U"3")
@@ -942,7 +948,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getAverageDistanceBetweenTracks, U"FormantModeler: Get average distance between tracks", 0)
+FORM (FormantModeler_getAverageDistanceBetweenTracks, U"FormantModeler: Get average distance between tracks", nullptr)
 	NATURAL (U"Track 1", U"2")
 	NATURAL (U"Track 2", U"3")
 	OPTIONMENU (U"Type of data", 1)
@@ -958,7 +964,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_getFormantsConstraintsFactor, U"FormantModeler: Get formants constraints factor", 0)
+FORM (FormantModeler_getFormantsConstraintsFactor, U"FormantModeler: Get formants constraints factor", nullptr)
 	REAL (U"Minimum F1 (Hz)", U"100.0")
 	REAL (U"Maximum F1 (Hz)", U"1200.0")
 	REAL (U"Minimum F2 (Hz)", U"0.0")
@@ -974,7 +980,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setDataWeighing, U"FormantModeler: Set data weighing", 0)
+FORM (FormantModeler_setDataWeighing, U"FormantModeler: Set data weighing", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	OPTIONMENU (U"Weigh data", 2)
@@ -991,7 +997,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setTolerance, U"FormantModeler: Set tolerance", 0)
+FORM (FormantModeler_setTolerance, U"FormantModeler: Set tolerance", nullptr)
 	REAL (U"Tolerance", U"1e-5")
 	OK
 DO
@@ -1001,7 +1007,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setParameterValueFixed, U"FormantModeler: Set parameter value fixed", 0)
+FORM (FormantModeler_setParameterValueFixed, U"FormantModeler: Set parameter value fixed", nullptr)
 	NATURAL (U"Formant number", U"1")
 	NATURAL (U"Parameter number", U"1")
 	REAL (U"Value", U"0.0")
@@ -1014,7 +1020,7 @@ DO
 END
 
 
-FORM (FormantModeler_setParameterFree, U"FormantModeler: Set parameter free", 0)
+FORM (FormantModeler_setParameterFree, U"FormantModeler: Set parameter free", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	INTEGER (U"left Parameter range", U"0")
@@ -1028,7 +1034,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setParameterValuesToZero, U"FormantModeler: Set parameter values to zero", 0)
+FORM (FormantModeler_setParameterValuesToZero, U"FormantModeler: Set parameter values to zero", nullptr)
 	INTEGER (U"left Formant range", U"0")
 	INTEGER (U"right Formant range", U"0")
 	REAL (U"Number of sigmas", U"1.0")
@@ -1041,7 +1047,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setDataPointValue, U"FormantModeler: Set data point value", 0)
+FORM (FormantModeler_setDataPointValue, U"FormantModeler: Set data point value", nullptr)
 	NATURAL (U"Formant index", U"1")
 	NATURAL (U"Data index", U"1")
 	REAL (U"Value", U"1.0")
@@ -1054,7 +1060,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setDataPointSigma, U"FormantModeler: Set data point sigma", 0)
+FORM (FormantModeler_setDataPointSigma, U"FormantModeler: Set data point sigma", nullptr)
 	NATURAL (U"Formant index", U"1")
 	NATURAL (U"Data index", U"1")
 	REAL (U"Sigma", U"10.0")
@@ -1067,7 +1073,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_setDataPointStatus, U"FormantModeler: Set data point status", 0)
+FORM (FormantModeler_setDataPointStatus, U"FormantModeler: Set data point status", nullptr)
 	NATURAL (U"Formant index", U"1")
 	NATURAL (U"Data index", U"1")
 	OPTIONMENU (U"Status", 1)
@@ -1090,7 +1096,7 @@ DIRECT (FormantModeler_fitModel)
 	}
 END
 
-FORM (FormantModeler_to_Covariance_parameters, U"", 0)
+FORM (FormantModeler_to_Covariance_parameters, U"", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -1102,7 +1108,7 @@ DO
 	}
 END
 
-FORM (FormantModeler_extractDataModeler, U"FormantModeler: Extract DataModeler", 0)
+FORM (FormantModeler_extractDataModeler, U"FormantModeler: Extract DataModeler", nullptr)
 	NATURAL (U"Formant number", U"1")
 	OK
 DO
@@ -1114,8 +1120,8 @@ DO
 	}
 END
 
-FORM (FormantModeler_to_Table_zscores, U"", 0)
-	BOOLEAN (U"Bandwidths as standard deviation", 1)
+FORM (FormantModeler_to_Table_zscores, U"", nullptr)
+	BOOLEAN (U"Bandwidths as standard deviation", true)
 	OK
 DO
 	LOOP {
@@ -1125,9 +1131,9 @@ DO
 	}
 END
 
-FORM (FormantModeler_processOutliers, U"", 0)
+FORM (FormantModeler_processOutliers, U"", nullptr)
 	POSITIVE (U"Number of sigmas", U"3.0")
-	BOOLEAN (U"Bandwidths as standard deviation", 1)
+	BOOLEAN (U"Bandwidths as standard deviation", true)
 	OK
 DO
 	LOOP {
@@ -1149,14 +1155,15 @@ DIRECT (OptimalCeilingTier_edit)
 	LOOP if (CLASS == classOptimalCeilingTier) {
 		iam (OptimalCeilingTier);
 		autoOptimalCeilingTierEditor editor = OptimalCeilingTierEditor_create (ID_AND_FULL_NAME, me, sound, true);
-		praat_installEditor (editor.transfer(), IOBJECT);
+		praat_installEditor (editor.get(), IOBJECT);
+		editor.releaseToUser();
 	}
 END
 
 
 /*************************** PitchModeler *************************************/
 
-FORM (Pitch_to_PitchModeler, U"Pitch: To PitchModeler", 0)
+FORM (Pitch_to_PitchModeler, U"Pitch: To PitchModeler", nullptr)
 	REAL (U"left Start time (s)", U"0.0")
 	REAL (U"right End time (s)", U"0.1")
 	INTEGER (U"Order of polynomials", U"2")
@@ -1170,13 +1177,13 @@ DO
 	}
 END
 
-FORM (PitchModeler_draw, U"PitchModeler: Draw", 0)
+FORM (PitchModeler_draw, U"PitchModeler: Draw", nullptr)
 	REAL (U"left Time range (s)", U"0.0")
 	REAL (U"right Time range (s)", U"0.0")
 	REAL (U"left Frequency range (Hz)", U"0.0")
 	REAL (U"right Frequency range (Hz)", U"500.0")
 	INTEGER (U"Order of polynomial for estimation", U"2")
-	BOOLEAN (U"Garnish", 1)
+	BOOLEAN (U"Garnish", true)
 	OK
 DO
 	autoPraatPicture picture;
@@ -1187,7 +1194,7 @@ DO
 	}
 END
 
-FORM (Sound_getOptimalFormantCeiling, U"Sound: Get optimal formant ceiling", 0)
+FORM (Sound_getOptimalFormantCeiling, U"Sound: Get optimal formant ceiling", nullptr)
 	REAL (U"left Time range (s)", U"0.1")
 	REAL (U"right Time range (s)", U"0.15")
 	POSITIVE (U"Window length (s)", U"0.015")
@@ -1221,7 +1228,7 @@ DO
 	}
 END
 
-FORM (Sound_to_Formant_interval, U"Sound: To Formant (interval)", 0)
+FORM (Sound_to_Formant_interval, U"Sound: To Formant (interval)", nullptr)
 	REAL (U"left Time range (s)", U"0.1")
 	REAL (U"right Time range (s)", U"0.15")
 	POSITIVE (U"Window length (s)", U"0.015")
@@ -1256,7 +1263,7 @@ DO
 	}
 END
 
-FORM (Sound_to_Formant_interval_constrained, U"Sound: To Formant (interval, constrained)", 0)
+FORM (Sound_to_Formant_interval_constrained, U"Sound: To Formant (interval, constrained)", nullptr)
 	REAL (U"left Time range (s)", U"0.1")
 	REAL (U"right Time range (s)", U"0.15")
 	POSITIVE (U"Window length (s)", U"0.015")
@@ -1298,7 +1305,7 @@ DO
 	}
 END
 
-FORM (Sound_to_Formant_interval_constrained_robust, U"Sound: To Formant (interval, constrained, robust)", 0)
+FORM (Sound_to_Formant_interval_constrained_robust, U"Sound: To Formant (interval, constrained, robust)", nullptr)
 	REAL (U"left Time range (s)", U"0.1")
 	REAL (U"right Time range (s)", U"0.15")
 	POSITIVE (U"Window length (s)", U"0.015")
@@ -1340,7 +1347,7 @@ DO
 	}
 END
 
-FORM (Sound_to_OptimalCeilingTier, U"", 0)
+FORM (Sound_to_OptimalCeilingTier, U"", nullptr)
 	POSITIVE (U"Window length (s)", U"0.015")
 	POSITIVE (U"Time step (s)", U"0.0025")
 	POSITIVE (U"left Maximum frequency range (Hz)", U"4500.0")
@@ -1370,7 +1377,7 @@ DO
 	}
 END
 
-FORM (Table_to_DataModeler, U"", 0)
+FORM (Table_to_DataModeler, U"", nullptr)
 	REAL (U"left X range", U"0.0")
 	REAL (U"right X range", U"0.0 (=auto)")
 	WORD (U"Column with X data", U"")
