@@ -1,20 +1,19 @@
 /* AmplitudeTier.cpp
  *
- * Copyright (C) 2003-2011,2014,2015 Paul Boersma
+ * Copyright (C) 2003-2011,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "AmplitudeTier.h"
@@ -24,7 +23,7 @@ Thing_implement (AmplitudeTier, RealTier, 0);
 autoAmplitudeTier AmplitudeTier_create (double tmin, double tmax) {
 	try {
 		autoAmplitudeTier me = Thing_new (AmplitudeTier);
-		RealTier_init (me.peek(), tmin, tmax);
+		RealTier_init (me.get(), tmin, tmax);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"AmplitudeTier not created.");
@@ -49,9 +48,9 @@ autoAmplitudeTier PointProcess_upto_AmplitudeTier (PointProcess me, double sound
 autoAmplitudeTier IntensityTier_to_AmplitudeTier (IntensityTier me) {
 	try {
 		autoAmplitudeTier thee = Thing_new (AmplitudeTier);
-		my structRealTier :: v_copy (thee.peek());
-		for (long i = 1; i <= thy numberOfPoints(); i ++) {
-			RealPoint point = thy point (i);
+		my structRealTier :: v_copy (thee.get());
+		for (long i = 1; i <= thy points.size; i ++) {
+			RealPoint point = thy points.at [i];
 			point -> value = pow (10.0, point -> value / 20.0) * 2.0e-5;
 		}
 		return thee;
@@ -64,9 +63,9 @@ autoIntensityTier AmplitudeTier_to_IntensityTier (AmplitudeTier me, double thres
 	try {
 		double threshold_Pa = pow (10.0, threshold_dB / 20.0) * 2.0e-5;   // often zero!
 		autoIntensityTier thee = Thing_new (IntensityTier);
-		my structRealTier :: v_copy (thee.peek());
-		for (long i = 1; i <= thy numberOfPoints(); i ++) {
-			RealPoint point = thy point (i);
+		my structRealTier :: v_copy (thee.get());
+		for (long i = 1; i <= thy points.size; i ++) {
+			RealPoint point = thy points.at [i];
 			double absoluteValue = fabs (point -> value);
 			point -> value = absoluteValue <= threshold_Pa ? threshold_dB : 20.0 * log10 (absoluteValue / 2.0e-5);
 		}
@@ -81,7 +80,7 @@ autoTableOfReal AmplitudeTier_downto_TableOfReal (AmplitudeTier me) {
 }
 
 void Sound_AmplitudeTier_multiply_inline (Sound me, AmplitudeTier amplitude) {
-	if (amplitude -> numberOfPoints() == 0) return;
+	if (amplitude -> points.size == 0) return;
 	for (long isamp = 1; isamp <= my nx; isamp ++) {
 		double t = my x1 + (isamp - 1) * my dx;
 		double factor = RealTier_getValueAtTime (amplitude, t);
@@ -94,8 +93,8 @@ void Sound_AmplitudeTier_multiply_inline (Sound me, AmplitudeTier amplitude) {
 autoSound Sound_AmplitudeTier_multiply (Sound me, AmplitudeTier amplitude) {
 	try {
 		autoSound thee = Data_copy (me);
-		Sound_AmplitudeTier_multiply_inline (thee.peek(), amplitude);
-		Vector_scale (thee.peek(), 0.9);
+		Sound_AmplitudeTier_multiply_inline (thee.get(), amplitude);
+		Vector_scale (thee.get(), 0.9);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not multiplied by ", amplitude, U".");
@@ -105,11 +104,11 @@ autoSound Sound_AmplitudeTier_multiply (Sound me, AmplitudeTier amplitude) {
 autoAmplitudeTier PointProcess_Sound_to_AmplitudeTier_point (PointProcess me, Sound thee) {
 	try {
 		long imin, imax, numberOfPeaks = PointProcess_getWindowPoints (me, my xmin, my xmax, & imin, & imax);
-		if (numberOfPeaks < 3) return nullptr;
+		if (numberOfPeaks < 3) return autoAmplitudeTier();
 		autoAmplitudeTier him = AmplitudeTier_create (my xmin, my xmax);
 		for (long i = imin; i <= imax; i ++) {
 			double value = Vector_getValueAtX (thee, my t [i], Vector_CHANNEL_AVERAGE, Vector_VALUE_INTERPOLATION_SINC700);
-			if (NUMdefined (value)) RealTier_addPoint (him.peek(), my t [i], value);
+			if (NUMdefined (value)) RealTier_addPoint (him.get(), my t [i], value);
 		}
 		return him;
 	} catch (MelderError) {
@@ -165,7 +164,7 @@ autoAmplitudeTier PointProcess_Sound_to_AmplitudeTier_period (PointProcess me, S
 			if (pmin == pmax || (p1 >= pmin && p1 <= pmax && p2 >= pmin && p2 <= pmax && intervalFactor <= maximumPeriodFactor)) {
 				double peak = Sound_getHannWindowedRms (thee, my t [i], 0.2 * p1, 0.2 * p2);
 				if (NUMdefined (peak) && peak > 0.0)
-					RealTier_addPoint (him.peek(), my t [i], peak);
+					RealTier_addPoint (him.get(), my t [i], peak);
 			}
 		}
 		return him;
@@ -176,8 +175,8 @@ autoAmplitudeTier PointProcess_Sound_to_AmplitudeTier_period (PointProcess me, S
 double AmplitudeTier_getShimmer_local (AmplitudeTier me, double pmin, double pmax, double maximumAmplitudeFactor) {
 	long numberOfPeaks = 0;
 	double numerator = 0.0, denominator = 0.0;
-	RealPoint *points = (RealPoint *) my points -> item;
-	for (long i = 2; i <= my points -> size; i ++) {
+	RealPoint *points = & my points.at [0];
+	for (long i = 2; i <= my points.size; i ++) {
 		double p = points [i] -> number - points [i - 1] -> number;
 		if (pmin == pmax || (p >= pmin && p <= pmax)) {
 			double a1 = points [i - 1] -> value, a2 = points [i] -> value;
@@ -191,7 +190,7 @@ double AmplitudeTier_getShimmer_local (AmplitudeTier me, double pmin, double pma
 	if (numberOfPeaks < 1) return NUMundefined;
 	numerator /= numberOfPeaks;
 	numberOfPeaks = 0;
-	for (long i = 1; i < my points -> size; i ++) {
+	for (long i = 1; i < my points.size; i ++) {
 		denominator += points [i] -> value;
 		numberOfPeaks ++;
 	}
@@ -202,8 +201,8 @@ double AmplitudeTier_getShimmer_local (AmplitudeTier me, double pmin, double pma
 double AmplitudeTier_getShimmer_local_dB (AmplitudeTier me, double pmin, double pmax, double maximumAmplitudeFactor) {
 	long numberOfPeaks = 0;
 	double result = 0.0;
-	RealPoint *points = (RealPoint *) my points -> item;
-	for (long i = 2; i <= my points -> size; i ++) {
+	RealPoint *points = & my points.at [0];
+	for (long i = 2; i <= my points.size; i ++) {
 		double p = points [i] -> number - points [i - 1] -> number;
 		if (pmin == pmax || (p >= pmin && p <= pmax)) {
 			double a1 = points [i - 1] -> value, a2 = points [i] -> value;
@@ -221,8 +220,8 @@ double AmplitudeTier_getShimmer_local_dB (AmplitudeTier me, double pmin, double
 double AmplitudeTier_getShimmer_apq3 (AmplitudeTier me, double pmin, double pmax, double maximumAmplitudeFactor) {
 	long numberOfPeaks = 0;
 	double numerator = 0.0, denominator = 0.0;
-	RealPoint *points = (RealPoint *) my points -> item;
-	for (long i = 2; i <= my points -> size - 1; i ++) {
+	RealPoint *points = & my points.at [0];
+	for (long i = 2; i <= my points.size - 1; i ++) {
 		double
 			p1 = points [i] -> number - points [i - 1] -> number,
 			p2 = points [i + 1] -> number - points [i] -> number;
@@ -239,7 +238,7 @@ double AmplitudeTier_getShimmer_apq3 (AmplitudeTier me, double pmin, double pmax
 	if (numberOfPeaks < 1) return NUMundefined;
 	numerator /= numberOfPeaks;
 	numberOfPeaks = 0;
-	for (long i = 1; i < my points -> size; i ++) {
+	for (long i = 1; i < my points.size; i ++) {
 		denominator += points [i] -> value;
 		numberOfPeaks ++;
 	}
@@ -250,8 +249,8 @@ double AmplitudeTier_getShimmer_apq3 (AmplitudeTier me, double pmin, double pmax
 double AmplitudeTier_getShimmer_apq5 (AmplitudeTier me, double pmin, double pmax, double maximumAmplitudeFactor) {
 	long numberOfPeaks = 0;
 	double numerator = 0.0, denominator = 0.0;
-	RealPoint *points = (RealPoint *) my points -> item;
-	for (long i = 3; i <= my points -> size - 2; i ++) {
+	RealPoint *points = & my points.at [0];
+	for (long i = 3; i <= my points.size - 2; i ++) {
 		double
 			p1 = points [i - 1] -> number - points [i - 2] -> number,
 			p2 = points [i] -> number - points [i - 1] -> number,
@@ -276,7 +275,7 @@ double AmplitudeTier_getShimmer_apq5 (AmplitudeTier me, double pmin, double pmax
 	if (numberOfPeaks < 1) return NUMundefined;
 	numerator /= numberOfPeaks;
 	numberOfPeaks = 0;
-	for (long i = 1; i < my points -> size; i ++) {
+	for (long i = 1; i < my points.size; i ++) {
 		denominator += points [i] -> value;
 		numberOfPeaks ++;
 	}
@@ -287,8 +286,8 @@ double AmplitudeTier_getShimmer_apq5 (AmplitudeTier me, double pmin, double pmax
 double AmplitudeTier_getShimmer_apq11 (AmplitudeTier me, double pmin, double pmax, double maximumAmplitudeFactor) {
 	long numberOfPeaks = 0;
 	double numerator = 0.0, denominator = 0.0;
-	RealPoint *points = (RealPoint *) my points -> item;
-	for (long i = 6; i <= my points -> size - 5; i ++) {
+	RealPoint *points = & my points.at [0];
+	for (long i = 6; i <= my points.size - 5; i ++) {
 		double
 			p1 = points [i - 4] -> number - points [i - 5] -> number,
 			p2 = points [i - 3] -> number - points [i - 4] -> number,
@@ -329,7 +328,7 @@ double AmplitudeTier_getShimmer_apq11 (AmplitudeTier me, double pmin, double pma
 	if (numberOfPeaks < 1) return NUMundefined;
 	numerator /= numberOfPeaks;
 	numberOfPeaks = 0;
-	for (long i = 1; i < my points -> size; i ++) {
+	for (long i = 1; i < my points.size; i ++) {
 		denominator += points [i] -> value;
 		numberOfPeaks ++;
 	}
@@ -351,14 +350,14 @@ autoSound AmplitudeTier_to_Sound (AmplitudeTier me, double samplingFrequency, lo
 		double *sound;
 		autoSound thee = Sound_create (1, my xmin, my xmax, sound_nt, dt, t1);
 		sound = thy z [1];
-		for (long it = 1; it <= my points -> size; it ++) {
-			RealPoint point = (RealPoint) my points -> item [it];
+		for (long it = 1; it <= my points.size; it ++) {
+			RealPoint point = my points.at [it];
 			double t = point -> number, amplitude = point -> value, angle, halfampsinangle;
-			long mid = Sampled_xToNearestIndex (thee.peek(), t), j;
+			long mid = Sampled_xToNearestIndex (thee.get(), t), j;
 			long begin = mid - interpolationDepth, end = mid + interpolationDepth;
 			if (begin < 1) begin = 1;
 			if (end > thy nx) end = thy nx;
-			angle = NUMpi * (Sampled_indexToX (thee.peek(), begin) - t) / thy dx;
+			angle = NUMpi * (Sampled_indexToX (thee.get(), begin) - t) / thy dx;
 			halfampsinangle = 0.5 * amplitude * sin (angle);
 			for (j = begin; j <= end; j ++) {
 				if (fabs (angle) < 1e-6)
