@@ -1,20 +1,19 @@
 /* ExperimentMFC.cpp
  *
- * Copyright (C) 2001-2011,2013 Paul Boersma
+ * Copyright (C) 2001-2011,2013,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -63,15 +62,18 @@
 #include "oo_DESCRIPTION.h"
 #include "ExperimentMFC_def.h"
 
-Thing_implement (ExperimentMFC, Daata, 6);
-
 #include "enums_getText.h"
 #include "Experiment_enums.h"
 #include "enums_getValue.h"
 #include "Experiment_enums.h"
 
+
+#pragma mark - class ExperimentMFC
+
+Thing_implement (ExperimentMFC, Daata, 6);
+
 static void readSound (ExperimentMFC me, const char32 *fileNameHead, const char32 *fileNameTail,
-	double medialSilenceDuration, char32 **name, Sound *sound)
+	double medialSilenceDuration, char32 **name, autoSound *sound)
 {
 	char32 fileNameBuffer [256], *fileNames = & fileNameBuffer [0];
 	Melder_sprint (fileNameBuffer,256, *name);
@@ -84,7 +86,7 @@ static void readSound (ExperimentMFC me, const char32 *fileNameHead, const char3
 	#if defined (_WIN32)
 		for (;;) { char32 *slash = str32chr (fileNames, U'/'); if (! slash) break; *slash = U'\\'; }
 	#endif
-	forget (*sound);
+	sound->reset();
 	char32 pathName [kMelder_MAXPATH+1];
 	/*
 	 * 'fileNames' can contain commas, which separate partial file names.
@@ -124,9 +126,9 @@ static void readSound (ExperimentMFC me, const char32 *fileNameHead, const char3
 		/*
 		 * Read the substimulus.
 		 */
-		autoSound substimulus = (Sound) Data_readFromFile (& file);   // Sound_readFromSoundFile (& file);
+		autoSound substimulus = Data_readFromFile (& file). static_cast_move<structSound>();
 		if (substimulus -> classInfo != classSound)
-			Melder_throw (U"File ", & file, U" contains a ", Thing_className (substimulus.peek()), U" instead of a sound.");
+			Melder_throw (U"File ", & file, U" contains a ", Thing_className (substimulus.get()), U" instead of a sound.");
 		/*
 		 * Check whether all sounds have the same number of channels.
 		 */
@@ -146,13 +148,10 @@ static void readSound (ExperimentMFC me, const char32 *fileNameHead, const char3
 		/*
 		 * Append the substimuli, perhaps with silent intervals.
 		 */
-		if (! *sound) {
-			*sound = substimulus.transfer();
+		if (*sound) {
+			*sound = Sounds_append (sound->get(), medialSilenceDuration, substimulus.get());
 		} else {
-			autoSound newStimulus = Sounds_append (*sound, medialSilenceDuration, substimulus.peek());
-			Melder_assert (sound == & (*sound));
-			forget (*sound);
-			*sound = newStimulus.transfer();
+			*sound = substimulus.move();
 		}
 		/*
 		 * Cycle.
@@ -336,29 +335,39 @@ static void playSound (ExperimentMFC me, Sound sound, Sound carrierBefore, Sound
 }
 
 void ExperimentMFC_playStimulus (ExperimentMFC me, long istim) {
-	playSound (me, my stimulus [istim]. sound,
-		my stimulusCarrierBefore. sound, my stimulusCarrierAfter. sound, my stimulusInitialSilenceDuration, my stimulusFinalSilenceDuration);
+	playSound (me, my stimulus [istim]. sound.get(),
+		my stimulusCarrierBefore. sound.get(), my stimulusCarrierAfter. sound.get(),
+		my stimulusInitialSilenceDuration, my stimulusFinalSilenceDuration);
 }
 
 void ExperimentMFC_playResponse (ExperimentMFC me, long iresp) {
-	playSound (me, my response [iresp]. sound,
-		my responseCarrierBefore. sound, my responseCarrierAfter. sound, my responseInitialSilenceDuration, my responseFinalSilenceDuration);
+	playSound (me, my response [iresp]. sound.get(),
+		my responseCarrierBefore. sound.get(), my responseCarrierAfter. sound.get(),
+		my responseInitialSilenceDuration, my responseFinalSilenceDuration);
 }
 
+
+#pragma mark - class ExperimentMFCList
+
+Thing_implement (ExperimentMFCList, Ordered, 0);
+
+
+#pragma mark - class ResultsMFC
+
 Thing_implement (ResultsMFC, Daata, 2);
 
-ResultsMFC ResultsMFC_create (long numberOfTrials) {
+autoResultsMFC ResultsMFC_create (long numberOfTrials) {
 	try {
 		autoResultsMFC me = Thing_new (ResultsMFC);
 		my numberOfTrials = numberOfTrials;
 		my result = NUMvector <structTrialMFC> (1, my numberOfTrials);
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"ResultsMFC not created.");
 	}
 }
 
-ResultsMFC ExperimentMFC_extractResults (ExperimentMFC me) {
+autoResultsMFC ExperimentMFC_extractResults (ExperimentMFC me) {
 	try {
 		if (my trial == 0 || my trial <= my numberOfTrials)
 			Melder_warning (U"The experiment was not finished. Only the first ", my trial - 1 + my pausing, U" responses are valid.");
@@ -372,13 +381,13 @@ ResultsMFC ExperimentMFC_extractResults (ExperimentMFC me) {
 			thy result [trial]. goodness = my goodnesses [trial];
 			thy result [trial]. reactionTime = my reactionTimes [trial];
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": results not extracted.");
 	}
 }
 
-ResultsMFC ResultsMFC_removeUnsharedStimuli (ResultsMFC me, ResultsMFC thee) {
+autoResultsMFC ResultsMFC_removeUnsharedStimuli (ResultsMFC me, ResultsMFC thee) {
 	try {
 		autoResultsMFC him = ResultsMFC_create (thy numberOfTrials);
 		his numberOfTrials = 0;
@@ -398,18 +407,18 @@ ResultsMFC ResultsMFC_removeUnsharedStimuli (ResultsMFC me, ResultsMFC thee) {
 		}
 		if (his numberOfTrials == 0)
 			Melder_throw (U"No shared stimuli.");
-		return him.transfer();
+		return him;
 	} catch (MelderError) {
 		Melder_throw (me, U" & ", thee, U": unshared stimuli not removed.");
 	}
 }
 
-Table ResultsMFCs_to_Table (Collection me) {
+autoTable ResultsMFCs_to_Table (OrderedOf<structResultsMFC>* me) {
 	try {
 		long irow = 0;
 		bool hasGoodnesses = false, hasReactionTimes = false;
 		for (long iresults = 1; iresults <= my size; iresults ++) {
-			ResultsMFC results = (ResultsMFC) my item [iresults];
+			ResultsMFC results = my at [iresults];
 			for (long itrial = 1; itrial <= results -> numberOfTrials; itrial ++) {
 				irow ++;
 				if (results -> result [itrial]. goodness != 0)
@@ -419,66 +428,66 @@ Table ResultsMFCs_to_Table (Collection me) {
 			}
 		}
 		autoTable thee = Table_create (irow, 3 + hasGoodnesses + hasReactionTimes);
-		Table_setColumnLabel (thee.peek(), 1, U"subject");
-		Table_setColumnLabel (thee.peek(), 2, U"stimulus");
-		Table_setColumnLabel (thee.peek(), 3, U"response");
+		Table_setColumnLabel (thee.get(), 1, U"subject");
+		Table_setColumnLabel (thee.get(), 2, U"stimulus");
+		Table_setColumnLabel (thee.get(), 3, U"response");
 		if (hasGoodnesses)
-			Table_setColumnLabel (thee.peek(), 4, U"goodness");
+			Table_setColumnLabel (thee.get(), 4, U"goodness");
 		if (hasReactionTimes)
-			Table_setColumnLabel (thee.peek(), 4 + hasGoodnesses, U"reactionTime");
+			Table_setColumnLabel (thee.get(), 4 + hasGoodnesses, U"reactionTime");
 		irow = 0;
 		for (long iresults = 1; iresults <= my size; iresults ++) {
-			ResultsMFC results = (ResultsMFC) my item [iresults];
+			ResultsMFC results = my at [iresults];
 			for (long itrial = 1; itrial <= results -> numberOfTrials; itrial ++) {
 				irow ++;
-				Table_setStringValue (thee.peek(), irow, 1, results -> name);
-				Table_setStringValue (thee.peek(), irow, 2, results -> result [itrial]. stimulus);
-				Table_setStringValue (thee.peek(), irow, 3, results -> result [itrial]. response);
+				Table_setStringValue (thee.get(), irow, 1, results -> name);
+				Table_setStringValue (thee.get(), irow, 2, results -> result [itrial]. stimulus);
+				Table_setStringValue (thee.get(), irow, 3, results -> result [itrial]. response);
 				if (hasGoodnesses) {
-					Table_setNumericValue (thee.peek(), irow, 4, results -> result [itrial]. goodness);
+					Table_setNumericValue (thee.get(), irow, 4, results -> result [itrial]. goodness);
 				}
 				if (hasReactionTimes) {
-					Table_setNumericValue (thee.peek(), irow, 4 + hasGoodnesses, results -> result [itrial]. reactionTime);
+					Table_setNumericValue (thee.get(), irow, 4 + hasGoodnesses, results -> result [itrial]. reactionTime);
 				}
 			}
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (U"ResultsMFC objects not collected to Table.");
 	}
 }
 
-Categories ResultsMFC_to_Categories_stimuli (ResultsMFC me) {
+autoCategories ResultsMFC_to_Categories_stimuli (ResultsMFC me) {
 	try {
 		autoCategories thee = Categories_create ();
 		for (long trial = 1; trial <= my numberOfTrials; trial ++) {
 			autoSimpleString category = SimpleString_create (my result [trial]. stimulus);
-			Collection_addItem (thee.peek(), category.transfer());
+			thy addItem_move (category.move());
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": stimuli not converted to Categories.");
 	}
 }
 
-Categories ResultsMFC_to_Categories_responses (ResultsMFC me) {
+autoCategories ResultsMFC_to_Categories_responses (ResultsMFC me) {
 	try {
 		autoCategories thee = Categories_create ();
 		for (long trial = 1; trial <= my numberOfTrials; trial ++) {
 			autoSimpleString category = SimpleString_create (my result [trial]. response);
-			Collection_addItem (thee.peek(), category.transfer());
+			thy addItem_move (category.move());
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": responses not converted to Categories.");
 	}
 }
 
-static int compare (SimpleString me, SimpleString thee) {
-	return str32cmp (my string, thy string);
-}
+
+#pragma mark - class Categories extensions
+
 void Categories_sort (Categories me) {
-	NUMsort_p (my size, my item, (int (*) (const void *, const void *)) compare);
+	my sort (SimpleString_compare);
 }
 
 double Categories_getEntropy (Categories me) {
@@ -486,9 +495,9 @@ double Categories_getEntropy (Categories me) {
 	char32 *previousString = nullptr;
 	double entropy = 0.0;
 	autoCategories thee = Data_copy (me);
-	Categories_sort (thee.peek());
+	Categories_sort (thee.get());
 	for (long i = 1; i <= thy size; i ++) {
-		SimpleString s = (SimpleString) thy item [i];
+		SimpleString s = thy at [i];
 		char32 *string = s -> string;
 		if (previousString && ! str32equ (string, previousString)) {
 			double p = (double) numberOfTokens / thy size;
