@@ -2,19 +2,18 @@
  *
  * Copyright (C) 2001-2011,2013,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -38,9 +37,13 @@ oo_DEFINE_STRUCT (SoundMFC)
 
 	oo_STRING (name)
 	#if !oo_READING && !oo_WRITING
-		oo_OBJECT (Sound, 0, sound)
+		oo_AUTO_OBJECT (Sound, 0, sound)
 	#endif
 		
+	#if oo_DESTROYING
+		sound.reset();
+	#endif
+
 oo_END_STRUCT (SoundMFC)
 #undef ooSTRUCT
 
@@ -53,9 +56,13 @@ oo_DEFINE_STRUCT (StimulusMFC)
 		oo_STRING (visibleText)
 	oo_ENDFROM
 	#if !oo_READING && !oo_WRITING
-		oo_OBJECT (Sound, 0, sound)
+		oo_AUTO_OBJECT (Sound, 0, sound)
 	#endif
-		
+
+	#if oo_DESTROYING
+		sound.reset();
+	#endif
+
 oo_END_STRUCT (StimulusMFC)
 #undef ooSTRUCT
 
@@ -76,7 +83,11 @@ oo_DEFINE_STRUCT (ResponseMFC)
 	oo_ENDFROM
 	oo_STRING (name)
 	#if !oo_READING && !oo_WRITING
-		oo_OBJECT (Sound, 0, sound)
+		oo_AUTO_OBJECT (Sound, 0, sound)
+	#endif
+
+	#if oo_DESTROYING
+		sound.reset();
 	#endif
 
 oo_END_STRUCT (ResponseMFC)
