@@ -2,19 +2,18 @@
  *
  * Copyright (C) 2008-2011,2012,2013,2014,2015 Paul Boersma & David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "FormantGridEditor.h"
@@ -32,34 +31,31 @@ Thing_implement (FormantGridEditor, FunctionEditor, 0);
 
 /********** MENU COMMANDS **********/
 
-static void menu_cb_removePoints (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
+static void menu_cb_removePoints (FormantGridEditor me, EDITOR_ARGS_DIRECT) {
 	Editor_save (me, U"Remove point(s)");
 	FormantGrid grid = (FormantGrid) my data;
-	Ordered tiers = my editingBandwidths ? grid -> bandwidths : grid -> formants;
-	RealTier tier = (RealTier) tiers -> item [my selectedFormant];
+	OrderedOf<structRealTier>* tiers = ( my editingBandwidths ? & grid -> bandwidths : & grid -> formants );
+	RealTier tier = tiers->at [my selectedFormant];
 	if (my d_startSelection == my d_endSelection)
-		AnyTier_removePointNear (tier, my d_startSelection);
+		AnyTier_removePointNear (tier->asAnyTier(), my d_startSelection);
 	else
-		AnyTier_removePointsBetween (tier, my d_startSelection, my d_endSelection);
+		AnyTier_removePointsBetween (tier->asAnyTier(), my d_startSelection, my d_endSelection);
 	FunctionEditor_redraw (me);
 	Editor_broadcastDataChanged (me);
 }
 
-static void menu_cb_addPointAtCursor (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
+static void menu_cb_addPointAtCursor (FormantGridEditor me, EDITOR_ARGS_DIRECT) {
 	Editor_save (me, U"Add point");
 	FormantGrid grid = (FormantGrid) my data;
-	Ordered tiers = my editingBandwidths ? grid -> bandwidths : grid -> formants;
-	RealTier tier = (RealTier) tiers -> item [my selectedFormant];
+	OrderedOf<structRealTier>* tiers = ( my editingBandwidths ? & grid -> bandwidths : & grid -> formants );
+	RealTier tier = tiers->at [my selectedFormant];
 	RealTier_addPoint (tier, 0.5 * (my d_startSelection + my d_endSelection), my ycursor);
 	FunctionEditor_redraw (me);
 	Editor_broadcastDataChanged (me);
 }
 
-static void menu_cb_addPointAt (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
-	EDITOR_FORM (U"Add point", 0)
+static void menu_cb_addPointAt (FormantGridEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Add point", nullptr)
 		REAL (U"Time (s)", U"0.0")
 		POSITIVE (U"Frequency (Hz)", U"200.0")
 	EDITOR_OK
@@ -68,17 +64,16 @@ static void menu_cb_addPointAt (EDITOR_ARGS) {
 	EDITOR_DO
 		Editor_save (me, U"Add point");
 		FormantGrid grid = (FormantGrid) my data;
-		Ordered tiers = my editingBandwidths ? grid -> bandwidths : grid -> formants;
-		RealTier tier = (RealTier) tiers -> item [my selectedFormant];
+		OrderedOf<structRealTier>* tiers = ( my editingBandwidths ? & grid -> bandwidths : & grid -> formants );
+		RealTier tier = tiers->at [my selectedFormant];
 		RealTier_addPoint (tier, GET_REAL (U"Time"), GET_REAL (U"Frequency"));
 		FunctionEditor_redraw (me);
 		Editor_broadcastDataChanged (me);
 	EDITOR_END
 }
 
-static void menu_cb_setFormantRange (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
-	EDITOR_FORM (U"Set formant range", 0)
+static void menu_cb_setFormantRange (FormantGridEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Set formant range", nullptr)
 		REAL (U"Minimum formant (Hz)", my default_formantFloor   ())
 		REAL (U"Maximum formant (Hz)", my default_formantCeiling ())
 	EDITOR_OK
@@ -91,9 +86,8 @@ static void menu_cb_setFormantRange (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_setBandwidthRange (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
-	EDITOR_FORM (U"Set bandwidth range", 0)
+static void menu_cb_setBandwidthRange (FormantGridEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Set bandwidth range", nullptr)
 		REAL (U"Minimum bandwidth (Hz)", my default_bandwidthFloor   ())
 		REAL (U"Maximum bandwidth (Hz)", my default_bandwidthCeiling ())
 	EDITOR_OK
@@ -106,8 +100,7 @@ static void menu_cb_setBandwidthRange (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_showBandwidths (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
+static void menu_cb_showBandwidths (FormantGridEditor me, EDITOR_ARGS_DIRECT) {
 	my editingBandwidths = ! my editingBandwidths;
 	GuiMenuItem_check (my d_bandwidthsToggle, my editingBandwidths);
 	FunctionEditor_redraw (me);
@@ -115,25 +108,24 @@ static void menu_cb_showBandwidths (EDITOR_ARGS) {
 
 static void selectFormantOrBandwidth (FormantGridEditor me, long iformant) {
 	FormantGrid grid = (FormantGrid) my data;
-	long numberOfFormants = grid -> formants -> size;
+	long numberOfFormants = grid -> formants.size;
 	if (iformant > numberOfFormants)
 		Melder_throw (U"Cannot select formant ", iformant, U", because the FormantGrid has only ", numberOfFormants, U" formants.");
 	my selectedFormant = iformant;
 	FunctionEditor_redraw (me);
 }
 
-static void menu_cb_selectFirst   (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 1); }
-static void menu_cb_selectSecond  (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 2); }
-static void menu_cb_selectThird   (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 3); }
-static void menu_cb_selectFourth  (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 4); }
-static void menu_cb_selectFifth   (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 5); }
-static void menu_cb_selectSixth   (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 6); }
-static void menu_cb_selectSeventh (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 7); }
-static void menu_cb_selectEighth  (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 8); }
-static void menu_cb_selectNinth   (EDITOR_ARGS) { EDITOR_IAM (FormantGridEditor); selectFormantOrBandwidth (me, 9); }
-static void menu_cb_selectFormantOrBandwidth (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
-	EDITOR_FORM (U"Select formant or bandwidth", 0)
+static void menu_cb_selectFirst   (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 1); }
+static void menu_cb_selectSecond  (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 2); }
+static void menu_cb_selectThird   (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 3); }
+static void menu_cb_selectFourth  (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 4); }
+static void menu_cb_selectFifth   (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 5); }
+static void menu_cb_selectSixth   (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 6); }
+static void menu_cb_selectSeventh (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 7); }
+static void menu_cb_selectEighth  (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 8); }
+static void menu_cb_selectNinth   (FormantGridEditor me, EDITOR_ARGS_DIRECT) { selectFormantOrBandwidth (me, 9); }
+static void menu_cb_selectFormantOrBandwidth (FormantGridEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Select formant or bandwidth", nullptr)
 		NATURAL (U"Formant number", U"1")
 	EDITOR_OK
 		SET_INTEGER (U"Formant number", my selectedFormant)
@@ -143,9 +135,8 @@ static void menu_cb_selectFormantOrBandwidth (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_pitchSettings (EDITOR_ARGS) {
-	EDITOR_IAM (FormantGridEditor);
-	EDITOR_FORM (U"Source pitch settings", 0)
+static void menu_cb_pitchSettings (FormantGridEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Source pitch settings", nullptr)
 		LABEL (U"", U"These settings apply to the pitch curve")
 		LABEL (U"", U"that you hear when playing the FormantGrid.")
 		REAL     (U"Starting time",       my default_source_pitch_tStart  ())
@@ -204,130 +195,128 @@ void structFormantGridEditor :: v_createMenus () {
 
 void structFormantGridEditor :: v_draw () {
 	FormantGrid grid = (FormantGrid) our data;
-	Ordered tiers = our editingBandwidths ? grid -> bandwidths : grid -> formants;
-	RealTier selectedTier = (RealTier) tiers -> item [selectedFormant];
+	OrderedOf<structRealTier>* tiers = ( our editingBandwidths ? & grid -> bandwidths : & grid -> formants );
+	RealTier selectedTier = tiers->at [our selectedFormant];
 	double ymin = our editingBandwidths ? our p_bandwidthFloor   : our p_formantFloor;
 	double ymax = our editingBandwidths ? our p_bandwidthCeiling : our p_formantCeiling;
-	Graphics_setColour (our d_graphics, Graphics_WHITE);
-	Graphics_setWindow (our d_graphics, 0, 1, 0, 1);
-	Graphics_fillRectangle (our d_graphics, 0, 1, 0, 1);
-	Graphics_setWindow (our d_graphics, our d_startWindow, our d_endWindow, ymin, ymax);
-	Graphics_setColour (our d_graphics, Graphics_RED);
-	Graphics_line (our d_graphics, our d_startWindow, our ycursor, our d_endWindow, our ycursor);
-	Graphics_setTextAlignment (our d_graphics, Graphics_RIGHT, Graphics_HALF);
-	Graphics_text (our d_graphics, our d_startWindow, our ycursor, Melder_float (Melder_half (our ycursor)));
-	Graphics_setColour (our d_graphics, Graphics_BLUE);
-	Graphics_setTextAlignment (our d_graphics, Graphics_LEFT, Graphics_TOP);
-	Graphics_text (our d_graphics, our d_endWindow, ymax, Melder_float (Melder_half (ymax)), U" Hz");
-	Graphics_setTextAlignment (our d_graphics, Graphics_LEFT, Graphics_HALF);
-	Graphics_text (our d_graphics, our d_endWindow, ymin, Melder_float (Melder_half (ymin)), U" Hz");
-	Graphics_setLineWidth (our d_graphics, 1);
-	Graphics_setColour (our d_graphics, Graphics_GREY);
-	for (long iformant = 1; iformant <= grid -> formants -> size; iformant ++) if (iformant != our selectedFormant) {
-		RealTier tier = (RealTier) tiers -> item [iformant];
-		long imin = AnyTier_timeToHighIndex (tier, our d_startWindow);
-		long imax = AnyTier_timeToLowIndex (tier, our d_endWindow);
-		long n = tier -> points -> size;
+	Graphics_setColour (our d_graphics.get(), Graphics_WHITE);
+	Graphics_setWindow (our d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+	Graphics_fillRectangle (our d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+	Graphics_setWindow (our d_graphics.get(), our d_startWindow, our d_endWindow, ymin, ymax);
+	Graphics_setColour (our d_graphics.get(), Graphics_RED);
+	Graphics_line (our d_graphics.get(), our d_startWindow, our ycursor, our d_endWindow, our ycursor);
+	Graphics_setTextAlignment (our d_graphics.get(), Graphics_RIGHT, Graphics_HALF);
+	Graphics_text (our d_graphics.get(), our d_startWindow, our ycursor, Melder_float (Melder_half (our ycursor)));
+	Graphics_setColour (our d_graphics.get(), Graphics_BLUE);
+	Graphics_setTextAlignment (our d_graphics.get(), Graphics_LEFT, Graphics_TOP);
+	Graphics_text (our d_graphics.get(), our d_endWindow, ymax, Melder_float (Melder_half (ymax)), U" Hz");
+	Graphics_setTextAlignment (our d_graphics.get(), Graphics_LEFT, Graphics_HALF);
+	Graphics_text (our d_graphics.get(), our d_endWindow, ymin, Melder_float (Melder_half (ymin)), U" Hz");
+	Graphics_setLineWidth (our d_graphics.get(), 1.0);
+	Graphics_setColour (our d_graphics.get(), Graphics_GREY);
+	for (long iformant = 1; iformant <= grid -> formants.size; iformant ++) if (iformant != our selectedFormant) {
+		RealTier tier = tiers->at [iformant];
+		long imin = AnyTier_timeToHighIndex (tier->asAnyTier(), our d_startWindow);
+		long imax = AnyTier_timeToLowIndex (tier->asAnyTier(), our d_endWindow);
+		long n = tier -> points.size;
 		if (n == 0) {
 		} else if (imax < imin) {
 			double yleft = RealTier_getValueAtTime (tier, our d_startWindow);
 			double yright = RealTier_getValueAtTime (tier, our d_endWindow);
-			Graphics_line (our d_graphics, our d_startWindow, yleft, our d_endWindow, yright);
+			Graphics_line (our d_graphics.get(), our d_startWindow, yleft, our d_endWindow, yright);
 		} else for (long i = imin; i <= imax; i ++) {
-			RealPoint point = (RealPoint) tier -> points -> item [i];
+			RealPoint point = tier -> points.at [i];
 			double t = point -> number, y = point -> value;
-			Graphics_fillCircle_mm (our d_graphics, t, y, 2);
+			Graphics_fillCircle_mm (our d_graphics.get(), t, y, 2.0);
 			if (i == 1)
-				Graphics_line (our d_graphics, our d_startWindow, y, t, y);
+				Graphics_line (our d_graphics.get(), our d_startWindow, y, t, y);
 			else if (i == imin)
-				Graphics_line (our d_graphics, t, y, our d_startWindow, RealTier_getValueAtTime (tier, our d_startWindow));
+				Graphics_line (our d_graphics.get(), t, y, our d_startWindow, RealTier_getValueAtTime (tier, our d_startWindow));
 			if (i == n)
-				Graphics_line (our d_graphics, t, y, our d_endWindow, y);
+				Graphics_line (our d_graphics.get(), t, y, our d_endWindow, y);
 			else if (i == imax)
-				Graphics_line (our d_graphics, t, y, our d_endWindow, RealTier_getValueAtTime (tier, our d_endWindow));
+				Graphics_line (our d_graphics.get(), t, y, our d_endWindow, RealTier_getValueAtTime (tier, our d_endWindow));
 			else {
-				RealPoint pointRight = (RealPoint) tier -> points -> item [i + 1];
-				Graphics_line (our d_graphics, t, y, pointRight -> number, pointRight -> value);
+				RealPoint pointRight = tier -> points.at [i + 1];
+				Graphics_line (our d_graphics.get(), t, y, pointRight -> number, pointRight -> value);
 			}
 		}
 	}
-	Graphics_setColour (our d_graphics, Graphics_BLUE);
-	long ifirstSelected = AnyTier_timeToHighIndex (selectedTier, our d_startSelection);
-	long ilastSelected = AnyTier_timeToLowIndex (selectedTier, our d_endSelection);
-	long n = selectedTier -> points -> size;
-	long imin = AnyTier_timeToHighIndex (selectedTier, our d_startWindow);
-	long imax = AnyTier_timeToLowIndex (selectedTier, our d_endWindow);
-	Graphics_setLineWidth (our d_graphics, 2);
+	Graphics_setColour (our d_graphics.get(), Graphics_BLUE);
+	long ifirstSelected = AnyTier_timeToHighIndex (selectedTier->asAnyTier(), our d_startSelection);
+	long ilastSelected = AnyTier_timeToLowIndex (selectedTier->asAnyTier(), our d_endSelection);
+	long n = selectedTier -> points.size;
+	long imin = AnyTier_timeToHighIndex (selectedTier->asAnyTier(), our d_startWindow);
+	long imax = AnyTier_timeToLowIndex (selectedTier->asAnyTier(), our d_endWindow);
+	Graphics_setLineWidth (our d_graphics.get(), 2.0);
 	if (n == 0) {
-		Graphics_setTextAlignment (our d_graphics, Graphics_CENTRE, Graphics_HALF);
-		Graphics_text (our d_graphics, 0.5 * (our d_startWindow + our d_endWindow),
+		Graphics_setTextAlignment (our d_graphics.get(), Graphics_CENTRE, Graphics_HALF);
+		Graphics_text (our d_graphics.get(), 0.5 * (our d_startWindow + our d_endWindow),
 			0.5 * (ymin + ymax), U"(no points in selected formant tier)");
 	} else if (imax < imin) {
 		double yleft = RealTier_getValueAtTime (selectedTier, our d_startWindow);
 		double yright = RealTier_getValueAtTime (selectedTier, our d_endWindow);
-		Graphics_line (our d_graphics, our d_startWindow, yleft, our d_endWindow, yright);
+		Graphics_line (our d_graphics.get(), our d_startWindow, yleft, our d_endWindow, yright);
 	} else for (long i = imin; i <= imax; i ++) {
-		RealPoint point = (RealPoint) selectedTier -> points -> item [i];
+		RealPoint point = selectedTier -> points.at [i];
 		double t = point -> number, y = point -> value;
 		if (i >= ifirstSelected && i <= ilastSelected)
-			Graphics_setColour (our d_graphics, Graphics_RED);
-		Graphics_fillCircle_mm (our d_graphics, t, y, 3);
-		Graphics_setColour (our d_graphics, Graphics_BLUE);
+			Graphics_setColour (our d_graphics.get(), Graphics_RED);
+		Graphics_fillCircle_mm (our d_graphics.get(), t, y, 3);
+		Graphics_setColour (our d_graphics.get(), Graphics_BLUE);
 		if (i == 1)
-			Graphics_line (our d_graphics, our d_startWindow, y, t, y);
+			Graphics_line (our d_graphics.get(), our d_startWindow, y, t, y);
 		else if (i == imin)
-			Graphics_line (our d_graphics, t, y, our d_startWindow, RealTier_getValueAtTime (selectedTier, our d_startWindow));
+			Graphics_line (our d_graphics.get(), t, y, our d_startWindow, RealTier_getValueAtTime (selectedTier, our d_startWindow));
 		if (i == n)
-			Graphics_line (our d_graphics, t, y, our d_endWindow, y);
+			Graphics_line (our d_graphics.get(), t, y, our d_endWindow, y);
 		else if (i == imax)
-			Graphics_line (our d_graphics, t, y, our d_endWindow, RealTier_getValueAtTime (selectedTier, our d_endWindow));
+			Graphics_line (our d_graphics.get(), t, y, our d_endWindow, RealTier_getValueAtTime (selectedTier, our d_endWindow));
 		else {
-			RealPoint pointRight = (RealPoint) selectedTier -> points -> item [i + 1];
-			Graphics_line (our d_graphics, t, y, pointRight -> number, pointRight -> value);
+			RealPoint pointRight = selectedTier -> points.at [i + 1];
+			Graphics_line (our d_graphics.get(), t, y, pointRight -> number, pointRight -> value);
 		}
 	}
-	Graphics_setLineWidth (our d_graphics, 1);
-	Graphics_setColour (our d_graphics, Graphics_BLACK);
+	Graphics_setLineWidth (our d_graphics.get(), 1.0);
+	Graphics_setColour (our d_graphics.get(), Graphics_BLACK);
 }
 
-static void drawWhileDragging (FormantGridEditor me, double xWC, double yWC, long first, long last, double dt, double dy) {
+static void drawWhileDragging (FormantGridEditor me, double /* xWC */, double /* yWC */, long first, long last, double dt, double dy) {
 	FormantGrid grid = (FormantGrid) my data;
-	Ordered tiers = my editingBandwidths ? grid -> bandwidths : grid -> formants;
-	RealTier tier = (RealTier) tiers -> item [my selectedFormant];
+	OrderedOf<structRealTier>* tiers = my editingBandwidths ? & grid -> bandwidths : & grid -> formants;
+	RealTier tier = tiers->at [my selectedFormant];
 	double ymin = my editingBandwidths ? my p_bandwidthFloor   : my p_formantFloor;
 	double ymax = my editingBandwidths ? my p_bandwidthCeiling : my p_formantCeiling;
-	(void) xWC;
-	(void) yWC;
 
 	/*
 	 * Draw all selected points as magenta empty circles, if inside the window.
 	 */
 	for (long i = first; i <= last; i ++) {
-		RealPoint point = (RealPoint) tier -> points -> item [i];
+		RealPoint point = tier -> points.at [i];
 		double t = point -> number + dt, y = point -> value + dy;
 		if (t >= my d_startWindow && t <= my d_endWindow)
-			Graphics_circle_mm (my d_graphics, t, y, 3);
+			Graphics_circle_mm (my d_graphics.get(), t, y, 3.0);
 	}
 
 	if (last == first) {
 		/*
 		 * Draw a crosshair with time and y.
 		 */
-		RealPoint point = (RealPoint) tier -> points -> item [first];
+		RealPoint point = tier -> points.at [first];
 		double t = point -> number + dt, y = point -> value + dy;
-		Graphics_line (my d_graphics, t, ymin, t, ymax - Graphics_dyMMtoWC (my d_graphics, 4.0));
-		Graphics_setTextAlignment (my d_graphics, kGraphics_horizontalAlignment_CENTRE, Graphics_TOP);
-		Graphics_text (my d_graphics, t, ymax, Melder_fixed (t, 6));
-		Graphics_line (my d_graphics, my d_startWindow, y, my d_endWindow, y);
-		Graphics_setTextAlignment (my d_graphics, Graphics_LEFT, Graphics_BOTTOM);
-		Graphics_text (my d_graphics, my d_startWindow, y, Melder_fixed (y, 6));
+		Graphics_line (my d_graphics.get(), t, ymin, t, ymax - Graphics_dyMMtoWC (my d_graphics.get(), 4.0));
+		Graphics_setTextAlignment (my d_graphics.get(), kGraphics_horizontalAlignment_CENTRE, Graphics_TOP);
+		Graphics_text (my d_graphics.get(), t, ymax, Melder_fixed (t, 6));
+		Graphics_line (my d_graphics.get(), my d_startWindow, y, my d_endWindow, y);
+		Graphics_setTextAlignment (my d_graphics.get(), Graphics_LEFT, Graphics_BOTTOM);
+		Graphics_text (my d_graphics.get(), my d_startWindow, y, Melder_fixed (y, 6));
 	}
 }
 
-int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPressed) {
+bool structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPressed) {
 	FormantGrid grid = (FormantGrid) our data;
-	Ordered tiers = editingBandwidths ? grid -> bandwidths : grid -> formants;
-	RealTier tier = (RealTier) tiers -> item [selectedFormant];
+	OrderedOf<structRealTier>* tiers = our editingBandwidths ? & grid -> bandwidths : & grid -> formants;
+	RealTier tier = tiers->at [selectedFormant];
 	double ymin = our editingBandwidths ? our p_bandwidthFloor   : our p_formantFloor;
 	double ymax = our editingBandwidths ? our p_bandwidthCeiling : our p_formantCeiling;
 	long inearestPoint, ifirstSelected, ilastSelected;
@@ -340,18 +329,18 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 	 */
 	//d_startSelection = d_endSelection = xWC;
 	our ycursor = (1.0 - yWC) * ymin + yWC * ymax;
-	Graphics_setWindow (our d_graphics, our d_startWindow, our d_endWindow, ymin, ymax);
+	Graphics_setWindow (our d_graphics.get(), our d_startWindow, our d_endWindow, ymin, ymax);
 	yWC = our ycursor;
 
 	/*
 	 * Clicked on a point?
 	 */
-	inearestPoint = AnyTier_timeToNearestIndex (tier, xWC);
+	inearestPoint = AnyTier_timeToNearestIndex (tier->asAnyTier(), xWC);
 	if (inearestPoint == 0) {
 		return FormantGridEditor_Parent :: v_click (xWC, yWC, shiftKeyPressed);
 	}
-	nearestPoint = (RealPoint) tier -> points -> item [inearestPoint];
-	if (Graphics_distanceWCtoMM (our d_graphics, xWC, yWC, nearestPoint -> number, nearestPoint -> value) > 1.5) {
+	nearestPoint = tier -> points.at [inearestPoint];
+	if (Graphics_distanceWCtoMM (our d_graphics.get(), xWC, yWC, nearestPoint -> number, nearestPoint -> value) > 1.5) {
 		return our FormantGridEditor_Parent :: v_click (xWC, yWC, shiftKeyPressed);
 	}
 
@@ -361,8 +350,8 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 	draggingSelection = shiftKeyPressed &&
 		nearestPoint -> number > our d_startSelection && nearestPoint -> number < our d_endSelection;
 	if (draggingSelection) {
-		ifirstSelected = AnyTier_timeToHighIndex (tier, our d_startSelection);
-		ilastSelected = AnyTier_timeToLowIndex (tier, our d_endSelection);
+		ifirstSelected = AnyTier_timeToHighIndex (tier->asAnyTier(), our d_startSelection);
+		ilastSelected = AnyTier_timeToLowIndex (tier->asAnyTier(), our d_endSelection);
 		Editor_save (this, U"Drag points");
 	} else {
 		ifirstSelected = ilastSelected = inearestPoint;
@@ -372,11 +361,11 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 	/*
 	 * Drag.
 	 */
-	Graphics_xorOn (our d_graphics, Graphics_MAROON);
+	Graphics_xorOn (our d_graphics.get(), Graphics_MAROON);
 	drawWhileDragging (this, xWC, yWC, ifirstSelected, ilastSelected, dt, df);
-	while (Graphics_mouseStillDown (our d_graphics)) {
+	while (Graphics_mouseStillDown (our d_graphics.get())) {
 		double xWC_new, yWC_new;
-		Graphics_getMouseLocation (our d_graphics, & xWC_new, & yWC_new);
+		Graphics_getMouseLocation (our d_graphics.get(), & xWC_new, & yWC_new);
 		if (xWC_new != xWC || yWC_new != yWC) {
 			drawWhileDragging (this, xWC, yWC, ifirstSelected, ilastSelected, dt, df);
 			dt += xWC_new - xWC, df += yWC_new - yWC;
@@ -384,7 +373,7 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 			drawWhileDragging (this, xWC, yWC, ifirstSelected, ilastSelected, dt, df);
 		}
 	}
-	Graphics_xorOff (our d_graphics);
+	Graphics_xorOff (our d_graphics.get());
 
 	/*
 	 * Dragged inside window?
@@ -394,21 +383,20 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 	/*
 	 * Points not dragged past neighbours?
 	 */
-	RealPoint *points = (RealPoint *) tier -> points -> item;
-	double newTime = points [ifirstSelected] -> number + dt;
+	double newTime = tier -> points.at [ifirstSelected] -> number + dt;
 	if (newTime < our tmin) return 1;   // outside domain
-	if (ifirstSelected > 1 && newTime <= points [ifirstSelected - 1] -> number)
+	if (ifirstSelected > 1 && newTime <= tier -> points.at [ifirstSelected - 1] -> number)
 		return 1;   // past left neighbour
-	newTime = points [ilastSelected] -> number + dt;
+	newTime = tier -> points.at [ilastSelected] -> number + dt;
 	if (newTime > our tmax) return 1;   // outside domain
-	if (ilastSelected < tier -> points -> size && newTime >= points [ilastSelected + 1] -> number)
-		return 1;   // past right neighbour
+	if (ilastSelected < tier -> points.size && newTime >= tier -> points.at [ilastSelected + 1] -> number)
+		return FunctionEditor_UPDATE_NEEDED;   // past right neighbour
 
 	/*
 	 * Drop.
 	 */
 	for (long i = ifirstSelected; i <= ilastSelected; i ++) {
-		RealPoint point = (RealPoint) tier -> points -> item [i];
+		RealPoint point = tier -> points.at [i];
 		point -> number += dt;
 		point -> value += df;
 	}
@@ -422,7 +410,7 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 		/*
 		 * Move crosshair to only selected formant point.
 		 */
-		RealPoint point = (RealPoint) tier -> points -> item [ifirstSelected];
+		RealPoint point = tier -> points.at [ifirstSelected];
 		our d_startSelection = our d_endSelection = point -> number;
 		our ycursor = point -> value;
 	} else {
@@ -434,7 +422,7 @@ int structFormantGridEditor :: v_click (double xWC, double yWC, bool shiftKeyPre
 	}
 
 	Editor_broadcastDataChanged (this);
-	return 1;   // update needed
+	return FunctionEditor_UPDATE_NEEDED;
 }
 
 void structFormantGridEditor :: v_play (double tmin, double tmax) {
@@ -449,7 +437,7 @@ void structFormantGridEditor :: v_play (double tmin, double tmax) {
 }
 
 void FormantGridEditor_init (FormantGridEditor me, const char32 *title, FormantGrid data) {
-	Melder_assert (data != nullptr);
+	Melder_assert (data);
 	Melder_assert (Thing_isa (data, classFormantGrid));
 	FunctionEditor_init (me, title, data);
 	my ycursor = 0.382 * my p_formantFloor + 0.618 * my p_formantCeiling;
@@ -459,7 +447,7 @@ void FormantGridEditor_init (FormantGridEditor me, const char32 *title, FormantG
 autoFormantGridEditor FormantGridEditor_create (const char32 *title, FormantGrid data) {
 	try {
 		autoFormantGridEditor me = Thing_new (FormantGridEditor);
-		FormantGridEditor_init (me.peek(), title, data);
+		FormantGridEditor_init (me.get(), title, data);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"FormantGrid window not created.");
