@@ -1,20 +1,19 @@
 /* RealTier.cpp
  *
- * Copyright (C) 1992-2012,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "RealTier.h"
@@ -47,14 +46,14 @@ autoRealPoint RealPoint_create (double time, double value) {
 	autoRealPoint me = Thing_new (RealPoint);
 	my number = time;
 	my value = value;
-	return me.transfer();
+	return me;
 }
 
 /********** class RealTier **********/
 
 void structRealTier :: v_info () {
 	structFunction :: v_info ();
-	MelderInfo_writeLine (U"Number of points: ", our numberOfPoints ());
+	MelderInfo_writeLine (U"Number of points: ", our points.size);
 	MelderInfo_writeLine (U"Minimum value: ", RealTier_getMinimumValue (this));
 	MelderInfo_writeLine (U"Maximum value: ", RealTier_getMaximumValue (this));
 }
@@ -69,34 +68,17 @@ double structRealTier :: v_getFunction1 (long irow, double x) {
 	return RealTier_getValueAtTime (this, x);
 }
 
-void structRealTier :: v_shiftX (double xfrom, double xto) {
-	RealTier_Parent :: v_shiftX (xfrom, xto);
-	for (long i = 1; i <= our numberOfPoints (); i ++) {
-		RealPoint point = our point (i);
-		NUMshift (& point -> number, xfrom, xto);
-	}
-}
-
-void structRealTier :: v_scaleX (double xminfrom, double xmaxfrom, double xminto, double xmaxto) {
-	RealTier_Parent :: v_scaleX (xminfrom, xmaxfrom, xminto, xmaxto);
-	for (long i = 1; i <= our numberOfPoints (); i ++) {
-		RealPoint point = our point (i);
-		NUMscale (& point -> number, xminfrom, xmaxfrom, xminto, xmaxto);
-	}
-}
-
-Thing_implement (RealTier, Function, 0);
+Thing_implement (RealTier, AnyTier, 0);
 
 void RealTier_init (RealTier me, double tmin, double tmax) {
 	my xmin = tmin;
 	my xmax = tmax;
-	my points = SortedSetOfDouble_create ();
 }
 
 autoRealTier RealTier_create (double tmin, double tmax) {
 	try {
 		autoRealTier me = Thing_new (RealTier);
-		RealTier_init (me.peek(), tmin, tmax);
+		RealTier_init (me.get(), tmin, tmax);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"RealTier not created.");
@@ -105,8 +87,8 @@ autoRealTier RealTier_create (double tmin, double tmax) {
 
 autoRealTier RealTier_createWithClass (double tmin, double tmax, ClassInfo klas) {
 	try {
-		autoRealTier me = static_cast <RealTier> (Thing_newFromClass (klas));
-		RealTier_init (me.peek(), tmin, tmax);
+		autoRealTier me = Thing_newFromClass (klas).static_cast_move <structRealTier> ();
+		RealTier_init (me.get(), tmin, tmax);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (klas -> className, U" not created.");
@@ -116,29 +98,29 @@ autoRealTier RealTier_createWithClass (double tmin, double tmax, ClassInfo klas)
 void RealTier_addPoint (RealTier me, double t, double value) {
 	try {
 		autoRealPoint point = RealPoint_create (t, value);
-		Collection_addItem (my points, point.transfer());
+		my points. addItem_move (point.move());
 	} catch (MelderError) {
 		Melder_throw (me, U": point not added.");
 	}
 }
 
 double RealTier_getValueAtIndex (RealTier me, long i) {
-	if (i < 1 || i > my numberOfPoints ()) return NUMundefined;
-	return my point (i) -> value;
+	if (i < 1 || i > my points.size) return NUMundefined;
+	return my points.at [i] -> value;
 }
 
 double RealTier_getValueAtTime (RealTier me, double t) {
-	long n = my numberOfPoints ();
+	long n = my points.size;
 	if (n == 0) return NUMundefined;
-	RealPoint pointRight = my point (1);
+	RealPoint pointRight = my points.at [1];
 	if (t <= pointRight -> number) return pointRight -> value;   // constant extrapolation
-	RealPoint pointLeft = my point (n);
+	RealPoint pointLeft = my points.at [n];
 	if (t >= pointLeft -> number) return pointLeft -> value;   // constant extrapolation
 	Melder_assert (n >= 2);
-	long ileft = AnyTier_timeToLowIndex (me, t), iright = ileft + 1;
+	long ileft = AnyTier_timeToLowIndex (me->asAnyTier(), t), iright = ileft + 1;
 	Melder_assert (ileft >= 1 && iright <= n);
-	pointLeft = my point (ileft);
-	pointRight = my point (iright);
+	pointLeft = my points.at [ileft];
+	pointRight = my points.at [iright];
 	double tleft = pointLeft -> number, fleft = pointLeft -> value;
 	double tright = pointRight -> number, fright = pointRight -> value;
 	return t == tright ? fright   // be very accurate
@@ -148,9 +130,9 @@ double RealTier_getValueAtTime (RealTier me, double t) {
 
 double RealTier_getMaximumValue (RealTier me) {
 	double result = NUMundefined;
-	long n = my numberOfPoints ();
+	long n = my points.size;
 	for (long i = 1; i <= n; i ++) {
-		RealPoint point = my point (i);
+		RealPoint point = my points.at [i];
 		if (result == NUMundefined || point -> value > result)
 			result = point -> value;
 	}
@@ -159,9 +141,9 @@ double RealTier_getMaximumValue (RealTier me) {
 
 double RealTier_getMinimumValue (RealTier me) {
 	double result = NUMundefined;
-	long n = my numberOfPoints ();
+	long n = my points.size;
 	for (long i = 1; i <= n; i ++) {
-		RealPoint point = my point (i);
+		RealPoint point = my points.at [i];
 		if (result == NUMundefined || point -> value < result)
 			result = point -> value;
 	}
@@ -169,14 +151,14 @@ double RealTier_getMinimumValue (RealTier me) {
 }
 
 double RealTier_getArea (RealTier me, double tmin, double tmax) {
-	long n = my numberOfPoints (), imin, imax;
-	RealPoint *points = my peekPoints ();
+	long n = my points.size, imin, imax;
+	//RealPoint *points = & my points [0];
 	if (n == 0) return NUMundefined;
-	if (n == 1) return (tmax - tmin) * points [1] -> value;
-	imin = AnyTier_timeToLowIndex (me, tmin);
-	if (imin == n) return (tmax - tmin) * points [n] -> value;
-	imax = AnyTier_timeToHighIndex (me, tmax);
-	if (imax == 1) return (tmax - tmin) * points [1] -> value;
+	if (n == 1) return (tmax - tmin) * my points.at [1] -> value;
+	imin = AnyTier_timeToLowIndex (me->asAnyTier(), tmin);
+	if (imin == n) return (tmax - tmin) * my points.at [n] -> value;
+	imax = AnyTier_timeToHighIndex (me->asAnyTier(), tmax);
+	if (imax == 1) return (tmax - tmin) * my points.at [1] -> value;
 	Melder_assert (imin < n);
 	Melder_assert (imax > 1);
 	/*
@@ -186,10 +168,20 @@ double RealTier_getArea (RealTier me, double tmin, double tmax) {
 	double area = 0.0;
 	for (long i = imin; i < imax; i ++) {
 		double tleft, fleft, tright, fright;
-		if (i == imin) tleft = tmin, fleft = RealTier_getValueAtTime (me, tmin);
-		else tleft = points [i] -> number, fleft = points [i] -> value;
-		if (i + 1 == imax) tright = tmax, fright = RealTier_getValueAtTime (me, tmax);
-		else tright = points [i + 1] -> number, fright = points [i + 1] -> value;
+		if (i == imin) {
+			tleft = tmin;
+			fleft = RealTier_getValueAtTime (me, tmin);
+		} else {
+			tleft = my points.at [i] -> number;
+			fleft = my points.at [i] -> value;
+		}
+		if (i + 1 == imax) {
+			tright = tmax;
+			fright = RealTier_getValueAtTime (me, tmax);
+		} else {
+			tright = my points.at [i + 1] -> number;
+			fright = my points.at [i + 1] -> value;
+		}
 		area += 0.5 * (fleft + fright) * (tright - tleft);
 	}
 	return area;
@@ -203,15 +195,14 @@ double RealTier_getMean_curve (RealTier me, double tmin, double tmax) {
 }
 
 double RealTier_getStandardDeviation_curve (RealTier me, double tmin, double tmax) {
-	long n = my numberOfPoints (), imin, imax;
-	RealPoint *points = my peekPoints ();
+	long n = my points.size, imin, imax;
 	double mean, integral = 0.0;
 	if (tmax <= tmin) { tmin = my xmin; tmax = my xmax; }   // autowindow
 	if (n == 0) return NUMundefined;
 	if (n == 1) return 0.0;
-	imin = AnyTier_timeToLowIndex (me, tmin);
+	imin = AnyTier_timeToLowIndex (me->asAnyTier(), tmin);
 	if (imin == n) return 0.0;
-	imax = AnyTier_timeToHighIndex (me, tmax);
+	imax = AnyTier_timeToHighIndex (me->asAnyTier(), tmax);
 	if (imax == 1) return 0.0;
 	Melder_assert (imin < n);
 	Melder_assert (imax > 1);
@@ -222,10 +213,20 @@ double RealTier_getStandardDeviation_curve (RealTier me, double tmin, double tma
 	mean = RealTier_getMean_curve (me, tmin, tmax);
 	for (long i = imin; i < imax; i ++) {
 		double tleft, fleft, tright, fright, sum, diff;
-		if (i == imin) tleft = tmin, fleft = RealTier_getValueAtTime (me, tmin);
-		else tleft = points [i] -> number, fleft = points [i] -> value - mean;
-		if (i + 1 == imax) tright = tmax, fright = RealTier_getValueAtTime (me, tmax);
-		else tright = points [i + 1] -> number, fright = points [i + 1] -> value - mean;
+		if (i == imin) {
+			tleft = tmin;
+			fleft = RealTier_getValueAtTime (me, tmin);
+		} else {
+			tleft = my points.at [i] -> number;
+			fleft = my points.at [i] -> value - mean;
+		}
+		if (i + 1 == imax) {
+			tright = tmax;
+			fright = RealTier_getValueAtTime (me, tmax);
+		} else {
+			tright = my points.at [i + 1] -> number;
+			fright = my points.at [i + 1] -> value - mean;
+		}
 		/*
 		 * The area is integral dt f^2
 		 *   = integral dt [f1 + (f2-f1)/(t2-t1) (t-t1)]^2
@@ -244,35 +245,33 @@ double RealTier_getStandardDeviation_curve (RealTier me, double tmin, double tma
 }
 
 double RealTier_getMean_points (RealTier me, double tmin, double tmax) {
-	long n = my numberOfPoints (), imin, imax;
+	long n = my points.size, imin, imax;
 	double sum = 0.0;
-	RealPoint *points = my peekPoints ();
 	if (tmax <= tmin) { tmin = my xmin; tmax = my xmax; }   // autowindow
-	n = AnyTier_getWindowPoints (me, tmin, tmax, & imin, & imax);
+	n = AnyTier_getWindowPoints (me->asAnyTier(), tmin, tmax, & imin, & imax);
 	if (n == 0) return NUMundefined;
 	for (long i = imin; i <= imax; i ++)
-		sum += points [i] -> value;
+		sum += my points.at [i] -> value;
 	return sum / n;
 }
 
 double RealTier_getStandardDeviation_points (RealTier me, double tmin, double tmax) {
-	long n = my numberOfPoints (), imin, imax;
+	long n = my points.size, imin, imax;
 	double mean, sum = 0.0;
-	RealPoint *points = my peekPoints ();
 	if (tmax <= tmin) { tmin = my xmin; tmax = my xmax; }   // autowindow
-	n = AnyTier_getWindowPoints (me, tmin, tmax, & imin, & imax);
+	n = AnyTier_getWindowPoints (me->asAnyTier(), tmin, tmax, & imin, & imax);
 	if (n < 2) return NUMundefined;
 	mean = RealTier_getMean_points (me, tmin, tmax);
 	for (long i = imin; i <= imax; i ++) {
-		double diff = points [i] -> value - mean;
+		double diff = my points.at [i] -> value - mean;
 		sum += diff * diff;
 	}
 	return sqrt (sum / (n - 1));
 }
 
 void RealTier_multiplyPart (RealTier me, double tmin, double tmax, double factor) {
-	for (long ipoint = 1; ipoint <= my numberOfPoints (); ipoint ++) {
-		RealPoint point = my point (ipoint);
+	for (long ipoint = 1; ipoint <= my points.size; ipoint ++) {
+		RealPoint point = my points.at [ipoint];
 		double t = point -> number;
 		if (t >= tmin && t <= tmax) {
 			point -> value *= factor;
@@ -285,19 +284,19 @@ void RealTier_draw (RealTier me, Graphics g, double tmin, double tmax, double fm
 {
 	bool drawLines = str32str (method, U"lines") || str32str (method, U"Lines");
 	bool drawSpeckles = str32str (method, U"speckles") || str32str (method, U"Speckles");
-	long n = my numberOfPoints (), imin, imax, i;
+	long n = my points.size, imin, imax, i;
 	if (tmax <= tmin) { tmin = my xmin; tmax = my xmax; }
 	Graphics_setWindow (g, tmin, tmax, fmin, fmax);
 	Graphics_setInner (g);
-	imin = AnyTier_timeToHighIndex (me, tmin);
-	imax = AnyTier_timeToLowIndex (me, tmax);
+	imin = AnyTier_timeToHighIndex (me->asAnyTier(), tmin);
+	imax = AnyTier_timeToLowIndex (me->asAnyTier(), tmax);
 	if (n == 0) {
 	} else if (imax < imin) {
 		double fleft = RealTier_getValueAtTime (me, tmin);
 		double fright = RealTier_getValueAtTime (me, tmax);
 		if (drawLines) Graphics_line (g, tmin, fleft, tmax, fright);
 	} else for (i = imin; i <= imax; i ++) {
-		RealPoint point = my point (i);
+		RealPoint point = my points.at [i];
 		double t = point -> number, f = point -> value;
 		if (drawSpeckles) Graphics_speckle (g, t, f);
 		if (drawLines) {
@@ -310,7 +309,7 @@ void RealTier_draw (RealTier me, Graphics g, double tmin, double tmax, double fm
 			else if (i == imax)
 				Graphics_line (g, t, f, tmax, RealTier_getValueAtTime (me, tmax));
 			else {
-				RealPoint pointRight = my point (i + 1);
+				RealPoint pointRight = my points.at [i + 1];
 				Graphics_line (g, t, f, pointRight -> number, pointRight -> value);
 			}
 		}
@@ -327,11 +326,11 @@ void RealTier_draw (RealTier me, Graphics g, double tmin, double tmax, double fm
 
 autoTableOfReal RealTier_downto_TableOfReal (RealTier me, const char32 *timeLabel, const char32 *valueLabel) {
 	try {
-		autoTableOfReal thee = TableOfReal_create (my numberOfPoints (), 2);
-		TableOfReal_setColumnLabel (thee.peek(), 1, timeLabel);
-		TableOfReal_setColumnLabel (thee.peek(), 2, valueLabel);
-		for (long i = 1; i <= my numberOfPoints (); i ++) {
-			RealPoint point = my point (i);
+		autoTableOfReal thee = TableOfReal_create (my points.size, 2);
+		TableOfReal_setColumnLabel (thee.get(), 1, timeLabel);
+		TableOfReal_setColumnLabel (thee.get(), 2, valueLabel);
+		for (long i = 1; i <= my points.size; i ++) {
+			RealPoint point = my points.at [i];
 			thy data [i] [1] = point -> number;
 			thy data [i] [2] = point -> value;
 		}
@@ -344,8 +343,8 @@ autoTableOfReal RealTier_downto_TableOfReal (RealTier me, const char32 *timeLabe
 void RealTier_interpolateQuadratically (RealTier me, long numberOfPointsPerParabola, int logarithmically) {
 	try {
 		autoRealTier thee = Data_copy (me);
-		for (long ipoint = 1; ipoint < my numberOfPoints (); ipoint ++) {
-			RealPoint point1 = my point (ipoint), point2 = my point (ipoint + 1);
+		for (long ipoint = 1; ipoint < my points.size; ipoint ++) {
+			RealPoint point1 = my points.at [ipoint], point2 = my points.at [ipoint + 1];
 			double time1 = point1 -> number, time2 = point2 -> number, tmid = 0.5 * (time1 + time2);
 			double value1 = point1 -> value, value2 = point2 -> value, valuemid;
 			double timeStep = (tmid - time1) / (numberOfPointsPerParabola + 1);
@@ -359,12 +358,12 @@ void RealTier_interpolateQuadratically (RealTier me, long numberOfPointsPerParab
 				double phase = (newTime - time1) / (tmid - time1);
 				double newValue = value1 + (valuemid - value1) * phase * phase;
 				if (logarithmically) newValue = exp (newValue);
-				RealTier_addPoint (thee.peek(), newTime, newValue);
+				RealTier_addPoint (thee.get(), newTime, newValue);
 			}
 			/*
 			 * The midpoint.
 			 */
-			RealTier_addPoint (thee.peek(), tmid, logarithmically ? exp (valuemid) : valuemid);
+			RealTier_addPoint (thee.get(), tmid, logarithmically ? exp (valuemid) : valuemid);
 			/*
 			 * Right from the midpoint.
 			 */
@@ -373,10 +372,10 @@ void RealTier_interpolateQuadratically (RealTier me, long numberOfPointsPerParab
 				double phase = (time2 - newTime) / (time2 - tmid);
 				double newValue = value2 + (valuemid - value2) * phase * phase;
 				if (logarithmically) newValue = exp (newValue);
-				RealTier_addPoint (thee.peek(), newTime, newValue);
+				RealTier_addPoint (thee.get(), newTime, newValue);
 			}
 		}
-		Thing_swap (me, thee.peek());
+		Thing_swap (me, thee.get());
 	} catch (MelderError) {
 		Melder_throw (me, U": not interpolated quadratically.");
 	}
@@ -384,18 +383,18 @@ void RealTier_interpolateQuadratically (RealTier me, long numberOfPointsPerParab
 
 autoTable RealTier_downto_Table (RealTier me, const char32 *indexText, const char32 *timeText, const char32 *valueText) {
 	try {
-		autoTable thee = Table_createWithoutColumnNames (my numberOfPoints (),
-			(indexText != NULL) + (timeText != NULL) + (valueText != NULL));
+		autoTable thee = Table_createWithoutColumnNames (my points.size,
+			(!! indexText) + (!! timeText) + (!! valueText));
 		long icol = 0;
-		if (indexText != NULL) Table_setColumnLabel (thee.peek(), ++ icol, indexText);
-		if (timeText  != NULL) Table_setColumnLabel (thee.peek(), ++ icol, timeText);
-		if (valueText != NULL) Table_setColumnLabel (thee.peek(), ++ icol, valueText);
-		for (long ipoint = 1; ipoint <= my numberOfPoints (); ipoint ++) {
-			RealPoint point = my point (ipoint);
+		if (indexText) Table_setColumnLabel (thee.get(), ++ icol, indexText);
+		if (timeText ) Table_setColumnLabel (thee.get(), ++ icol, timeText);
+		if (valueText) Table_setColumnLabel (thee.get(), ++ icol, valueText);
+		for (long ipoint = 1; ipoint <= my points.size; ipoint ++) {
+			RealPoint point = my points.at [ipoint];
 			icol = 0;
-			if (indexText != NULL) Table_setNumericValue (thee.peek(), ipoint, ++ icol, ipoint);
-			if (timeText != NULL)  Table_setNumericValue (thee.peek(), ipoint, ++ icol, point -> number);
-			if (valueText != NULL) Table_setNumericValue (thee.peek(), ipoint, ++ icol, point -> value);
+			if (indexText) Table_setNumericValue (thee.get(), ipoint, ++ icol, ipoint);
+			if (timeText)  Table_setNumericValue (thee.get(), ipoint, ++ icol, point -> number);
+			if (valueText) Table_setNumericValue (thee.get(), ipoint, ++ icol, point -> value);
 		}
 		return thee;
 	} catch (MelderError) {
@@ -407,7 +406,7 @@ autoRealTier Vector_to_RealTier (Vector me, long channel, ClassInfo klas) {
 	try {
 		autoRealTier thee = RealTier_createWithClass (my xmin, my xmax, klas);
 		for (long i = 1; i <= my nx; i ++) {
-			RealTier_addPoint (thee.peek(), Sampled_indexToX (me, i), my z [channel] [i]);
+			RealTier_addPoint (thee.get(), Sampled_indexToX (me, i), my z [channel] [i]);
 		}
 		return thee;
 	} catch (MelderError) {
@@ -424,7 +423,7 @@ autoRealTier Vector_to_RealTier_peaks (Vector me, long channel, ClassInfo klas)
 				double x, maximum;
 				Vector_getMaximumAndX (me, my x1 + (i - 2.5) * my dx, my x1 + (i + 0.5) * my dx,
 					channel, NUM_PEAK_INTERPOLATE_PARABOLIC, & maximum, & x);
-				RealTier_addPoint (thee.peek(), x, maximum);
+				RealTier_addPoint (thee.get(), x, maximum);
 			}
 		}
 		return thee;
@@ -442,7 +441,7 @@ autoRealTier Vector_to_RealTier_valleys (Vector me, long channel, ClassInfo klas
 				double x, minimum;
 				Vector_getMinimumAndX (me, my x1 + (i - 2.5) * my dx, my x1 + (i + 0.5) * my dx,
 					channel, NUM_PEAK_INTERPOLATE_PARABOLIC, & minimum, & x);
-				RealTier_addPoint (thee.peek(), x, minimum);
+				RealTier_addPoint (thee.get(), x, minimum);
 			}
 		}
 		return thee;
@@ -455,7 +454,7 @@ autoRealTier PointProcess_upto_RealTier (PointProcess me, double value, ClassInf
 	try {
 		autoRealTier thee = RealTier_createWithClass (my xmin, my xmax, klas);
 		for (long i = 1; i <= my nt; i ++) {
-			RealTier_addPoint (thee.peek(), my t [i], value);
+			RealTier_addPoint (thee.get(), my t [i], value);
 		}
 		return thee;
 	} catch (MelderError) {
@@ -466,13 +465,13 @@ autoRealTier PointProcess_upto_RealTier (PointProcess me, double value, ClassInf
 void RealTier_formula (RealTier me, const char32 *expression, Interpreter interpreter, RealTier thee) {
 	try {
 		Formula_compile (interpreter, me, expression, kFormula_EXPRESSION_TYPE_NUMERIC, true);
-		if (thee == NULL) thee = me;
-		for (long icol = 1; icol <= my numberOfPoints (); icol ++) {
+		if (! thee) thee = me;
+		for (long icol = 1; icol <= my points.size; icol ++) {
 			struct Formula_Result result;
 			Formula_run (0, icol, & result);
 			if (result. result.numericResult == NUMundefined)
 				Melder_throw (U"Cannot put an undefined value into the tier.");
-			thy point (icol) -> value = result. result.numericResult;
+			thy points.at [icol] -> value = result. result.numericResult;
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": formula not completed.");
@@ -480,10 +479,10 @@ void RealTier_formula (RealTier me, const char32 *expression, Interpreter interp
 }
 
 void RealTier_removePointsBelow (RealTier me, double level) {
-	for (long ipoint = my numberOfPoints (); ipoint > 0; ipoint --) {
-		RealPoint point = my point (ipoint);
+	for (long ipoint = my points.size; ipoint > 0; ipoint --) {
+		RealPoint point = my points.at [ipoint];
 		if (point -> value < level) {
-			AnyTier_removePoint (me, ipoint);
+			AnyTier_removePoint (me->asAnyTier(), ipoint);
 		}
 	}
 }
