@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1992-2012,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -30,30 +29,25 @@ oo_END_CLASS (RealPoint)
 #define ooSTRUCT RealTier
 oo_DEFINE_CLASS (RealTier, Function)
 
-	oo_COLLECTION (SortedSetOfDouble, points, RealPoint, 0)
+	oo_COLLECTION_OF (SortedSetOfDoubleOf, points, RealPoint, 0)   // a kind of AnyTier though
 
 	#if oo_DECLARING
-		long numberOfPoints () // accessor
-			{ return points -> size; }
-		RealPoint * peekPoints () // accessor
-			{ return reinterpret_cast <RealPoint *> (points -> item); }
-		RealPoint point (long ipoint) // accessor
-			{ return static_cast <RealPoint> (points -> item [ipoint]); }
+		AnyTier_METHODS
 
 		void v_info ()
 			override;
 		bool v_hasGetNx ()
 			override { return true; }
 		double v_getNx ()
-			override { return numberOfPoints (); }
+			override { return points.size; }
 		bool v_hasGetX ()
 			override { return true; }
 		double v_getX (long ix)
-			override { return point (ix) -> number; }
+			override { return points.at [ix] -> number; }
 		bool v_hasGetNcol ()
 			override { return true; }
 		double v_getNcol ()
-			override { return numberOfPoints (); }
+			override { return points.size; }
 		bool v_hasGetVector ()
 			override { return true; }
 		double v_getVector (long irow, long icol)
@@ -64,10 +58,6 @@ oo_DEFINE_CLASS (RealTier, Function)
 			override;
 		const char32 * v_getUnitText (long /* ilevel */, int /* unit */, unsigned long /* flags */)
 			override { return U"Time (s)"; }
-		void v_shiftX (double xfrom, double xto)
-			override;
-		void v_scaleX (double xminfrom, double xmaxfrom, double xminto, double xmaxto)
-			override;
 	#endif
 
 oo_END_CLASS (RealTier)
