@@ -1,20 +1,19 @@
 /* Sound.cpp
  *
- * Copyright (C) 1992-2012,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -34,7 +33,7 @@
 
 Thing_implement (Sound, Vector, 2);
 
-Sound Sound_clipboard;
+autoSound Sound_clipboard;
 
 void structSound :: v_info () {
 	structDaata :: v_info ();
@@ -126,7 +125,7 @@ double structSound :: v_getFunction2 (double x, double y) {
 autoSound Sound_create (long numberOfChannels, double xmin, double xmax, long nx, double dx, double x1) {
 	try {
 		autoSound me = Thing_new (Sound);
-		Matrix_init (me.peek(), xmin, xmax, nx, dx, x1, 1, numberOfChannels, numberOfChannels, 1, 1);
+		Matrix_init (me.get(), xmin, xmax, nx, dx, x1, 1, numberOfChannels, numberOfChannels, 1, 1);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Sound not created.");
@@ -183,12 +182,12 @@ autoSound Sound_convertToStereo (Sound me) {
 	}
 }
 
-autoSound Sounds_combineToStereo (Collection me) {
+autoSound Sounds_combineToStereo (OrderedOf<structSound>* me) {
 	try {
 		long totalNumberOfChannels = 0;
 		double sharedSamplingPeriod = 0.0;
 		for (long isound = 1; isound <= my size; isound ++) {
-			Sound sound = (Sound) my item [isound];
+			Sound sound = my at [isound];
 			totalNumberOfChannels += sound -> ny;
 			if (sharedSamplingPeriod == 0.0) {
 				sharedSamplingPeriod = sound -> dx;
@@ -199,7 +198,7 @@ autoSound Sounds_combineToStereo (Collection me) {
 		}
 		double sharedMinimumTime = NUMundefined, sharedMaximumTime = NUMundefined;
 		for (long isound = 1; isound <= my size; isound ++) {
-			Sound sound = (Sound) my item [isound];
+			Sound sound = my at [isound];
 			if (isound == 1) {
 				sharedMinimumTime = sound -> xmin;
 				sharedMaximumTime = sound -> xmax;
@@ -212,7 +211,7 @@ autoSound Sounds_combineToStereo (Collection me) {
 		long sharedNumberOfSamples = 0;
 		double sumOfFirstTimes = 0.0;
 		for (long isound = 1; isound <= my size; isound ++) {
-			Sound sound = (Sound) my item [isound];
+			Sound sound = my at [isound];
 			numberOfInitialZeroes [isound] = floor ((sound -> xmin - sharedMinimumTime) / sharedSamplingPeriod);
 			double newFirstTime = sound -> x1 - sound -> dx * numberOfInitialZeroes [isound];
 			sumOfFirstTimes += newFirstTime;
@@ -224,7 +223,7 @@ autoSound Sounds_combineToStereo (Collection me) {
 			sharedNumberOfSamples, sharedSamplingPeriod, sharedTimeOfFirstSample);
 		long channelNumber = 0;
 		for (long isound = 1; isound <= my size; isound ++) {
-			Sound sound = (Sound) my item [isound];
+			Sound sound = my at [isound];
 			long offset = (long) floor (numberOfInitialZeroes [isound]);
 			for (long ichan = 1; ichan <= sound -> ny; ichan ++) {
 				channelNumber ++;
@@ -321,7 +320,7 @@ autoSound Matrix_to_Sound_mono (Matrix me, long row) {
 autoSound Matrix_to_Sound (Matrix me) {
 	try {
 		autoSound thee = Thing_new (Sound);
-		my structMatrix :: v_copy (thee.peek());
+		my structMatrix :: v_copy (thee.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Sound.");
@@ -331,7 +330,7 @@ autoSound Matrix_to_Sound (Matrix me) {
 autoMatrix Sound_to_Matrix (Sound me) {
 	try {
 		autoMatrix thee = Thing_new (Matrix);
-		my structMatrix :: v_copy (thee.peek());
+		my structMatrix :: v_copy (thee.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Matrix.");
@@ -395,7 +394,7 @@ autoSound Sound_resample (Sound me, double samplingFrequency, long precision) {
 					to [i] = data [i + antiTurnAround] * factor;
 				}
 			}
-			me = filtered.peek();   // reference copy; remove at end
+			me = filtered.get();   // reference copy; remove at end
 		}
 		autoSound thee = Sound_create (my ny, my xmin, my xmax, numberOfSamples, 1.0 / samplingFrequency,
 			0.5 * (my xmin + my xmax - (numberOfSamples - 1) / samplingFrequency));
@@ -404,7 +403,7 @@ autoSound Sound_resample (Sound me, double samplingFrequency, long precision) {
 			double *to = thy z [channel];
 			if (precision <= 1) {
 				for (long i = 1; i <= numberOfSamples; i ++) {
-					double x = Sampled_indexToX (thee.peek(), i);
+					double x = Sampled_indexToX (thee.get(), i);
 					double index = Sampled_xToIndex (me, x);
 					long leftSample = (long) floor (index);
 					double fraction = index - leftSample;
@@ -413,7 +412,7 @@ autoSound Sound_resample (Sound me, double samplingFrequency, long precision) {
 				}
 			} else {
 				for (long i = 1; i <= numberOfSamples; i ++) {
-					double x = Sampled_indexToX (thee.peek(), i);
+					double x = Sampled_indexToX (thee.get(), i);
 					double index = Sampled_xToIndex (me, x);
 					to [i] = NUM_interpolate_sinc (my z [channel], my nx, index, precision);
 				}
@@ -443,12 +442,12 @@ autoSound Sounds_append (Sound me, double silenceDuration, Sound thee) {
 	}
 }
 
-autoSound Sounds_concatenate_e (Collection me, double overlapTime) {
+autoSound Sounds_concatenate (OrderedOf<structSound>& list, double overlapTime) {
 	try {
 		long numberOfChannels = 0, nx = 0, numberOfSmoothingSamples;
 		double dx = 0.0;
-		for (long i = 1; i <= my size; i ++) {
-			Sound sound = (Sound) my item [i];
+		for (long i = 1; i <= list.size; i ++) {
+			Sound sound = list.at [i];
 			if (numberOfChannels == 0) {
 				numberOfChannels = sound -> ny;
 			} else if (sound -> ny != numberOfChannels) {
@@ -473,12 +472,12 @@ autoSound Sounds_concatenate_e (Collection me, double overlapTime) {
 			}
 		}
 		nx = 0;
-		for (long i = 1; i <= my size; i ++) {
-			Sound sound = (Sound) my item [i];
+		for (long i = 1; i <= list.size; i ++) {
+			Sound sound = list.at [i];
 			if (numberOfSmoothingSamples > 2 * sound -> nx)
 				Melder_throw (U"At least one of the sounds is shorter than twice the overlap time.\nChoose a shorter overlap time.");
 			bool thisIsTheFirstSound = ( i == 1 );
-			bool thisIsTheLastSound = ( i == my size );
+			bool thisIsTheLastSound = ( i == list.size );
 			bool weNeedSmoothingAtTheStartOfThisSound = ! thisIsTheFirstSound;
 			bool weNeedSmoothingAtTheEndOfThisSound = ! thisIsTheLastSound;
 			long numberOfSmoothingSamplesAtTheStartOfThisSound = weNeedSmoothingAtTheStartOfThisSound ? numberOfSmoothingSamples : 0;
@@ -501,7 +500,7 @@ autoSound Sounds_concatenate_e (Collection me, double overlapTime) {
 			}
 			nx += sound -> nx - numberOfSmoothingSamplesAtTheEndOfThisSound;
 		}
-		thy nx -= numberOfSmoothingSamples * (my size - 1);
+		thy nx -= numberOfSmoothingSamples * (list.size - 1);
 		Melder_assert (thy nx == nx);
 		thy xmax = thy nx * dx;
 		return thee;
@@ -567,19 +566,19 @@ autoSound Sounds_convolve (Sound me, Sound thee, enum kSounds_convolve_scaling s
 		}
 		switch (scaling) {
 			case kSounds_convolve_scaling_INTEGRAL: {
-				Vector_multiplyByScalar (him.peek(), my dx / nfft);
+				Vector_multiplyByScalar (him.get(), my dx / nfft);
 			} break;
 			case kSounds_convolve_scaling_SUM: {
-				Vector_multiplyByScalar (him.peek(), 1.0 / nfft);
+				Vector_multiplyByScalar (him.get(), 1.0 / nfft);
 			} break;
 			case kSounds_convolve_scaling_NORMALIZE: {
 				double normalizationFactor = Matrix_getNorm (me) * Matrix_getNorm (thee);
 				if (normalizationFactor != 0.0) {
-					Vector_multiplyByScalar (him.peek(), 1.0 / nfft / normalizationFactor);
+					Vector_multiplyByScalar (him.get(), 1.0 / nfft / normalizationFactor);
 				}
 			} break;
 			case kSounds_convolve_scaling_PEAK_099: {
-				Vector_scale (him.peek(), 0.99);
+				Vector_scale (him.get(), 0.99);
 			} break;
 			default: Melder_fatal (U"Sounds_convolve: unimplemented scaling ", scaling);
 		}
@@ -650,19 +649,19 @@ autoSound Sounds_crossCorrelate (Sound me, Sound thee, enum kSounds_convolve_sca
 		}
 		switch (scaling) {
 			case kSounds_convolve_scaling_INTEGRAL: {
-				Vector_multiplyByScalar (him.peek(), my dx / nfft);
+				Vector_multiplyByScalar (him.get(), my dx / nfft);
 			} break;
 			case kSounds_convolve_scaling_SUM: {
-				Vector_multiplyByScalar (him.peek(), 1.0 / nfft);
+				Vector_multiplyByScalar (him.get(), 1.0 / nfft);
 			} break;
 			case kSounds_convolve_scaling_NORMALIZE: {
 				double normalizationFactor = Matrix_getNorm (me) * Matrix_getNorm (thee);
 				if (normalizationFactor != 0.0) {
-					Vector_multiplyByScalar (him.peek(), 1.0 / nfft / normalizationFactor);
+					Vector_multiplyByScalar (him.get(), 1.0 / nfft / normalizationFactor);
 				}
 			} break;
 			case kSounds_convolve_scaling_PEAK_099: {
-				Vector_scale (him.peek(), 0.99);
+				Vector_scale (him.get(), 0.99);
 			} break;
 			default: Melder_fatal (U"Sounds_crossCorrelate: unimplemented scaling ", scaling);
 		}
@@ -721,19 +720,19 @@ autoSound Sound_autoCorrelate (Sound me, enum kSounds_convolve_scaling scaling,
 		}
 		switch (scaling) {
 			case kSounds_convolve_scaling_INTEGRAL: {
-				Vector_multiplyByScalar (thee.peek(), my dx / nfft);
+				Vector_multiplyByScalar (thee.get(), my dx / nfft);
 			} break;
 			case kSounds_convolve_scaling_SUM: {
-				Vector_multiplyByScalar (thee.peek(), 1.0 / nfft);
+				Vector_multiplyByScalar (thee.get(), 1.0 / nfft);
 			} break;
 			case kSounds_convolve_scaling_NORMALIZE: {
 				double normalizationFactor = Matrix_getNorm (me) * Matrix_getNorm (me);
 				if (normalizationFactor != 0.0) {
-					Vector_multiplyByScalar (thee.peek(), 1.0 / nfft / normalizationFactor);
+					Vector_multiplyByScalar (thee.get(), 1.0 / nfft / normalizationFactor);
 				}
 			} break;
 			case kSounds_convolve_scaling_PEAK_099: {
-				Vector_scale (thee.peek(), 0.99);
+				Vector_scale (thee.get(), 0.99);
 			} break;
 			default: Melder_fatal (U"Sounds_autoCorrelate: unimplemented scaling ", scaling);
 		}
@@ -873,7 +872,7 @@ double Sound_getNearestZeroCrossing (Sound me, double position, long channel) {
 		position - leftZero < rightZero - position ? leftZero : rightZero;
 }
 
-void Sound_setZero (Sound me, double tmin_in, double tmax_in, int roundTimesToNearestZeroCrossing) {
+void Sound_setZero (Sound me, double tmin_in, double tmax_in, bool roundTimesToNearestZeroCrossing) {
 	Function_unidirectionalAutowindow (me, & tmin_in, & tmax_in);
 	Function_intersectRangeWithDomain (me, & tmin_in, & tmax_in);
 	for (long channel = 1; channel <= my ny; channel ++) {
@@ -952,7 +951,7 @@ autoSound Sound_createFromToneComplex (double startingTime, double endTime, doub
 			1.0 / sampleRate, startingTime + 0.5 / sampleRate);
 		double *amplitude = my z [1];
 		for (long isamp = 1; isamp <= my nx; isamp ++) {
-			double value = 0.0, t = Sampled_indexToX (me.peek(), isamp);
+			double value = 0.0, t = Sampled_indexToX (me.get(), isamp);
 			double omegaStepT = omegaStep * t, firstOmegaT = firstOmega * t;
 			if (phase == Sound_TONE_COMPLEX_SINE)
 				for (long icomp = 1; icomp <= numberOfComponents; icomp ++)
@@ -1096,7 +1095,7 @@ autoSound Sound_extractPart (Sound me, double t1, double t2, enum kSound_windowS
 		/*
 		 * Multiply by a window that extends throughout the target domain.
 		 */
-		Sound_multiplyByWindow (thee.peek(), windowShape);
+		Sound_multiplyByWindow (thee.get(), windowShape);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": part not extracted.");
@@ -1165,7 +1164,7 @@ void Sound_filterWithFormants (Sound me, double tmin, double tmax,
 autoSound Sound_filter_oneFormant (Sound me, double frequency, double bandwidth) {
 	try {
 		autoSound thee = Data_copy (me);
-		Sound_filterWithOneFormantInline (thee.peek(), frequency, bandwidth);
+		Sound_filterWithOneFormantInline (thee.get(), frequency, bandwidth);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not filtered (one formant).");
@@ -1182,8 +1181,8 @@ void Sound_filterWithOneFormantInline (Sound me, double frequency, double bandwi
 autoSound Sound_filter_preemphasis (Sound me, double frequency) {
 	try {
 		autoSound thee = Data_copy (me);
-		Sound_preEmphasis (thee.peek(), frequency);
-		Matrix_scaleAbsoluteExtremum (thee.peek(), 0.99);
+		Sound_preEmphasis (thee.get(), frequency);
+		Matrix_scaleAbsoluteExtremum (thee.get(), 0.99);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not filtered (pre-emphasis).");
@@ -1193,8 +1192,8 @@ autoSound Sound_filter_preemphasis (Sound me, double frequency) {
 autoSound Sound_filter_deemphasis (Sound me, double frequency) {
 	try {
 		autoSound thee = Data_copy (me);
-		Sound_deEmphasis (thee.peek(), frequency);
-		Matrix_scaleAbsoluteExtremum (thee.peek(), 0.99);
+		Sound_deEmphasis (thee.get(), frequency);
+		Matrix_scaleAbsoluteExtremum (thee.get(), 0.99);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not filtered (de-emphasis).");
