@@ -1,20 +1,19 @@
 /* SoundRecorder.cpp
  *
- * Copyright (C) 1992-2011,2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2011,2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 /* Linux code originally by Darryl Purnell, Pretoria */
 /* GTK conversion includes work by Franz Brauße */
@@ -27,7 +26,7 @@
  * this would seem a good candidate for object-oriented programming
  * (one audio manager and several audio drivers).
  * However, the places where sound recorders are similar and where they are different,
- * are hard to predict. For this reason, everything is done with system #ifdefs.
+ * are hard to predict. For this reason, everything is done with platform #ifdefs.
  */
 
 #include <errno.h>
@@ -101,7 +100,7 @@ static void win_fillHeader (SoundRecorder me, int which) {
 	my waveHeader [which]. lpData = which == 0 ? (char *) my buffer : which == 1 ? (char *) my buffertje1: (char *) my buffertje2;
 	my waveHeader [which]. dwBufferLength = which == 0 ? my nmax * my waveFormat. nChannels * 2 : 1000 * my waveFormat. nChannels * 2;
 	my waveHeader [which]. dwLoops = 0;
-	my waveHeader [which]. lpNext = NULL;
+	my waveHeader [which]. lpNext = nullptr;
 	my waveHeader [which]. reserved = 0;
 }
 static void win_waveInCheck (SoundRecorder me) {
@@ -191,7 +190,7 @@ static void stopRecording (SoundRecorder me) {
 			if (my inputUsesPortAudio) {
 				Pa_StopStream (my portaudioStream);
 				Pa_CloseStream (my portaudioStream);
-				my portaudioStream = NULL;
+				my portaudioStream = nullptr;
 			} else {
 				#if defined (_WIN32)
 					/*
@@ -220,12 +219,12 @@ static void stopRecording (SoundRecorder me) {
 	} catch (MelderError) {
 		Melder_flushError (U"Cannot stop recording.");
 	}
-	Graphics_setWindow (my graphics, 0.0, 1.0, 0.0, 1.0);
-	Graphics_setColour (my graphics, Graphics_WHITE);
-	Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, 1.0);
+	Graphics_setWindow (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
+	Graphics_setColour (my graphics.get(), Graphics_WHITE);
+	Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 }
 
-void structSoundRecorder :: v_destroy () {
+void structSoundRecorder :: v_destroy () noexcept {
 	stopRecording (this);   // must occur before freeing my buffer
 	MelderAudio_stopPlaying (MelderAudio_IMPLICIT);   // must also occur before freeing my buffer
 	#if cocoa
@@ -252,57 +251,56 @@ void structSoundRecorder :: v_destroy () {
 			if (our fd != -1) close (our fd);
 		#endif
 	}
-	forget (our graphics);
 	SoundRecorder_Parent :: v_destroy ();
 }
 
 static void showMaximum (SoundRecorder me, int channel, double maximum) {
 	maximum /= 32768.0;
-	Graphics_setWindow (my graphics,
+	Graphics_setWindow (my graphics.get(),
 		my numberOfChannels == 1 || channel == 1 ? 0.0 : -1.0,
 		my numberOfChannels == 1 || channel == 2 ? 1.0 : 2.0,
 		-0.1, 1.1);
-	Graphics_setGrey (my graphics, 0.9);
-	Graphics_fillRectangle (my graphics, 0.0, 1.0, maximum, 1.0);
-	Graphics_setColour (my graphics, Graphics_GREEN);
+	Graphics_setGrey (my graphics.get(), 0.9);
+	Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, maximum, 1.0);
+	Graphics_setColour (my graphics.get(), Graphics_GREEN);
 	if (maximum < 0.75) {
-		Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, maximum);
+		Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, maximum);
 	} else {
-		Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, 0.75);
-		Graphics_setColour (my graphics, Graphics_YELLOW);
+		Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, 0.75);
+		Graphics_setColour (my graphics.get(), Graphics_YELLOW);
 		if (maximum < 0.92) {
-			Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.75, maximum);
+			Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.75, maximum);
 		} else {
-			Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.75, 0.92);
-			Graphics_setColour (my graphics, Graphics_RED);
-			Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.92, maximum);
+			Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.75, 0.92);
+			Graphics_setColour (my graphics.get(), Graphics_RED);
+			Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.92, maximum);
 		}
 	}
 }
 
 static void showMeter (SoundRecorder me, short *buffer, long nsamp) {
-	Melder_assert (my graphics != NULL);
+	Melder_assert (my graphics);
 	if (nsamp < 1) {
-		Graphics_setWindow (my graphics, 0.0, 1.0, 0.0, 1.0);
+		Graphics_setWindow (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 		#if defined (macintosh)
-			Graphics_setColour (my graphics, Graphics_WHITE);
-			Graphics_fillRectangle (my graphics, 0.2, 0.8, 0.3, 0.7);
+			Graphics_setColour (my graphics.get(), Graphics_WHITE);
+			Graphics_fillRectangle (my graphics.get(), 0.2, 0.8, 0.3, 0.7);
 		#endif
-		Graphics_setTextAlignment (my graphics, Graphics_CENTRE, Graphics_HALF);
-		Graphics_setColour (my graphics, Graphics_BLACK);
-		Graphics_text (my graphics, 0.5, 0.5, U"Not recording.");
+		Graphics_setTextAlignment (my graphics.get(), Graphics_CENTRE, Graphics_HALF);
+		Graphics_setColour (my graphics.get(), Graphics_BLACK);
+		Graphics_text (my graphics.get(), 0.5, 0.5, U"Not recording.");
 		return;
 	}
 	if (my p_meter_which == kSoundRecorder_meter_INTENSITY) {
-		short leftMaximum = 0, rightMaximum = 0;
+		int leftMaximum = 0, rightMaximum = 0;
 		if (my numberOfChannels == 1) {
 			for (long i = 0; i < nsamp; i ++) {
-				short value = buffer [i];
+				int value = buffer [i];
 				if (abs (value) > leftMaximum) leftMaximum = abs (value);
 			}
 		} else {
 			for (long i = 0; i < nsamp; i ++) {
-				long left = buffer [i+i], right = buffer [i+i+1];
+				int left = buffer [i+i], right = buffer [i+i+1];
 				if (abs (left) > leftMaximum) leftMaximum = abs (left);
 				if (abs (right) > rightMaximum) rightMaximum = abs (right);
 			}
@@ -331,22 +329,22 @@ static void showMeter (SoundRecorder me, short *buffer, long nsamp) {
 				sound -> z [ichan] [isamp] = * (p ++) / 32768.0;
 			}
 		}
-		Sound_multiplyByWindow (sound.peek(), kSound_windowShape_KAISER_2);
-		double intensity = Sound_getIntensity_dB (sound.peek());
-		autoSpectrum spectrum = Sound_to_Spectrum (sound.peek(), true);
-		double centreOfGravity = Spectrum_getCentreOfGravity (spectrum.peek(), 1.0);
+		Sound_multiplyByWindow (sound.get(), kSound_windowShape_KAISER_2);
+		double intensity = Sound_getIntensity_dB (sound.get());
+		autoSpectrum spectrum = Sound_to_Spectrum (sound.get(), true);
+		double centreOfGravity = Spectrum_getCentreOfGravity (spectrum.get(), 1.0);
 		trace (nsamp, U" samples, intensity ", intensity, U" dB, centre of gravity ", centreOfGravity, U" Hz");
-		Graphics_setWindow (my graphics,
+		Graphics_setWindow (my graphics.get(),
 			my p_meter_centreOfGravity_minimum, my p_meter_centreOfGravity_maximum,
 			my p_meter_intensity_minimum, my p_meter_intensity_maximum);
-		Graphics_setColour (my graphics, Graphics_WHITE);
-		Graphics_fillRectangle (my graphics,
+		Graphics_setColour (my graphics.get(), Graphics_WHITE);
+		Graphics_fillRectangle (my graphics.get(),
 			my p_meter_centreOfGravity_minimum, my p_meter_centreOfGravity_maximum,
 			my p_meter_intensity_minimum, my p_meter_intensity_maximum);
-		Graphics_setColour (my graphics, Graphics_BLACK);
-		Graphics_fillCircle_mm (my graphics, centreOfGravity, intensity, 3.0);
+		Graphics_setColour (my graphics.get(), Graphics_BLACK);
+		Graphics_fillCircle_mm (my graphics.get(), centreOfGravity, intensity, 3.0);
 	}
-	Graphics_flushWs (my graphics);
+	Graphics_flushWs (my graphics.get());
 }
 
 static bool tooManySamplesInBufferToReturnToGui (SoundRecorder me) {
@@ -361,17 +359,17 @@ static long getMyNsamp (SoundRecorder me) {
 
 #if cocoa
 	#define WORKPROC_RETURN  void
-	#define WORKPROC_ARGS  CFRunLoopTimerRef /*timer*/, void *void_me
+	#define WORKPROC_ARGS  CFRunLoopTimerRef /*timer*/, void *void_SoundRecorder
 #elif gtk
 	#define WORKPROC_RETURN  gboolean
-	#define WORKPROC_ARGS  void *void_me
+	#define WORKPROC_ARGS  void *void_SoundRecorder
 #else
 	#define WORKPROC_RETURN  bool
-	#define WORKPROC_ARGS  void *void_me
+	#define WORKPROC_ARGS  void *void_SoundRecorder
 #endif
 
 static WORKPROC_RETURN workProc (WORKPROC_ARGS) {
-	iam (SoundRecorder);
+	SoundRecorder me = static_cast <SoundRecorder> (void_SoundRecorder);
 	try {
 		short buffertje [step*2];
 		int stepje = 0;
@@ -480,7 +478,7 @@ static WORKPROC_RETURN workProc (WORKPROC_ARGS) {
 				showMeter (me, my buffer + firstSample * my numberOfChannels, lastSample - firstSample);
 				GuiScale_setValue (my progressScale, 1000.0 * ((double) lastSample / (double) my nmax));
 			} else {
-				showMeter (me, NULL, 0);
+				showMeter (me, nullptr, 0);
 			}
 		}
 	} catch (MelderError) {
@@ -496,11 +494,11 @@ static WORKPROC_RETURN workProc (WORKPROC_ARGS) {
 }
 
 static int portaudioStreamCallback (
-    const void *input, void *output,
+    const void *input, void * /* output */,
     unsigned long frameCount,
-    const PaStreamCallbackTimeInfo* timeInfo,
-    PaStreamCallbackFlags statusFlags,
-    void *void_me)
+    const PaStreamCallbackTimeInfo* /* timeInfo */,
+    PaStreamCallbackFlags /* statusFlags */,
+    void *void_SoundRecorder)
 {
 	/*
 	 * This procedure may be called at interrupt time.
@@ -509,10 +507,7 @@ static int portaudioStreamCallback (
 	 * The only thing it changes is my nsamp;
 	 * the workProc will therefore have to take some care in accessing my nsamp (see there).
 	 */
-	iam (SoundRecorder);
-	(void) output;
-	(void) timeInfo;
-	(void) statusFlags;
+	SoundRecorder me = static_cast <SoundRecorder> (void_SoundRecorder);
 	if (Melder_debug == 20)
 		Melder_casual (U"The PortAudio stream callback receives ", frameCount, U" frames.");
 	Melder_assert (my nsamp <= my nmax);
@@ -531,8 +526,7 @@ static int portaudioStreamCallback (
 	return paContinue;
 }
 
-static void gui_button_cb_record (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_record (SoundRecorder me, GuiButtonEvent /* event */) {
 	try {
 		if (my recording) return;
 		my nsamp = 0;
@@ -556,7 +550,7 @@ static void gui_button_cb_record (I, GuiButtonEvent /* event */) {
 				#endif
 				if (Melder_debug == 20)
 					Melder_casual (U"Before Pa_OpenStream");
-				PaError err = Pa_OpenStream (& my portaudioStream, & streamParameters, NULL,
+				PaError err = Pa_OpenStream (& my portaudioStream, & streamParameters, nullptr,
 					theControlPanel. sampleRate, 0, paNoFlag, portaudioStreamCallback, (void *) me);
 				if (Melder_debug == 20)
 					Melder_casual (U"Pa_OpenStream returns ", (int) err);
@@ -579,27 +573,25 @@ static void gui_button_cb_record (I, GuiButtonEvent /* event */) {
 				#endif
 			}
 		}
-		Graphics_setWindow (my graphics, 0.0, 1.0, 0.0, 1.0);
-		Graphics_setColour (my graphics, Graphics_WHITE);
-		Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, 1.0);
+		Graphics_setWindow (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setColour (my graphics.get(), Graphics_WHITE);
+		Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 	} catch (MelderError) {
-		Graphics_setWindow (my graphics, 0.0, 1.0, 0.0, 1.0);
-		Graphics_setColour (my graphics, Graphics_WHITE);
-		Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, 1.0);
+		Graphics_setWindow (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setColour (my graphics.get(), Graphics_WHITE);
+		Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 		my recording = false;
 		Melder_flushError (U"Cannot record.");
 	}
 }
 
-static void gui_button_cb_stop (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_stop (SoundRecorder me, GuiButtonEvent /* event */) {
 	stopRecording (me);
 }
 
-static void gui_button_cb_play (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_play (SoundRecorder me, GuiButtonEvent /* event */) {
 	if (my recording || my nsamp == 0) return;
-	MelderAudio_play16 (my buffer, theControlPanel. sampleRate, my fakeMono ? my nsamp / 2 : my nsamp, my fakeMono ? 2 : my numberOfChannels, NULL, NULL);
+	MelderAudio_play16 (my buffer, theControlPanel. sampleRate, my fakeMono ? my nsamp / 2 : my nsamp, my fakeMono ? 2 : my numberOfChannels, nullptr, nullptr);
 }
 
 static void publish (SoundRecorder me) {
@@ -628,25 +620,22 @@ static void publish (SoundRecorder me) {
 	}
 	if (my soundName) {
 		autostring32 name = GuiText_getString (my soundName);
-		Thing_setName (sound.peek(), name.peek());
+		Thing_setName (sound.get(), name.peek());
 	}
-	Editor_broadcastPublication (me, sound.transfer());
+	Editor_broadcastPublication (me, sound.move());
 }
 
-static void gui_button_cb_cancel (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_cancel (SoundRecorder me, GuiButtonEvent /* event */) {
 	stopRecording (me);
 	forget (me);
 }
 
-static void gui_button_cb_apply (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_apply (SoundRecorder me, GuiButtonEvent /* event */) {
 	stopRecording (me);
 	publish (me);
 }
 
-static void gui_button_cb_ok (I, GuiButtonEvent /* event */) {
-	iam (SoundRecorder);
+static void gui_button_cb_ok (SoundRecorder me, GuiButtonEvent /* event */) {
 	stopRecording (me);
 	publish (me);
 	forget (me);
@@ -689,7 +678,7 @@ static void initialize (SoundRecorder me) {
 						Melder_throw (U"Cannot open audio device.\n"
 							U"Please switch on PortAudio in the Sound Recording Preferences.");
 				}
-				ioctl (my fd, SNDCTL_DSP_RESET, NULL);
+				ioctl (my fd, SNDCTL_DSP_RESET, nullptr);
 				ioctl (my fd, SNDCTL_DSP_SPEED, & sampleRate);
 				ioctl (my fd, SNDCTL_DSP_SAMPLESIZE, & sampleSize);
 				ioctl (my fd, SNDCTL_DSP_CHANNELS, (val = channels, & val));
@@ -717,8 +706,7 @@ static void initialize (SoundRecorder me) {
 	}
 }
 
-static void gui_radiobutton_cb_input (I, GuiRadioButtonEvent event) {
-	iam (SoundRecorder);
+static void gui_radiobutton_cb_input (SoundRecorder me, GuiRadioButtonEvent event) {
 	Melder_casual (U"SoundRecorder:"
 		U" setting the input source from ", theControlPanel. inputSource,
 		U" to ", event -> position, U".");
@@ -750,8 +738,7 @@ static void gui_radiobutton_cb_input (I, GuiRadioButtonEvent event) {
 	}
 }
 
-static void gui_radiobutton_cb_fsamp (I, GuiRadioButtonEvent event) {
-	iam (SoundRecorder);
+static void gui_radiobutton_cb_fsamp (SoundRecorder me, GuiRadioButtonEvent event) {
 	if (my recording) return;
 	try {
 		double fsamp = NUMundefined;
@@ -797,13 +784,12 @@ static void gui_radiobutton_cb_fsamp (I, GuiRadioButtonEvent event) {
 	}
 }
 
-static void gui_drawingarea_cb_resize (I, GuiDrawingAreaResizeEvent event) {
-	iam (SoundRecorder);
-	if (my graphics == NULL) return;   // Could be the case in the very beginning.
-	Graphics_setWsViewport (my graphics, 0, event -> width, 0, event -> height);
-	Graphics_setWsWindow (my graphics, 0, event -> width, 0, event -> height);
-	Graphics_setViewport (my graphics, 0, event -> width, 0, event -> height);
-	Graphics_updateWs (my graphics);
+static void gui_drawingarea_cb_resize (SoundRecorder me, GuiDrawingArea_ResizeEvent event) {
+	if (! my graphics) return;   // could be the case in the very beginning
+	Graphics_setWsViewport (my graphics.get(), 0, event -> width, 0, event -> height);
+	Graphics_setWsWindow (my graphics.get(), 0, event -> width, 0, event -> height);
+	Graphics_setViewport (my graphics.get(), 0, event -> width, 0, event -> height);
+	Graphics_updateWs (my graphics.get());
 }
 
 void structSoundRecorder :: v_createChildren ()
@@ -816,10 +802,10 @@ void structSoundRecorder :: v_createChildren ()
 	GuiRadioGroup_begin ();
 	y += Gui_RADIOBUTTON_HEIGHT + Gui_RADIOBUTTON_SPACING;
 	monoButton = GuiRadioButton_createShown (d_windowForm, 20, 170, y, y + Gui_RADIOBUTTON_HEIGHT,
-		U"Mono", NULL, NULL, 0);
+		U"Mono", nullptr, nullptr, 0);
 	y += Gui_RADIOBUTTON_HEIGHT + Gui_RADIOBUTTON_SPACING;
 	stereoButton = GuiRadioButton_createShown (d_windowForm, 20, 170, y, y + Gui_RADIOBUTTON_HEIGHT,
-		U"Stereo", NULL, NULL, 0);
+		U"Stereo", nullptr, nullptr, 0);
 	GuiRadioGroup_end ();
 
 	/* Input source */
@@ -848,7 +834,7 @@ void structSoundRecorder :: v_createChildren ()
 	GuiLabel_createShown (d_windowForm, 170, -170, y, y + Gui_LABEL_HEIGHT, U"Meter", GuiLabel_CENTRE);
 	y += Gui_LABEL_HEIGHT;
 	meter = GuiDrawingArea_createShown (d_windowForm, 170, -170, y, -150,
-		NULL, NULL, NULL, gui_drawingarea_cb_resize, this, GuiDrawingArea_BORDER);
+		nullptr, nullptr, nullptr, gui_drawingarea_cb_resize, this, GuiDrawingArea_BORDER);
 
 	/* Sampling frequency */
 
@@ -926,9 +912,8 @@ static void writeAudioFile (SoundRecorder me, MelderFile file, int audioFileType
 	}
 }
 
-static void menu_cb_writeWav (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
-	EDITOR_FORM_WRITE (U"Save as WAV file", 0)
+static void menu_cb_writeWav (SoundRecorder me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save as WAV file", nullptr)
 		char32 *name = GuiText_getString (my soundName);
 		Melder_sprint (defaultName,300, name, U".wav");
 		Melder_free (name);
@@ -937,9 +922,8 @@ static void menu_cb_writeWav (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_writeAifc (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
-	EDITOR_FORM_WRITE (U"Save as AIFC file", 0)
+static void menu_cb_writeAifc (SoundRecorder me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save as AIFC file", nullptr)
 		char32 *name = GuiText_getString (my soundName);
 		Melder_sprint (defaultName,300, name, U".aifc");
 		Melder_free (name);
@@ -948,9 +932,8 @@ static void menu_cb_writeAifc (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_writeNextSun (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
-	EDITOR_FORM_WRITE (U"Save as NeXT/Sun file", 0)
+static void menu_cb_writeNextSun (SoundRecorder me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save as NeXT/Sun file", nullptr)
 		char32 *name = GuiText_getString (my soundName);
 		Melder_sprint (defaultName,300, name, U".au");
 		Melder_free (name);
@@ -959,9 +942,8 @@ static void menu_cb_writeNextSun (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_writeNist (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
-	EDITOR_FORM_WRITE (U"Save as NIST file", 0)
+static void menu_cb_writeNist (SoundRecorder me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save as NIST file", nullptr)
 		char32 *name = GuiText_getString (my soundName);
 		Melder_sprint (defaultName,300, name, U".nist");
 		Melder_free (name);
@@ -977,18 +959,16 @@ static void updateMenus (SoundRecorder me) {
 		my p_meter_which == kSoundRecorder_meter_CENTRE_OF_GRAVITY_VERSUS_INTENSITY);
 }
 
-static void menu_cb_intensity (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
+static void menu_cb_intensity (SoundRecorder me, EDITOR_ARGS_DIRECT) {
 	my pref_meter_which () = my p_meter_which = kSoundRecorder_meter_INTENSITY;
 	updateMenus (me);
 }
-static void menu_cb_centreOfGravityVersusIntensity (EDITOR_ARGS) {
-	EDITOR_IAM (SoundRecorder);
+static void menu_cb_centreOfGravityVersusIntensity (SoundRecorder me, EDITOR_ARGS_DIRECT) {
 	my pref_meter_which () = my p_meter_which = kSoundRecorder_meter_CENTRE_OF_GRAVITY_VERSUS_INTENSITY;
 	updateMenus (me);
 }
 
-static void menu_cb_SoundRecorder_help (EDITOR_ARGS) { EDITOR_IAM (SoundRecorder); Melder_help (U"SoundRecorder"); }
+static void menu_cb_SoundRecorder_help (SoundRecorder, EDITOR_ARGS_DIRECT) { Melder_help (U"SoundRecorder"); }
 
 void structSoundRecorder :: v_createMenus () {
 	SoundRecorder_Parent :: v_createMenus ();
@@ -1067,7 +1047,7 @@ autoSoundRecorder SoundRecorder_create (int numberOfChannels) {
 		 */
 		if (preferences.bufferSizeInMegabytes < 1) preferences.bufferSizeInMegabytes = 1;   // validate preferences
 		if (preferences.bufferSizeInMegabytes > 1000) preferences.bufferSizeInMegabytes = 1000;
-		if (my buffer == NULL) {
+		if (! my buffer) {
 			long nmax_bytes_pref = preferences.bufferSizeInMegabytes * 1000000;
 			long nmax_bytes = my inputUsesPortAudio ? nmax_bytes_pref :
 				#if defined (_WIN32)
@@ -1090,7 +1070,7 @@ autoSoundRecorder SoundRecorder_create (int numberOfChannels) {
 				}
 			}
 		}
-		Melder_assert (my buffer != NULL);
+		Melder_assert (my buffer);
 
 		/*
 		 * Count the number of input devices and sources.
@@ -1179,31 +1159,31 @@ autoSoundRecorder SoundRecorder_create (int numberOfChannels) {
 		 * Some systems take initial values from the system control panel
 		 * (automatically in the workProc), other systems from theControlPanel.
 		 */
-		initialize (me.peek());
+		initialize (me.get());
 
-		Editor_init (me.peek(), 100, 100, 600, 500, U"SoundRecorder", NULL);
+		Editor_init (me.get(), 100, 100, 600, 500, U"SoundRecorder", nullptr);
 		my graphics = Graphics_create_xmdrawingarea (my meter);
 		Melder_assert (my graphics);
-		Graphics_setWindow (my graphics, 0.0, 1.0, 0.0, 1.0);
-		Graphics_setColour (my graphics, Graphics_WHITE);
-		Graphics_fillRectangle (my graphics, 0.0, 1.0, 0.0, 1.0);
+		Graphics_setWindow (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setColour (my graphics.get(), Graphics_WHITE);
+		Graphics_fillRectangle (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 
-struct structGuiDrawingAreaResizeEvent event = { my meter, 0 };
+struct structGuiDrawingArea_ResizeEvent event { my meter, 0 };
 event. width  = GuiControl_getWidth  (my meter);
 event. height = GuiControl_getHeight (my meter);
-gui_drawingarea_cb_resize (me.peek(), & event);
+gui_drawingarea_cb_resize (me.get(), & event);
 
 		#if cocoa
-			CFRunLoopTimerContext context = { 0, me.peek(), NULL, NULL, NULL };
-			my d_cocoaTimer = CFRunLoopTimerCreate (NULL, CFAbsoluteTimeGetCurrent () + 0.02,
+			CFRunLoopTimerContext context = { 0, me.get(), nullptr, nullptr, nullptr };
+			my d_cocoaTimer = CFRunLoopTimerCreate (nullptr, CFAbsoluteTimeGetCurrent () + 0.02,
 				0.02, 0, 0, workProc, & context);
 			CFRunLoopAddTimer (CFRunLoopGetCurrent (), my d_cocoaTimer, kCFRunLoopCommonModes);
 		#elif gtk
-			g_idle_add (workProc, me.peek());
+			g_idle_add (workProc, me.get());
 		#elif motif
-			my workProcId = GuiAddWorkProc (workProc, me.peek());
+			my workProcId = GuiAddWorkProc (workProc, me.get());
 		#endif
-		updateMenus (me.peek());
+		updateMenus (me.get());
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"SoundRecorder not created.");
