@@ -1,40 +1,19 @@
 /* Sound_audio.cpp
  *
- * Copyright (C) 1992-2011,2015 Paul Boersma
+ * Copyright (C) 1992-2011,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-/*
- * pb 2002/07/16 GPL
- * pb 2003/09/14 Sound_recordFixedTime records in stereo if mono is not available
- * pb 2003/12/06 use sys/soundcard.h instead of linux/soundcard.h for FreeBSD compatibility
- * pb 2005/04/24 Sound_recordFixedTime: Firewire Solo 1264
- * pb 2005/06/16 removed previous change (System Preferences handles this)
- * pb 2005/10/13 edition for OpenBSD
- * pb 2006/10/28 erased MacOS 9 stuff
- * pb 2006/12/20 Sound_playPart and Sound_play allow stereo
- * pb 2006/12/30 Sound_playPart and Sound_play allow better stereo
- * pb 2007/01/07 PortAudio
- * Stefan de Konink 2007/12/02 big-endian Linux
- * pb 2008/01/19 double
- * pb 2008/07/07 split zero padding between silenceBefore and silenceAfter
- * fb 2010/02/26 fix resource leak fd_mixer in case of error during init
- * pb 2010/04/20 Sound_recordFixedTime for Linux: repair
- * pb 2010/11/02 Sound_recordFixedTime for Linux: repair bug from 1998
- * pb 2011/06/07 C++
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <errno.h>
@@ -51,9 +30,6 @@
 
 #if defined (macintosh)
 	#include "macport_on.h"
-    #if useCarbon
-        #include <Carbon/Carbon.h>
-    #endif
 	#include "pa_mac_core.h"
 	#include "macport_off.h"
 #elif defined (_WIN32)
@@ -152,7 +128,7 @@ autoSound Sound_recordFixedTime (int inputSource, double gain, double balance, d
 		#else
 			MelderAudio_getInputSoundSystem () == kMelder_inputSoundSystem_ALSA_VIA_PORTAUDIO;
 		#endif
-	PaStream *portaudioStream = NULL;
+	PaStream *portaudioStream = nullptr;
 	#if defined (macintosh)
 	#elif defined (_WIN32)
 		HWAVEIN hWaveIn = 0;
@@ -249,7 +225,7 @@ autoSound Sound_recordFixedTime (int inputSource, double gain, double balance, d
 				/* The device immediately started recording into its buffer, but probably at the wrong rate etc. */
 				/* Pause and flush this rubbish. */
 				#if defined (linux)
-					ioctl (fd, SNDCTL_DSP_RESET, NULL);
+					ioctl (fd, SNDCTL_DSP_RESET, nullptr);
 				#endif
 			#endif
 		}
@@ -381,7 +357,7 @@ autoSound Sound_recordFixedTime (int inputSource, double gain, double balance, d
 			info. numberOfSamples = numberOfSamples;
 			info. numberOfSamplesRead = 0;
 			info. buffer = buffer.peek();
-			PaError err = Pa_OpenStream (& portaudioStream, & streamParameters, NULL,
+			PaError err = Pa_OpenStream (& portaudioStream, & streamParameters, nullptr,
 				sampleRate, 0, paNoFlag, portaudioStreamCallback, (void *) & info);
 			if (err)
 				Melder_throw (U"open ", Melder_peek8to32 (Pa_GetErrorText (err)));
@@ -415,7 +391,7 @@ for (i = 1; i <= numberOfSamples; i ++) trace (U"Recorded ", buffer [i]);
 				waveHeader. lpData = (char *) & buffer [1];
 				waveHeader. dwBufferLength = numberOfSamples * 2;
 				waveHeader. dwLoops = 0;
-				waveHeader. lpNext = NULL;
+				waveHeader. lpNext = nullptr;
 				waveHeader. reserved = 0;
 				err = waveInPrepareHeader (hWaveIn, & waveHeader, sizeof (WAVEHDR));
 				if (err != MMSYSERR_NOERROR)
@@ -503,9 +479,9 @@ for (i = 1; i <= numberOfSamples; i ++) trace (U"Recorded ", buffer [i]);
 static struct SoundPlay {
 	long numberOfSamples, i1, i2, silenceBefore, silenceAfter;
 	double tmin, tmax, dt, t1;
-	int (*callback) (void *closure, int phase, double tmin, double tmax, double t);
-	void *closure;
-	short *buffer;
+	Sound_PlayCallback callback;
+	Thing boss;
+	int16 *buffer;
 } thePlayingSound;
 
 static bool melderPlayCallback (void *closure, long samplesPlayed) {
@@ -515,22 +491,21 @@ static bool melderPlayCallback (void *closure, long samplesPlayed) {
 		samplesPlayed >= my silenceBefore + my numberOfSamples ? my tmax :
 		my t1 + (my i1 - 1.5 + samplesPlayed - my silenceBefore) * my dt;
 	if (! MelderAudio_isPlaying) {
-		NUMvector_free (my buffer, 1), my buffer = NULL;
+		NUMvector_free (my buffer, 1), my buffer = nullptr;
 		phase = 3;
 	}
 	if (my callback)
-		return my callback (my closure, phase, my tmin, my tmax, t);
+		return my callback (my boss, phase, my tmin, my tmax, t);
 	return true;
 }
 
-void Sound_playPart (Sound me, double tmin, double tmax,
-	int (*callback) (void *closure, int phase, double tmin, double tmax, double t), void *closure)
+void Sound_playPart (Sound me, double tmin, double tmax, Sound_PlayCallback callback, Thing boss)
 {
 	try {
 		long ifsamp = lround (1.0 / my dx), bestSampleRate = MelderAudio_getOutputBestSampleRate (ifsamp);
 		if (ifsamp == bestSampleRate) {
 			struct SoundPlay *thee = (struct SoundPlay *) & thePlayingSound;
-			double *fromLeft = my z [1], *fromRight = my ny > 1 ? my z [2] : NULL;
+			double *fromLeft = my z [1], *fromRight = my ny > 1 ? my z [2] : nullptr;
 			MelderAudio_stopPlaying (MelderAudio_IMPLICIT);
 			long i1, i2;
 			if ((thy numberOfSamples = Matrix_getWindowSamplesX (me, tmin, tmax, & i1, & i2)) < 1) return;
@@ -539,7 +514,7 @@ void Sound_playPart (Sound me, double tmin, double tmax,
 			thy dt = my dx;
 			thy t1 = my x1;
 			thy callback = callback;
-			thy closure = closure;
+			thy boss = boss;
 			thy silenceBefore = (long) (ifsamp * MelderAudio_getOutputSilenceBefore ());
 			thy silenceAfter = (long) (ifsamp * MelderAudio_getOutputSilenceAfter ());
 			int numberOfChannels = my ny;
@@ -568,20 +543,19 @@ void Sound_playPart (Sound me, double tmin, double tmax,
 					* ++ to = value < -32768 ? -32768 : value > 32767 ? 32767 : value;
 				}
 			}
-			if (thy callback) thy callback (thy closure, 1, tmin, tmax, tmin);
+			if (thy callback) thy callback (thy boss, 1, tmin, tmax, tmin);
 			MelderAudio_play16 (thy buffer + 1, ifsamp,
 				thy silenceBefore + thy numberOfSamples + thy silenceAfter, numberOfChannels, melderPlayCallback, thee);
 		} else {
 			autoSound resampled = Sound_resample (me, bestSampleRate, 1);
-			Sound_playPart (resampled.peek(), tmin, tmax, callback, closure);   // recursively
+			Sound_playPart (resampled.get(), tmin, tmax, callback, boss);   // recursively
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": not played.");
 	}
 }
 
-void Sound_play (Sound me,
-	int (*playCallback) (void *playClosure, int phase, double tmin, double tmax, double t), void *playClosure)
+void Sound_play (Sound me, Sound_PlayCallback playCallback, Thing playClosure)
 {
 	Sound_playPart (me, my xmin, my xmax, playCallback, playClosure);
 }
