@@ -4,19 +4,18 @@
  *
  * Copyright (C) 1992-2012,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "AnyTier.h"
@@ -25,26 +24,25 @@
 #include "TableOfReal.h"
 #include "Table.h"
 
+Collection_define (FunctionList, OrderedOf, Function) {
+};
+
 #include "TextGrid_def.h"
 
-oo_CLASS_CREATE (TextPoint, AnyPoint);
 autoTextPoint TextPoint_create (double time, const char32 *mark);
 
 void TextPoint_setText (TextPoint me, const char32 *text);
 
-oo_CLASS_CREATE (TextInterval, Function);
 autoTextInterval TextInterval_create (double tmin, double tmax, const char32 *text);
 
 void TextInterval_setText (TextInterval me, const char32 *text);
 
-oo_CLASS_CREATE (TextTier, Function);
 autoTextTier TextTier_create (double tmin, double tmax);
 
 void TextTier_addPoint (TextTier me, double time, const char32 *mark);
 autoTextTier TextTier_readFromXwaves (MelderFile file);
 autoPointProcess TextTier_getPoints (TextTier me, const char32 *text);
 
-oo_CLASS_CREATE (IntervalTier, Function);
 autoIntervalTier IntervalTier_create (double tmin, double tmax);
 autoIntervalTier IntervalTier_readFromXwaves (MelderFile file);
 void IntervalTier_writeToXwaves (IntervalTier me, MelderFile file);
@@ -63,7 +61,6 @@ void IntervalTier_removeLeftBoundary (IntervalTier me, long iinterval);
 
 void TextTier_removePoint (TextTier me, long ipoint);
 
-oo_CLASS_CREATE (TextGrid, Function);
 autoTextGrid TextGrid_createWithoutTiers (double tmin, double tmax);
 autoTextGrid TextGrid_create (double tmin, double tmax, const char32 *tierNames, const char32 *pointTiers);
 
@@ -86,11 +83,11 @@ IntervalTier TextGrid_checkSpecifiedTierIsIntervalTier (TextGrid me, long tierNu
 TextTier TextGrid_checkSpecifiedTierIsPointTier (TextGrid me, long tierNumber);
 
 void TextGrid_addTier_copy (TextGrid me, Function tier);
-autoTextGrid TextGrid_merge (Collection textGrids);
+autoTextGrid TextGrids_merge (OrderedOf<structTextGrid>* textGrids);
 autoTextGrid TextGrid_extractPart (TextGrid me, double tmin, double tmax, int preserveTimes);
 
 autoTextGrid Label_to_TextGrid (Label me, double duration);
-autoTextGrid Label_Function_to_TextGrid (Label me, Any function);
+autoTextGrid Label_Function_to_TextGrid (Label me, Function function);
 
 autoTextTier PointProcess_upto_TextTier (PointProcess me, const char32 *text);
 autoTableOfReal IntervalTier_downto_TableOfReal (IntervalTier me, const char32 *label);
@@ -130,7 +127,7 @@ autoTable TextGrid_downto_Table (TextGrid me, bool includeLineNumbers, int timeD
 void TextGrid_list (TextGrid me, bool includeLineNumbers, int timeDecimals, bool includeTierNames, bool includeEmptyIntervals);
 
 void TextGrid_correctRoundingErrors (TextGrid me);
-autoTextGrid TextGrids_concatenate (Collection me);
+autoTextGrid TextGrids_concatenate (OrderedOf<structTextGrid>* me);
 
 /* End of file TextGrid.h */
 #endif
