@@ -1,20 +1,19 @@
 /* TimeSoundEditor.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "TimeSoundEditor.h"
@@ -37,9 +36,9 @@ Thing_implement (TimeSoundEditor, FunctionEditor, 0);
 
 /********** Thing methods **********/
 
-void structTimeSoundEditor :: v_destroy () {
-	if (d_ownSound)
-		forget (d_sound.data);
+void structTimeSoundEditor :: v_destroy () noexcept {
+	if (our d_ownSound)
+		forget (our d_sound.data);
 	TimeSoundEditor_Parent :: v_destroy ();
 }
 
@@ -51,9 +50,8 @@ void structTimeSoundEditor :: v_info () {
 
 /***** FILE MENU *****/
 
-static void menu_cb_DrawVisibleSound (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM (U"Draw visible sound", 0)
+static void menu_cb_DrawVisibleSound (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Draw visible sound", nullptr)
 		my v_form_pictureWindow (cmd);
 		LABEL (U"", U"Sound:")
 		BOOLEAN (U"Preserve times", my default_picture_preserveTimes ());
@@ -78,22 +76,21 @@ static void menu_cb_DrawVisibleSound (EDITOR_ARGS) {
 		my v_do_pictureMargins (cmd);
 		my v_do_pictureSelection (cmd);
 		my pref_picture_garnish () = GET_INTEGER (U"Garnish");
-		if (my d_longSound.data == NULL && my d_sound.data == NULL)
+		if (! my d_longSound.data && ! my d_sound.data)
 			Melder_throw (U"There is no sound to draw.");
 		autoSound publish = my d_longSound.data ?
 			LongSound_extractPart (my d_longSound.data, my d_startWindow, my d_endWindow, my pref_picture_preserveTimes ()) :
 			Sound_extractPart (my d_sound.data, my d_startWindow, my d_endWindow, kSound_windowShape_RECTANGULAR, 1.0, my pref_picture_preserveTimes ());
 		Editor_openPraatPicture (me);
-		Sound_draw (publish.peek(), my pictureGraphics, 0.0, 0.0, my pref_picture_bottom (), my pref_picture_top (),
+		Sound_draw (publish.get(), my pictureGraphics, 0.0, 0.0, my pref_picture_bottom (), my pref_picture_top (),
 			my pref_picture_garnish (), U"Curve");
 		FunctionEditor_garnish (me);
 		Editor_closePraatPicture (me);
 	EDITOR_END
 }
 
-static void menu_cb_DrawSelectedSound (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM (U"Draw selected sound", 0)
+static void menu_cb_DrawSelectedSound (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Draw selected sound", nullptr)
 		my v_form_pictureWindow (cmd);
 		LABEL (U"", U"Sound:")
 		BOOLEAN (U"Preserve times", my default_picture_preserveTimes ());
@@ -115,13 +112,13 @@ static void menu_cb_DrawSelectedSound (EDITOR_ARGS) {
 		my pref_picture_top () = GET_REAL (U"right Vertical range");
 		my v_do_pictureMargins (cmd);
 		my pref_picture_garnish () = GET_INTEGER (U"Garnish");
-		if (my d_longSound.data == NULL && my d_sound.data == NULL)
+		if (! my d_longSound.data && ! my d_sound.data)
 			Melder_throw (U"There is no sound to draw.");
 		autoSound publish = my d_longSound.data ?
 			LongSound_extractPart (my d_longSound.data, my d_startSelection, my d_endSelection, my pref_picture_preserveTimes ()) :
 			Sound_extractPart (my d_sound.data, my d_startSelection, my d_endSelection, kSound_windowShape_RECTANGULAR, 1.0, my pref_picture_preserveTimes ());
 		Editor_openPraatPicture (me);
-		Sound_draw (publish.peek(), my pictureGraphics, 0.0, 0.0, my pref_picture_bottom (), my pref_picture_top (),
+		Sound_draw (publish.get(), my pictureGraphics, 0.0, 0.0, my pref_picture_bottom (), my pref_picture_top (),
 			my pref_picture_garnish (), U"Curve");
 		Editor_closePraatPicture (me);
 	EDITOR_END
@@ -136,22 +133,19 @@ static void do_ExtractSelectedSound (TimeSoundEditor me, bool preserveTimes) {
 	} else if (my d_sound.data) {
 		extract = Sound_extractPart (my d_sound.data, my d_startSelection, my d_endSelection, kSound_windowShape_RECTANGULAR, 1.0, preserveTimes);
 	}
-	Editor_broadcastPublication (me, extract.transfer());
+	Editor_broadcastPublication (me, extract.move());
 }
 
-static void menu_cb_ExtractSelectedSound_timeFromZero (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
+static void menu_cb_ExtractSelectedSound_timeFromZero (TimeSoundEditor me, EDITOR_ARGS_DIRECT) {
 	do_ExtractSelectedSound (me, false);
 }
 
-static void menu_cb_ExtractSelectedSound_preserveTimes (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
+static void menu_cb_ExtractSelectedSound_preserveTimes (TimeSoundEditor me, EDITOR_ARGS_DIRECT) {
 	do_ExtractSelectedSound (me, true);
 }
 
-static void menu_cb_ExtractSelectedSound_windowed (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM (U"Extract selected sound (windowed)", 0)
+static void menu_cb_ExtractSelectedSound_windowed (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Extract selected sound (windowed)", nullptr)
 		WORD (U"Name", U"slice")
 		OPTIONMENU_ENUM (U"Window shape", kSound_windowShape, my default_extract_windowShape ())
 		POSITIVE (U"Relative width", my default_extract_relativeWidth ())
@@ -162,32 +156,31 @@ static void menu_cb_ExtractSelectedSound_windowed (EDITOR_ARGS) {
 		SET_INTEGER (U"Preserve times", my pref_extract_preserveTimes ())
 	EDITOR_DO
 		Sound sound = my d_sound.data;
-		Melder_assert (sound != NULL);
+		Melder_assert (sound);
 		my pref_extract_windowShape () = GET_ENUM (kSound_windowShape, U"Window shape");
 		my pref_extract_relativeWidth () = GET_REAL (U"Relative width");
 		my pref_extract_preserveTimes () = GET_INTEGER (U"Preserve times");
 		autoSound extract = Sound_extractPart (sound, my d_startSelection, my d_endSelection, my pref_extract_windowShape (),
 			my pref_extract_relativeWidth (), my pref_extract_preserveTimes ());
-		Thing_setName (extract.peek(), GET_STRING (U"Name"));
-		Editor_broadcastPublication (me, extract.transfer());
+		Thing_setName (extract.get(), GET_STRING (U"Name"));
+		Editor_broadcastPublication (me, extract.move());
 	EDITOR_END
 }
 
-static void menu_cb_ExtractSelectedSoundForOverlap (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM (U"Extract selected sound for overlap)", 0)
+static void menu_cb_ExtractSelectedSoundForOverlap (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Extract selected sound for overlap)", nullptr)
 		WORD (U"Name", U"slice")
 		POSITIVE (U"Overlap (s)", my default_extract_overlap ())
 	EDITOR_OK
 		SET_REAL (U"Overlap", my pref_extract_overlap ())
 	EDITOR_DO
 		Sound sound = my d_sound.data;
-		Melder_assert (sound != NULL);
+		Melder_assert (sound);
 		my pref_extract_overlap () = GET_REAL (U"Overlap");
 		autoSound extract = Sound_extractPartForOverlap (sound, my d_startSelection, my d_endSelection,
 			my pref_extract_overlap ());
-		Thing_setName (extract.peek(), GET_STRING (U"Name"));
-		Editor_broadcastPublication (me, extract.transfer());
+		Thing_setName (extract.get(), GET_STRING (U"Name"));
+		Editor_broadcastPublication (me, extract.move());
 	EDITOR_END
 }
 
@@ -214,79 +207,71 @@ static void do_write (TimeSoundEditor me, MelderFile file, int format, int numbe
 					save -> z [channel] [i - offset] = sound -> z [channel] [i];
 				}
 			}
-			Sound_writeToAudioFile (save.peek(), file, format, numberOfBitsPerSamplePoint);
+			Sound_writeToAudioFile (save.get(), file, format, numberOfBitsPerSamplePoint);
 		}
 	}
 }
 
-static void menu_cb_WriteWav (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as WAV file", 0)
+static void menu_cb_WriteWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as WAV file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".wav");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_WAV, 16);
 	EDITOR_END
 }
 
-static void menu_cb_SaveAs24BitWav (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as 24-bit WAV file", 0)
-		Melder_assert (my d_longSound.data == NULL && my d_sound.data != NULL);
+static void menu_cb_SaveAs24BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as 24-bit WAV file", nullptr)
+		Melder_assert (! my d_longSound.data && my d_sound.data);
 		Melder_sprint (defaultName,300, my d_sound.data -> name, U".wav");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_WAV, 24);
 	EDITOR_END
 }
 
-static void menu_cb_SaveAs32BitWav (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as 32-bit WAV file", 0)
-		Melder_assert (my d_longSound.data == NULL && my d_sound.data != NULL);
+static void menu_cb_SaveAs32BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as 32-bit WAV file", nullptr)
+		Melder_assert (! my d_longSound.data && my d_sound.data);
 		Melder_sprint (defaultName,300, my d_sound.data -> name, U".wav");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_WAV, 32);
 	EDITOR_END
 }
 
-static void menu_cb_WriteAiff (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as AIFF file", 0)
+static void menu_cb_WriteAiff (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as AIFF file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".aiff");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_AIFF, 16);
 	EDITOR_END
 }
 
-static void menu_cb_WriteAifc (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as AIFC file", 0)
+static void menu_cb_WriteAifc (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as AIFC file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".aifc");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_AIFC, 16);
 	EDITOR_END
 }
 
-static void menu_cb_WriteNextSun (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as NeXT/Sun file", 0)
+static void menu_cb_WriteNextSun (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as NeXT/Sun file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".au");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_NEXT_SUN, 16);
 	EDITOR_END
 }
 
-static void menu_cb_WriteNist (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as NIST file", 0)
+static void menu_cb_WriteNist (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as NIST file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".nist");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_NIST, 16);
 	EDITOR_END
 }
 
-static void menu_cb_WriteFlac (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM_WRITE (U"Save selected sound as FLAC file", 0)
+static void menu_cb_WriteFlac (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save selected sound as FLAC file", nullptr)
 		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".flac");
 	EDITOR_DO_WRITE
 		do_write (me, file, Melder_FLAC, 16);
@@ -356,39 +341,36 @@ void structTimeSoundEditor :: v_createMenuItems_file_write (EditorMenu menu) {
 void structTimeSoundEditor :: v_createMenuItems_file (EditorMenu menu) {
 	TimeSoundEditor_Parent :: v_createMenuItems_file (menu);
 	v_createMenuItems_file_draw (menu);
-	EditorMenu_addCommand (menu, U"-- after file draw --", 0, NULL);
+	EditorMenu_addCommand (menu, U"-- after file draw --", 0, nullptr);
 	v_createMenuItems_file_extract (menu);
-	EditorMenu_addCommand (menu, U"-- after file extract --", 0, NULL);
+	EditorMenu_addCommand (menu, U"-- after file extract --", 0, nullptr);
 	v_createMenuItems_file_write (menu);
-	EditorMenu_addCommand (menu, U"-- after file write --", 0, NULL);
+	EditorMenu_addCommand (menu, U"-- after file write --", 0, nullptr);
 }
 
 /********** QUERY MENU **********/
 
-static void menu_cb_SoundInfo (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
+static void menu_cb_SoundInfo (TimeSoundEditor me, EDITOR_ARGS_DIRECT) {
 	Thing_info (my d_sound.data);
 }
 
-static void menu_cb_LongSoundInfo (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
+static void menu_cb_LongSoundInfo (TimeSoundEditor me, EDITOR_ARGS_DIRECT) {
 	Thing_info (my d_longSound.data);
 }
 
 void structTimeSoundEditor :: v_createMenuItems_query_info (EditorMenu menu) {
 	TimeSoundEditor_Parent :: v_createMenuItems_query_info (menu);
-	if (d_sound.data != NULL && d_sound.data != data) {
+	if (d_sound.data && d_sound.data != data) {
 		EditorMenu_addCommand (menu, U"Sound info", 0, menu_cb_SoundInfo);
-	} else if (d_longSound.data != NULL && d_longSound.data != data) {
+	} else if (d_longSound.data && d_longSound.data != data) {
 		EditorMenu_addCommand (menu, U"LongSound info", 0, menu_cb_LongSoundInfo);
 	}
 }
 
 /********** VIEW MENU **********/
 
-static void menu_cb_soundScaling (EDITOR_ARGS) {
-	EDITOR_IAM (TimeSoundEditor);
-	EDITOR_FORM (U"Sound scaling", 0)
+static void menu_cb_soundScaling (TimeSoundEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Sound scaling", nullptr)
 		OPTIONMENU_ENUM (U"Scaling strategy", kTimeSoundEditor_scalingStrategy, my default_sound_scalingStrategy ())
 		LABEL (U"", U"For \"fixed height\":");
 		POSITIVE (U"Height", my default_sound_scaling_height ())
@@ -417,12 +399,17 @@ void structTimeSoundEditor :: v_createMenuItems_view (EditorMenu menu) {
 
 void structTimeSoundEditor :: v_createMenuItems_view_sound (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"Sound scaling...", 0, menu_cb_soundScaling);
-	EditorMenu_addCommand (menu, U"-- sound view --", 0, 0);
+	EditorMenu_addCommand (menu, U"-- sound view --", 0, nullptr);
 }
 
 void structTimeSoundEditor :: v_updateMenuItems_file () {
-	Sampled sound = d_sound.data != NULL ? (Sampled) d_sound.data : (Sampled) d_longSound.data;
-	if (sound == NULL) return;
+	Sampled sound;
+	if (d_sound.data) {
+		sound = d_sound.data;
+	} else {
+		sound = d_longSound.data;
+	}
+	if (! sound) return;
 	long first, last, selectedSamples = Sampled_getWindowSamples (sound, d_startSelection, d_endSelection, & first, & last);
 	if (drawButton) {
 		GuiThing_setSensitive (drawButton, selectedSamples != 0);
@@ -446,32 +433,32 @@ void structTimeSoundEditor :: v_updateMenuItems_file () {
 void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double globalMaximum) {
 	Sound sound = my d_sound.data;
 	LongSound longSound = my d_longSound.data;
-	Melder_assert ((sound == NULL) != (longSound == NULL));
+	Melder_assert (!! sound != !! longSound);
 	int nchan = sound ? sound -> ny : longSound -> numberOfChannels;
 	bool cursorVisible = my d_startSelection == my d_endSelection && my d_startSelection >= my d_startWindow && my d_startSelection <= my d_endWindow;
-	Graphics_setColour (my d_graphics, Graphics_BLACK);
+	Graphics_setColour (my d_graphics.get(), Graphics_BLACK);
 	bool fits;
 	try {
 		fits = sound ? true : LongSound_haveWindow (longSound, my d_startWindow, my d_endWindow);
 	} catch (MelderError) {
-		int outOfMemory = str32str (Melder_getError (), U"memory") != NULL;
+		bool outOfMemory = !! str32str (Melder_getError (), U"memory");
 		if (Melder_debug == 9) Melder_flushError (); else Melder_clearError ();
-		Graphics_setWindow (my d_graphics, 0, 1, 0, 1);
-		Graphics_setTextAlignment (my d_graphics, Graphics_CENTRE, Graphics_HALF);
-		Graphics_text (my d_graphics, 0.5, 0.5, outOfMemory ? U"(out of memory)" : U"(cannot read sound file)");
+		Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setTextAlignment (my d_graphics.get(), Graphics_CENTRE, Graphics_HALF);
+		Graphics_text (my d_graphics.get(), 0.5, 0.5, outOfMemory ? U"(out of memory)" : U"(cannot read sound file)");
 		return;
 	}
 	if (! fits) {
-		Graphics_setWindow (my d_graphics, 0, 1, 0, 1);
-		Graphics_setTextAlignment (my d_graphics, Graphics_CENTRE, Graphics_HALF);
-		Graphics_text (my d_graphics, 0.5, 0.5, U"(window too large; zoom in to see the data)");
+		Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setTextAlignment (my d_graphics.get(), Graphics_CENTRE, Graphics_HALF);
+		Graphics_text (my d_graphics.get(), 0.5, 0.5, U"(window too large; zoom in to see the data)");
 		return;
 	}
 	long first, last;
 	if (Sampled_getWindowSamples (sound ? (Sampled) sound : (Sampled) longSound, my d_startWindow, my d_endWindow, & first, & last) <= 1) {
-		Graphics_setWindow (my d_graphics, 0, 1, 0, 1);
-		Graphics_setTextAlignment (my d_graphics, Graphics_CENTRE, Graphics_HALF);
-		Graphics_text (my d_graphics, 0.5, 0.5, U"(zoom out to see the data)");
+		Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setTextAlignment (my d_graphics.get(), Graphics_CENTRE, Graphics_HALF);
+		Graphics_text (my d_graphics.get(), 0.5, 0.5, U"(zoom out to see the data)");
 		return;
 	}
 	const int numberOfVisibleChannels = nchan > 8 ? 8 : nchan;
@@ -505,7 +492,7 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		 */
 		double ymin = (double) (numberOfVisibleChannels - ichan + my d_sound.channelOffset) / numberOfVisibleChannels;
 		double ymax = (double) (numberOfVisibleChannels + 1 - ichan + my d_sound.channelOffset) / numberOfVisibleChannels;
-		Graphics_Viewport vp = Graphics_insetViewport (my d_graphics, 0, 1, ymin, ymax);
+		Graphics_Viewport vp = Graphics_insetViewport (my d_graphics.get(), 0.0, 1.0, ymin, ymax);
 		bool horizontal = false;
 		double minimum = sound ? globalMinimum : -1.0, maximum = sound ? globalMaximum : 1.0;
 		if (my p_sound_scalingStrategy == kTimeSoundEditor_scalingStrategy_BY_WINDOW) {
@@ -544,42 +531,42 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 			minimum = my p_sound_scaling_minimum;
 			maximum = my p_sound_scaling_maximum;
 		}
-		if (minimum == maximum) { horizontal = true; minimum -= 1; maximum += 1;}
-		Graphics_setWindow (my d_graphics, my d_startWindow, my d_endWindow, minimum, maximum);
+		if (minimum == maximum) { horizontal = true; minimum -= 1.0; maximum += 1.0;}
+		Graphics_setWindow (my d_graphics.get(), my d_startWindow, my d_endWindow, minimum, maximum);
 		if (horizontal) {
-			Graphics_setTextAlignment (my d_graphics, Graphics_RIGHT, Graphics_HALF);
+			Graphics_setTextAlignment (my d_graphics.get(), Graphics_RIGHT, Graphics_HALF);
 			double mid = 0.5 * (minimum + maximum);
-			Graphics_text (my d_graphics, my d_startWindow, mid, Melder_float (Melder_half (mid)));
+			Graphics_text (my d_graphics.get(), my d_startWindow, mid, Melder_float (Melder_half (mid)));
 		} else {
-			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || Graphics_dyWCtoMM (my d_graphics, cursorFunctionValue - minimum) > 5.0) {
-				Graphics_setTextAlignment (my d_graphics, Graphics_RIGHT, Graphics_BOTTOM);
-				Graphics_text (my d_graphics, my d_startWindow, minimum, Melder_float (Melder_half (minimum)));
+			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || Graphics_dyWCtoMM (my d_graphics.get(), cursorFunctionValue - minimum) > 5.0) {
+				Graphics_setTextAlignment (my d_graphics.get(), Graphics_RIGHT, Graphics_BOTTOM);
+				Graphics_text (my d_graphics.get(), my d_startWindow, minimum, Melder_float (Melder_half (minimum)));
 			}
-			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || Graphics_dyWCtoMM (my d_graphics, maximum - cursorFunctionValue) > 5.0) {
-				Graphics_setTextAlignment (my d_graphics, Graphics_RIGHT, Graphics_TOP);
-				Graphics_text (my d_graphics, my d_startWindow, maximum, Melder_float (Melder_half (maximum)));
+			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || Graphics_dyWCtoMM (my d_graphics.get(), maximum - cursorFunctionValue) > 5.0) {
+				Graphics_setTextAlignment (my d_graphics.get(), Graphics_RIGHT, Graphics_TOP);
+				Graphics_text (my d_graphics.get(), my d_startWindow, maximum, Melder_float (Melder_half (maximum)));
 			}
 		}
 		if (minimum < 0 && maximum > 0 && ! horizontal) {
-			Graphics_setWindow (my d_graphics, 0, 1, minimum, maximum);
-			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || fabs (Graphics_dyWCtoMM (my d_graphics, cursorFunctionValue - 0.0)) > 3.0) {
-				Graphics_setTextAlignment (my d_graphics, Graphics_RIGHT, Graphics_HALF);
-				Graphics_text (my d_graphics, 0, 0, U"0");
+			Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, minimum, maximum);
+			if (! cursorVisible || ! NUMdefined (cursorFunctionValue) || fabs (Graphics_dyWCtoMM (my d_graphics.get(), cursorFunctionValue - 0.0)) > 3.0) {
+				Graphics_setTextAlignment (my d_graphics.get(), Graphics_RIGHT, Graphics_HALF);
+				Graphics_text (my d_graphics.get(), 0.0, 0.0, U"0");
 			}
-			Graphics_setColour (my d_graphics, Graphics_CYAN);
-			Graphics_setLineType (my d_graphics, Graphics_DOTTED);
-			Graphics_line (my d_graphics, 0, 0, 1, 0);
-			Graphics_setLineType (my d_graphics, Graphics_DRAWN);
+			Graphics_setColour (my d_graphics.get(), Graphics_CYAN);
+			Graphics_setLineType (my d_graphics.get(), Graphics_DOTTED);
+			Graphics_line (my d_graphics.get(), 0.0, 0.0, 1.0, 0.0);
+			Graphics_setLineType (my d_graphics.get(), Graphics_DRAWN);
 		}
 		/*
 		 * Garnish the drawing area of each channel.
 		 */
-		Graphics_setWindow (my d_graphics, 0, 1, 0, 1);
-		Graphics_setColour (my d_graphics, Graphics_CYAN);
-		Graphics_innerRectangle (my d_graphics, 0, 1, 0, 1);
-		Graphics_setColour (my d_graphics, Graphics_BLACK);
+		Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setColour (my d_graphics.get(), Graphics_CYAN);
+		Graphics_innerRectangle (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+		Graphics_setColour (my d_graphics.get(), Graphics_BLACK);
 		if (nchan > 1) {
-			Graphics_setTextAlignment (my d_graphics, Graphics_LEFT, Graphics_HALF);
+			Graphics_setTextAlignment (my d_graphics.get(), Graphics_LEFT, Graphics_HALF);
 			const char32 *channelName = my v_getChannelName (ichan);
 			static MelderString channelLabel;
 			MelderString_copy (& channelLabel, ( channelName ? U"ch" : U"Channel " ), ichan);
@@ -590,53 +577,53 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 			} else if (ichan >= 8 && ichan - my d_sound.channelOffset == 8 && ichan < nchan) {
 				MelderString_append (& channelLabel, U" " UNITEXT_DOWNWARDS_ARROW);
 			}
-			Graphics_text (my d_graphics, 1, 0.5, channelLabel.string);
+			Graphics_text (my d_graphics.get(), 1.0, 0.5, channelLabel.string);
 		}
 		/*
 		 * Draw a very thin separator line underneath.
 		 */
 		if (ichan < nchan) {
-			/*Graphics_setColour (d_graphics, Graphics_BLACK);*/
-			Graphics_line (my d_graphics, 0, 0, 1, 0);
+			/*Graphics_setColour (d_graphics.get(), Graphics_BLACK);*/
+			Graphics_line (my d_graphics.get(), 0.0, 0.0, 1.0, 0.0);
 		}
 		/*
 		 * Draw the samples.
 		 */
 		/*if (ichan == 1) FunctionEditor_SoundAnalysis_drawPulses (this);*/
 		if (sound) {
-			Graphics_setWindow (my d_graphics, my d_startWindow, my d_endWindow, minimum, maximum);
+			Graphics_setWindow (my d_graphics.get(), my d_startWindow, my d_endWindow, minimum, maximum);
 			if (cursorVisible && NUMdefined (cursorFunctionValue))
 				FunctionEditor_drawCursorFunctionValue (me, cursorFunctionValue, Melder_float (Melder_half (cursorFunctionValue)), U"");
-			Graphics_setColour (my d_graphics, Graphics_BLACK);
-			Graphics_function (my d_graphics, sound -> z [ichan], first, last,
+			Graphics_setColour (my d_graphics.get(), Graphics_BLACK);
+			Graphics_function (my d_graphics.get(), sound -> z [ichan], first, last,
 				Sampled_indexToX (sound, first), Sampled_indexToX (sound, last));
 		} else {
-			Graphics_setWindow (my d_graphics, my d_startWindow, my d_endWindow, minimum * 32768, maximum * 32768);
-			Graphics_function16 (my d_graphics,
+			Graphics_setWindow (my d_graphics.get(), my d_startWindow, my d_endWindow, minimum * 32768, maximum * 32768);
+			Graphics_function16 (my d_graphics.get(),
 				longSound -> buffer - longSound -> imin * nchan + (ichan - 1), nchan - 1, first, last,
 				Sampled_indexToX (longSound, first), Sampled_indexToX (longSound, last));
 		}
-		Graphics_resetViewport (my d_graphics, vp);
+		Graphics_resetViewport (my d_graphics.get(), vp);
 	}
-	Graphics_setWindow (my d_graphics, 0, 1, 0, 1);
-	Graphics_rectangle (my d_graphics, 0, 1, 0, 1);
+	Graphics_setWindow (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
+	Graphics_rectangle (my d_graphics.get(), 0.0, 1.0, 0.0, 1.0);
 }
 
-int structTimeSoundEditor :: v_click (double xbegin, double ybegin, bool shiftKeyPressed) {
+bool structTimeSoundEditor :: v_click (double xbegin, double ybegin, bool shiftKeyPressed) {
 	Sound sound = d_sound.data;
 	LongSound longSound = d_longSound.data;
-	if ((sound == NULL) != (longSound == NULL)) {
+	if (!! sound != !! longSound) {
 		ybegin = (ybegin - v_getBottomOfSoundArea ()) / (1.0 - v_getBottomOfSoundArea ());
 		int nchan = sound ? sound -> ny : longSound -> numberOfChannels;
 		if (nchan > 8) {
 			trace (xbegin, U" ", ybegin, U" ", nchan, U" ", d_sound.channelOffset);
 			if (xbegin >= d_endWindow && ybegin > 0.875 && ybegin <= 1.000 && d_sound.channelOffset > 0) {
 				d_sound.channelOffset -= 8;
-				return 1;
+				return FunctionEditor_UPDATE_NEEDED;
 			}
 			if (xbegin >= d_endWindow && ybegin > 0.000 && ybegin <= 0.125 && d_sound.channelOffset < nchan - 8) {
 				d_sound.channelOffset += 8;
-				return 1;
+				return FunctionEditor_UPDATE_NEEDED;
 			}
 		}
 	}
@@ -645,10 +632,10 @@ int structTimeSoundEditor :: v_click (double xbegin, double ybegin, bool shiftKe
 
 void TimeSoundEditor_init (TimeSoundEditor me, const char32 *title, Function data, Sampled sound, bool ownSound) {
 	my d_ownSound = ownSound;
-	if (sound != NULL) {
+	if (sound) {
 		if (ownSound) {
 			Melder_assert (Thing_isa (sound, classSound));
-			my d_sound.data = Data_copy ((Sound) sound);   // deep copy; ownership transferred
+			my d_sound.data = Data_copy ((Sound) sound).releaseToAmbiguousOwner();   // deep copy; ownership transferred
 			Matrix_getWindowExtrema (my d_sound.data, 1, my d_sound.data -> nx, 1, my d_sound.data -> ny, & my d_sound.minimum, & my d_sound.maximum);
 		} else if (Thing_isa (sound, classSound)) {
 			my d_sound.data = (Sound) sound;   // reference copy; ownership not transferred
