@@ -1,20 +1,19 @@
 /* Network.cpp
  *
- * Copyright (C) 2009-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 2009-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -98,17 +97,17 @@ void Network_init (Network me, double spreadingRate, enum kNetwork_activityClipp
 	my connections = NUMvector <structNetworkConnection> (1, numberOfConnections);
 }
 
-Network Network_create (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
+autoNetwork Network_create (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
 	double minimumActivity, double maximumActivity, double activityLeak,
 	double learningRate, double minimumWeight, double maximumWeight, double weightLeak,
 	double xmin, double xmax, double ymin, double ymax, long numberOfNodes, long numberOfConnections)
 {
 	try {
 		autoNetwork me = Thing_new (Network);
-		Network_init (me.peek(), spreadingRate, activityClippingRule, minimumActivity, maximumActivity, activityLeak,
+		Network_init (me.get(), spreadingRate, activityClippingRule, minimumActivity, maximumActivity, activityLeak,
 			learningRate, minimumWeight, maximumWeight, weightLeak,
 			xmin, xmax, ymin, ymax, numberOfNodes, numberOfConnections);
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Network not created.");
 	}
@@ -277,7 +276,7 @@ void Network_normalizeWeights (Network me, long nodeMin, long nodeMax, long node
 	}
 }
 
-Network Network_create_rectangle (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
+autoNetwork Network_create_rectangle (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
 	double minimumActivity, double maximumActivity, double activityLeak,
 	double learningRate, double minimumWeight, double maximumWeight, double weightLeak,
 	long numberOfRows, long numberOfColumns, bool bottomRowClamped,
@@ -321,13 +320,13 @@ Network Network_create_rectangle (double spreadingRate, enum kNetwork_activityCl
 			}
 		}
 		Melder_assert (iconn == my numberOfConnections);
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Rectangular network not created.");
 	}
 }
 
-Network Network_create_rectangle_vertical (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
+autoNetwork Network_create_rectangle_vertical (double spreadingRate, enum kNetwork_activityClippingRule activityClippingRule,
 	double minimumActivity, double maximumActivity, double activityLeak,
 	double learningRate, double minimumWeight, double maximumWeight, double weightLeak,
 	long numberOfRows, long numberOfColumns, bool bottomRowClamped,
@@ -364,7 +363,7 @@ Network Network_create_rectangle_vertical (double spreadingRate, enum kNetwork_a
 			}
 		}
 		Melder_assert (iconn == my numberOfConnections);
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Vertical rectangular network not created.");
 	}
@@ -482,7 +481,7 @@ void Network_setActivityClippingRule (Network me, enum kNetwork_activityClipping
 	Network_zeroActivities (me, 0, 0);
 }
 
-Table Network_nodes_downto_Table (Network me, long fromNodeNumber, long toNodeNumber,
+autoTable Network_nodes_downto_Table (Network me, long fromNodeNumber, long toNodeNumber,
 	bool includeNodeNumbers,
 	bool includeX, bool includeY, int positionDecimals,
 	bool includeClamped,
@@ -498,23 +497,23 @@ Table Network_nodes_downto_Table (Network me, long fromNodeNumber, long toNodeNu
 		autoTable thee = Table_createWithoutColumnNames (numberOfNodes,
 			includeNodeNumbers + includeX + includeY + includeClamped + includeActivity + includeExcitation);
 		long icol = 0;
-		if (includeNodeNumbers) Table_setColumnLabel (thee.peek(), ++ icol, U"node");
-		if (includeX)           Table_setColumnLabel (thee.peek(), ++ icol, U"x");
-		if (includeY)           Table_setColumnLabel (thee.peek(), ++ icol, U"y");
-		if (includeClamped)     Table_setColumnLabel (thee.peek(), ++ icol, U"clamped");
-		if (includeActivity)    Table_setColumnLabel (thee.peek(), ++ icol, U"activity");
-		if (includeExcitation)  Table_setColumnLabel (thee.peek(), ++ icol, U"excitation");
+		if (includeNodeNumbers) Table_setColumnLabel (thee.get(), ++ icol, U"node");
+		if (includeX)           Table_setColumnLabel (thee.get(), ++ icol, U"x");
+		if (includeY)           Table_setColumnLabel (thee.get(), ++ icol, U"y");
+		if (includeClamped)     Table_setColumnLabel (thee.get(), ++ icol, U"clamped");
+		if (includeActivity)    Table_setColumnLabel (thee.get(), ++ icol, U"activity");
+		if (includeExcitation)  Table_setColumnLabel (thee.get(), ++ icol, U"excitation");
 		for (long inode = fromNodeNumber; inode <= toNodeNumber; inode ++) {
 			NetworkNode node = & my nodes [inode];
 			icol = 0;
-			if (includeNodeNumbers) Table_setNumericValue (thee.peek(), inode, ++ icol, inode);
-			if (includeX)           Table_setStringValue  (thee.peek(), inode, ++ icol, Melder_fixed (node -> x, positionDecimals));
-			if (includeY)           Table_setStringValue  (thee.peek(), inode, ++ icol, Melder_fixed (node -> y, positionDecimals));
-			if (includeClamped)     Table_setNumericValue (thee.peek(), inode, ++ icol, node -> clamped);
-			if (includeActivity)    Table_setStringValue  (thee.peek(), inode, ++ icol, Melder_fixed (node -> activity,   activityDecimals));
-			if (includeExcitation)  Table_setStringValue  (thee.peek(), inode, ++ icol, Melder_fixed (node -> excitation, activityDecimals));
+			if (includeNodeNumbers) Table_setNumericValue (thee.get(), inode, ++ icol, inode);
+			if (includeX)           Table_setStringValue  (thee.get(), inode, ++ icol, Melder_fixed (node -> x, positionDecimals));
+			if (includeY)           Table_setStringValue  (thee.get(), inode, ++ icol, Melder_fixed (node -> y, positionDecimals));
+			if (includeClamped)     Table_setNumericValue (thee.get(), inode, ++ icol, node -> clamped);
+			if (includeActivity)    Table_setStringValue  (thee.get(), inode, ++ icol, Melder_fixed (node -> activity,   activityDecimals));
+			if (includeExcitation)  Table_setStringValue  (thee.get(), inode, ++ icol, Melder_fixed (node -> excitation, activityDecimals));
 		}
-		return thee.transfer();
+		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Table.");
 	}
@@ -529,7 +528,7 @@ void Network_listNodes (Network me, long fromNodeNumber, long toNodeNumber,
 	try {
 		autoTable table = Network_nodes_downto_Table (me, fromNodeNumber, toNodeNumber, includeNodeNumbers,
 			includeX, includeY, positionDecimals, includeClamped, includeActivity, includeExcitation, activityDecimals);
-		Table_list (table.peek(), false);
+		Table_list (table.get(), false);
 	} catch (MelderError) {
 		Melder_throw (me, U": not listed.");
 	}
