@@ -1,20 +1,19 @@
 /* DataEditor.cpp
  *
- * Copyright (C) 1995-2012,2015 Paul Boersma
+ * Copyright (C) 1995-2012,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define NAME_X  30
@@ -46,15 +45,15 @@ static int stringLengths [] = { 0,
 	15, 27, 35, 59,
 	33, 33, 8, 6, 60, 60 };
 
-static VectorEditor VectorEditor_create (DataEditor root, const char32 *title, void *address,
+static void VectorEditor_create (DataEditor root, const char32 *title, void *address,
 	Data_Description description, long minimum, long maximum);
 
-static MatrixEditor MatrixEditor_create (DataEditor root, const char32 *title, void *address,
+static void MatrixEditor_create (DataEditor root, const char32 *title, void *address,
 	Data_Description description, long min1, long max1, long min2, long max2);
 
-static StructEditor StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
+static void StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
 
-static ClassEditor ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
+static void ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
 
 static inline const char32 * strip_d (const char32 *s) {
 	return s && s [0] == U'd' && s [1] == U'_' ? & s [2] : & s [0];
@@ -64,13 +63,13 @@ static inline const char32 * strip_d (const char32 *s) {
 
 Thing_implement (DataSubEditor, Editor, 0);
 
-void structDataSubEditor :: v_destroy () {
+void structDataSubEditor :: v_destroy () noexcept {
 	for (int i = 1; i <= kDataSubEditor_MAXNUM_ROWS; i ++)
 		Melder_free (d_fieldData [i]. history);
-	if (d_root && d_root -> d_children)
-		for (int i = d_root -> d_children -> size; i > 0; i --)
-			if (d_root -> d_children -> item [i] == this)
-				Collection_subtractItem (d_root -> d_children, i);
+	if (our root)
+		for (int i = our root -> children.size; i > 0; i --)
+			if (our root -> children.at [i] == this)
+				our root -> children.subtractItem_ref (i);
 	DataSubEditor_Parent :: v_destroy ();
 }
 
@@ -107,9 +106,7 @@ static Data_Description DataSubEditor_findNumberUse (DataSubEditor me, const cha
 	return nullptr;
 }
 
-static void gui_button_cb_change (I, GuiButtonEvent event) {
-	(void) event;
-	iam (DataSubEditor);
+static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */) {
 	int irow = 1;
 	for (; irow <= kDataSubEditor_MAXNUM_ROWS; irow ++) {
 		#if motif
@@ -117,7 +114,7 @@ static void gui_button_cb_change (I, GuiButtonEvent event) {
 		#elif gtk
 			gboolean visible;
 			g_object_get (G_OBJECT (my d_fieldData [irow]. text), "visible", & visible, nullptr);
-		#elif defined (macintosh) && ! useCarbon
+		#elif defined (macintosh)
 			bool visible = ! [(GuiCocoaTextField *) my d_fieldData [irow]. text -> d_widget   isHidden];
 		#else
 			bool visible = false;
@@ -213,10 +210,10 @@ static void gui_button_cb_change (I, GuiButtonEvent event) {
 	 * 1. The owner (creator) of our root DataEditor: so that she can notify other editors, if any.
 	 * 2. All our sibling DataSubEditors.
 	 */
-	Editor_broadcastDataChanged (my d_root);
+	Editor_broadcastDataChanged (my root);
 	update (me);
-	for (int isub = 1; isub <= my d_root -> d_children -> size; isub ++) {
-		DataSubEditor subeditor = (DataSubEditor) my d_root -> d_children -> item [isub];
+	for (int isub = 1; isub <= my root -> children.size; isub ++) {
+		DataSubEditor subeditor = my root -> children.at [isub];
 		if (subeditor != me) update (subeditor);
 	}
 	return;
@@ -224,20 +221,16 @@ error:
 	Melder_flushError (U"Edit field \"", strip_d (my d_fieldData [irow]. description -> name), U"\" or click \"Cancel\".");
 }
 
-static void gui_button_cb_cancel (I, GuiButtonEvent event) {
-	(void) event;
-	iam (DataSubEditor);
+static void gui_button_cb_cancel (DataSubEditor me, GuiButtonEvent /* event */) {
 	update (me);
 }
 
-static void gui_cb_scroll (I, GuiScrollBarEvent event) {
-	iam (DataSubEditor);
+static void gui_cb_scroll (DataSubEditor me, GuiScrollBarEvent event) {
 	my d_topField = GuiScrollBar_getValue (event -> scrollBar) + 1;
 	update (me);
 }
 
-static void gui_button_cb_open (I, GuiButtonEvent event) {
-	iam (DataSubEditor);
+static void gui_button_cb_open (DataSubEditor me, GuiButtonEvent event) {
 	int ifield = 0;
 	static MelderString name { 0 };
 	MelderString_empty (& name);
@@ -259,27 +252,27 @@ static void gui_button_cb_open (I, GuiButtonEvent event) {
 	if (fieldData -> description -> rank == 1 || fieldData -> description -> rank == 3 || fieldData -> description -> rank < 0) {
 		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name),
 			U" [", fieldData -> minimum, U"..", fieldData -> maximum, U"]");
-		VectorEditor_create (my d_root, name.string, fieldData -> address,
+		VectorEditor_create (my root, name.string, fieldData -> address,
 			fieldData -> description, fieldData -> minimum, fieldData -> maximum);
 	} else if (fieldData -> description -> rank == 2) {
 		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name),
 			U" [", fieldData -> minimum, U"..", fieldData -> maximum, U"]");
 		MelderString_append (& name, U" [", fieldData -> min2, U"..", fieldData -> max2, U"]");
-		MatrixEditor_create (my d_root, name.string, fieldData -> address, fieldData -> description,
+		MatrixEditor_create (my root, name.string, fieldData -> address, fieldData -> description,
 			fieldData -> minimum, fieldData -> maximum, fieldData -> min2, fieldData -> max2);
 	} else if (fieldData -> description -> type == structwa) {
 		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name));
-		StructEditor_create (my d_root, name.string, fieldData -> address,
+		StructEditor_create (my root, name.string, fieldData -> address,
 			* (Data_Description *) fieldData -> description -> tagType);
 	} else if (fieldData -> description -> type == objectwa ||
 	           fieldData -> description -> type == autoobjectwa ||
-			   fieldData -> description -> type == collectionwa ||
+	           fieldData -> description -> type == collectionofwa ||
 			   fieldData -> description -> type == autocollectionwa) {
 		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name));
-		ClassEditor_create (my d_root, name.string, fieldData -> address,
+		ClassEditor_create (my root, name.string, fieldData -> address,
 			Class_getDescription ((ClassInfo) fieldData -> description -> tagType));
 	} else /*if (fieldData -> description -> type == inheritwa)*/ {
-		ClassEditor_create (my d_root, fieldData -> history, fieldData -> address,
+		ClassEditor_create (my root, fieldData -> history, fieldData -> address,
 			fieldData -> description);
 /*	} else {
 		Melder_casual (
@@ -317,7 +310,7 @@ void structDataSubEditor :: v_createChildren () {
 	}
 }
 
-static void menu_cb_help (EDITOR_ARGS) { EDITOR_IAM (DataSubEditor); Melder_help (U"Inspect"); }
+static void menu_cb_help (DataSubEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Inspect"); }
 
 void structDataSubEditor :: v_createHelpMenuItems (EditorMenu menu) {
 	DataSubEditor_Parent :: v_createHelpMenuItems (menu);
@@ -325,9 +318,9 @@ void structDataSubEditor :: v_createHelpMenuItems (EditorMenu menu) {
 }
 
 static void DataSubEditor_init (DataSubEditor me, DataEditor root, const char32 *title, void *address, Data_Description description) {
-	my d_root = root;
+	my root = root;
 	if (me != root) {
-		Collection_addItem (root -> d_children, me);
+		root -> children.addItem_ref (me);
 	}
 	my d_address = address;
 	my d_description = description;
@@ -477,8 +470,17 @@ static void showStructMember (
 		fieldData -> rank = 0;
 		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
-	} else if (type == objectwa || type == autoobjectwa || type == collectionwa || type == autocollectionwa) {
-		fieldData -> address = * (Daata *) memberAddress;   // indirect
+	} else if (type == objectwa || type == autoobjectwa || type == autocollectionwa) {
+		fieldData -> address = * (Daata *) memberAddress;   // indirect  // FIXME: not guaranteed for auto objects
+		if (! fieldData -> address) return;   // no button if no object
+		fieldData -> description = memberDescription;
+		fieldData -> rank = 0;
+		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		GuiThing_show (fieldData -> button);
+	} else if (type == collectionofwa) {
+		fieldData -> address = (Daata) memberAddress;   // direct  // FIXME: not guaranteed for auto objects
+		Melder_casual (U"Daata ", Melder_pointer (fieldData -> address));
+		Melder_casual (U"Class ", ((Daata) fieldData -> address) -> classInfo -> className);
 		if (! fieldData -> address) return;   // no button if no object
 		fieldData -> description = memberDescription;
 		fieldData -> rank = 0;
@@ -506,11 +508,11 @@ static void StructEditor_init (StructEditor me, DataEditor root, const char32 *t
 	DataSubEditor_init (me, root, title, address, description);
 }
 
-static StructEditor StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
 	try {
 		autoStructEditor me = Thing_new (StructEditor);
-		StructEditor_init (me.peek(), root, title, address, description);
-		return me.transfer();
+		StructEditor_init (me.get(), root, title, address, description);
+		return me.releaseToUser();
 	} catch (MelderError) {
 		Melder_throw (U"Struct inspector window not created.");
 	}
@@ -609,15 +611,15 @@ void structVectorEditor :: v_showMembers () {
 	}
 }
 
-static VectorEditor VectorEditor_create (DataEditor root, const char32 *title, void *address,
+static void VectorEditor_create (DataEditor root, const char32 *title, void *address,
 	Data_Description description, long minimum, long maximum)
 {
 	try {
 		autoVectorEditor me = Thing_new (VectorEditor);
 		my d_minimum = minimum;
 		my d_maximum = maximum;
-		DataSubEditor_init (me.peek(), root, title, address, description);
-		return me.transfer();
+		DataSubEditor_init (me.get(), root, title, address, description);
+		return me.releaseToUser();
 	} catch (MelderError) {
 		Melder_throw (U"Vector inspector window not created.");
 	}
@@ -668,7 +670,7 @@ void structMatrixEditor :: v_showMembers () {
 	}
 }
 
-static MatrixEditor MatrixEditor_create (DataEditor root, const char32 *title, void *address,
+static void MatrixEditor_create (DataEditor root, const char32 *title, void *address,
 	Data_Description description, long min1, long max1, long min2, long max2)
 {
 	try {
@@ -677,8 +679,8 @@ static MatrixEditor MatrixEditor_create (DataEditor root, const char32 *title, v
 		my d_maximum = max1;
 		my d_min2 = min2;
 		my d_max2 = max2;
-		DataSubEditor_init (me.peek(), root, title, address, description);
-		return me.transfer();
+		DataSubEditor_init (me.get(), root, title, address, description);
+		return me.releaseToUser();
 	} catch (MelderError) {
 		Melder_throw (U"Matrix inspector window not created.");
 	}
@@ -712,11 +714,11 @@ static void ClassEditor_init (ClassEditor me, DataEditor root, const char32 *tit
 	StructEditor_init (me, root, title, address, description);
 }
 
-static ClassEditor ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
 	try {
 		autoClassEditor me = Thing_new (ClassEditor);
-		ClassEditor_init (me.peek(), root, title, address, description);
-		return me.transfer();
+		ClassEditor_init (me.get(), root, title, address, description);
+		return me.releaseToUser();
 	} catch (MelderError) {
 		Melder_throw (U"Class inspector window not created.");
 	}
@@ -726,45 +728,84 @@ static ClassEditor ClassEditor_create (DataEditor root, const char32 *title, voi
 
 Thing_implement (DataEditor, ClassEditor, 0);
 
-void structDataEditor :: v_destroy () {
+static void DataEditor_destroyAllChildren (DataEditor me) {
+	/*
+		To destroy all children, we travel them from youngest to oldest,
+		because the array of children will change from under us:
+	*/
+	for (int i = my children.size; i >= 1; i --) {
+		/*
+			An optimization coming!
+			
+			Instead of
+				DataSubEditor child = my children [i];
+				forget (child);
+			we isolate the child from the parent before destroying the child,
+			so that the child won't try to remove the reference
+			that the parent has to her.
+			So first we make the parent forget the moribund child,
+			which prevents a dangling pointer:
+		*/
+		DataSubEditor child = my children.subtractItem_ref (i);
+		/*
+			That was fast, because subtracting the last item involves no shifting
+			of the remaining items.
 
-	/* Tell my children not to notify me when they die. */
-
-	for (int i = 1; i <= d_children -> size; i ++) {
-		DataSubEditor child = (DataSubEditor) d_children -> item [i];
-		child -> d_root = nullptr;
+			Second, we make the child forget the parent,
+			so that the child won't try to remove the reference
+			that the parent had to her (but no longer has):
+		*/
+		child -> root = nullptr;
+		/*
+			The child is now fully isolated, so we are ready to destroy her:
+		*/
+		forget (child);
+		/*
+			This procedure was an optimization because if we just destroyed each child,
+			each child would remove itself from the array by (1) searching for itself
+			and (2) shifting the remaining children, both of which have a complexity
+			that is linear in the number of children. So we would end up with quadratic complexity,
+			whereas the procedure that we use above has linear complexity.
+			
+			This linear complexity makes this procedure good enough for `v_destroy()`
+			(where obtaining linear complexity would have been easy anyway),
+			and nice enough for `v_dataChanged()`.
+			
+			Something to note is that this procedure doesn't care whether the autoCollection
+			`children` owns its items or not.
+		*/
 	}
+}
 
-	forget (d_children);
+void structDataEditor :: v_destroy () noexcept {
+	DataEditor_destroyAllChildren (this);
 	DataEditor_Parent :: v_destroy ();
 }
 
 void structDataEditor :: v_dataChanged () {
 	/*
-	 * Someone else changed our data.
-	 * We know that the top-level data is still accessible.
-	 */
+		Someone else changed our data.
+		We know that the top-level data is still accessible,
+		so we update the top-level window to show the change:
+	*/
 	update (this);
 	/*
-	 * But all structure may have changed,
-	 * so that we do not know if any of the subeditors contain valid data.
-	 */
-	for (int i = d_children -> size; i >= 1; i --) {
-		DataSubEditor subeditor = (DataSubEditor) d_children -> item [i];
-		Collection_subtractItem (d_children, i);
-		forget (subeditor);
-	}
+		Changing the data may have changed any part of the *structure* of the data,
+		so we do not know if the data visible in any of the subeditors is still valid.
+		We follow the most straightforward solution, which is to simply close all the child windows,
+		which guarantees the removal of all dangling visual representations:
+	*/
+	DataEditor_destroyAllChildren (this);
 }
 
-DataEditor DataEditor_create (const char32 *title, Daata data) {
+autoDataEditor DataEditor_create (const char32 *title, Daata data) {
 	try {
 		ClassInfo klas = data -> classInfo;
 		if (Class_getDescription (klas) == nullptr)
 			Melder_throw (U"Class ", klas -> className, U" cannot be inspected.");
 		autoDataEditor me = Thing_new (DataEditor);
-		my d_children = Collection_create (classDataSubEditor, 10);
-		ClassEditor_init (me.peek(), me.peek(), title, data, Class_getDescription (klas));
-		return me.transfer();
+		ClassEditor_init (me.get(), me.get(), title, data, Class_getDescription (klas));
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Inspector window not created.");
 	}
