@@ -1,20 +1,19 @@
 /* Editor.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <time.h>
@@ -42,10 +41,9 @@ Thing_implement (Editor, Thing, 0);
 
 Thing_implement (EditorCommand, Thing, 0);
 
-void structEditorCommand :: v_destroy () {
+void structEditorCommand :: v_destroy () noexcept {
 	Melder_free (our itemTitle);
 	Melder_free (our script);
-	forget (our d_uiform);
 	EditorCommand_Parent :: v_destroy ();
 }
 
@@ -53,16 +51,14 @@ void structEditorCommand :: v_destroy () {
 
 Thing_implement (EditorMenu, Thing, 0);
 
-void structEditorMenu :: v_destroy () {
+void structEditorMenu :: v_destroy () noexcept {
 	Melder_free (our menuTitle);
-	forget (our commands);
 	EditorMenu_Parent :: v_destroy ();
 }
 
 /********** functions **********/
 
-static void commonCallback (GUI_ARGS) {
-	GUI_IAM (EditorCommand);
+static void commonCallback (EditorCommand me, GuiMenuItemEvent /* event */) {
 	if (my d_editor && my d_editor -> v_scriptable () && ! str32str (my itemTitle, U"...")) {
 		UiHistory_write (U"\n");
 		UiHistory_write_colonize (my itemTitle);
@@ -77,20 +73,19 @@ static void commonCallback (GUI_ARGS) {
 	}
 }
 
-GuiMenuItem EditorMenu_addCommand (EditorMenu me, const char32 *itemTitle /* cattable */, long flags,
-	void (*commandCallback) (Editor me, EditorCommand cmd, UiForm sendingForm, int narg, Stackel args, const char32 *sendingString, Interpreter interpreter))
+GuiMenuItem EditorMenu_addCommand (EditorMenu me, const char32 *itemTitle /* cattable */, long flags, EditorCommandCallback commandCallback)
 {
 	autoEditorCommand thee = Thing_new (EditorCommand);
 	thy d_editor = my d_editor;
 	thy menu = me;
 	thy itemTitle = Melder_dup (itemTitle);
 	thy itemWidget =
-		commandCallback == nullptr ? GuiMenu_addSeparator (my menuWidget) :
+		! commandCallback ? GuiMenu_addSeparator (my menuWidget) :
 		flags & Editor_HIDDEN ? nullptr :
-		GuiMenu_addItem (my menuWidget, itemTitle, flags, commonCallback, thee.peek());   // DANGLE BUG: me can be killed by Collection_addItem(), but EditorCommand::destroy doesn't remove the item
+		GuiMenu_addItem (my menuWidget, itemTitle, flags, commonCallback, thee.get());   // DANGLE BUG: me can be killed by Collection_addItem(), but EditorCommand::destroy doesn't remove the item
 	thy commandCallback = commandCallback;
 	GuiMenuItem result = thy itemWidget;
-	Collection_addItem (my commands, thee.transfer());
+	my commands. addItem_move (thee.move());
 	return result;
 }
 
@@ -101,21 +96,17 @@ EditorMenu Editor_addMenu (Editor me, const char32 *menuTitle, long flags) {
 	thy d_editor = me;
 	thy menuTitle = Melder_dup (menuTitle);
 	thy menuWidget = GuiMenu_createInWindow (my d_windowForm, menuTitle, flags);
-	thy commands = Ordered_create ();
-	EditorMenu result = thee.peek();
-	Collection_addItem (my menus, thee.transfer());
-	return result;
+	return my menus. addItem_move (thee.move());
 }
 
 /*GuiObject EditorMenu_getMenuWidget (EditorMenu me) { return my menuWidget; }*/
 
-GuiMenuItem Editor_addCommand (Editor me, const char32 *menuTitle, const char32 *itemTitle, long flags,
-	void (*commandCallback) (Editor me, EditorCommand cmd, UiForm sendingForm, int narg, Stackel args, const char32 *sendingString, Interpreter interpreter))
+GuiMenuItem Editor_addCommand (Editor me, const char32 *menuTitle, const char32 *itemTitle, long flags, EditorCommandCallback commandCallback)
 {
 	try {
-		long numberOfMenus = my menus -> size;
+		long numberOfMenus = my menus.size;
 		for (long imenu = 1; imenu <= numberOfMenus; imenu ++) {
-			EditorMenu menu = (EditorMenu) my menus -> item [imenu];
+			EditorMenu menu = my menus.at [imenu];
 			if (str32equ (menuTitle, menu -> menuTitle))
 				return EditorMenu_addCommand (menu, itemTitle, flags, commandCallback);
 		}
@@ -125,28 +116,25 @@ GuiMenuItem Editor_addCommand (Editor me, const char32 *menuTitle, const char32
 	}
 }
 
-static void Editor_scriptCallback (Editor me, EditorCommand cmd, UiForm sendingForm, int narg, Stackel args, const char32 *sendingString, Interpreter interpreter) {
-	(void) sendingForm;
-	(void) narg;
-	(void) args;
-	(void) sendingString;
-	(void) interpreter;
+static void Editor_scriptCallback (Editor me, EditorCommand cmd, UiForm /* sendingForm */,
+	int /* narg */, Stackel /* args */, const char32 * /* sendingString */, Interpreter /* interpreter */)
+{
 	DO_RunTheScriptFromAnyAddedEditorCommand (me, cmd -> script);
 }
 
 GuiMenuItem Editor_addCommandScript (Editor me, const char32 *menuTitle, const char32 *itemTitle, long flags,
 	const char32 *script)
 {
-	long numberOfMenus = my menus -> size;
+	long numberOfMenus = my menus.size;
 	for (long imenu = 1; imenu <= numberOfMenus; imenu ++) {
-		EditorMenu menu = (EditorMenu) my menus -> item [imenu];
+		EditorMenu menu = my menus.at [imenu];
 		if (str32equ (menuTitle, menu -> menuTitle)) {
 			autoEditorCommand cmd = Thing_new (EditorCommand);
 			cmd -> d_editor = me;
 			cmd -> menu = menu;
 			cmd -> itemTitle = Melder_dup_f (itemTitle);
 			cmd -> itemWidget = script == nullptr ? GuiMenu_addSeparator (menu -> menuWidget) :
-				GuiMenu_addItem (menu -> menuWidget, itemTitle, flags, commonCallback, cmd.peek());   // DANGLE BUG
+				GuiMenu_addItem (menu -> menuWidget, itemTitle, flags, commonCallback, cmd.get());   // DANGLE BUG
 			cmd -> commandCallback = Editor_scriptCallback;
 			if (str32len (script) == 0) {
 				cmd -> script = Melder_dup_f (U"");
@@ -156,7 +144,7 @@ GuiMenuItem Editor_addCommandScript (Editor me, const char32 *menuTitle, const c
 				cmd -> script = Melder_dup_f (Melder_fileToPath (& file));
 			}
 			GuiMenuItem result = cmd -> itemWidget;
-			Collection_addItem (menu -> commands, cmd.transfer());
+			menu -> commands. addItem_move (cmd.move());
 			return result;
 		}
 	}
@@ -169,9 +157,9 @@ GuiMenuItem Editor_addCommandScript (Editor me, const char32 *menuTitle, const c
 }
 
 void Editor_setMenuSensitive (Editor me, const char32 *menuTitle, int sensitive) {
-	int numberOfMenus = my menus -> size;
+	int numberOfMenus = my menus.size;
 	for (int imenu = 1; imenu <= numberOfMenus; imenu ++) {
-		EditorMenu menu = (EditorMenu) my menus -> item [imenu];
+		EditorMenu menu = my menus.at [imenu];
 		if (str32equ (menuTitle, menu -> menuTitle)) {
 			GuiThing_setSensitive (menu -> menuWidget, sensitive);
 			return;
@@ -180,13 +168,13 @@ void Editor_setMenuSensitive (Editor me, const char32 *menuTitle, int sensitive)
 }
 
 EditorCommand Editor_getMenuCommand (Editor me, const char32 *menuTitle, const char32 *itemTitle) {
-	int numberOfMenus = my menus -> size;
+	int numberOfMenus = my menus.size;
 	for (int imenu = 1; imenu <= numberOfMenus; imenu ++) {
-		EditorMenu menu = (EditorMenu) my menus -> item [imenu];
+		EditorMenu menu = my menus.at [imenu];
 		if (str32equ (menuTitle, menu -> menuTitle)) {
-			int numberOfCommands = menu -> commands -> size, icommand;
+			int numberOfCommands = menu -> commands.size, icommand;
 			for (icommand = 1; icommand <= numberOfCommands; icommand ++) {
-				EditorCommand command = (EditorCommand) menu -> commands -> item [icommand];
+				EditorCommand command = menu -> commands.at [icommand];
 				if (str32equ (itemTitle, command -> itemTitle))
 					return command;
 			}
@@ -196,12 +184,12 @@ EditorCommand Editor_getMenuCommand (Editor me, const char32 *menuTitle, const c
 }
 
 void Editor_doMenuCommand (Editor me, const char32 *commandTitle, int narg, Stackel args, const char32 *arguments, Interpreter interpreter) {
-	int numberOfMenus = my menus -> size;
+	int numberOfMenus = my menus.size;
 	for (int imenu = 1; imenu <= numberOfMenus; imenu ++) {
-		EditorMenu menu = (EditorMenu) my menus -> item [imenu];
-		long numberOfCommands = menu -> commands -> size;
+		EditorMenu menu = my menus.at [imenu];
+		long numberOfCommands = menu -> commands.size;
 		for (long icommand = 1; icommand <= numberOfCommands; icommand ++) {
-			EditorCommand command = (EditorCommand) menu -> commands -> item [icommand];
+			EditorCommand command = menu -> commands.at [icommand];
 			if (str32equ (commandTitle, command -> itemTitle)) {
 				command -> commandCallback (me, command, nullptr, narg, args, arguments, interpreter);
 				return;
@@ -213,14 +201,15 @@ void Editor_doMenuCommand (Editor me, const char32 *commandTitle, int narg, Stac
 
 /********** class Editor **********/
 
-void structEditor :: v_destroy () {
+void structEditor :: v_destroy () noexcept {
 	trace (U"enter");
 	MelderAudio_stopPlaying (MelderAudio_IMPLICIT);
 	/*
 	 * The following command must be performed before the shell is destroyed.
 	 * Otherwise, we would be forgetting dangling command dialogs here.
 	 */
-	forget (our menus);
+	our menus.removeAllItems();
+
 	Editor_broadcastDestruction (this);
 	if (our d_windowForm) {
 		#if gtk
@@ -229,9 +218,9 @@ void structEditor :: v_destroy () {
 				gtk_widget_destroy (GTK_WIDGET (our d_windowForm -> d_gtkWindow));
 			}
 		#elif cocoa
-			if (our d_windowForm -> d_cocoaWindow) {
-				NSWindow *cocoaWindow = our d_windowForm -> d_cocoaWindow;
-				//d_windowForm -> d_cocoaWindow = nullptr;
+			if (our d_windowForm -> d_cocoaShell) {
+				NSWindow *cocoaWindow = our d_windowForm -> d_cocoaShell;
+				//d_windowForm -> d_cocoaShell = nullptr;
 				[cocoaWindow close];
 			}
 		#elif motif
@@ -240,7 +229,6 @@ void structEditor :: v_destroy () {
 			}
 		#endif
 	}
-	forget (our previousData);
 	if (our d_ownData) forget (our data);
 	Melder_free (our callbackSocket);
 	Editor_Parent :: v_destroy ();
@@ -264,17 +252,15 @@ void structEditor :: v_nameChanged () {
 
 void structEditor :: v_saveData () {
 	if (! our data) return;
-	forget (our previousData);
 	our previousData = Data_copy (our data);
 }
 
 void structEditor :: v_restoreData () {
 	if (our data && our previousData)
-		Thing_swap (our data, our previousData);
+		Thing_swap (our data, our previousData.get());
 }
 
-static void menu_cb_sendBackToCallingProgram (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_sendBackToCallingProgram (Editor me, EDITOR_ARGS_DIRECT) {
 	if (my data) {
 		extern structMelderDir praatDir;
 		structMelderFile file = { 0 };
@@ -285,13 +271,11 @@ static void menu_cb_sendBackToCallingProgram (EDITOR_ARGS) {
 	my v_goAway ();
 }
 
-static void menu_cb_close (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_close (Editor me, EDITOR_ARGS_DIRECT) {
 	my v_goAway ();
 }
 
-static void menu_cb_undo (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_undo (Editor me, EDITOR_ARGS_DIRECT) {
 	my v_restoreData ();
 	if (str32nequ (my undoText, U"Undo", 4)) my undoText [0] = U'R', my undoText [1] = U'e';
 	else if (str32nequ (my undoText, U"Redo", 4)) my undoText [0] = U'U', my undoText [1] = U'n';
@@ -314,24 +298,22 @@ static void menu_cb_undo (EDITOR_ARGS) {
 	Editor_broadcastDataChanged (me);
 }
 
-static void menu_cb_searchManual (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_searchManual (Editor /* me */, EDITOR_ARGS_DIRECT) {
 	Melder_search ();
 }
 
-static void menu_cb_newScript (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
-	(void) ScriptEditor_createFromText (me, nullptr);
-}
-
-static void menu_cb_openScript (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_newScript (Editor me, EDITOR_ARGS_DIRECT) {
 	autoScriptEditor scriptEditor = ScriptEditor_createFromText (me, nullptr);
-	TextEditor_showOpen (scriptEditor.transfer());
+	scriptEditor.releaseToUser();
 }
 
-void structEditor :: v_createMenuItems_file (EditorMenu menu) {
-	(void) menu;
+static void menu_cb_openScript (Editor me, EDITOR_ARGS_DIRECT) {
+	autoScriptEditor scriptEditor = ScriptEditor_createFromText (me, nullptr);
+	TextEditor_showOpen (scriptEditor.get());
+	scriptEditor.releaseToUser();
+}
+
+void structEditor :: v_createMenuItems_file (EditorMenu /* menu */) {
 }
 
 void structEditor :: v_createMenuItems_edit (EditorMenu menu) {
@@ -339,13 +321,11 @@ void structEditor :: v_createMenuItems_edit (EditorMenu menu) {
 		our undoButton = EditorMenu_addCommand (menu, U"Cannot undo", GuiMenu_INSENSITIVE + 'Z', menu_cb_undo);
 }
 
-static void menu_cb_settingsReport (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_settingsReport (Editor me, EDITOR_ARGS_DIRECT) {
 	Thing_info (me);
 }
 
-static void menu_cb_info (EDITOR_ARGS) {
-	EDITOR_IAM (Editor);
+static void menu_cb_info (Editor me, EDITOR_ARGS_DIRECT) {
 	if (my data) Thing_info (my data);
 }
 
@@ -376,7 +356,7 @@ void structEditor :: v_createMenus () {
 
 void structEditor :: v_form_pictureWindow (EditorCommand cmd) {
 	LABEL (U"", U"Picture window:")
-	BOOLEAN (U"Erase first", 1);
+	BOOLEAN (U"Erase first", true);
 }
 void structEditor :: v_ok_pictureWindow (EditorCommand cmd) {
 	SET_INTEGER (U"Erase first", pref_picture_eraseFirst ());
@@ -386,7 +366,7 @@ void structEditor :: v_do_pictureWindow (EditorCommand cmd) {
 }
 
 void structEditor :: v_form_pictureMargins (EditorCommand cmd) {
-	Any radio = 0;
+	UiField radio;
 	LABEL (U"", U"Margins:")
 	OPTIONMENU_ENUM (U"Write name at top", kEditor_writeNameAtTop, kEditor_writeNameAtTop_DEFAULT);
 }
@@ -397,9 +377,8 @@ void structEditor :: v_do_pictureMargins (EditorCommand cmd) {
 	pref_picture_writeNameAtTop () = GET_ENUM (kEditor_writeNameAtTop, U"Write name at top");
 }
 
-static void gui_window_cb_goAway (I) {
-	iam (Editor);
-	Melder_assert (me != nullptr);
+static void gui_window_cb_goAway (Editor me) {
+	Melder_assert (me);
 	Melder_assert (Thing_isa (me, classEditor));
 	my v_goAway ();
 }
@@ -480,7 +459,6 @@ void Editor_init (Editor me, int x, int y, int width, int height, const char32 *
 	/* Create menus. */
 
 	if (my v_hasMenuBar ()) {
-		my menus = Ordered_create ();
 		GuiWindow_addMenuBar (my d_windowForm);
 	}
 
