@@ -2,21 +2,20 @@
 #define _Gui_h_
 /* Gui.h
  *
- * Copyright (C) 1993-2011,2012,2013,2014,2015 Paul Boersma, 2013 Tom Naughton
+ * Copyright (C) 1993-2011,2012,2013,2014,2015,2016 Paul Boersma, 2013 Tom Naughton
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -35,15 +34,9 @@
 	#define motif 1
 	#define cocoa 0
 #elif defined (macintosh)
-	#if useCarbon
-		#define gtk 0
-		#define motif 1
-		#define cocoa 0
-	#else
-		#define gtk 0
-		#define motif 0
-		#define cocoa 1
-	#endif
+	#define gtk 0
+	#define motif 0
+	#define cocoa 1
 #endif
 
 #include "Collection.h"
@@ -54,9 +47,6 @@
 	#include <cairo/cairo.h>
 #elif defined (macintosh)
 	#include "macport_on.h"
-    #if useCarbon
-        #include <Carbon/Carbon.h>
-    #endif
     #include <Cocoa/Cocoa.h>
 	#include "macport_off.h"
 #elif defined (_WIN32)
@@ -66,10 +56,6 @@
 	#include "winport_off.h"
 #endif
 
-#define GUI_ARGS  void *void_me, GuiMenuItemEvent event
-
-#define GUI_IAM(klas)  (void) void_me; (void) event; iam (klas);
-
 #define Gui_LEFT_DIALOG_SPACING  20
 #define Gui_RIGHT_DIALOG_SPACING  20
 #define Gui_TOP_DIALOG_SPACING  14
@@ -105,7 +91,7 @@
 #elif cocoa
 	Thing_declare (GuiThing);
 	@protocol GuiCocoaAny
-		- (GuiThing) userData;
+		- (GuiThing) getUserData;
 		- (void) setUserData: (GuiThing) userData;
 	@end
 	typedef NSObject <GuiCocoaAny> *GuiObject;
@@ -130,20 +116,16 @@
 		- (void) magnifyBy: (double) step;
 	@end
 	@interface GuiCocoaScrolledWindow : NSScrollView <GuiCocoaAny> @end
+	@interface GuiCocoaShell : NSWindow <GuiCocoaAny> @end
 	@interface GuiCocoaTextField : NSTextField <GuiCocoaAny> @end
 	@interface GuiCocoaTextView : NSTextView <GuiCocoaAny, NSTextViewDelegate> @end
-	@interface GuiCocoaWindow : NSWindow <GuiCocoaAny> @end
 #elif motif
-	typedef class structGuiObject *GuiObject;   // Opaque
+	typedef class structGuiObject *GuiObject;   // opaque
 
 	/*
 	 * Definitions of X11 types.
 	 */
-	#if defined (macintosh)
-		typedef struct EventRecord XEvent;
-	#else
-		typedef MSG XEvent;
-	#endif
+	typedef MSG XEvent;
 	typedef unsigned char Boolean;
 	typedef long Cardinal;
 	typedef unsigned int Dimension;
@@ -291,19 +273,9 @@
 	#define XmToggleButtonGetState XmToggleButtonGadgetGetState
 	void XmToggleButtonGadgetSetState (GuiObject widget, Boolean value, Boolean notify);
 	#define XmToggleButtonSetState XmToggleButtonGadgetSetState
-	void XmUpdateDisplay (GuiObject dummy);
 
-	#if defined (macintosh)
-		void motif_mac_defaultFont ();
-		void GuiMac_clipOn (GuiObject widget);   /* Clip to the inner area of a drawingArea (for drawing);
-			used by graphics drivers for Macintosh (clipping is automatic for Xwindows). */
-		int GuiMac_clipOn_graphicsContext (GuiObject me, void *graphicsContext);
-		void GuiMac_clipOff ();
-		void motif_mac_setUserMessageCallback (int (*userMessageCallback) (char32 *message));
-	#elif defined (_WIN32)
-		bool motif_win_mouseStillDown ();
-		void motif_win_setUserMessageCallback (int (*userMessageCallback) (void));
-	#endif
+	bool motif_win_mouseStillDown ();
+	void motif_win_setUserMessageCallback (int (*userMessageCallback) (void));
 #else
 	typedef void *GuiObject;
 #endif
@@ -322,7 +294,7 @@ Thing_define (GuiThing, Thing) {
 	GuiThing d_parent;
 	GuiObject d_widget;
 
-	void v_destroy ()
+	void v_destroy () noexcept
 		override;
 
 	virtual void v_show ();
@@ -352,19 +324,21 @@ void GuiControl_setSize (GuiControl me, int width, int height);
 Thing_define (GuiForm, GuiControl) {
 };
 
+typedef MelderCallback <void, structThing /* boss */> GuiShell_GoAwayCallback;
+
 Thing_define (GuiShell, GuiForm) {
 	int d_width, d_height;
 	#if gtk
 		GtkWindow *d_gtkWindow;
 	#elif cocoa
-		GuiCocoaWindow *d_cocoaWindow;
+		GuiCocoaShell *d_cocoaShell;
 	#elif motif
 		GuiObject d_xmShell;
 	#endif
-	void (*d_goAwayCallback) (void *boss);
-	void *d_goAwayBoss;
+	GuiShell_GoAwayCallback d_goAwayCallback;
+	Thing d_goAwayBoss;
 
-	void v_destroy ()
+	void v_destroy () noexcept
 		override;
 };
 
@@ -382,9 +356,11 @@ typedef struct structGuiButtonEvent {
 	bool shiftKeyPressed, commandKeyPressed, optionKeyPressed, extraControlKeyPressed;
 } *GuiButtonEvent;
 
+typedef MelderCallback <void, structThing /* boss */, GuiButtonEvent> GuiButton_ActivateCallback;
+
 Thing_define (GuiButton, GuiControl) {
-	void (*d_activateCallback) (void *boss, GuiButtonEvent event);
-	void *d_activateBoss;
+	GuiButton_ActivateCallback d_activateCallback;
+	Thing d_activateBoss;
 	GuiMenu d_menu;   // for cascade buttons
 };
 
@@ -396,12 +372,12 @@ Thing_define (GuiButton, GuiControl) {
 GuiButton GuiButton_create      (GuiForm parent,
 	int left, int right, int top, int bottom,
 	const char32 *text,
-	void (*activateCallback) (void *boss, GuiButtonEvent event), void *boss,
+	GuiButton_ActivateCallback activateCallback, Thing boss,
 	uint32 flags);
 GuiButton GuiButton_createShown (GuiForm parent,
 	int left, int right, int top, int bottom,
 	const char32 *text,
-	void (*activateCallback) (void *boss, GuiButtonEvent event), void *boss,
+	GuiButton_ActivateCallback activateCallback, Thing boss,
 	uint32 flags);
 
 void GuiButton_setText (GuiButton me, const char32 *text /* cattable */);
@@ -414,9 +390,11 @@ typedef struct structGuiCheckButtonEvent {
 	GuiCheckButton toggle;
 } *GuiCheckButtonEvent;
 
+typedef MelderCallback <void, structThing /* boss */, GuiCheckButtonEvent> GuiCheckButton_ValueChangedCallback;
+
 Thing_define (GuiCheckButton, GuiControl) {
-	void (*d_valueChangedCallback) (void *boss, GuiCheckButtonEvent event);
-	void *d_valueChangedBoss;
+	GuiCheckButton_ValueChangedCallback d_valueChangedCallback;
+	Thing d_valueChangedBoss;
 };
 
 /* GuiCheckButton creation flags: */
@@ -425,12 +403,12 @@ Thing_define (GuiCheckButton, GuiControl) {
 GuiCheckButton GuiCheckButton_create      (GuiForm parent,
 	int left, int right, int top, int bottom,
 	const char32 *text,
-	void (*valueChangedCallback) (void *boss, GuiCheckButtonEvent event), void *boss,
+	GuiCheckButton_ValueChangedCallback valueChangedCallback, Thing boss,
 	uint32 flags);
 GuiCheckButton GuiCheckButton_createShown (GuiForm parent,
 	int left, int right, int top, int bottom,
 	const char32 *text,
-	void (*valueChangedCallback) (void *boss, GuiCheckButtonEvent event), void *boss,
+	GuiCheckButton_ValueChangedCallback valueChangedCallback, Thing boss,
 	uint32 flags);
 
 bool GuiCheckButton_getValue (GuiCheckButton me);
@@ -446,7 +424,7 @@ Thing_define (GuiDialog, GuiShell) {
 GuiDialog GuiDialog_create (GuiWindow parent,
 	int x, int y, int width, int height,
 	const char32 *title,
-	void (*goAwayCallback) (void *goAwayBoss), void *goAwayBoss,
+	GuiShell_GoAwayCallback goAwayCallback, Thing goAwayBoss,
 	uint32 flags);
 
 /********** GuiDrawingArea **********/
@@ -454,73 +432,78 @@ GuiDialog GuiDialog_create (GuiWindow parent,
 Thing_declare (GuiDrawingArea);
 Thing_declare (GuiScrollBar);
 
-typedef struct structGuiDrawingAreaExposeEvent {
+typedef struct structGuiDrawingArea_ExposeEvent {
 	GuiDrawingArea widget;
 	int x, y, width, height;
-} *GuiDrawingAreaExposeEvent;
-typedef struct structGuiDrawingAreaClickEvent {
+} *GuiDrawingArea_ExposeEvent;
+typedef struct structGuiDrawingArea_ClickEvent {
 	GuiDrawingArea widget;
 	int x, y;
 	bool shiftKeyPressed, commandKeyPressed, optionKeyPressed, extraControlKeyPressed;
 	int button;
-} *GuiDrawingAreaClickEvent;
-typedef struct structGuiDrawingAreaKeyEvent {
+} *GuiDrawingArea_ClickEvent;
+typedef struct structGuiDrawingArea_KeyEvent {
 	GuiDrawingArea widget;
 	char32 key;
 	bool shiftKeyPressed, commandKeyPressed, optionKeyPressed, extraControlKeyPressed;
-} *GuiDrawingAreaKeyEvent;
-typedef struct structGuiDrawingAreaResizeEvent {
+} *GuiDrawingArea_KeyEvent;
+typedef struct structGuiDrawingArea_ResizeEvent {
 	GuiDrawingArea widget;
 	int width, height;
-} *GuiDrawingAreaResizeEvent;
+} *GuiDrawingArea_ResizeEvent;
+
+typedef MelderCallback <void, structThing /* boss */, GuiDrawingArea_ExposeEvent> GuiDrawingArea_ExposeCallback;
+typedef MelderCallback <void, structThing /* boss */, GuiDrawingArea_ClickEvent > GuiDrawingArea_ClickCallback;
+typedef MelderCallback <void, structThing /* boss */, GuiDrawingArea_KeyEvent   > GuiDrawingArea_KeyCallback;
+typedef MelderCallback <void, structThing /* boss */, GuiDrawingArea_ResizeEvent> GuiDrawingArea_ResizeCallback;
 
 Thing_define (GuiDrawingArea, GuiControl) {
 	GuiScrollBar d_horizontalScrollBar, d_verticalScrollBar;   // for swiping
-	void (*d_exposeCallback) (void *boss, GuiDrawingAreaExposeEvent event);
-	void *d_exposeBoss;
-	void (*d_clickCallback) (void *boss, GuiDrawingAreaClickEvent event);
-	void *d_clickBoss;
-	void (*d_keyCallback) (void *boss, GuiDrawingAreaKeyEvent event);
-	void *d_keyBoss;
-	void (*d_resizeCallback) (void *boss, GuiDrawingAreaResizeEvent event);
-	void *d_resizeBoss;
+	GuiDrawingArea_ExposeCallback d_exposeCallback;
+	Thing d_exposeBoss;
+	GuiDrawingArea_ClickCallback d_clickCallback;
+	Thing d_clickBoss;
+	GuiDrawingArea_KeyCallback d_keyCallback;
+	Thing d_keyBoss;
+	GuiDrawingArea_ResizeCallback d_resizeCallback;
+	Thing d_resizeBoss;
 };
 
 /* GuiDrawingArea creation flags: */
 #define GuiDrawingArea_BORDER  1
 GuiDrawingArea GuiDrawingArea_create (GuiForm parent, int left, int right, int top, int bottom,
-	void (*exposeCallback) (void *boss, GuiDrawingAreaExposeEvent event),
-	void (*clickCallback) (void *boss, GuiDrawingAreaClickEvent event),
-	void (*keyCallback) (void *boss, GuiDrawingAreaKeyEvent event),
-	void (*resizeCallback) (void *boss, GuiDrawingAreaResizeEvent event), void *boss,
+	GuiDrawingArea_ExposeCallback exposeCallback,
+	GuiDrawingArea_ClickCallback clickCallback,
+	GuiDrawingArea_KeyCallback keyCallback,
+	GuiDrawingArea_ResizeCallback resizeCallback, Thing boss,
 	uint32 flags);
 GuiDrawingArea GuiDrawingArea_createShown (GuiForm parent, int left, int right, int top, int bottom,
-	void (*exposeCallback) (void *boss, GuiDrawingAreaExposeEvent event),
-	void (*clickCallback) (void *boss, GuiDrawingAreaClickEvent event),
-	void (*keyCallback) (void *boss, GuiDrawingAreaKeyEvent event),
-	void (*resizeCallback) (void *boss, GuiDrawingAreaResizeEvent event), void *boss,
+	GuiDrawingArea_ExposeCallback exposeCallback,
+	GuiDrawingArea_ClickCallback clickCallback,
+	GuiDrawingArea_KeyCallback keyCallback,
+	GuiDrawingArea_ResizeCallback resizeCallback, Thing boss,
 	uint32 flags);
 GuiDrawingArea GuiDrawingArea_create (GuiScrolledWindow parent, int width, int height,
-	void (*exposeCallback) (void *boss, GuiDrawingAreaExposeEvent event),
-	void (*clickCallback) (void *boss, GuiDrawingAreaClickEvent event),
-	void (*keyCallback) (void *boss, GuiDrawingAreaKeyEvent event),
-	void (*resizeCallback) (void *boss, GuiDrawingAreaResizeEvent event), void *boss,
+	GuiDrawingArea_ExposeCallback exposeCallback,
+	GuiDrawingArea_ClickCallback clickCallback,
+	GuiDrawingArea_KeyCallback keyCallback,
+	GuiDrawingArea_ResizeCallback resizeCallback, Thing boss,
 	uint32 flags);
 GuiDrawingArea GuiDrawingArea_createShown (GuiScrolledWindow parent, int width, int height,
-	void (*exposeCallback) (void *boss, GuiDrawingAreaExposeEvent event),
-	void (*clickCallback) (void *boss, GuiDrawingAreaClickEvent event),
-	void (*keyCallback) (void *boss, GuiDrawingAreaKeyEvent event),
-	void (*resizeCallback) (void *boss, GuiDrawingAreaResizeEvent event), void *boss,
+	GuiDrawingArea_ExposeCallback exposeCallback,
+	GuiDrawingArea_ClickCallback clickCallback,
+	GuiDrawingArea_KeyCallback keyCallback,
+	GuiDrawingArea_ResizeCallback resizeCallback, Thing boss,
 	uint32 flags);
 
 void GuiDrawingArea_setSwipable (GuiDrawingArea me, GuiScrollBar horizontalScrollBar, GuiScrollBar verticalScrollBar);
-void GuiDrawingArea_setExposeCallback (GuiDrawingArea me, void (*callback) (void *boss, GuiDrawingAreaExposeEvent event), void *boss);
-void GuiDrawingArea_setClickCallback  (GuiDrawingArea me, void (*callback) (void *boss, GuiDrawingAreaClickEvent  event), void *boss);
-void GuiDrawingArea_setResizeCallback (GuiDrawingArea me, void (*callback) (void *boss, GuiDrawingAreaResizeEvent event), void *boss);
+void GuiDrawingArea_setExposeCallback (GuiDrawingArea me, GuiDrawingArea_ExposeCallback callback, Thing boss);
+void GuiDrawingArea_setClickCallback  (GuiDrawingArea me, GuiDrawingArea_ClickCallback  callback, Thing boss);
+void GuiDrawingArea_setResizeCallback (GuiDrawingArea me, GuiDrawingArea_ResizeCallback callback, Thing boss);
 
 /********** GuiFileSelect **********/
 
-SortedSetOfString GuiFileSelect_getInfileNames (GuiWindow parent, const char32 *title, bool allowMultipleFiles);
+autoStringSet GuiFileSelect_getInfileNames (GuiWindow parent, const char32 *title, bool allowMultipleFiles);
 char32 * GuiFileSelect_getOutfileName (GuiWindow parent, const char32 *title, const char32 *defaultName);
 char32 * GuiFileSelect_getDirectoryName (GuiWindow parent, const char32 *title);
 
@@ -550,22 +533,31 @@ void GuiLabel_setText (GuiLabel me, const char32 *text /* cattable */);
 Thing_declare (GuiList);
 Thing_declare (GuiScrolledWindow);
 
-typedef struct structGuiListEvent {
+typedef struct structGuiList_SelectionChangedEvent {
 	GuiList list;
-} *GuiListEvent;
+} *GuiList_SelectionChangedEvent;
+typedef MelderCallback <void, structThing /* boss */, GuiList_SelectionChangedEvent> GuiList_SelectionChangedCallback;
+
+typedef struct structGuiList_DoubleClickEvent {
+	GuiList list;
+} *GuiList_DoubleClickEvent;
+typedef MelderCallback <void, structThing /* boss */, GuiList_DoubleClickEvent> GuiList_DoubleClickCallback;
+
+typedef struct structGuiList_ScrollEvent {
+	GuiList list;
+} *GuiList_ScrollEvent;
+typedef MelderCallback <void, structThing /* boss */, GuiList_ScrollEvent> GuiList_ScrollCallback;
 
 Thing_define (GuiList, GuiControl) {
 	bool d_allowMultipleSelection;
-	void (*d_selectionChangedCallback) (void *boss, GuiListEvent event);
-	void *d_selectionChangedBoss;
-	void (*d_doubleClickCallback) (void *boss, GuiListEvent event);
-	void *d_doubleClickBoss;
+	GuiList_SelectionChangedCallback d_selectionChangedCallback;
+	Thing d_selectionChangedBoss;
+	GuiList_DoubleClickCallback d_doubleClickCallback;
+	Thing d_doubleClickBoss;
+	GuiList_ScrollCallback d_scrollCallback;
+	Thing d_scrollBoss;
 	#if gtk
 		GtkListStore *d_liststore;
-	#elif cocoa
-	#elif motif && useCarbon
-		GuiObject d_xmScrolled, d_xmList;
-		ListHandle d_macListHandle;
 	#endif
 };
 
@@ -580,20 +572,34 @@ long GuiList_getBottomPosition (GuiList me);
 long GuiList_getNumberOfItems (GuiList me);
 long * GuiList_getSelectedPositions (GuiList me, long *numberOfSelected);
 long GuiList_getTopPosition (GuiList me);
+
+/**
+	Inserts a new item into a GuiList at a given position.
+	@param me
+		The GuiList into which the new item is inserted.
+	@param itemText
+		The text of the new item. Cattable.
+	@param position
+		The position of the new item in the list after insertion.
+		A value of 1 therefore puts the new item at the top of the list.
+		A value of 0 is special: the item is put at the bottom of the list.
+ */
 void GuiList_insertItem  (GuiList me, const char32 *itemText /* cattable */, long position);
+
 void GuiList_replaceItem (GuiList me, const char32 *itemText /* cattable */, long position);
 void GuiList_setTopPosition (GuiList me, long topPosition);
 void GuiList_selectItem (GuiList me, long position);
-void GuiList_setSelectionChangedCallback (GuiList me, void (*callback) (void *boss, GuiListEvent event), void *boss);
-void GuiList_setDoubleClickCallback (GuiList me, void (*callback) (void *boss, GuiListEvent event), void *boss);
+void GuiList_setSelectionChangedCallback (GuiList me, GuiList_SelectionChangedCallback callback, Thing boss);
+void GuiList_setDoubleClickCallback (GuiList me, GuiList_DoubleClickCallback callback, Thing boss);
+void GuiList_setScrollCallback (GuiList me, GuiList_ScrollCallback callback, Thing boss);
 
 /********** GuiMenu **********/
 
 Thing_declare (GuiMenuItem);
 
 Thing_define (GuiMenu, GuiThing) {
-	GuiMenuItem d_menuItem;
-	GuiButton d_cascadeButton;
+	autoGuiMenuItem d_menuItem;
+	autoGuiButton d_cascadeButton;
 	#if gtk
 		GtkMenuItem *d_gtkMenuTitle;
 	#elif cocoa
@@ -605,7 +611,7 @@ Thing_define (GuiMenu, GuiThing) {
 		GuiObject d_xmMenuBar;   // in case the menu is in a form
 	#endif
 
-	void v_destroy ()
+	void v_destroy () noexcept
 		override;
 	void v_show ()
 		override;
@@ -630,10 +636,12 @@ typedef struct structGuiMenuItemEvent {
 	bool shiftKeyPressed, commandKeyPressed, optionKeyPressed, extraControlKeyPressed;
 } *GuiMenuItemEvent;
 
+typedef MelderCallback <void, structThing /* boss */, GuiMenuItemEvent> GuiMenuItemCallback;
+
 Thing_define (GuiMenuItem, GuiThing) {
 	GuiMenu d_menu;
-	void (*d_commandCallback) (void *boss, GuiMenuItemEvent event);
-	void *d_boss;
+	GuiMenuItemCallback d_callback;
+	Thing d_boss;
 	#if gtk
 		bool d_callbackBlocked;
 	#endif
@@ -683,8 +691,8 @@ Thing_define (GuiMenuItem, GuiThing) {
 // or any ASCII character (preferably a letter or digit) between 32 and 126
 
 GuiMenuItem GuiMenu_addItem (GuiMenu menu, const char32 *title, uint32 flags,
-	void (*commandCallback) (void *boss, GuiMenuItemEvent event), void *boss);
-/* Flags is a combination of the above defines. */
+	GuiMenuItemCallback callback, Thing boss);
+/* Flags is a combination of the above defines (both layout and accelerators). */
 GuiMenuItem GuiMenu_addSeparator (GuiMenu menu);
 
 void GuiMenuItem_check (GuiMenuItem me, bool check);
@@ -693,7 +701,7 @@ void GuiMenuItem_check (GuiMenuItem me, bool check);
 
 Thing_define (GuiOptionMenu, GuiControl) {
 	int d_value;
-	Ordered d_options;   // of SimpleString
+	OrderedOf<structGuiMenuItem> d_options;
 	#if gtk
 		//GtkComboBox *d_gtkCascadeButton;
 	#elif cocoa
@@ -717,7 +725,7 @@ void GuiOptionMenu_setValue (GuiOptionMenu me, int value);
 
 Thing_declare (GuiProgressBar);
 
-Thing_define (GuiProgressBar, GuiControl) { public:
+Thing_define (GuiProgressBar, GuiControl) {
 	#if cocoa
 		GuiCocoaProgressBar *d_cocoaProgressBar;
 	#endif
@@ -737,10 +745,12 @@ typedef struct structGuiRadioButtonEvent {
 	int position;
 } *GuiRadioButtonEvent;
 
-Thing_define (GuiRadioButton, GuiControl) { public:
+typedef MelderCallback <void, structThing /* boss */, GuiRadioButtonEvent> GuiRadioButtonCallback;
+
+Thing_define (GuiRadioButton, GuiControl) {
 	GuiRadioButton d_previous, d_next;   // there's a linked list of grouped radio buttons
-	void (*d_valueChangedCallback) (void *boss, GuiRadioButtonEvent event);
-	void *d_valueChangedBoss;
+	GuiRadioButtonCallback d_valueChangedCallback;
+	Thing d_valueChangedBoss;
 	#if cocoa
 		GuiCocoaRadioButton *d_cocoaRadioButton;
 	#endif
@@ -750,9 +760,9 @@ Thing_define (GuiRadioButton, GuiControl) { public:
 #define GuiRadioButton_SET  1
 #define GuiRadioButton_INSENSITIVE  2
 GuiRadioButton GuiRadioButton_create      (GuiForm parent, int left, int right, int top, int bottom,
-	const char32 *buttonText, void (*valueChangedCallback) (void *boss, GuiRadioButtonEvent event), void *valueChangedBoss, uint32 flags);
+	const char32 *buttonText, GuiRadioButtonCallback valueChangedCallback, Thing valueChangedBoss, uint32 flags);
 GuiRadioButton GuiRadioButton_createShown (GuiForm parent, int left, int right, int top, int bottom,
-	const char32 *buttonText, void (*valueChangedCallback) (void *boss, GuiRadioButtonEvent event), void *valueChangedBoss, uint32 flags);
+	const char32 *buttonText, GuiRadioButtonCallback valueChangedCallback, Thing valueChangedBoss, uint32 flags);
 
 void GuiRadioGroup_begin ();
 void GuiRadioGroup_end ();
@@ -786,19 +796,21 @@ typedef struct structGuiScrollBarEvent {
 	GuiScrollBar scrollBar;
 } *GuiScrollBarEvent;
 
-Thing_define (GuiScrollBar, GuiControl) { public:
-	void (*d_valueChangedCallback) (void *boss, GuiScrollBarEvent event);
-	void *d_valueChangedBoss;
+typedef MelderCallback <void, structThing /* boss */, GuiScrollBarEvent> GuiScrollBarCallback;
+
+Thing_define (GuiScrollBar, GuiControl) {
+	GuiScrollBarCallback d_valueChangedCallback;
+	Thing d_valueChangedBoss;
 };
 
 /* GuiScrollBar creation flags: */
 #define GuiScrollBar_HORIZONTAL  1
 GuiScrollBar GuiScrollBar_create      (GuiForm parent, int left, int right, int top, int bottom,
 	double minimum, double maximum, double value, double sliderSize, double increment, double pageIncrement,
-	void (*valueChangedCallback) (void *boss, GuiScrollBarEvent event), void *valueChangedBoss, uint32 flags);
+	GuiScrollBarCallback valueChangedCallback, Thing valueChangedBoss, uint32 flags);
 GuiScrollBar GuiScrollBar_createShown (GuiForm parent, int left, int right, int top, int bottom,
 	double minimum, double maximum, double value, double sliderSize, double increment, double pageIncrement,
-	void (*valueChangedCallback) (void *boss, GuiScrollBarEvent event), void *valueChangedBoss, uint32 flags);
+	GuiScrollBarCallback valueChangedCallback, Thing valueChangedBoss, uint32 flags);
 
 int GuiScrollBar_getValue (GuiScrollBar me);
 int GuiScrollBar_getSliderSize (GuiScrollBar me);
@@ -825,6 +837,8 @@ typedef struct structGuiTextEvent {
 	GuiText text;
 } *GuiTextEvent;
 
+typedef MelderCallback <void, structThing /* boss */, GuiTextEvent> GuiText_ChangedCallback;
+
 #if gtk
 	typedef gchar * history_data;
 #else
@@ -839,9 +853,9 @@ struct _history_entry_s {
 	bool type_del : 1;
 };
 
-Thing_define (GuiText, GuiControl) { public:
-	void (*d_changeCallback) (void *boss, GuiTextEvent event);
-	void *d_changeBoss;
+Thing_define (GuiText, GuiControl) {
+	GuiText_ChangedCallback d_changedCallback;
+	Thing d_changedBoss;
 	#if cocoa
 		GuiCocoaScrolledWindow *d_cocoaScrollView;
 		GuiCocoaTextView *d_cocoaTextView;
@@ -876,7 +890,7 @@ void GuiText_redo (GuiText me);
 void GuiText_remove (GuiText me);
 void GuiText_replace (GuiText me, long from_pos, long to_pos, const char32 *value);
 void GuiText_scrollToSelection (GuiText me);
-void GuiText_setChangeCallback (GuiText me, void (*changeCallback) (void *boss, GuiTextEvent event), void *changeBoss);
+void GuiText_setChangedCallback (GuiText me, GuiText_ChangedCallback changedCallback, Thing changedBoss);
 void GuiText_setFontSize (GuiText me, int size);
 void GuiText_setRedoItem (GuiText me, GuiMenuItem item);
 void GuiText_setSelection (GuiText me, long first, long last);
@@ -886,17 +900,17 @@ void GuiText_undo (GuiText me);
 
 /********** GuiWindow **********/
 
-Thing_define (GuiWindow, GuiShell) { public:
+Thing_define (GuiWindow, GuiShell) {
 	#if gtk
 		GtkMenuBar *d_gtkMenuBar;
 	#elif cocoa
 		int d_menuBarWidth;
-		void (*d_tabCallback) (void *boss, GuiMenuItemEvent event);
-		void *d_tabBoss;
-		void (*d_shiftTabCallback) (void *boss, GuiMenuItemEvent event);
-		void *d_shiftTabBoss;
-		void (*d_optionBackspaceCallback) (void *boss, GuiMenuItemEvent event);
-		void *d_optionBackspaceBoss;
+		GuiMenuItemCallback d_tabCallback;
+		Thing d_tabBoss;
+		GuiMenuItemCallback d_shiftTabCallback;
+		Thing d_shiftTabBoss;
+		GuiMenuItemCallback d_optionBackspaceCallback;
+		Thing d_optionBackspaceBoss;
 	#elif motif
 		GuiObject d_xmMenuBar;
 	#endif
@@ -905,7 +919,7 @@ Thing_define (GuiWindow, GuiShell) { public:
 /* GuiWindow creation flags: */
 #define GuiWindow_FULLSCREEN  1
 GuiWindow GuiWindow_create (int x, int y, int width, int height, int minimumWidth, int minimumHeight,
-	const char32 *title /* cattable */, void (*goAwayCallback) (void *goAwayBoss), void *goAwayBoss, uint32 flags);
+	const char32 *title /* cattable */, GuiShell_GoAwayCallback goAwayCallback, Thing goAwayBoss, uint32 flags);
 	// returns a Form widget that has a new Shell parent.
 
 void GuiWindow_addMenuBar (GuiWindow me);
