@@ -1,29 +1,26 @@
 /* GuiMenu.cpp
  *
- * Copyright (C) 1992-2012,2013,2015 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse, 2013 Tom Naughton
+ * Copyright (C) 1992-2012,2013,2015,2016 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse, 2013 Tom Naughton
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "GuiP.h"
 
 Thing_implement (GuiMenu, GuiThing, 0);
 
-void structGuiMenu :: v_destroy () {
-	forget (our d_cascadeButton);
-	forget (our d_menuItem);
+void structGuiMenu :: v_destroy () noexcept {
 	our GuiMenu_Parent :: v_destroy ();   // if (d_widget) { _GuiObject_setUserData (d_widget, nullptr); GuiObject_destroy (d_widget); }
 }
 
@@ -42,7 +39,7 @@ void structGuiMenu :: v_destroy () {
 		(void) void_me;
 		GuiMenu me = (GuiMenu) _GuiObject_getUserData (widget);
 		if (! me) return;
-		trace (U"destroying GuiButton ", Melder_pointer (my d_cascadeButton));
+		trace (U"destroying GuiButton ", Melder_pointer (my d_cascadeButton.get()));
 		gtk_widget_destroy (GTK_WIDGET (my d_widget));
 	}
 #elif cocoa
@@ -75,16 +72,23 @@ void structGuiMenu :: v_destroy () {
 			}
 			if (character == NSTabCharacter) {
 				NSWindow *cocoaKeyWindow = [NSApp keyWindow];
-				if ([cocoaKeyWindow class] == [GuiCocoaWindow class]) {
-					GuiWindow window = (GuiWindow) [(GuiCocoaWindow *) cocoaKeyWindow userData];
-					if (window -> d_tabCallback) {
-						try {
-							struct structGuiMenuItemEvent event = { nullptr, 0 };
-							window -> d_tabCallback (window -> d_tabBoss, & event);
-						} catch (MelderError) {
-							Melder_flushError (U"Tab key not completely handled.");
+				if ([cocoaKeyWindow class] == [GuiCocoaShell class]) {
+					GuiShell shell = (GuiShell) [(GuiCocoaShell *) cocoaKeyWindow   getUserData];
+					if (shell -> classInfo == classGuiWindow) {
+						GuiWindow window = (GuiWindow) shell;
+						if (window -> d_tabCallback) {
+							try {
+								struct structGuiMenuItemEvent event { nullptr, false, false, false, false };
+								window -> d_tabCallback (window -> d_tabBoss, & event);
+							} catch (MelderError) {
+								Melder_flushError (U"Tab key not completely handled.");
+							}
+							return;
 						}
-						return;
+					} else {
+						/*
+							We're in a dialog. Do nothing. Send the event on.
+						*/
 					}
 				}
 			} else if (character == NSBackTabCharacter) {
@@ -96,44 +100,58 @@ void structGuiMenu :: v_destroy () {
 				 * one can get here as well by pressing Ctrl-Y (Y is the 25th letter in the alphabet).
 				 */
 				NSWindow *cocoaKeyWindow = [NSApp keyWindow];
-				if ([cocoaKeyWindow class] == [GuiCocoaWindow class]) {
-					GuiWindow window = (GuiWindow) [(GuiCocoaWindow *) cocoaKeyWindow userData];
-					if ([nsEvent modifierFlags] & NSShiftKeyMask) {
+				if ([cocoaKeyWindow class] == [GuiCocoaShell class]) {
+					GuiShell shell = (GuiShell) [(GuiCocoaShell *) cocoaKeyWindow   getUserData];
+					if (shell -> classInfo == classGuiWindow) {
+						GuiWindow window = (GuiWindow) shell;
+						if ([nsEvent modifierFlags] & NSShiftKeyMask) {
+							/*
+								Make sure we got here by Shift-Tab rather than Ctrl-Y.
+							*/
+							if (window -> d_shiftTabCallback) {
+								try {
+									struct structGuiMenuItemEvent event { nullptr, false, false, false, false };
+									window -> d_shiftTabCallback (window -> d_shiftTabBoss, & event);
+								} catch (MelderError) {
+									Melder_flushError (U"Tab key not completely handled.");
+								}
+								return;
+							}
+						} else {
+							/*
+							 * We probably got in this branch by pressing Ctrl-Y.
+							 * People sometimes press that because it means "yank" (= Paste) in Emacs,
+							 * and indeed sending this key combination on, as we do here,
+							 * implements (together with Ctrl-K = "kil" = Cut)
+							 * a special cut & paste operation in text fields.
+							 */
+							// do nothing, i.e. send on
+						}
+					} else {
 						/*
-						 * Make sure we got here by Shift-Tab rather than Ctrl-Y.
-						 */
-						if (window -> d_shiftTabCallback) {
+							We're in a dialog. Do nothing. Send on.
+						*/
+					}
+				}
+			} else if (character == NSDeleteCharacter) {
+				NSWindow *cocoaKeyWindow = [NSApp keyWindow];
+				if ([cocoaKeyWindow class] == [GuiCocoaShell class]) {
+					GuiShell shell = (GuiShell) [(GuiCocoaShell *) cocoaKeyWindow   getUserData];
+					if (shell -> classInfo == classGuiWindow) {
+						GuiWindow window = (GuiWindow) shell;
+						if (([nsEvent modifierFlags] & NSAlternateKeyMask) && window -> d_optionBackspaceCallback) {
 							try {
-								struct structGuiMenuItemEvent event = { nullptr, 0 };
-								window -> d_shiftTabCallback (window -> d_shiftTabBoss, & event);
+								struct structGuiMenuItemEvent event { nullptr, false, false, false, false };
+								window -> d_optionBackspaceCallback (window -> d_optionBackspaceBoss, & event);
 							} catch (MelderError) {
-								Melder_flushError (U"Tab key not completely handled.");
+								Melder_flushError (U"Option-Backspace not completely handled.");
 							}
 							return;
 						}
 					} else {
 						/*
-						 * We probably got in this branch by pressing Ctrl-Y.
-						 * People sometimes press that because it means "yank" (= Paste) in Emacs,
-						 * and indeed sending this key combination on, as we do here,
-						 * implements (together with Ctrl-K = "kil" = Cut)
-						 * a special cut & paste operation in text fields.
-						 */
-						// do nothing, i.e. send on
-					}
-				}
-			} else if (character == NSDeleteCharacter) {
-				NSWindow *cocoaKeyWindow = [NSApp keyWindow];
-				if ([cocoaKeyWindow class] == [GuiCocoaWindow class]) {
-					GuiWindow window = (GuiWindow) [(GuiCocoaWindow *) cocoaKeyWindow userData];
-					if (([nsEvent modifierFlags] & NSAlternateKeyMask) && window -> d_optionBackspaceCallback) {
-						try {
-							struct structGuiMenuItemEvent event = { nullptr, 0 };
-							window -> d_optionBackspaceCallback (window -> d_optionBackspaceBoss, & event);
-						} catch (MelderError) {
-							Melder_flushError (U"Option-Backspace not completely handled.");
-						}
-						return;
+							We're in a dialog. Do nothing. Send on.
+						*/
 					}
 				}
 			}
@@ -241,40 +259,40 @@ void GuiMenu_empty (GuiMenu me) {
 		GuiMenu d_userData;
 	}
 	- (void) dealloc {   // override
-		GuiMenu me = self -> d_userData;
+		GuiMenu me = d_userData;
 		forget (me);
 		trace (U"deleting a menu button");
 		[super dealloc];
 	}
-	- (GuiThing) userData {
-		return self -> d_userData;
+	- (GuiThing) getUserData {
+		return d_userData;
 	}
 	- (void) setUserData: (GuiThing) userData {
 		Melder_assert (userData == nullptr || Thing_isa (userData, classGuiMenu));
-		self -> d_userData = static_cast <GuiMenu> (userData);
+		d_userData = static_cast <GuiMenu> (userData);
 	}
 	@end
 	@implementation GuiCocoaMenu {
 		GuiMenu d_userData;
 	}
 	- (void) dealloc {   // override
-		GuiMenu me = self -> d_userData;
+		GuiMenu me = d_userData;
 		forget (me);
 		trace (U"deleting a menu");
 		[super dealloc];
 	}
-	- (GuiThing) userData {
-		return self -> d_userData;
+	- (GuiThing) getUserData {
+		return d_userData;
 	}
 	- (void) setUserData: (GuiThing) userData {
 		Melder_assert (userData == nullptr || Thing_isa (userData, classGuiMenu));
-		self -> d_userData = static_cast <GuiMenu> (userData);
+		d_userData = static_cast <GuiMenu> (userData);
 	}
 	@end
 #endif
 
 GuiMenu GuiMenu_createInWindow (GuiWindow window, const char32 *title, uint32 flags) {
-	GuiMenu me = Thing_new (GuiMenu);
+	autoGuiMenu me = Thing_new (GuiMenu);
 	my d_shell = window;
 	my d_parent = window;
 	#if gtk
@@ -289,7 +307,7 @@ GuiMenu GuiMenu_createInWindow (GuiWindow window, const char32 *title, uint32 fl
 		GtkAccelGroup *ag = (GtkAccelGroup *) g_object_get_data (G_OBJECT (window -> d_gtkMenuBar), "accel-group");
 		gtk_menu_set_accel_group (GTK_MENU (my d_widget), ag);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (my d_gtkMenuTitle), GTK_WIDGET (my d_widget));
-		_GuiObject_setUserData (my d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
 	#elif cocoa
 		if (! theMenuBar) {
 			int numberOfMenus = [[[NSApp mainMenu] itemArray] count];
@@ -301,7 +319,7 @@ GuiMenu GuiMenu_createInWindow (GuiWindow window, const char32 *title, uint32 fl
 		my d_cocoaMenu = [[GuiCocoaMenu alloc]
 			initWithTitle: (NSString *) Melder_peek32toCfstring (title)];
 		my d_widget = my d_cocoaMenu;
-		[my d_cocoaMenu   setUserData: me];
+		[my d_cocoaMenu   setUserData: me.get()];
 		[my d_cocoaMenu   setAutoenablesItems: NO];
 		if (! window) {
 			/*
@@ -387,26 +405,26 @@ GuiMenu GuiMenu_createInWindow (GuiWindow window, const char32 *title, uint32 fl
 			XtVaSetValues (my d_xmMenuTitle, XmNsubMenuId, my d_widget, nullptr);
 			XtManageChild (my d_xmMenuTitle);
 		}
-		_GuiObject_setUserData (my d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
 	#endif
 
 	#if gtk
-		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me);
+		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me.get());
 	#elif cocoa
 	#elif motif
-		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me);
+		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me.get());
 	#endif
-	return me;
+	return me.releaseToAmbiguousOwner();
 }
 
 GuiMenu GuiMenu_createInMenu (GuiMenu supermenu, const char32 *title, uint32 flags) {
-	GuiMenu me = Thing_new (GuiMenu);
+	autoGuiMenu me = Thing_new (GuiMenu);
 	my d_shell = supermenu -> d_shell;
 	my d_parent = supermenu;
 	my d_menuItem = Thing_new (GuiMenuItem);
 	my d_menuItem -> d_shell = my d_shell;
 	my d_menuItem -> d_parent = supermenu;
-	my d_menuItem -> d_menu = me;
+	my d_menuItem -> d_menu = me.get();
 	#if gtk
 		my d_menuItem -> d_widget = gtk_menu_item_new_with_label (Melder_peek32to8 (title));
 		my d_widget = gtk_menu_new ();
@@ -418,7 +436,7 @@ GuiMenu GuiMenu_createInMenu (GuiMenu supermenu, const char32 *title, uint32 fla
 		gtk_menu_shell_append (GTK_MENU_SHELL (supermenu -> d_widget), GTK_WIDGET (my d_menuItem -> d_widget));
 		gtk_widget_show (GTK_WIDGET (my d_widget));
 		gtk_widget_show (GTK_WIDGET (my d_menuItem -> d_widget));
-		_GuiObject_setUserData (my d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
 	#elif cocoa
 		trace (U"creating menu item ", title);
 		NSMenuItem *item = [[NSMenuItem alloc]
@@ -432,9 +450,9 @@ GuiMenu GuiMenu_createInMenu (GuiMenu supermenu, const char32 *title, uint32 fla
 		trace (U"creating menu ", title);
 		my d_cocoaMenu = [[GuiCocoaMenu alloc]
 			initWithTitle: (NSString *) Melder_peek32toCfstring (title)];
-		[my d_cocoaMenu   setUserData: me];
+		[my d_cocoaMenu   setUserData: me.get()];
 		[my d_cocoaMenu   setAutoenablesItems: NO];
-		trace (U"adding the new menu ", Melder_pointer (me), U" to its supermenu ", Melder_pointer (supermenu));
+		trace (U"adding the new menu ", Melder_pointer (me.get()), U" to its supermenu ", Melder_pointer (supermenu));
 		[supermenu -> d_cocoaMenu   setSubmenu: my d_cocoaMenu   forItem: item];   // the supermenu will retain the menu...
 		Melder_assert ([my d_cocoaMenu retainCount] == 2);
 		[my d_cocoaMenu release];   // ... so we can release the menu already, even before returning it
@@ -447,16 +465,16 @@ GuiMenu GuiMenu_createInMenu (GuiMenu supermenu, const char32 *title, uint32 fla
 			XtSetSensitive (my d_menuItem -> d_widget, False);
 		XtVaSetValues (my d_menuItem -> d_widget, XmNsubMenuId, my d_widget, nullptr);
 		XtManageChild (my d_menuItem -> d_widget);
-		_GuiObject_setUserData (my d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
 	#endif
 
 	#if gtk
-		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me);
+		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me.get());
 	#elif cocoa
 	#elif motif
-		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me);
+		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me.get());
 	#endif
-	return me;
+	return me.releaseToAmbiguousOwner();
 }
 
 #if gtk
@@ -491,13 +509,13 @@ static gint button_press (GtkWidget *widget, GdkEvent *event)
 #endif
 
 GuiMenu GuiMenu_createInForm (GuiForm form, int left, int right, int top, int bottom, const char32 *title, uint32 flags) {
-	GuiMenu me = Thing_new (GuiMenu);
+	autoGuiMenu me = Thing_new (GuiMenu);
 	my d_shell = form -> d_shell;
 	my d_parent = form;
 	my d_cascadeButton = Thing_new (GuiButton);
 	my d_cascadeButton -> d_shell = my d_shell;
 	my d_cascadeButton -> d_parent = form;
-	my d_cascadeButton -> d_menu = me;
+	my d_cascadeButton -> d_menu = me.get();
 	#if gtk
 		my d_cascadeButton -> d_widget = gtk_button_new_with_label (Melder_peek32to8 (title));
 		my d_cascadeButton -> v_positionInForm (my d_cascadeButton -> d_widget, left, right, top, bottom, form);
@@ -512,12 +530,12 @@ GuiMenu GuiMenu_createInForm (GuiForm form, int left, int right, int top, int bo
 		g_object_set_data (G_OBJECT (my d_widget), "button", my d_cascadeButton -> d_widget);
 		gtk_menu_attach_to_widget (GTK_MENU (my d_widget), GTK_WIDGET (my d_cascadeButton -> d_widget), nullptr);
 		gtk_button_set_alignment (GTK_BUTTON (my d_cascadeButton -> d_widget), 0.0f, 0.5f);
-		_GuiObject_setUserData (my d_widget, me);
-		_GuiObject_setUserData (my d_cascadeButton -> d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
+		_GuiObject_setUserData (my d_cascadeButton -> d_widget, me.get());
 	#elif cocoa
 		my d_cascadeButton -> d_widget = my d_cocoaMenuButton = [[GuiCocoaMenuButton alloc] init];
 		my d_cascadeButton -> v_positionInForm (my d_cocoaMenuButton, left, right, top, bottom, form);
-		[my d_cocoaMenuButton   setUserData: me];
+		[my d_cocoaMenuButton   setUserData: me.get()];
 		[my d_cocoaMenuButton   setPullsDown: YES];
 		[my d_cocoaMenuButton   setAutoenablesItems: NO];
 		[my d_cocoaMenuButton   setBezelStyle: NSShadowlessSquareBezelStyle];
@@ -550,17 +568,17 @@ GuiMenu GuiMenu_createInForm (GuiForm form, int left, int right, int top, int bo
 		XtVaSetValues (my d_cascadeButton -> d_widget, XmNsubMenuId, my d_widget, nullptr);
 		XtManageChild (my d_cascadeButton -> d_widget);
 		XtManageChild (my d_xmMenuBar);
-		_GuiObject_setUserData (my d_widget, me);
+		_GuiObject_setUserData (my d_widget, me.get());
 	#endif
 
 	#if gtk
-		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me);
-		g_signal_connect (G_OBJECT (my d_cascadeButton -> d_widget), "destroy", G_CALLBACK (_guiGtkMenuCascadeButton_destroyCallback), me);
+		g_signal_connect (G_OBJECT (my d_widget), "destroy", G_CALLBACK (_guiGtkMenu_destroyCallback), me.get());
+		g_signal_connect (G_OBJECT (my d_cascadeButton -> d_widget), "destroy", G_CALLBACK (_guiGtkMenuCascadeButton_destroyCallback), me.get());
 	#elif cocoa
 	#elif motif
-		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me);
+		XtAddCallback (my d_widget, XmNdestroyCallback, _guiMotifMenu_destroyCallback, me.get());
 	#endif
-	return me;
+	return me.releaseToAmbiguousOwner();
 };
 
 /* End of file GuiMenu.cpp */
