@@ -1,20 +1,19 @@
 /* ManPages.cpp
  *
- * Copyright (C) 1996-2012,2014,2015 Paul Boersma
+ * Copyright (C) 1996-2012,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <ctype.h>
@@ -36,10 +35,10 @@ static int isSingleWordCharacter (int c) {
 
 static long lookUp_unsorted (ManPages me, const char32 *title);
 
-void structManPages :: v_destroy () {
-	if (dynamic && pages) {
-		for (long ipage = 1; ipage <= pages -> size; ipage ++) {
-			ManPage page = (ManPage) pages -> item [ipage];
+void structManPages :: v_destroy () noexcept {
+	if (our dynamic) {
+		for (long ipage = 1; ipage <= our pages.size; ipage ++) {
+			ManPage page = our pages.at [ipage];
 			Melder_free (page -> title);
 			Melder_free (page -> author);
 			if (page -> paragraphs) {
@@ -54,11 +53,10 @@ void structManPages :: v_destroy () {
 			}
 		}
 	}
-	if (pages && titles)
-		for (long ipage = 1; ipage <= pages -> size; ipage ++) {
+	if (our titles)
+		for (long ipage = 1; ipage <= our pages.size; ipage ++) {
 			Melder_free (titles [ipage]);
 		}
-	forget (pages);
 	NUMvector_free <const char32 *> (titles, 1);
 	ManPages_Parent :: v_destroy ();
 }
@@ -116,13 +114,14 @@ static void readOnePage (ManPages me, MelderReadText text) {
 		return;
 	}
 
-	ManPage page = Thing_new (ManPage);
-	page -> title = title;
+	autoManPage autopage = Thing_new (ManPage);
+	autopage -> title = title;
+	ManPage page = autopage.get();
 
 	/*
 	 * Add the page early, so that lookUp can find it.
 	 */
-	Collection_addItem (my pages, page);
+	my pages. addItem_move (autopage.move());
 
 	try {
 		page -> author = texgetw2 (text);
@@ -235,15 +234,13 @@ static void readOnePage (ManPages me, MelderReadText text) {
 	page -> paragraphs = (ManPage_Paragraph) Melder_realloc (page -> paragraphs, (int64) sizeof (struct structManPage_Paragraph) * (par - page -> paragraphs));
 }
 void structManPages :: v_readText (MelderReadText text, int /*formatVersion*/) {
-	dynamic = true;
-	pages = Ordered_create ();
+	our dynamic = true;
 	MelderDir_copy (& Data_directoryBeingRead, & rootDirectory);
 	readOnePage (this, text);
 }
 
-ManPages ManPages_create () {
-	ManPages me = Thing_new (ManPages);
-	my pages = Ordered_create ();
+autoManPages ManPages_create () {
+	autoManPages me = Thing_new (ManPages);
 	return me;
 }
 
@@ -255,7 +252,7 @@ void ManPages_addPage (ManPages me, const char32 *title, const char32 *author, l
 	page -> paragraphs = & paragraphs [0];
 	page -> author = author;
 	page -> date = date;
-	Collection_addItem (my pages, page.transfer());
+	my pages. addItem_move (page.move());
 }
 
 static int pageCompare (const void *first, const void *second) {
@@ -277,8 +274,8 @@ static long lookUp_unsorted (ManPages me, const char32 *title) {
 	/*
 	 * First try to match an unaltered 'title' with the titles of the man pages.
 	 */
-	for (i = 1; i <= my pages -> size; i ++) {
-		ManPage page = (ManPage) my pages -> item [i];
+	for (i = 1; i <= my pages.size; i ++) {
+		ManPage page = my pages.at [i];
 		if (str32equ (page -> title, title)) return i;
 	}
 
@@ -289,8 +286,8 @@ static long lookUp_unsorted (ManPages me, const char32 *title) {
 		char32 upperTitle [300];
 		Melder_sprint (upperTitle,300, title);
 		upperTitle [0] = toupper (upperTitle [0]);
-		for (i = 1; i <= my pages -> size; i ++) {
-			ManPage page = (ManPage) my pages -> item [i];
+		for (i = 1; i <= my pages.size; i ++) {
+			ManPage page = my pages.at [i];
 			if (str32equ (page -> title, upperTitle)) return i;
 		}
 	}
@@ -298,19 +295,21 @@ static long lookUp_unsorted (ManPages me, const char32 *title) {
 }
 
 static long lookUp_sorted (ManPages me, const char32 *title) {
-	static ManPage dummy;
+	static autoManPage dummy;
 	ManPage *page;
 	if (! dummy) dummy = Thing_new (ManPage);
 	dummy -> title = title;
-	page = (ManPage *) bsearch (& dummy, & my pages -> item [1], my pages -> size, sizeof (ManPage), pageCompare);
-	if (page) return (page - (ManPage *) & my pages -> item [1]) + 1;
+	page = (ManPage *) bsearch (& dummy, & my pages.at [1], my pages.size, sizeof (ManPage), pageCompare);   // noexcept
+	dummy -> title = nullptr;   // undangle
+	if (page) return (page - & my pages.at [1]) + 1;
 	if (islower (title [0]) || isupper (title [0])) {
 		char32 caseSwitchedTitle [300];
 		Melder_sprint (caseSwitchedTitle,300, title);
 		caseSwitchedTitle [0] = islower (title [0]) ? toupper (caseSwitchedTitle [0]) : tolower (caseSwitchedTitle [0]);
 		dummy -> title = caseSwitchedTitle;
-		page = (ManPage *) bsearch (& dummy, & my pages -> item [1], my pages -> size, sizeof (ManPage), pageCompare);
-		if (page) return (page - (ManPage *) & my pages -> item [1]) + 1;
+		page = (ManPage *) bsearch (& dummy, & my pages.at [1], my pages.size, sizeof (ManPage), pageCompare);   // noexcept
+		dummy -> title = nullptr;   // undangle
+		if (page) return (page - & my pages.at [1]) + 1;
 	}
 	return 0;
 }
@@ -319,14 +318,14 @@ static void grind (ManPages me) {
 	long ipage, ndangle = 0, jpage, grandNlinks, ilinkHither, ilinkThither;
 	long *grandLinksHither, *grandLinksThither;
 
-	qsort (& my pages -> item [1], my pages -> size, sizeof (ManPage), pageCompare);
+	qsort (& my pages.at [1], my pages.size, sizeof (ManPage), pageCompare);
 
 	/*
 	 * First pass: count and check links: fill in nlinksHither and nlinksThither.
 	 */
 	grandNlinks = 0;
-	for (ipage = 1; ipage <= my pages -> size; ipage ++) {
-		ManPage page = (ManPage) my pages -> item [ipage];
+	for (ipage = 1; ipage <= my pages.size; ipage ++) {
+		ManPage page = my pages.at [ipage];
 		int ipar;
 		for (ipar = 0; page -> paragraphs [ipar]. type; ipar ++) {
 			const char32 *text = page -> paragraphs [ipar]. text, *p;
@@ -336,7 +335,7 @@ static void grind (ManPages me) {
 					continue;   // ignore "FILE" links
 				if ((jpage = lookUp_sorted (me, link)) != 0) {
 					page -> nlinksThither ++;
-					((ManPage) my pages -> item [jpage]) -> nlinksHither ++;
+					my pages.at [jpage] -> nlinksHither ++;
 					grandNlinks ++;
 				} else {
 					MelderInfo_writeLine (U"Page \"", page -> title, U"\" contains a dangling link to \"", link, U"\".");
@@ -361,8 +360,8 @@ static void grind (ManPages me) {
 		return;
 	}
 	ilinkHither = ilinkThither = 0;
-	for (ipage = 1; ipage <= my pages -> size; ipage ++) {
-		ManPage page = (ManPage) my pages -> item [ipage];
+	for (ipage = 1; ipage <= my pages.size; ipage ++) {
+		ManPage page = my pages.at [ipage];
 		page -> linksHither = grandLinksHither + ilinkHither;
 		page -> linksThither = grandLinksThither + ilinkThither;
 		ilinkHither += page -> nlinksHither;
@@ -376,8 +375,8 @@ static void grind (ManPages me) {
 	 * Third pass: remember the links: fill in linksThither [1..nlinksThither] and linksHither [1..nlinksHither].
 	 * Rebuild nlinksHither and nlinksThither.
 	 */
-	for (ipage = 1; ipage <= my pages -> size; ipage ++) {
-		ManPage page = (ManPage) my pages -> item [ipage];
+	for (ipage = 1; ipage <= my pages.size; ipage ++) {
+		ManPage page = my pages.at [ipage];
 		for (int ipar = 0; page -> paragraphs [ipar]. type; ipar ++) {
 			const char32 *text = page -> paragraphs [ipar]. text, *p;
 			char32 link [301];
@@ -393,7 +392,7 @@ static void grind (ManPages me) {
 						}
 					}
 					if (! alreadyPresent) {
-						ManPage otherPage = (ManPage) my pages -> item [jpage];
+						ManPage otherPage = my pages.at [jpage];
 						page -> linksThither [++ page -> nlinksThither] = jpage;
 						otherPage -> linksHither [++ otherPage -> nlinksHither] = ipage;
 					}
@@ -406,7 +405,7 @@ static void grind (ManPages me) {
 }
 
 long ManPages_uniqueLinksHither (ManPages me, long ipage) {
-	ManPage page = (ManPage) my pages -> item [ipage];
+	ManPage page = my pages.at [ipage];
 	long result = page -> nlinksHither, ilinkHither, ilinkThither;
 	for (ilinkHither = 1; ilinkHither <= page -> nlinksHither; ilinkHither ++) {
 		long link = page -> linksHither [ilinkHither];
@@ -423,8 +422,8 @@ long ManPages_lookUp (ManPages me, const char32 *title) {
 
 static long ManPages_lookUp_caseSensitive (ManPages me, const char32 *title) {
 	if (! my ground) grind (me);
-	for (long i = 1; i <= my pages -> size; i ++) {
-		ManPage page = (ManPage) my pages -> item [i];
+	for (long i = 1; i <= my pages.size; i ++) {
+		ManPage page = my pages.at [i];
 		if (str32equ (page -> title, title)) return i;
 	}
 	return 0;
@@ -433,13 +432,13 @@ static long ManPages_lookUp_caseSensitive (ManPages me, const char32 *title) {
 const char32 **ManPages_getTitles (ManPages me, long *numberOfTitles) {
 	if (! my ground) grind (me);
 	if (! my titles) {
-		my titles = NUMvector <const char32 *> (1, my pages -> size);   // TODO
-		for (long i = 1; i <= my pages -> size; i ++) {
-			ManPage page = (ManPage) my pages -> item [i];
+		my titles = NUMvector <const char32 *> (1, my pages.size);   // TODO
+		for (long i = 1; i <= my pages.size; i ++) {
+			ManPage page = my pages.at [i];
 			my titles [i] = Melder_dup_f (page -> title);
 		}
 	}
-	*numberOfTitles = my pages -> size;
+	*numberOfTitles = my pages.size;
 	return my titles;
 }
 
@@ -498,17 +497,18 @@ static void writeParagraphsAsHtml (ManPages me, MelderFile file, ManPage_Paragra
 			pdfFile. path [str32len (pdfFile. path) - 5] = U'\0';   // delete extension ".html"
 			str32cpy (pdfFile. path + str32len (pdfFile. path),
 				Melder_cat (U"_", numberOfPictures, U".pdf"));
-			Graphics graphics = Graphics_create_pdffile (& pdfFile, 100, 0.0, paragraph -> width, 0.0, paragraph -> height);
-			Graphics_setFont (graphics, kGraphics_font_TIMES);
-			Graphics_setFontStyle (graphics, 0);
-			Graphics_setFontSize (graphics, 12);
-			Graphics_setWrapWidth (graphics, 0);
-			Graphics_setViewport (graphics, 0.0, paragraph -> width, 0.0, paragraph -> height);
-			paragraph -> draw (graphics);
-			Graphics_setViewport (graphics, 0, 1, 0, 1);
-			Graphics_setWindow (graphics, 0, 1, 0, 1);
-			Graphics_setTextAlignment (graphics, Graphics_LEFT, Graphics_BOTTOM);
-			forget (graphics);
+			{// scope
+				autoGraphics graphics = Graphics_create_pdffile (& pdfFile, 100, 0.0, paragraph -> width, 0.0, paragraph -> height);
+				Graphics_setFont (graphics.get(), kGraphics_font_TIMES);
+				Graphics_setFontStyle (graphics.get(), 0);
+				Graphics_setFontSize (graphics.get(), 12);
+				Graphics_setWrapWidth (graphics.get(), 0);
+				Graphics_setViewport (graphics.get(), 0.0, paragraph -> width, 0.0, paragraph -> height);
+				paragraph -> draw (graphics.get());
+				Graphics_setViewport (graphics.get(), 0, 1, 0, 1);
+				Graphics_setWindow (graphics.get(), 0, 1, 0, 1);
+				Graphics_setTextAlignment (graphics.get(), Graphics_LEFT, Graphics_BOTTOM);
+			}
 			structMelderFile tiffFile;
 			MelderFile_copy (file, & tiffFile);
 			tiffFile. path [str32len (tiffFile. path) - 5] = U'\0';   // delete extension ".html"
@@ -530,65 +530,66 @@ static void writeParagraphsAsHtml (ManPages me, MelderFile file, ManPage_Paragra
 			pdfFile. path [str32len (pdfFile. path) - 5] = U'\0';   // delete extension ".html"
 			str32cpy (pdfFile. path + str32len (pdfFile.path),
 				Melder_cat (U"_", numberOfPictures, U".pdf"));
-			Graphics graphics = Graphics_create_pdffile (& pdfFile, 100, 0.0, paragraph -> width, 0.0, paragraph -> height);
-			Graphics_setFont (graphics, kGraphics_font_TIMES);
-			Graphics_setFontStyle (graphics, 0);
-			Graphics_setFontSize (graphics, 12);
-			Graphics_setWrapWidth (graphics, 0);
-			static structPraatApplication praatApplication;
-			static structPraatObjects praatObjects;
-			static structPraatPicture praatPicture;
-			theCurrentPraatApplication = & praatApplication;
-			theCurrentPraatApplication -> batch = true;
-			theCurrentPraatApplication -> topShell = theForegroundPraatApplication. topShell;   // needed for UiForm_create () in dialogs
-			theCurrentPraatObjects = (PraatObjects) & praatObjects;
-			theCurrentPraatPicture = (PraatPicture) & praatPicture;
-			theCurrentPraatPicture -> graphics = graphics;
-			theCurrentPraatPicture -> font = kGraphics_font_TIMES;
-			theCurrentPraatPicture -> fontSize = 12;
-			theCurrentPraatPicture -> lineType = Graphics_DRAWN;
-			theCurrentPraatPicture -> colour = Graphics_BLACK;
-			theCurrentPraatPicture -> lineWidth = 1.0;
-			theCurrentPraatPicture -> arrowSize = 1.0;
-			theCurrentPraatPicture -> speckleSize = 1.0;
-			theCurrentPraatPicture -> x1NDC = 0;
-			theCurrentPraatPicture -> x2NDC = paragraph -> width;
-			theCurrentPraatPicture -> y1NDC = 0;
-			theCurrentPraatPicture -> y2NDC = paragraph -> height;
-			Graphics_setViewport (graphics, theCurrentPraatPicture -> x1NDC, theCurrentPraatPicture -> x2NDC, theCurrentPraatPicture -> y1NDC, theCurrentPraatPicture -> y2NDC);			
-			Graphics_setWindow (graphics, 0.0, 1.0, 0.0, 1.0);
-			long x1DC, y1DC, x2DC, y2DC;
-			Graphics_WCtoDC (graphics, 0.0, 0.0, & x1DC, & y2DC);
-			Graphics_WCtoDC (graphics, 1.0, 1.0, & x2DC, & y1DC);
-			Graphics_resetWsViewport (graphics, x1DC, x2DC, y1DC, y2DC);
-			Graphics_setWsWindow (graphics, 0, paragraph -> width, 0, paragraph -> height);
-			theCurrentPraatPicture -> x1NDC = 0;
-			theCurrentPraatPicture -> x2NDC = paragraph -> width;
-			theCurrentPraatPicture -> y1NDC = 0;
-			theCurrentPraatPicture -> y2NDC = paragraph -> height;
-			Graphics_setViewport (graphics, theCurrentPraatPicture -> x1NDC, theCurrentPraatPicture -> x2NDC, theCurrentPraatPicture -> y1NDC, theCurrentPraatPicture -> y2NDC);			
 			{// scope
-				autoMelderProgressOff progress;
-				autoMelderWarningOff warning;
-				autoMelderSaveDefaultDir saveDir;
-				if (! MelderDir_isNull (& my rootDirectory)) {
-					Melder_setDefaultDir (& my rootDirectory);
-				}
-				try {
-					autostring32 text = Melder_dup (p);
-					Interpreter_run (interpreter.peek(), text.peek());
-				} catch (MelderError) {
-					trace (U"interpreter fails on ", pdfFile. path);
-					Melder_flushError ();
+				autoGraphics graphics = Graphics_create_pdffile (& pdfFile, 100, 0.0, paragraph -> width, 0.0, paragraph -> height);
+				Graphics_setFont (graphics.get(), kGraphics_font_TIMES);
+				Graphics_setFontStyle (graphics.get(), 0);
+				Graphics_setFontSize (graphics.get(), 12);
+				Graphics_setWrapWidth (graphics.get(), 0);
+				static structPraatApplication praatApplication;
+				static structPraatObjects praatObjects;
+				static structPraatPicture praatPicture;
+				theCurrentPraatApplication = & praatApplication;
+				theCurrentPraatApplication -> batch = true;
+				theCurrentPraatApplication -> topShell = theForegroundPraatApplication. topShell;   // needed for UiForm_create () in dialogs
+				theCurrentPraatObjects = (PraatObjects) & praatObjects;
+				theCurrentPraatPicture = (PraatPicture) & praatPicture;
+				theCurrentPraatPicture -> graphics = graphics.get();   // FIXME: should be move()?
+				theCurrentPraatPicture -> font = kGraphics_font_TIMES;
+				theCurrentPraatPicture -> fontSize = 12;
+				theCurrentPraatPicture -> lineType = Graphics_DRAWN;
+				theCurrentPraatPicture -> colour = Graphics_BLACK;
+				theCurrentPraatPicture -> lineWidth = 1.0;
+				theCurrentPraatPicture -> arrowSize = 1.0;
+				theCurrentPraatPicture -> speckleSize = 1.0;
+				theCurrentPraatPicture -> x1NDC = 0.0;
+				theCurrentPraatPicture -> x2NDC = paragraph -> width;
+				theCurrentPraatPicture -> y1NDC = 0.0;
+				theCurrentPraatPicture -> y2NDC = paragraph -> height;
+				Graphics_setViewport (graphics.get(), theCurrentPraatPicture -> x1NDC, theCurrentPraatPicture -> x2NDC, theCurrentPraatPicture -> y1NDC, theCurrentPraatPicture -> y2NDC);
+				Graphics_setWindow (graphics.get(), 0.0, 1.0, 0.0, 1.0);
+				long x1DC, y1DC, x2DC, y2DC;
+				Graphics_WCtoDC (graphics.get(), 0.0, 0.0, & x1DC, & y2DC);
+				Graphics_WCtoDC (graphics.get(), 1.0, 1.0, & x2DC, & y1DC);
+				Graphics_resetWsViewport (graphics.get(), x1DC, x2DC, y1DC, y2DC);
+				Graphics_setWsWindow (graphics.get(), 0.0, paragraph -> width, 0.0, paragraph -> height);
+				theCurrentPraatPicture -> x1NDC = 0.0;
+				theCurrentPraatPicture -> x2NDC = paragraph -> width;
+				theCurrentPraatPicture -> y1NDC = 0.0;
+				theCurrentPraatPicture -> y2NDC = paragraph -> height;
+				Graphics_setViewport (graphics.get(), theCurrentPraatPicture -> x1NDC, theCurrentPraatPicture -> x2NDC, theCurrentPraatPicture -> y1NDC, theCurrentPraatPicture -> y2NDC);
+				{// scope
+					autoMelderProgressOff progress;
+					autoMelderWarningOff warning;
+					autoMelderSaveDefaultDir saveDir;
+					if (! MelderDir_isNull (& my rootDirectory)) {
+						Melder_setDefaultDir (& my rootDirectory);
+					}
+					try {
+						autostring32 text = Melder_dup (p);
+						Interpreter_run (interpreter.get(), text.peek());
+					} catch (MelderError) {
+						trace (U"interpreter fails on ", pdfFile. path);
+						Melder_flushError ();
+					}
 				}
+				Graphics_setViewport (graphics.get(), 0.0, 1.0, 0.0, 1.0);
+				Graphics_setWindow (graphics.get(), 0.0, 1.0, 0.0, 1.0);
+				Graphics_setTextAlignment (graphics.get(), Graphics_LEFT, Graphics_BOTTOM);
 			}
-			Graphics_setViewport (graphics, 0, 1, 0, 1);
-			Graphics_setWindow (graphics, 0, 1, 0, 1);
-			Graphics_setTextAlignment (graphics, Graphics_LEFT, Graphics_BOTTOM);
-			forget (graphics);
 			structMelderFile tiffFile;
 			MelderFile_copy (file, & tiffFile);
-			tiffFile. path [str32len (tiffFile. path) - 5] = '\0';   // delete extension ".html"
+			tiffFile. path [str32len (tiffFile. path) - 5] = U'\0';   // delete extension ".html"
 			str32cpy (tiffFile. path + str32len (tiffFile. path),
 				Melder_cat (U"_", numberOfPictures, U".png"));
 			system (Melder_peek32to8 (Melder_cat (U"/usr/local/bin/gs -q -dNOPAUSE "
@@ -779,7 +780,7 @@ static const char32 *month [] =
 	  U"July", U"August", U"September", U"October", U"November", U"December" };
 
 static void writePageAsHtml (ManPages me, MelderFile file, long ipage, MelderString *buffer) {
-	ManPage page = (ManPage) my pages -> item [ipage];
+	ManPage page = my pages.at [ipage];
 	ManPage_Paragraph paragraphs = page -> paragraphs;
 	MelderString_append (buffer, U"<html><head><meta name=\"robots\" content=\"index,follow\">"
 		U"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
@@ -805,7 +806,7 @@ static void writePageAsHtml (ManPages me, MelderFile file, long ipage, MelderStr
 				if (page -> linksThither [jlink] == link)
 					alreadyShown = true;
 			if (! alreadyShown) {
-				const char32 *title = ((ManPage) my pages -> item [page -> linksHither [ilink]]) -> title, *p;
+				const char32 *title = my pages.at [page -> linksHither [ilink]] -> title, *p;
 				MelderString_append (buffer, U"<li><a href=\"");
 				for (p = title; *p; p ++) {
 					if (p - title >= LONGEST_FILE_NAME) break;
@@ -839,8 +840,8 @@ void ManPages_writeOneToHtmlFile (ManPages me, long ipage, MelderFile file) {
 void ManPages_writeAllToHtmlDir (ManPages me, const char32 *dirPath) {
 	structMelderDir dir;
 	Melder_pathToDir (dirPath, & dir);
-	for (long ipage = 1; ipage <= my pages -> size; ipage ++) {
-		ManPage page = (ManPage) my pages -> item [ipage];
+	for (long ipage = 1; ipage <= my pages.size; ipage ++) {
+		ManPage page = my pages.at [ipage];
 		char32 fileName [256];
 		Melder_assert (str32len (page -> title) < 256 - 100);
 		trace (U"page ", ipage, U": ", page -> title);
