@@ -1,38 +1,19 @@
 /* Manual.cpp
  *
- * Copyright (C) 1996-2011,2014,2015 Paul Boersma
+ * Copyright (C) 1996-2011,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-/*
- * pb 2003/03/09 searching: more points for multiple occurrences within a paragraph
- * pb 2003/10/03 praat_executeFromFile without arguments
- * pb 2003/11/26 use recording time from file
- * pb 2003/11/30 removed newline from date
- * pb 2004/02/08 allow arguments in scripts
- * pb 2005/05/08 embedded scripts (for pictures)
- * pb 2005/07/19 moved navigation buttons to the top, removed page label and horizontal scroll bar
- * pb 2006/10/20 embedded scripts: not on opening page
- * pb 2007/06/10 wchar
- * pb 2007/08/12 wchar
- * pb 2008/01/19 double
- * pb 2008/03/20 split off Help menu
- * pb 2008/03/21 new Editor API
- * pb 2011/04/06 C++
- * pb 2011/07/05 C++
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <ctype.h>
@@ -55,12 +36,11 @@ static const char32 *month [] =
 	{ U"", U"January", U"February", U"March", U"April", U"May", U"June",
 	  U"July", U"August", U"September", U"October", U"November", U"December" };
 
-static void menu_cb_writeOneToHtmlFile (EDITOR_ARGS) {
-	EDITOR_IAM (Manual);
-	EDITOR_FORM_WRITE (U"Save as HTML file", 0)
+static void menu_cb_writeOneToHtmlFile (Manual me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM_WRITE (U"Save as HTML file", nullptr)
 		ManPages manPages = (ManPages) my data;
 		autoMelderString buffer;
-		MelderString_copy (& buffer, ((ManPage) manPages -> pages -> item [my path]) -> title);
+		MelderString_copy (& buffer, manPages -> pages.at [my path] -> title);
 		char32 *p = buffer.string;
 		while (*p) { if (! isalnum ((int) *p) && *p != U'_') *p = U'_'; p ++; }
 		MelderString_append (& buffer, U".html");
@@ -70,13 +50,12 @@ static void menu_cb_writeOneToHtmlFile (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_writeAllToHtmlDir (EDITOR_ARGS) {
-	EDITOR_IAM (Manual);
-	EDITOR_FORM (U"Save all pages as HTML files", 0)
+static void menu_cb_writeAllToHtmlDir (Manual me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Save all pages as HTML files", nullptr)
 		LABEL (U"", U"Type a directory name:")
 		TEXTFIELD (U"directory", U"")
 	EDITOR_OK
-		structMelderDir currentDirectory = { { 0 } };
+		structMelderDir currentDirectory { { 0 } };
 		Melder_getDefaultDir (& currentDirectory);
 		SET_STRING (U"directory", Melder_dirToPath (& currentDirectory))
 	EDITOR_DO
@@ -85,13 +64,12 @@ static void menu_cb_writeAllToHtmlDir (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_searchForPageList (EDITOR_ARGS) {
-	EDITOR_IAM (Manual);
-	EDITOR_FORM (U"Search for page", 0)
+static void menu_cb_searchForPageList (Manual me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Search for page", nullptr)
 		ManPages manPages = (ManPages) my data;
 		long numberOfPages;
 		const char32 **pages = ManPages_getTitles (manPages, & numberOfPages);
-		LIST (U"Page", manPages -> pages -> size, pages, 1)
+		LIST (U"Page", manPages -> pages.size, pages, 1)
 	EDITOR_OK
 	EDITOR_DO
 		HyperPage_goToPage_i (me, GET_INTEGER (U"Page"));
@@ -103,20 +81,20 @@ void structManual :: v_draw () {
 	ManPage page;
 	ManPage_Paragraph paragraph;
 	#if motif
-	Graphics_clearWs (g);
+	Graphics_clearWs (our graphics.get());
 	#endif
 	if (our path == SEARCH_PAGE) {
 		HyperPage_pageTitle (this, U"Best matches");
 		HyperPage_intro (this, U"The best matches to your query seem to be:");
 		for (int i = 1; i <= our numberOfMatches; i ++) {
 			char32 link [300];
-			page = (ManPage) manPages -> pages -> item [matches [i]];
+			page = manPages -> pages.at [matches [i]];
 			Melder_sprint (link,300, U"• @@", page -> title);
 			HyperPage_listItem (this, link);
 		}
 		return;
 	}
-	page = (ManPage) manPages -> pages -> item [path];
+	page = manPages -> pages.at [path];
 	if (! our paragraphs) return;
 	HyperPage_pageTitle (this, page -> title);
 	for (paragraph = & page -> paragraphs [0]; paragraph -> type != 0; paragraph ++) {
@@ -171,7 +149,7 @@ void structManual :: v_draw () {
 				if (page -> linksThither [jlink] == link)
 					alreadyShown = true;
 			if (! alreadyShown) {
-				const char32 *title = ((ManPage) manPages -> pages -> item [page -> linksHither [ilink]]) -> title;
+				const char32 *title = manPages -> pages.at [page -> linksHither [ilink]] -> title;
 				char32 linkText [304];
 				Melder_sprint (linkText, 304, U"@@", title, U"@");
 				HyperPage_listItem (this, linkText);
@@ -198,17 +176,17 @@ void structManual :: v_draw () {
 
 /********** PRINTING **********/
 
-static void print (I, Graphics graphics) {
+static void print (void *void_me, Graphics graphics) {
 	iam (Manual);
 	ManPages manPages = (ManPages) my data;
-	long numberOfPages = manPages -> pages -> size, savePage = my path;
+	long numberOfPages = manPages -> pages.size, savePage = my path;
 	my ps = graphics;
 	Graphics_setDollarSignIsCode (my ps, true);
 	Graphics_setAtSignIsLink (my ps, true);
 	my printing = true;
 	HyperPage_initSheetOfPaper ((HyperPage) me);
 	for (long ipage = 1; ipage <= numberOfPages; ipage ++) {
-		ManPage page = (ManPage) manPages -> pages -> item [ipage];
+		ManPage page = manPages -> pages.at [ipage];
 		if (my printPagesStartingWith == nullptr ||
 		    Melder_stringMatchesCriterion (page -> title, kMelder_string_STARTS_WITH, my printPagesStartingWith))
 		{
@@ -229,8 +207,7 @@ static void print (I, Graphics graphics) {
 	my printPagesStartingWith = nullptr;
 }
 
-static void menu_cb_printRange (EDITOR_ARGS) {
-	EDITOR_IAM (Manual);
+static void menu_cb_printRange (Manual me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Print range", 0)
 		SENTENCE (U"Left or inside header", U"")
 		SENTENCE (U"Middle header", U"")
@@ -258,8 +235,8 @@ static void menu_cb_printRange (EDITOR_ARGS) {
 		SET_STRING (U"Left or inside header", date)
 		SET_STRING (U"Right or outside header", my name)
 		if (my d_printingPageNumber) SET_INTEGER (U"First page number", my d_printingPageNumber + 1)
-		if (my path >= 1 && my path <= manPages -> pages -> size) {
-			ManPage page = (ManPage) manPages -> pages -> item [my path];
+		if (my path >= 1 && my path <= manPages -> pages.size) {
+			ManPage page = manPages -> pages.at [my path];
 			SET_STRING (U"Print pages starting with", page -> title);
 		}
 	EDITOR_DO
@@ -283,7 +260,7 @@ static double *goodnessOfMatch;
 
 static double searchToken (ManPages me, long ipage, char32 *token) {
 	double goodness = 0.0;
-	ManPage page = (ManPage) my pages -> item [ipage];
+	ManPage page = my pages.at [ipage];
 	struct structManPage_Paragraph *par = & page -> paragraphs [0];
 	if (! token [0]) return 1.0;
 	/*
@@ -320,7 +297,7 @@ static double searchToken (ManPages me, long ipage, char32 *token) {
 
 static void search (Manual me, const char32 *query) {
 	ManPages manPages = (ManPages) my data;
-	long numberOfPages = manPages -> pages -> size;
+	long numberOfPages = manPages -> pages.size;
 	static MelderString searchText { 0 };
 	MelderString_copy (& searchText, query);
 	for (char32 *p = & searchText.string [0]; *p != U'\0'; p ++) {
@@ -366,24 +343,20 @@ void Manual_search (Manual me, const char32 *query) {
 	search (me, query);
 }
 
-static void gui_button_cb_home (I, GuiButtonEvent event) {
-	(void) event;
-	iam (Manual);
+static void gui_button_cb_home (Manual me, GuiButtonEvent /* event */) {
 	ManPages pages = (ManPages) my data;
 	long iHome = ManPages_lookUp (pages, U"Intro");
 	HyperPage_goToPage_i (me, iHome ? iHome : 1);
 }
  
-static void gui_button_cb_record (I, GuiButtonEvent event) {
-	(void) event;
-	iam (Manual);
+static void gui_button_cb_record (Manual me, GuiButtonEvent /* event */) {
 	ManPages manPages = (ManPages) my data;
-	ManPage manPage = (ManPage) (my path < 1 ? nullptr : manPages -> pages -> item [my path]);
+	ManPage manPage = ( my path < 1 ? nullptr : manPages -> pages.at [my path] );
 	GuiThing_setSensitive (my recordButton,  false);
 	GuiThing_setSensitive (my playButton,    false);
 	GuiThing_setSensitive (my publishButton, false);
-	#if motif
-		XmUpdateDisplay (my d_windowForm -> d_xmShell);
+	#if defined (_WIN32)
+		GdiFlush ();
 	#endif
 	if (! Melder_record (manPage == nullptr ? 1.0 : manPage -> recordingTime)) Melder_flushError ();
 	GuiThing_setSensitive (my recordButton,  true);
@@ -391,14 +364,12 @@ static void gui_button_cb_record (I, GuiButtonEvent event) {
 	GuiThing_setSensitive (my publishButton, true);
 }
 
-static void gui_button_cb_play (I, GuiButtonEvent event) {
-	(void) event;
-	iam (Manual);
+static void gui_button_cb_play (Manual me, GuiButtonEvent /* event */) {
 	GuiThing_setSensitive (my recordButton,  false);
 	GuiThing_setSensitive (my playButton,    false);
 	GuiThing_setSensitive (my publishButton, false);
-	#if motif
-		XmUpdateDisplay (my d_windowForm -> d_xmShell);
+	#if defined (_WIN32)
+		GdiFlush ();
 	#endif
 	Melder_play ();
 	GuiThing_setSensitive (my recordButton,  true);
@@ -406,10 +377,7 @@ static void gui_button_cb_play (I, GuiButtonEvent event) {
 	GuiThing_setSensitive (my publishButton, true);
 }
 
-static void gui_button_cb_publish (I, GuiButtonEvent event) {
-	(void) event;
-	iam (Manual);
-	(void) me;
+static void gui_button_cb_publish (Manual /* me */, GuiButtonEvent /* event */) {
 	Melder_publishPlayed ();
 }
 
@@ -419,14 +387,7 @@ static void do_search (Manual me) {
 	Melder_free (query);
 }
 
-static void gui_button_cb_search (I, GuiButtonEvent event) {
-	(void) event;
-	iam (Manual);
-	do_search (me);
-}
-
-static void gui_cb_search (GUI_ARGS) {
-	GUI_IAM (Manual);
+static void gui_button_cb_search (Manual me, GuiButtonEvent /* event */) {
 	do_search (me);
 }
 
@@ -455,7 +416,7 @@ void structManual :: v_createChildren () {
 	our searchText = GuiText_createShown (our d_windowForm, 274+69 + STRING_SPACING, 452 + STRING_SPACING - 2, y, y + Gui_TEXTFIELD_HEIGHT, 0);
 }
 
-static void menu_cb_help (EDITOR_ARGS) { EDITOR_IAM (Manual); HyperPage_goToPage (me, U"Manual"); }
+static void menu_cb_help (Manual me, EDITOR_ARGS_DIRECT) { HyperPage_goToPage (me, U"Manual"); }
 
 void structManual :: v_createMenus () {
 	Manual_Parent :: v_createMenus ();
@@ -480,7 +441,7 @@ void structManual :: v_defaultHeaders (EditorCommand cmd) {
 		char32 string [400];
 		static const char32 *shortMonth [] =
 			{ U"Jan", U"Feb", U"Mar", U"Apr", U"May", U"Jun", U"Jul", U"Aug", U"Sep", U"Oct", U"Nov", U"Dec" };
-		ManPage page = (ManPage) manPages -> pages -> item [my path];
+		ManPage page = manPages -> pages.at [my path];
 		long date = page -> date;
 		SET_STRING (U"Right or outside header", page -> title)
 		SET_STRING (U"Left or inside footer", page -> author)
@@ -493,7 +454,7 @@ void structManual :: v_defaultHeaders (EditorCommand cmd) {
 
 long structManual :: v_getNumberOfPages () {
 	ManPages manPages = (ManPages) our data;
-	return manPages -> pages -> size;
+	return manPages -> pages.size;
 }
 
 long structManual :: v_getCurrentPageNumber () {
@@ -502,7 +463,7 @@ long structManual :: v_getCurrentPageNumber () {
 
 void structManual :: v_goToPage_i (long pageNumber) {
 	ManPages manPages = (ManPages) our data;
-	if (pageNumber < 1 || pageNumber > manPages -> pages -> size) {
+	if (pageNumber < 1 || pageNumber > manPages -> pages.size) {
 		if (pageNumber == SEARCH_PAGE) {
 			our path = SEARCH_PAGE;
 			Melder_free (our currentPageTitle);
@@ -510,7 +471,7 @@ void structManual :: v_goToPage_i (long pageNumber) {
 		} else Melder_throw (U"Page ", pageNumber, U" not found.");
 	}
 	our path = pageNumber;
-	ManPage page = (ManPage) manPages -> pages -> item [path];
+	ManPage page = manPages -> pages.at [path];
 	our paragraphs = page -> paragraphs;
 	our numberOfParagraphs = 0;
 	ManPage_Paragraph par = paragraphs;
@@ -521,12 +482,12 @@ void structManual :: v_goToPage_i (long pageNumber) {
 
 int structManual :: v_goToPage (const char32 *title) {
 	ManPages manPages = (ManPages) our data;
-	if (title [0] == '\\' && title [1] == 'F' && title [2] == 'I') {
+	if (title [0] == U'\\' && title [1] == U'F' && title [2] == U'I') {
 		structMelderFile file = { 0 };
 		MelderDir_relativePathToFile (& manPages -> rootDirectory, title + 3, & file);
 		Melder_recordFromFile (& file);
 		return -1;
-	} else if (title [0] == '\\' && title [1] == 'S' && title [2] == 'C') {
+	} else if (title [0] == U'\\' && title [1] == U'S' && title [2] == U'C') {
 		autoMelderSetDefaultDir dir (& manPages -> rootDirectory);
 		autoPraatBackground background;
 		try {
@@ -554,14 +515,14 @@ void Manual_init (Manual me, const char32 *title, Daata data, bool ownData) {
 	if (! (i = ManPages_lookUp (manPages, title)))
 		Melder_throw (U"Page \"", title, U"\" not found.");
 	my path = i;
-	page = (ManPage) manPages -> pages -> item [i];
+	page = manPages -> pages.at [i];
 	my paragraphs = page -> paragraphs;
 	my numberOfParagraphs = 0;
 	par = my paragraphs;
 	while ((par ++) -> type) my numberOfParagraphs ++;
 
-	if (((ManPage) manPages -> pages -> item [1]) -> title [0] == '-') {
-		Melder_sprint (windowTitle,101, & ((ManPage) manPages -> pages -> item [1]) -> title [1]);
+	if (manPages -> pages.at [1] -> title [0] == U'-') {
+		Melder_sprint (windowTitle,101, & manPages -> pages.at [1] -> title [1]);
 		if (windowTitle [str32len (windowTitle) - 1] == U'-') windowTitle [str32len (windowTitle) - 1] = U'\0';
 	} else {
 		Melder_sprint (windowTitle,101, U"Manual");
@@ -572,11 +533,11 @@ void Manual_init (Manual me, const char32 *title, Daata data, bool ownData) {
 	my history [0]. page = Melder_dup_f (title);   // BAD
 }
 
-Manual Manual_create (const char32 *title, Daata data, bool ownData) {
+autoManual Manual_create (const char32 *title, Daata data, bool ownData) {
 	try {
 		autoManual me = Thing_new (Manual);
-		Manual_init (me.peek(), title, data, ownData);
-		return me.transfer();
+		Manual_init (me.get(), title, data, ownData);
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Manual window not created.");
 	}
