@@ -1,20 +1,19 @@
 /* ScriptEditor.cpp
  *
- * Copyright (C) 1997-2012,2013,2015 Paul Boersma
+ * Copyright (C) 1997-2012,2013,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ScriptEditor.h"
@@ -24,22 +23,20 @@
 
 Thing_implement (ScriptEditor, TextEditor, 0);
 
-static Collection theScriptEditors;
+static CollectionOf <structScriptEditor> theReferencesToAllOpenScriptEditors;
 
 bool ScriptEditors_dirty () {
-	if (! theScriptEditors) return false;
-	for (long i = 1; i <= theScriptEditors -> size; i ++) {
-		ScriptEditor me = (ScriptEditor) theScriptEditors -> item [i];
+	for (long i = 1; i <= theReferencesToAllOpenScriptEditors.size; i ++) {
+		ScriptEditor me = theReferencesToAllOpenScriptEditors.at [i];
 		if (my dirty) return true;
 	}
 	return false;
 }
 
-void structScriptEditor :: v_destroy () {
+void structScriptEditor :: v_destroy () noexcept {
 	Melder_free (environmentName);
-	forget (interpreter);
-	forget (argsDialog);
-	if (theScriptEditors) Collection_undangleItem (theScriptEditors, this);
+	our argsDialog.reset();   // don't delay till delete
+	theReferencesToAllOpenScriptEditors. undangleItem (this);
 	ScriptEditor_Parent :: v_destroy ();
 }
 
@@ -64,14 +61,10 @@ void structScriptEditor :: v_goAway () {
 	}
 }
 
-static void args_ok (UiForm sendingForm, int narg_dummy, Stackel args_dummy, const char32 *sendingString_dummy, Interpreter interpreter_dummy, const char32 *invokingButtonTitle, bool modified_dummy, I) {
+static void args_ok (UiForm sendingForm, int /* narg */, Stackel /* args */, const char32 * /* sendingString */,
+	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, void *void_me)
+{
 	iam (ScriptEditor);
-	(void) narg_dummy;
-	(void) args_dummy;
-	(void) sendingString_dummy;
-	(void) interpreter_dummy;
-	(void) invokingButtonTitle;
-	(void) modified_dummy;
 	autostring32 text = GuiText_getString (my textWidget);
 	structMelderFile file = { 0 };
 	if (my name [0]) {
@@ -80,23 +73,19 @@ static void args_ok (UiForm sendingForm, int narg_dummy, Stackel args_dummy, con
 	}
 	Melder_includeIncludeFiles (& text);
 
-	Interpreter_getArgumentsFromDialog (my interpreter, sendingForm);
+	Interpreter_getArgumentsFromDialog (my interpreter.get(), sendingForm);
 
 	autoPraatBackground background;
 	if (my name [0]) MelderFile_setDefaultDir (& file);
-	Interpreter_run (my interpreter, text.peek());
+	Interpreter_run (my interpreter.get(), text.peek());
 }
 
-static void args_ok_selectionOnly (UiForm sendingForm, int narg_dummy, Stackel args_dummy, const char32 *sendingString_dummy, Interpreter interpreter_dummy, const char32 *invokingButtonTitle, bool modified_dummy, I) {
+static void args_ok_selectionOnly (UiForm sendingForm, int /* narg */, Stackel /* args */, const char32 * /* sendingString */,
+	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, void *void_me)
+{
 	iam (ScriptEditor);
-	(void) narg_dummy;
-	(void) args_dummy;
-	(void) sendingString_dummy;
-	(void) interpreter_dummy;
-	(void) invokingButtonTitle;
-	(void) modified_dummy;
 	autostring32 text = GuiText_getSelection (my textWidget);
-	if (text.peek() == NULL)
+	if (! text.peek())
 		Melder_throw (U"No text is selected any longer.\nPlease reselect or click Cancel.");
 	structMelderFile file = { 0 };
 	if (my name [0]) {
@@ -105,15 +94,14 @@ static void args_ok_selectionOnly (UiForm sendingForm, int narg_dummy, Stackel a
 	}
 	Melder_includeIncludeFiles (& text);
 
-	Interpreter_getArgumentsFromDialog (my interpreter, sendingForm);
+	Interpreter_getArgumentsFromDialog (my interpreter.get(), sendingForm);
 
 	autoPraatBackground background;
 	if (my name [0]) MelderFile_setDefaultDir (& file);
-	Interpreter_run (my interpreter, text.peek());
+	Interpreter_run (my interpreter.get(), text.peek());
 }
 
-static void menu_cb_run (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_run (ScriptEditor me, EDITOR_ARGS_DIRECT) {
 	if (my interpreter -> running)
 		Melder_throw (U"The script is already running (paused). Please close or continue the pause or demo window.");
 	autostring32 text = GuiText_getString (my textWidget);
@@ -124,28 +112,26 @@ static void menu_cb_run (EDITOR_ARGS) {
 		MelderFile_setDefaultDir (& file);
 	}
 	Melder_includeIncludeFiles (& text);
-	int npar = Interpreter_readParameters (my interpreter, text.peek());
+	int npar = Interpreter_readParameters (my interpreter.get(), text.peek());
 	if (npar) {
 		/*
 		 * Pop up a dialog box for querying the arguments.
 		 */
-		forget (my argsDialog);
-		my argsDialog = Interpreter_createForm (my interpreter, my d_windowForm, NULL, args_ok, me, false);
-		UiForm_do (my argsDialog, false);
+		my argsDialog = autoUiForm (Interpreter_createForm (my interpreter.get(), my d_windowForm, nullptr, args_ok, me, false));
+		UiForm_do (my argsDialog.get(), false);
 	} else {
 		autoPraatBackground background;
 		if (my name [0]) MelderFile_setDefaultDir (& file);
 		trace (U"Running the following script (2):\n", text.peek());
-		Interpreter_run (my interpreter, text.peek());
+		Interpreter_run (my interpreter.get(), text.peek());
 	}
 }
 
-static void menu_cb_runSelection (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_runSelection (ScriptEditor me, EDITOR_ARGS_DIRECT) {
 	if (my interpreter -> running)
 		Melder_throw (U"The script is already running (paused). Please close or continue the pause or demo window.");
 	autostring32 text = GuiText_getSelection (my textWidget);
-	if (text.peek() == NULL)
+	if (! text.peek())
 		Melder_throw (U"No text selected.");
 	structMelderFile file = { 0 };
 	if (my name [0]) {
@@ -153,23 +139,21 @@ static void menu_cb_runSelection (EDITOR_ARGS) {
 		MelderFile_setDefaultDir (& file);
 	}
 	Melder_includeIncludeFiles (& text);
-	int npar = Interpreter_readParameters (my interpreter, text.peek());
+	int npar = Interpreter_readParameters (my interpreter.get(), text.peek());
 	if (npar) {
 		/*
 		 * Pop up a dialog box for querying the arguments.
 		 */
-		forget (my argsDialog);
-		my argsDialog = Interpreter_createForm (my interpreter, my d_windowForm, NULL, args_ok_selectionOnly, me, true);
-		UiForm_do (my argsDialog, false);
+		my argsDialog = autoUiForm (Interpreter_createForm (my interpreter.get(), my d_windowForm, nullptr, args_ok_selectionOnly, me, true));
+		UiForm_do (my argsDialog.get(), false);
 	} else {
 		autoPraatBackground background;
 		if (my name [0]) MelderFile_setDefaultDir (& file);
-		Interpreter_run (my interpreter, text.peek());
+		Interpreter_run (my interpreter.get(), text.peek());
 	}
 }
 
-static void menu_cb_addToMenu (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_addToMenu (ScriptEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Add to menu", U"Add to fixed menu...")
 		WORD (U"Window", U"?")
 		SENTENCE (U"Menu", U"File")
@@ -192,8 +176,7 @@ static void menu_cb_addToMenu (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_addToFixedMenu (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_addToFixedMenu (ScriptEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Add to fixed menu", U"Add to fixed menu...");
 		RADIO (U"Window", 1)
 			RADIOBUTTON (U"Objects")
@@ -217,8 +200,7 @@ static void menu_cb_addToFixedMenu (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_addToDynamicMenu (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_addToDynamicMenu (ScriptEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Add to dynamic menu", U"Add to dynamic menu...")
 		WORD (U"Class 1", U"Sound")
 		INTEGER (U"Number 1", U"0")
@@ -245,15 +227,13 @@ static void menu_cb_addToDynamicMenu (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_clearHistory (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_clearHistory (ScriptEditor /* me */, EDITOR_ARGS_DIRECT) {
 	UiHistory_clear ();
 }
 
-static void menu_cb_pasteHistory (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_pasteHistory (ScriptEditor me, EDITOR_ARGS_DIRECT) {
 	char32 *history = UiHistory_get ();
-	if (history == NULL || history [0] == U'\0')
+	if (! history || history [0] == U'\0')
 		Melder_throw (U"No history.");
 	long length = str32len (history);
 	if (history [length - 1] != U'\n') {
@@ -273,8 +253,7 @@ static void menu_cb_pasteHistory (EDITOR_ARGS) {
 	GuiText_scrollToSelection (my textWidget);
 }
 
-static void menu_cb_expandIncludeFiles (EDITOR_ARGS) {
-	EDITOR_IAM (ScriptEditor);
+static void menu_cb_expandIncludeFiles (ScriptEditor me, EDITOR_ARGS_DIRECT) {
 	structMelderFile file = { 0 };
 	autostring32 text = GuiText_getString (my textWidget);
 	if (my name [0]) {
@@ -285,16 +264,16 @@ static void menu_cb_expandIncludeFiles (EDITOR_ARGS) {
 	GuiText_setString (my textWidget, text.peek());
 }
 
-static void menu_cb_AboutScriptEditor (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"ScriptEditor"); }
-static void menu_cb_ScriptingTutorial (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Scripting"); }
-static void menu_cb_ScriptingExamples (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Scripting examples"); }
-static void menu_cb_PraatScript (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Praat script"); }
-static void menu_cb_FormulasTutorial (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Formulas"); }
-static void menu_cb_DemoWindow (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Demo window"); }
-static void menu_cb_TheHistoryMechanism (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"History mechanism"); }
-static void menu_cb_InitializationScripts (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Initialization script"); }
-static void menu_cb_AddingToAFixedMenu (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Add to fixed menu..."); }
-static void menu_cb_AddingToADynamicMenu (EDITOR_ARGS) { EDITOR_IAM (ScriptEditor); Melder_help (U"Add to dynamic menu..."); }
+static void menu_cb_AboutScriptEditor (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"ScriptEditor"); }
+static void menu_cb_ScriptingTutorial (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Scripting"); }
+static void menu_cb_ScriptingExamples (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Scripting examples"); }
+static void menu_cb_PraatScript (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Praat script"); }
+static void menu_cb_FormulasTutorial (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Formulas"); }
+static void menu_cb_DemoWindow (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Demo window"); }
+static void menu_cb_TheHistoryMechanism (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"History mechanism"); }
+static void menu_cb_InitializationScripts (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Initialization script"); }
+static void menu_cb_AddingToAFixedMenu (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Add to fixed menu..."); }
+static void menu_cb_AddingToADynamicMenu (ScriptEditor, EDITOR_ARGS_DIRECT) { Melder_help (U"Add to dynamic menu..."); }
 
 void structScriptEditor :: v_createMenus () {
 	ScriptEditor_Parent :: v_createMenus ();
@@ -304,11 +283,11 @@ void structScriptEditor :: v_createMenus () {
 		Editor_addCommand (this, U"File", U"Add to fixed menu...", 0, menu_cb_addToFixedMenu);
 		Editor_addCommand (this, U"File", U"Add to dynamic menu...", 0, menu_cb_addToDynamicMenu);
 	}
-	Editor_addCommand (this, U"File", U"-- close --", 0, NULL);
-	Editor_addCommand (this, U"Edit", U"-- history --", 0, 0);
+	Editor_addCommand (this, U"File", U"-- close --", 0, nullptr);
+	Editor_addCommand (this, U"Edit", U"-- history --", 0, nullptr);
 	Editor_addCommand (this, U"Edit", U"Clear history", 0, menu_cb_clearHistory);
 	Editor_addCommand (this, U"Edit", U"Paste history", 'H', menu_cb_pasteHistory);
-	Editor_addCommand (this, U"Convert", U"-- expand --", 0, 0);
+	Editor_addCommand (this, U"Convert", U"-- expand --", 0, nullptr);
 	Editor_addCommand (this, U"Convert", U"Expand include files", 0, menu_cb_expandIncludeFiles);
 	Editor_addMenu (this, U"Run", 0);
 	Editor_addCommand (this, U"Run", U"Run", 'R', menu_cb_run);
@@ -323,58 +302,52 @@ void structScriptEditor :: v_createHelpMenuItems (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"Praat script", 0, menu_cb_PraatScript);
 	EditorMenu_addCommand (menu, U"Formulas tutorial", 0, menu_cb_FormulasTutorial);
 	EditorMenu_addCommand (menu, U"Demo window", 0, menu_cb_DemoWindow);
-	EditorMenu_addCommand (menu, U"-- help history --", 0, NULL);
+	EditorMenu_addCommand (menu, U"-- help history --", 0, nullptr);
 	EditorMenu_addCommand (menu, U"The History mechanism", 0, menu_cb_TheHistoryMechanism);
 	EditorMenu_addCommand (menu, U"Initialization scripts", 0, menu_cb_InitializationScripts);
-	EditorMenu_addCommand (menu, U"-- help add --", 0, NULL);
+	EditorMenu_addCommand (menu, U"-- help add --", 0, nullptr);
 	EditorMenu_addCommand (menu, U"Adding to a fixed menu", 0, menu_cb_AddingToAFixedMenu);
 	EditorMenu_addCommand (menu, U"Adding to a dynamic menu", 0, menu_cb_AddingToADynamicMenu);
 }
 
 void ScriptEditor_init (ScriptEditor me, Editor environment, const char32 *initialText) {
-	if (environment != NULL) {
+	if (environment) {
 		my environmentName = Melder_dup (environment -> name);
 		my editorClass = environment -> classInfo;
 	}
 	TextEditor_init (me, initialText);
 	my interpreter = Interpreter_createFromEnvironment (environment);
-	if (theScriptEditors == NULL) {
-		theScriptEditors = Collection_create (NULL, 10);
-		Collection_dontOwnItems (theScriptEditors);
-	}
-	Collection_addItem (theScriptEditors, me);
+	theReferencesToAllOpenScriptEditors. addItem_ref (me);
 }
 
-ScriptEditor ScriptEditor_createFromText (Editor environment, const char32 *initialText) {
+autoScriptEditor ScriptEditor_createFromText (Editor environment, const char32 *initialText) {
 	try {
 		autoScriptEditor me = Thing_new (ScriptEditor);
-		ScriptEditor_init (me.peek(), environment, initialText);
-		return me.transfer();
+		ScriptEditor_init (me.get(), environment, initialText);
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Script window not created.");
 	}
 }
 
-ScriptEditor ScriptEditor_createFromScript (Editor environment, Script script) {
+autoScriptEditor ScriptEditor_createFromScript_canBeNull (Editor environment, Script script) {
 	try {
-		if (theScriptEditors) {
-			for (long ieditor = 1; ieditor <= theScriptEditors -> size; ieditor ++) {
-				ScriptEditor editor = (ScriptEditor) theScriptEditors -> item [ieditor];
-				if (MelderFile_equal (& script -> file, & editor -> file)) {
-					Editor_raise (editor);
-					Melder_appendError (U"The script ", & script -> file, U" is already open and has been moved to the front.");
-					if (editor -> dirty)
-						Melder_appendError (U"Choose \"Reopen from disk\" if you want to revert to the old version.");
-					Melder_flushError ();
-					return NULL;   // safe NULL
-				}
+		for (long ieditor = 1; ieditor <= theReferencesToAllOpenScriptEditors.size; ieditor ++) {
+			ScriptEditor editor = theReferencesToAllOpenScriptEditors.at [ieditor];
+			if (MelderFile_equal (& script -> file, & editor -> file)) {
+				Editor_raise (editor);
+				Melder_appendError (U"The script ", & script -> file, U" is already open and has been moved to the front.");
+				if (editor -> dirty)
+					Melder_appendError (U"Choose \"Reopen from disk\" if you want to revert to the old version.");
+				Melder_flushError ();
+				return autoScriptEditor();   // safe null
 			}
 		}
 		autostring32 text = MelderFile_readText (& script -> file);
 		autoScriptEditor me = ScriptEditor_createFromText (environment, text.peek());
 		MelderFile_copy (& script -> file, & my file);
-		Thing_setName (me.peek(), Melder_fileToPath (& script -> file));
-		return me.transfer();
+		Thing_setName (me.get(), Melder_fileToPath (& script -> file));
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Script window not created.");
 	}
