@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1992-2012,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 //#define USE_STAT  1
@@ -91,17 +90,17 @@ void structStrings :: v_info () {
 const char32 * structStrings :: v_getVectorStr (long icol) {
 	if (icol < 1 || icol > our numberOfStrings) return U"";
 	char32 *stringValue = strings [icol];
-	return stringValue == NULL ? U"" : stringValue;
+	return stringValue ? stringValue : U"";
 }
 
 #define Strings_createAsFileOrDirectoryList_TYPE_FILE  0
 #define Strings_createAsFileOrDirectoryList_TYPE_DIRECTORY  1
-static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattable */, int type) {
+static autoStrings Strings_createAsFileOrDirectoryList (const char32 *path /* cattable */, int type) {
 	#if USE_STAT
 		/*
 		 * Initialize.
 		 */
-		DIR *d = NULL;
+		DIR *d = nullptr;
 		try {
 			autoMelderString filePath, searchDirectory, left, right;
 			/*
@@ -112,11 +111,11 @@ static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattab
 			 */
 			MelderString_copy (& searchDirectory, path);
 			char32 *asterisk = str32rchr (searchDirectory. string, '*');
-			if (asterisk != NULL) {
+			if (asterisk) {
 				*asterisk = '\0';
 				searchDirectory. length = asterisk - searchDirectory. string;   // probably superfluous, but correct
 				char32 *lastSlash = str32rchr (searchDirectory. string, Melder_DIRECTORY_SEPARATOR);
-				if (lastSlash != NULL) {
+				if (lastSlash) {
 					*lastSlash = '\0';   // This fixes searchDirectory.
 					searchDirectory. length = lastSlash - searchDirectory. string;   // probably superfluous, but correct
 					MelderString_copy (& left, lastSlash + 1);
@@ -126,19 +125,19 @@ static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattab
 				}
 				MelderString_copy (& right, asterisk + 1);
 			}
-			char buffer8 [1+kMelder_MAXPATH];
+			char buffer8 [kMelder_MAXPATH+1];
 			Melder_str32To8bitFileRepresentation_inline (searchDirectory. string, buffer8);
 			d = opendir (buffer8 [0] ? buffer8 : ".");
-			if (d == NULL)
+			if (! d)
 				Melder_throw (U"Cannot open directory ", searchDirectory. string, U".");
 			//Melder_casual (U"opened");
 			autoStrings me = Thing_new (Strings);
 			my strings = NUMvector <char32 *> (1, 1000000);
 			struct dirent *entry;
-			while ((entry = readdir (d)) != NULL) {
+			while (!! (entry = readdir (d))) {
 				MelderString_copy (& filePath, searchDirectory. string [0] ? searchDirectory. string : U".");
 				MelderString_appendCharacter (& filePath, Melder_DIRECTORY_SEPARATOR);
-				char32 buffer32 [1+kMelder_MAXPATH];
+				char32 buffer32 [kMelder_MAXPATH+1];
 				Melder_8bitFileRepresentationToStr32_inline (entry -> d_name, buffer32);
 				MelderString_append (& filePath, buffer32);
 				//Melder_casual (U"read ", filePath. string);
@@ -154,7 +153,7 @@ static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattab
 					(type == Strings_createAsFileOrDirectoryList_TYPE_DIRECTORY && S_ISDIR (stats. st_mode)))
 				{
 					Melder_8bitFileRepresentationToStr32_inline (entry -> d_name, buffer32);
-					unsigned long length = str32len (buffer32);
+					int64 length = str32len (buffer32);
 					if (buffer32 [0] != U'.' &&
 						(left. length == 0 || str32nequ (buffer32, left. string, left. length)) &&
 						(right. length == 0 || (length >= right. length && str32equ (buffer32 + (length - right. length), right. string))))
@@ -164,20 +163,21 @@ static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattab
 				}
 			}
 			closedir (d);
-			Strings_sort (me.peek());
-			return me.transfer();
+			Strings_sort (me.get());
+			return me;
 		} catch (MelderError) {
 			if (d) closedir (d);   // "finally"
 			throw;
 		}
 	#elif defined (_WIN32)
 		try {
-			char32 searchPath [1+kMelder_MAXPATH];
-			int len = str32len (path), hasAsterisk = str32chr (path, U'*') != NULL;
+			char32 searchPath [kMelder_MAXPATH+1];
+			int len = str32len (path);
+			bool hasAsterisk = !! str32chr (path, U'*');
 			bool endsInSeparator = ( len != 0 && path [len - 1] == U'\\' );
 			autoStrings me = Thing_new (Strings);
 			my strings = NUMvector <char32 *> (1, 1000000);
-			Melder_sprint (searchPath, 1+kMelder_MAXPATH, path, hasAsterisk || endsInSeparator ? U"" : U"\\", hasAsterisk ? U"" : U"*");
+			Melder_sprint (searchPath, kMelder_MAXPATH+1, path, hasAsterisk || endsInSeparator ? U"" : U"\\", hasAsterisk ? U"" : U"*");
 			WIN32_FIND_DATAW findData;
 			HANDLE searchHandle = FindFirstFileW (Melder_peek32toW (searchPath), & findData);
 			if (searchHandle != INVALID_HANDLE_VALUE) {
@@ -194,15 +194,15 @@ static Strings Strings_createAsFileOrDirectoryList (const char32 *path /* cattab
 				} while (FindNextFileW (searchHandle, & findData));
 				FindClose (searchHandle);
 			}
-			Strings_sort (me.peek());
-			return me.transfer();
+			Strings_sort (me.get());
+			return me;
 		} catch (MelderError) {
 			throw;
 		}
 	#endif
 }
 
-Strings Strings_createAsFileList (const char32 *path /* cattable */) {
+autoStrings Strings_createAsFileList (const char32 *path /* cattable */) {
 	try {
 		return Strings_createAsFileOrDirectoryList (path, Strings_createAsFileOrDirectoryList_TYPE_FILE);
 	} catch (MelderError) {
@@ -210,7 +210,7 @@ Strings Strings_createAsFileList (const char32 *path /* cattable */) {
 	}
 }
 
-Strings Strings_createAsDirectoryList (const char32 *path /* cattable */) {
+autoStrings Strings_createAsDirectoryList (const char32 *path /* cattable */) {
 	try {
 		return Strings_createAsFileOrDirectoryList (path, Strings_createAsFileOrDirectoryList_TYPE_DIRECTORY);
 	} catch (MelderError) {
@@ -218,7 +218,7 @@ Strings Strings_createAsDirectoryList (const char32 *path /* cattable */) {
 	}
 }
 
-Strings Strings_readFromRawTextFile (MelderFile file) {
+autoStrings Strings_readFromRawTextFile (MelderFile file) {
 	try {
 		autoMelderReadText text = MelderReadText_createFromFile (file);
 
@@ -241,7 +241,7 @@ Strings Strings_readFromRawTextFile (MelderFile file) {
 			char32 *line = MelderReadText_readLine (text.peek());
 			my strings [i] = Melder_dup (line);
 		}
-		return me.transfer();
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Strings not read from raw text file ", file, U".");
 	}
