@@ -1,20 +1,19 @@
 /* TextEditor.cpp
  *
- * Copyright (C) 1997-2012,2013,2015 Paul Boersma, 2010 Franz Brausse
+ * Copyright (C) 1997-2012,2013,2015,2016 Paul Boersma, 2010 Franz Brausse
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "TextEditor.h"
@@ -32,18 +31,14 @@ Thing_implement (TextEditor, Editor, 0);
 #include "prefs_copyToInstance.h"
 #include "TextEditor_prefs.h"
 
-static Collection theOpenTextEditors = NULL;
+static CollectionOf <structTextEditor> theReferencesToAllOpenTextEditors;
 
 /***** TextEditor methods *****/
 
-void structTextEditor :: v_destroy () {
-	forget (our openDialog);
-	forget (our saveDialog);
-	forget (our printDialog);
-	forget (our findDialog);
-	if (theOpenTextEditors) {
-		Collection_undangleItem (theOpenTextEditors, this);
-	}
+void structTextEditor :: v_destroy () noexcept {
+	our openDialog.reset();   // don't delay till delete
+	our saveDialog.reset();   // don't delay till delete
+	theReferencesToAllOpenTextEditors. undangleItem (this);
 	TextEditor_Parent :: v_destroy ();
 }
 
@@ -69,16 +64,14 @@ void structTextEditor :: v_nameChanged () {
 }
 
 static void openDocument (TextEditor me, MelderFile file) {
-	if (theOpenTextEditors) {
-		for (long ieditor = 1; ieditor <= theOpenTextEditors -> size; ieditor ++) {
-			TextEditor editor = (TextEditor) theOpenTextEditors -> item [ieditor];
-			if (editor != me && MelderFile_equal (file, & editor -> file)) {
-				Editor_raise (editor);
-				Melder_appendError (U"Text file ", file, U" is already open.");
-				forget (me);   // don't forget me before Melder_appendError, because "file" is owned by one of my dialogs
-				Melder_flushError ();
-				return;
-			}
+	for (long ieditor = 1; ieditor <= theReferencesToAllOpenTextEditors.size; ieditor ++) {
+		TextEditor editor = theReferencesToAllOpenTextEditors.at [ieditor];
+		if (editor != me && MelderFile_equal (file, & editor -> file)) {
+			Editor_raise (editor);
+			Melder_appendError (U"Text file ", file, U" is already open.");
+			forget (me);   // don't forget me before Melder_appendError, because "file" is owned by one of my dialogs
+			Melder_flushError ();
+			return;
 		}
 	}
 	autostring32 text = MelderFile_readText (file);
@@ -111,40 +104,37 @@ static void closeDocument (TextEditor me) {
 }
 
 static void cb_open_ok (UiForm sendingForm, int /* narg */, Stackel /* args */, const char32 * /* sendingString */,
-	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, I)
+	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, void *void_me)
 {
 	iam (TextEditor);
 	MelderFile file = UiFile_getFile (sendingForm);
 	openDocument (me, file);
 }
 
-static void cb_showOpen (EditorCommand cmd, UiForm /* sendingForm */, const char32 * /* sendingString */, Interpreter /* interpreter */) {
+static void cb_showOpen (EditorCommand cmd) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	if (! my openDialog)
-		my openDialog = UiInfile_create (my d_windowForm, U"Open", cb_open_ok, me, NULL, NULL, false);
-	UiInfile_do (my openDialog);
+		my openDialog = autoUiForm (UiInfile_create (my d_windowForm, U"Open", cb_open_ok, me, nullptr, nullptr, false));
+	UiInfile_do (my openDialog.get());
 }
 
 static void cb_saveAs_ok (UiForm sendingForm, int /* narg */, Stackel /* args */, const char32 * /* sendingString */,
-	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, I)
+	Interpreter /* interpreter */, const char32 * /* invokingButtonTitle */, bool /* modified */, void *void_me)
 {
 	iam (TextEditor);
 	MelderFile file = UiFile_getFile (sendingForm);
 	saveDocument (me, file);
 }
 
-static void menu_cb_saveAs (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_saveAs (TextEditor me, EDITOR_ARGS_DIRECT) {
 	if (! my saveDialog)
-		my saveDialog = UiOutfile_create (my d_windowForm, U"Save", cb_saveAs_ok, me, NULL, NULL);
+		my saveDialog = autoUiForm (UiOutfile_create (my d_windowForm, U"Save", cb_saveAs_ok, me, nullptr, nullptr));
 	char32 defaultName [300];
 	Melder_sprint (defaultName,300, ! my v_fileBased () ? U"info.txt" : my name [0] ? MelderFile_name (& my file) : U"");
-	UiOutfile_do (my saveDialog, defaultName);
+	UiOutfile_do (my saveDialog.get(), defaultName);
 }
 
-static void gui_button_cb_saveAndOpen (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_saveAndOpen (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyOpenDialog);
 	if (my name [0]) {
@@ -154,37 +144,32 @@ static void gui_button_cb_saveAndOpen (I, GuiButtonEvent event) {
 			Melder_flushError ();
 			return;
 		}
-		cb_showOpen (cmd, NULL, NULL, NULL);
+		cb_showOpen (cmd);
 	} else {
-		menu_cb_saveAs (me, cmd, NULL, 0, NULL, NULL, NULL);
+		menu_cb_saveAs (me, cmd, nullptr, 0, nullptr, nullptr, nullptr);
 	}
 }
 
-static void gui_button_cb_cancelOpen (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_cancelOpen (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyOpenDialog);
 }
 
-static void gui_button_cb_discardAndOpen (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_discardAndOpen (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyOpenDialog);
-	cb_showOpen (cmd, NULL, NULL, NULL);
+	cb_showOpen (cmd);
 }
 
-static void menu_cb_open (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_open (TextEditor me, EDITOR_ARGS_CMD) {
 	if (my dirty) {
-		if (my dirtyOpenDialog == NULL) {
+		if (! my dirtyOpenDialog) {
 			int buttonWidth = 120, buttonSpacing = 20;
 			my dirtyOpenDialog = GuiDialog_create (my d_windowForm,
 				150, 70,
 				Gui_LEFT_DIALOG_SPACING + 3 * buttonWidth + 2 * buttonSpacing + Gui_RIGHT_DIALOG_SPACING,
 				Gui_TOP_DIALOG_SPACING + Gui_TEXTFIELD_HEIGHT + Gui_VERTICAL_DIALOG_SPACING_SAME + 2 * Gui_BOTTOM_DIALOG_SPACING + Gui_PUSHBUTTON_HEIGHT,
-				U"Text changed", NULL, NULL, GuiDialog_MODAL);
+				U"Text changed", nullptr, nullptr, GuiDialog_MODAL);
 			GuiLabel_createShown (my dirtyOpenDialog,
 				Gui_LEFT_DIALOG_SPACING, - Gui_RIGHT_DIALOG_SPACING,
 				Gui_TOP_DIALOG_SPACING, Gui_TOP_DIALOG_SPACING + Gui_LABEL_HEIGHT,
@@ -204,13 +189,11 @@ static void menu_cb_open (EDITOR_ARGS) {
 		}
 		GuiThing_show (my dirtyOpenDialog);
 	} else {
-		cb_showOpen (cmd, sendingForm, sendingString, interpreter);
+		cb_showOpen (cmd);
 	}
 }
 
-static void gui_button_cb_saveAndNew (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_saveAndNew (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyNewDialog);
 	if (my name [0]) {
@@ -222,34 +205,29 @@ static void gui_button_cb_saveAndNew (I, GuiButtonEvent event) {
 		}
 		newDocument (me);
 	} else {
-		menu_cb_saveAs (me, cmd, NULL, 0, NULL, NULL, NULL);
+		menu_cb_saveAs (me, cmd, nullptr, 0, nullptr, nullptr, nullptr);
 	}
 }
 
-static void gui_button_cb_cancelNew (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_cancelNew (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyNewDialog);
 }
 
-static void gui_button_cb_discardAndNew (I, GuiButtonEvent event) {
-	(void) event;
-	EditorCommand cmd = (EditorCommand) void_me;
+static void gui_button_cb_discardAndNew (EditorCommand cmd, GuiButtonEvent /* event */) {
 	TextEditor me = (TextEditor) cmd -> d_editor;
 	GuiThing_hide (my dirtyNewDialog);
 	newDocument (me);
 }
 
-static void menu_cb_new (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_new (TextEditor me, EDITOR_ARGS_CMD) {
 	if (my v_fileBased () && my dirty) {
 		if (! my dirtyNewDialog) {
 			int buttonWidth = 120, buttonSpacing = 20;
 			my dirtyNewDialog = GuiDialog_create (my d_windowForm,
 				150, 70, Gui_LEFT_DIALOG_SPACING + 3 * buttonWidth + 2 * buttonSpacing + Gui_RIGHT_DIALOG_SPACING,
 					Gui_TOP_DIALOG_SPACING + Gui_TEXTFIELD_HEIGHT + Gui_VERTICAL_DIALOG_SPACING_SAME + 2 * Gui_BOTTOM_DIALOG_SPACING + Gui_PUSHBUTTON_HEIGHT,
-				U"Text changed", NULL, NULL, GuiDialog_MODAL);
+				U"Text changed", nullptr, nullptr, GuiDialog_MODAL);
 			GuiLabel_createShown (my dirtyNewDialog,
 				Gui_LEFT_DIALOG_SPACING, - Gui_RIGHT_DIALOG_SPACING,
 				Gui_TOP_DIALOG_SPACING, Gui_TOP_DIALOG_SPACING + Gui_LABEL_HEIGHT,
@@ -273,13 +251,11 @@ static void menu_cb_new (EDITOR_ARGS) {
 	}
 }
 
-static void menu_cb_clear (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_clear (TextEditor me, EDITOR_ARGS_DIRECT) {
 	my v_clear ();
 }
 
-static void menu_cb_save (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_save (TextEditor me, EDITOR_ARGS_CMD) {
 	if (my name [0]) {
 		try {
 			saveDocument (me, & my file);
@@ -288,12 +264,11 @@ static void menu_cb_save (EDITOR_ARGS) {
 			return;
 		}
 	} else {
-		menu_cb_saveAs (me, cmd, NULL, 0, NULL, NULL, NULL);
+		menu_cb_saveAs (me, cmd, nullptr, 0, nullptr, nullptr, nullptr);
 	}
 }
 
-static void menu_cb_reopen (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_reopen (TextEditor me, EDITOR_ARGS_DIRECT) {
 	if (my name [0]) {
 		try {
 			openDocument (me, & my file);
@@ -306,9 +281,7 @@ static void menu_cb_reopen (EDITOR_ARGS) {
 	}
 }
 
-static void gui_button_cb_saveAndClose (I, GuiButtonEvent event) {
-	(void) event;
-	iam (TextEditor);
+static void gui_button_cb_saveAndClose (TextEditor me, GuiButtonEvent /* event */) {
 	GuiThing_hide (my dirtyCloseDialog);
 	if (my name [0]) {
 		try {
@@ -319,19 +292,15 @@ static void gui_button_cb_saveAndClose (I, GuiButtonEvent event) {
 		}
 		closeDocument (me);
 	} else {
-		menu_cb_saveAs (me, Editor_getMenuCommand (me, U"File", U"Save as..."), NULL, 0, NULL, NULL, NULL);
+		menu_cb_saveAs (me, Editor_getMenuCommand (me, U"File", U"Save as..."), nullptr, 0, nullptr, nullptr, nullptr);
 	}
 }
 
-static void gui_button_cb_cancelClose (I, GuiButtonEvent event) {
-	(void) event;
-	iam (TextEditor);
+static void gui_button_cb_cancelClose (TextEditor me, GuiButtonEvent /* event */) {
 	GuiThing_hide (my dirtyCloseDialog);
 }
 
-static void gui_button_cb_discardAndClose (I, GuiButtonEvent event) {
-	(void) event;
-	iam (TextEditor);
+static void gui_button_cb_discardAndClose (TextEditor me, GuiButtonEvent /* event */) {
 	GuiThing_hide (my dirtyCloseDialog);
 	closeDocument (me);
 }
@@ -343,7 +312,7 @@ void structTextEditor :: v_goAway () {
 			dirtyCloseDialog = GuiDialog_create (d_windowForm,
 				150, 70, Gui_LEFT_DIALOG_SPACING + 3 * buttonWidth + 2 * buttonSpacing + Gui_RIGHT_DIALOG_SPACING,
 					Gui_TOP_DIALOG_SPACING + Gui_TEXTFIELD_HEIGHT + Gui_VERTICAL_DIALOG_SPACING_SAME + 2 * Gui_BOTTOM_DIALOG_SPACING + Gui_PUSHBUTTON_HEIGHT,
-				U"Text changed", NULL, NULL, GuiDialog_MODAL);
+				U"Text changed", nullptr, nullptr, GuiDialog_MODAL);
 			GuiLabel_createShown (dirtyCloseDialog,
 				Gui_LEFT_DIALOG_SPACING, - Gui_RIGHT_DIALOG_SPACING,
 				Gui_TOP_DIALOG_SPACING, Gui_TOP_DIALOG_SPACING + Gui_LABEL_HEIGHT,
@@ -367,33 +336,27 @@ void structTextEditor :: v_goAway () {
 	}
 }
 
-static void menu_cb_undo (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_undo (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_undo (my textWidget);
 }
 
-static void menu_cb_redo (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_redo (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_redo (my textWidget);
 }
 
-static void menu_cb_cut (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_cut (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_cut (my textWidget);  // use ((XmAnyCallbackStruct *) call) -> event -> xbutton. time
 }
 
-static void menu_cb_copy (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_copy (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_copy (my textWidget);
 }
 
-static void menu_cb_paste (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_paste (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_paste (my textWidget);
 }
 
-static void menu_cb_erase (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_erase (TextEditor me, EDITOR_ARGS_DIRECT) {
 	GuiText_remove (my textWidget);
 }
 
@@ -425,13 +388,13 @@ static bool getSelectedLines (TextEditor me, long *firstLine, long *lastLine) {
 	return true;
 }
 
-static char32 *theFindString = NULL, *theReplaceString = NULL;
+static char32 *theFindString = nullptr, *theReplaceString = nullptr;
 static void do_find (TextEditor me) {
-	if (theFindString == NULL) return;   // e.g. when the user does "Find again" before having done any "Find"
+	if (! theFindString) return;   // e.g. when the user does "Find again" before having done any "Find"
 	long left, right;
 	autostring32 text = GuiText_getStringAndSelectionPosition (my textWidget, & left, & right);
 	char32 *location = str32str (& text [right], theFindString);
-	if (location != NULL) {
+	if (location) {
 		long index = location - text.peek();
 		GuiText_setSelection (my textWidget, index, index + str32len (theFindString));
 		GuiText_scrollToSelection (my textWidget);
@@ -441,7 +404,7 @@ static void do_find (TextEditor me) {
 	} else {
 		/* Try from the start of the document. */
 		location = str32str (text.peek(), theFindString);
-		if (location != NULL) {
+		if (location) {
 			long index = location - text.peek();
 			GuiText_setSelection (my textWidget, index, index + str32len (theFindString));
 			GuiText_scrollToSelection (my textWidget);
@@ -455,7 +418,7 @@ static void do_find (TextEditor me) {
 }
 
 static void do_replace (TextEditor me) {
-	if (theReplaceString == NULL) return;   // e.g. when the user does "Replace again" before having done any "Replace"
+	if (! theReplaceString) return;   // e.g. when the user does "Replace again" before having done any "Replace"
 	autostring32 selection = GuiText_getSelection (my textWidget);
 	if (! Melder_equ (selection.peek(), theFindString)) {
 		do_find (me);
@@ -471,13 +434,12 @@ static void do_replace (TextEditor me) {
 	#endif
 }
 
-static void menu_cb_find (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_find (TextEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Find", 0)
 		LABEL (U"", U"Find:")
 		TEXTFIELD (U"findString", U"")
 	EDITOR_OK
-		if (theFindString != NULL) SET_STRING (U"findString", theFindString);
+		if (theFindString) SET_STRING (U"findString", theFindString);
 	EDITOR_DO
 		Melder_free (theFindString);
 		theFindString = Melder_dup_f (GET_STRING (U"findString"));
@@ -485,13 +447,11 @@ static void menu_cb_find (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_findAgain (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_findAgain (TextEditor me, EDITOR_ARGS_DIRECT) {
 	do_find (me);
 }
 
-static void menu_cb_replace (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_replace (TextEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Find", 0)
 		LABEL (U"", U"This is a \"slow\" find-and-replace method;")
 		LABEL (U"", U"if the selected text is identical to the Find string,")
@@ -503,8 +463,8 @@ static void menu_cb_replace (EDITOR_ARGS) {
 		LABEL (U"", U"Replace with:")
 		TEXTFIELD (U"replaceString", U"")
 	EDITOR_OK
-		if (theFindString != NULL) SET_STRING (U"findString", theFindString);
-		if (theReplaceString != NULL) SET_STRING (U"replaceString", theReplaceString);
+		if (theFindString) SET_STRING (U"findString", theFindString);
+		if (theReplaceString) SET_STRING (U"replaceString", theReplaceString);
 	EDITOR_DO
 		Melder_free (theFindString);
 		theFindString = Melder_dup (GET_STRING (U"findString"));
@@ -514,13 +474,11 @@ static void menu_cb_replace (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_replaceAgain (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_replaceAgain (TextEditor me, EDITOR_ARGS_DIRECT) {
 	do_replace (me);
 }
 
-static void menu_cb_whereAmI (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_whereAmI (TextEditor me, EDITOR_ARGS_DIRECT) {
 	long numberOfLinesLeft, numberOfLinesRight;
 	if (! getSelectedLines (me, & numberOfLinesLeft, & numberOfLinesRight)) {
 		Melder_information (U"The cursor is on line ", numberOfLinesLeft, U".");
@@ -531,8 +489,7 @@ static void menu_cb_whereAmI (EDITOR_ARGS) {
 	}
 }
 
-static void menu_cb_goToLine (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_goToLine (TextEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Go to line", 0)
 		NATURAL (U"Line", U"1")
 	EDITOR_OK
@@ -567,8 +524,7 @@ static void menu_cb_goToLine (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void menu_cb_convertToCString (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
+static void menu_cb_convertToCString (TextEditor me, EDITOR_ARGS_DIRECT) {
 	autostring32 text = GuiText_getString (my textWidget);
 	char32 buffer [2] = U" ";
 	const char32 *hex [16] = { U"0", U"1", U"2", U"3", U"4", U"5", U"6", U"7", U"8", U"9", U"A", U"B", U"C", U"D", U"E", U"F" };
@@ -615,14 +571,13 @@ static void setFontSize (TextEditor me, int fontSize) {
 	updateSizeMenu (me);
 }
 
-static void menu_cb_10 (EDITOR_ARGS) { EDITOR_IAM (TextEditor); setFontSize (me, 10); }
-static void menu_cb_12 (EDITOR_ARGS) { EDITOR_IAM (TextEditor); setFontSize (me, 12); }
-static void menu_cb_14 (EDITOR_ARGS) { EDITOR_IAM (TextEditor); setFontSize (me, 14); }
-static void menu_cb_18 (EDITOR_ARGS) { EDITOR_IAM (TextEditor); setFontSize (me, 18); }
-static void menu_cb_24 (EDITOR_ARGS) { EDITOR_IAM (TextEditor); setFontSize (me, 24); }
-static void menu_cb_fontSize (EDITOR_ARGS) {
-	EDITOR_IAM (TextEditor);
-	EDITOR_FORM (U"Text window: Font size", 0)
+static void menu_cb_10 (TextEditor me, EDITOR_ARGS_DIRECT) { setFontSize (me, 10); }
+static void menu_cb_12 (TextEditor me, EDITOR_ARGS_DIRECT) { setFontSize (me, 12); }
+static void menu_cb_14 (TextEditor me, EDITOR_ARGS_DIRECT) { setFontSize (me, 14); }
+static void menu_cb_18 (TextEditor me, EDITOR_ARGS_DIRECT) { setFontSize (me, 18); }
+static void menu_cb_24 (TextEditor me, EDITOR_ARGS_DIRECT) { setFontSize (me, 24); }
+static void menu_cb_fontSize (TextEditor me, EDITOR_ARGS_FORM) {
+	EDITOR_FORM (U"Text window: Font size", nullptr)
 		NATURAL (U"Font size (points)", U"12")
 	EDITOR_OK
 		SET_INTEGER (U"Font size", (long) my p_fontSize);
@@ -631,9 +586,7 @@ static void menu_cb_fontSize (EDITOR_ARGS) {
 	EDITOR_END
 }
 
-static void gui_text_cb_change (I, GuiTextEvent event) {
-	(void) event;
-	iam (TextEditor);
+static void gui_text_cb_changed (TextEditor me, GuiTextEvent /* event */) {
 	if (! my dirty) {
 		my dirty = true;
 		my v_nameChanged ();
@@ -642,11 +595,12 @@ static void gui_text_cb_change (I, GuiTextEvent event) {
 
 void structTextEditor :: v_createChildren () {
 	textWidget = GuiText_createShown (d_windowForm, 0, 0, Machine_getMenuBarHeight (), 0, GuiText_SCROLLED);
-	GuiText_setChangeCallback (textWidget, gui_text_cb_change, this);
+	GuiText_setChangedCallback (textWidget, gui_text_cb_changed, this);
 }
 
 void structTextEditor :: v_createMenus () {
 	TextEditor_Parent :: v_createMenus ();
+
 	if (v_fileBased ()) {
 		Editor_addCommand (this, U"File", U"New", 'N', menu_cb_new);
 		Editor_addCommand (this, U"File", U"Open...", 'O', menu_cb_open);
@@ -654,71 +608,66 @@ void structTextEditor :: v_createMenus () {
 	} else {
 		Editor_addCommand (this, U"File", U"Clear", 'N', menu_cb_clear);
 	}
-	Editor_addCommand (this, U"File", U"-- save --", 0, NULL);
+	Editor_addCommand (this, U"File", U"-- save --", 0, nullptr);
 	if (v_fileBased ()) {
 		Editor_addCommand (this, U"File", U"Save", 'S', menu_cb_save);
 		Editor_addCommand (this, U"File", U"Save as...", 0, menu_cb_saveAs);
 	} else {
 		Editor_addCommand (this, U"File", U"Save as...", 'S', menu_cb_saveAs);
 	}
-	Editor_addCommand (this, U"File", U"-- close --", 0, NULL);
+	Editor_addCommand (this, U"File", U"-- close --", 0, nullptr);
 	GuiText_setUndoItem (textWidget, Editor_addCommand (this, U"Edit", U"Undo", 'Z', menu_cb_undo));
 	GuiText_setRedoItem (textWidget, Editor_addCommand (this, U"Edit", U"Redo", 'Y', menu_cb_redo));
-	Editor_addCommand (this, U"Edit", U"-- cut copy paste --", 0, NULL);
+	Editor_addCommand (this, U"Edit", U"-- cut copy paste --", 0, nullptr);
 	Editor_addCommand (this, U"Edit", U"Cut", 'X', menu_cb_cut);
 	Editor_addCommand (this, U"Edit", U"Copy", 'C', menu_cb_copy);
 	Editor_addCommand (this, U"Edit", U"Paste", 'V', menu_cb_paste);
 	Editor_addCommand (this, U"Edit", U"Erase", 0, menu_cb_erase);
+
 	Editor_addMenu (this, U"Search", 0);
 	Editor_addCommand (this, U"Search", U"Find...", 'F', menu_cb_find);
 	Editor_addCommand (this, U"Search", U"Find again", 'G', menu_cb_findAgain);
 	Editor_addCommand (this, U"Search", U"Replace...", GuiMenu_SHIFT + 'F', menu_cb_replace);
 	Editor_addCommand (this, U"Search", U"Replace again", GuiMenu_SHIFT + 'G', menu_cb_replaceAgain);
-	Editor_addCommand (this, U"Search", U"-- line --", 0, NULL);
+	Editor_addCommand (this, U"Search", U"-- line --", 0, nullptr);
 	Editor_addCommand (this, U"Search", U"Where am I?", 0, menu_cb_whereAmI);
 	Editor_addCommand (this, U"Search", U"Go to line...", 'L', menu_cb_goToLine);
+
 	Editor_addMenu (this, U"Convert", 0);
 	Editor_addCommand (this, U"Convert", U"Convert to C string", 0, menu_cb_convertToCString);
-	#if defined (macintosh) || defined (UNIX) || defined (_WIN32)
-		Editor_addMenu (this, U"Font", 0);
-		Editor_addCommand (this, U"Font", U"Font size...", 0, menu_cb_fontSize);
-		fontSizeButton_10 = Editor_addCommand (this, U"Font", U"10", GuiMenu_CHECKBUTTON, menu_cb_10);
-		fontSizeButton_12 = Editor_addCommand (this, U"Font", U"12", GuiMenu_CHECKBUTTON, menu_cb_12);
-		fontSizeButton_14 = Editor_addCommand (this, U"Font", U"14", GuiMenu_CHECKBUTTON, menu_cb_14);
-		fontSizeButton_18 = Editor_addCommand (this, U"Font", U"18", GuiMenu_CHECKBUTTON, menu_cb_18);
-		fontSizeButton_24 = Editor_addCommand (this, U"Font", U"24", GuiMenu_CHECKBUTTON, menu_cb_24);
-	#endif
+
+	Editor_addMenu (this, U"Font", 0);
+	Editor_addCommand (this, U"Font", U"Font size...", 0, menu_cb_fontSize);
+	fontSizeButton_10 = Editor_addCommand (this, U"Font", U"10", GuiMenu_CHECKBUTTON, menu_cb_10);
+	fontSizeButton_12 = Editor_addCommand (this, U"Font", U"12", GuiMenu_CHECKBUTTON, menu_cb_12);
+	fontSizeButton_14 = Editor_addCommand (this, U"Font", U"14", GuiMenu_CHECKBUTTON, menu_cb_14);
+	fontSizeButton_18 = Editor_addCommand (this, U"Font", U"18", GuiMenu_CHECKBUTTON, menu_cb_18);
+	fontSizeButton_24 = Editor_addCommand (this, U"Font", U"24", GuiMenu_CHECKBUTTON, menu_cb_24);
 }
 
 void TextEditor_init (TextEditor me, const char32 *initialText) {
-	Editor_init (me, 0, 0, 600, 400, U"", NULL);
+	Editor_init (me, 0, 0, 600, 400, U"", nullptr);
 	setFontSize (me, my p_fontSize);
 	if (initialText) {
 		GuiText_setString (my textWidget, initialText);
 		my dirty = false;   // was set to true in valueChanged callback
 		Thing_setName (me, U"");
 	}
-	if (theOpenTextEditors == NULL) {
-		theOpenTextEditors = Collection_create (classTextEditor, 100);
-		Collection_dontOwnItems (theOpenTextEditors);
-	}
-	if (theOpenTextEditors != NULL) {
-		Collection_addItem (theOpenTextEditors, me);
-	}
+	theReferencesToAllOpenTextEditors. addItem_ref (me);
 }
 
-TextEditor TextEditor_create (const char32 *initialText) {
+autoTextEditor TextEditor_create (const char32 *initialText) {
 	try {
 		autoTextEditor me = Thing_new (TextEditor);
-		TextEditor_init (me.peek(), initialText);
-		return me.transfer();
+		TextEditor_init (me.get(), initialText);
+		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Text window not created.");
 	}
 }
 
 void TextEditor_showOpen (TextEditor me) {
-	cb_showOpen (Editor_getMenuCommand (me, U"File", U"Open..."), NULL, NULL, NULL);
+	cb_showOpen (Editor_getMenuCommand (me, U"File", U"Open..."));
 }
 
 /* End of file TextEditor.cpp */
