@@ -2,28 +2,27 @@
 #define _Thing_h_
 /* Thing.h
  *
- * Copyright (C) 1992-2011,2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2011,2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* The root class of all objects. */
 
 /* Anyone who uses Thing can also use: */
 	/* Arrays with any bounds and 1 or 2 indices, math, and numerics: */
-		#include "NUM.h"   /* Including math.h */
+		#include "../num/NUM.h"   /* Including math.h */
 	/* The messaging mechanism: */
 		#include "melder.h"   /* Including stdio.h string.h etc. */
 	/* The macros for struct and class definitions: */
@@ -35,23 +34,16 @@
 
 #define _Thing_auto_DEBUG  0
 
-/* Public. */
-
-typedef void *Any;   /* Prevent compile-time type checking. */
-
 /*
-	Use the macros 'I' and 'thou' for objects in the formal parameter lists
+	Use the macro 'I' for an object in the formal parameter lists
 	(if the explicit type cannot be used).
-	Use the macros 'iam' and 'thouart'
+	Use the macro 'iam'
 	as the first declaration in a function definition.
-	After this, the object 'me' or 'thee' has the right class (for the compiler),
-	so that you can use the macros 'my' and 'thy' to refer to members.
+	After this, the object 'me' has the right class (for the compiler),
+	so that you can use the macro 'my' to refer to members.
 	Example: int Person_getAge (I) { iam (Person); return my age; }
 */
-#define I  Any void_me
-#define thou  Any void_thee
 #define iam(klas)  klas me = (klas) void_me
-#define thouart(klas)  klas thee = (klas) void_thee
 #define my  me ->
 #define thy  thee ->
 #define his  him ->
@@ -80,12 +72,12 @@ struct structClassInfo {
 #define Thing_declare(klas) \
 	typedef struct struct##klas *klas; \
 	typedef _Thing_auto <struct##klas> auto##klas; \
+	extern struct structClassInfo theClassInfo_##klas; \
 	extern ClassInfo class##klas
 
 #define Thing_define(klas,parentKlas) \
 	Thing_declare (klas); \
 	typedef struct##parentKlas klas##_Parent; \
-	extern struct structClassInfo theClassInfo_##klas; \
 	struct struct##klas : public struct##parentKlas
 
 #define Thing_implement(klas,parentKlas,version) \
@@ -108,14 +100,14 @@ struct structThing {
 
 	/*
 	 * If a Thing has members of type autoThing,
-	 * then we want the destructors of autoThing to be called automatically whenever Thing is `delete`d.
+	 * then we want the destructors of autoThing to be called automatically whenever a Thing is `delete`d.
 	 * For this to happen, it is necessary that every Thing itself has a destructor.
 	 * We therefore define a destructor here,
 	 * and we make it virtual to ensure that every subclass has its own automatic version.
 	 */
-	virtual ~structThing () { }
+	virtual ~structThing () noexcept { }
 
-	virtual void v_destroy () { Melder_free (name); };
+	virtual void v_destroy () noexcept { Melder_free (name); };
 		/*
 		 * derived::v_destroy calls base::v_destroy at end
 		 */
@@ -176,24 +168,6 @@ bool Thing_isSubclass (ClassInfo klas, ClassInfo ancestor);
 void Thing_info (Thing me);
 void Thing_infoWithIdAndFile (Thing me, unsigned long id, MelderFile file);
 
-#define Thing_new(Klas)  static_cast<Klas> (Thing_newFromClass (class##Klas))
-/*
-	Function:
-		return a new object of class 'klas'.
-	Postconditions:
-		result -> classInfo == class'klas';
-		other members are 0.
-*/
-
-Thing Thing_newFromClass (ClassInfo klas);
-/*
-	Function:
-		return a new object of class 'klas'.
-	Postconditions:
-		result -> classInfo == 'klas';
-		other members are 0.
-*/
-
 void Thing_recognizeClassesByName (ClassInfo readableClass, ...);
 /*
 	Function:
@@ -215,25 +189,14 @@ void Thing_recognizeClassesByName (ClassInfo readableClass, ...);
 void Thing_recognizeClassByOtherName (ClassInfo readableClass, const char32 *otherName);
 long Thing_listReadableClasses ();
 
-Thing Thing_newFromClassName (const char32 *className, int *p_formatVersion);
-/*
-	Function:
-		return a new object of class 'className', or null if the class name is not recognized.
-	Postconditions:
-		result -> classInfo == class'className';
-		other members are 0.
-	Side effect:
-		see Thing_classFromClassName.
-*/
-
 ClassInfo Thing_classFromClassName (const char32 *className, int *formatVersion);
 /*
 	Function:
 		Return the class info table of class 'className', or null if it is not recognized.
-		E.g. the value returned from Thing_classFromClassName (L"PietjePuk")
+		E.g. the value returned from Thing_classFromClassName (U"PietjePuk")
 		will be equal to classPietjePuk.
 	Side effect:
-		If 'className' equals L"PietjePuk 300", the value returned will be classPietjePuk,
+		If 'className' equals U"PietjePuk 300", the value returned will be classPietjePuk,
 		and formatVersion (if not null) will be set to 300.
 */
 
@@ -278,7 +241,7 @@ void * _Thing_check (Thing me, ClassInfo table, const char *fileName, int line);
 
 /* For debugging. */
 
-long Thing_getTotalNumberOfThings ();
+extern long theTotalNumberOfThings;
 /* This number is 0 initially, increments at every successful `new', and decrements at every `forget'. */
 
 template <class T>
@@ -286,35 +249,41 @@ class _Thing_auto {
 	T *ptr;
 public:
 	/*
-	 * The default constructor.
-	 * Things like
-	 *    autoPitch pitch;
-	 * should initialize the pointer to null.
-	 */
+		A default constructor, as in
+			autoPitch pitch;
+		should initialize the pointer to null.
+	*/
 	_Thing_auto () : ptr (nullptr) {
 		#if _Thing_auto_DEBUG
 			fprintf (stderr, "default constructor\n");
 		#endif
 	}
 	/*
-	 * Things like
-	 *    autoPitch pitch (Pitch_create (...));
-	 * and
-	 *    autoPitch pitch = Pitch_create (...);
-	 * should work.
-	 */
-	_Thing_auto (T *newPtr) : ptr (newPtr) {
+		An awkward explicit construction from a pointer, like
+			extern Pitch getPitch ();
+			autoPitch pitch (getPitch ());
+		should work. However, such a case is probably a mistake,
+		because getPitch() should return an autoPitch if it
+		owns the Pitch, and if getPitch() doesn't own the Pitch,
+		an autoPitch should not receive it.
+		For this reason, the more intuitive syntax in
+			autoPitch pitch = getPitch ();
+		is forbidden, by declaring the construction-from-pointer as "explicit".
+	*/
+	explicit _Thing_auto (T *newPtr) : ptr (newPtr) {
 		#if _Thing_auto_DEBUG
 			if (our ptr)
-				fprintf (stderr, "constructor %p %s\n",
+				fprintf (stderr, "constructor from pointer %p %s\n",
 					our ptr, Melder_peek32to8 (our ptr -> classInfo -> className));
 		#endif
 	}
 	/*
-	 * pitch should be destroyed when going out of scope,
-	 * both at the end of the try block and when a throw occurs.
-	 */
-	~_Thing_auto () {
+		After
+			autoPitch pitch1 = Pitch_create ();
+		pitch1 should be destroyed when going out of scope,
+		either at the end of the try block or whenever a throw occurs.
+	*/
+	~_Thing_auto () noexcept {
 		#if _Thing_auto_DEBUG
 			fprintf (stderr, "destructor %p %s\n",
 				our ptr, our ptr ? Melder_peek32to8 (our ptr -> classInfo -> className) : "(class unknown)");
@@ -324,41 +293,48 @@ public:
 			our ptr = nullptr;
 		}
 	}
-	T* get () const {
-		return our ptr;
-	}
-	T* peek () const {
+	T* get () const noexcept {
 		return our ptr;
 	}
 	/*
-	 * The expression
-	 *    pitch.d_ptr -> xmin
-	 * should be abbreviatable by
-	 *    pitch -> xmin
-	 */
-	T* operator-> () const {   // as r-value
+		The expression
+			pitch.d_ptr -> xmin
+		should be abbreviatable by
+			pitch -> xmin
+	*/
+	T* operator-> () const noexcept {   // as r-value
 		return our ptr;
 	}
-	T& operator* () const {   // as l-value
+	/*
+	T& operator* () const noexcept {   // as l-value
 		return *our ptr;
-	}
+	}*/
 	/*
 	 * After construction, there are two ways to access the pointer: with and without transfer of ownership.
 	 *
 	 * Without transfer:
-	 *    Pitch_draw (pitch.peek());
+	 *    Pitch_draw (pitch.get());
 	 *
 	 * With transfer:
-	 *    return thee.transfer();
+	 *    return thee;
 	 * and
-	 *    *out_pitch = pitch.transfer();
-	 *    *out_pulses = pulses.transfer();
+	 *    *out_pitch = pitch.move();
+	 *    *out_pulses = pulses.move();
 	 * and
-	 *    Collection_addItem (me, pitch.transfer());
+	 *    Collection_addItem_move (me, pitch.move());
 	 * and
-	 *    praat_new (pitch.transfer(), my name);
+	 *    praat_new (pitch.move(), my name);
 	 */
-	T* transfer () {
+	void releaseToUser () noexcept {
+		our ptr = nullptr;   // make the pointer non-automatic again
+	}
+	/*
+		Sometimes the ownership is determined by a flag such as _ownItems or _ownData or _ownSound.
+		In that case, the autoThing has be released as a raw Thing pointer,
+		and the ambiguous owner may become responsible for destruction the object.
+		In Praat, this happens with Collection items and with some editors.
+	*/
+	T* releaseToAmbiguousOwner () noexcept {
 		T* temp = our ptr;
 		our ptr = nullptr;   // make the pointer non-automatic again
 		return temp;
@@ -374,20 +350,24 @@ public:
 	T* clone () const {
 		return static_cast<T *> (Data_copy (our ptr));
 	}
+	void reset () noexcept {
+		_Thing_forget (our ptr);
+		our ptr = nullptr;
+	}
 	/*
 	 * Replacing a pointer in an existing autoThing should be an exceptional phenomenon,
 	 * and therefore has to be done explicitly (rather than via an assignment),
 	 * so that you can easily spot ugly places in your source code.
 	 * In order not to leak memory, the old object is destroyed.
 	 */
-	void reset (T* newPtr = nullptr) noexcept {
+	/*void reset (T* newPtr = nullptr) noexcept {
 		_Thing_forget (our ptr);
 		our ptr = newPtr;
-	}
-	void zero () {
+	}*/
+	void _zero () noexcept {
 		our ptr = nullptr;
 	}
-	explicit operator bool () const {
+	explicit operator bool () const noexcept {
 		return !! our ptr;
 	}
 	bool operator== (_Thing_auto<T> other) const noexcept {
@@ -434,13 +414,13 @@ public:
 		#endif
 		other. ptr = nullptr;
 	}
-	template <class Y> _Thing_auto<T> (_Thing_auto<Y>&& other) noexcept : ptr (other.peek()) {
+	template <class Y> _Thing_auto<T> (_Thing_auto<Y>&& other) noexcept : ptr (other.get()) {
 		#if _Thing_auto_DEBUG
 			if (our ptr)
 				fprintf (stderr, "move constructor %p from other class %s\n",
 					our ptr, Melder_peek32to8 (our ptr -> classInfo -> className));
 		#endif
-		other.zero();
+		other. _zero();
 	}
 	/*
 	 * The compiler should treat assignments from _Thing_auto r-values, as in
@@ -455,7 +435,7 @@ public:
 				fprintf (stderr, "move assignment before %p from same class %s\n",
 					our ptr, our ptr ? Melder_peek32to8 (our ptr -> classInfo -> className) : "(class unknown)");
 			#endif
-			_Thing_forget (our ptr);
+			if (our ptr) _Thing_forget (our ptr);
 			our ptr = other. ptr;
 			#if _Thing_auto_DEBUG
 				fprintf (stderr, "move assignment after %p from same class %s\n",
@@ -466,26 +446,26 @@ public:
 		return *this;
 	}
 	template <class Y> _Thing_auto<T>& operator= (_Thing_auto<Y>&& other) noexcept {
-		if (other.peek() != our ptr) {
+		if (other.get() != our ptr) {
 			#if _Thing_auto_DEBUG
 				fprintf (stderr, "move assignment before %p from other class %s\n",
 					our ptr, our ptr ? Melder_peek32to8 (our ptr -> classInfo -> className) : "(class unknown)");
 			#endif
-			_Thing_forget (our ptr);
-			our ptr = other.peek();
+			if (our ptr) _Thing_forget (our ptr);
+			our ptr = other.get();
 			#if _Thing_auto_DEBUG
 				fprintf (stderr, "move assignment after %p from other class %s\n",
 					our ptr, our ptr ? Melder_peek32to8 (our ptr -> classInfo -> className) : "(class unknown)");
 			#endif
-			other.zero();
+			other. _zero();
 		}
 		return *this;
 	}
 	/*
 	 * Move semantics from l-values can be achieved with move syntax:
-	 *    autoPitch pitch2 = pitch.move();   // calls the move constructor and therefore nullifies pitch
+	 *    autoPitch pitch2 = pitch1.move();   // calls the move constructor and therefore nullifies pitch1
 	 *
-	 *    pitch2 = pitch.move();   // performs move assignment and therefore nullifies pitch
+	 *    pitch2 = pitch1.move();   // performs move assignment and therefore nullifies pitch1
 	 */
 	_Thing_auto<T>&& move () noexcept { return static_cast <_Thing_auto<T>&&> (*this); }
 	/*
@@ -499,45 +479,78 @@ public:
 	 * returns a moved `thee` in `pitch`. This works because return values from automatic (i.e. non-static) variables are r-values.
 	 *
 	 * In function arguments, transfer of ownership works only explicitly:
-	 *    extern void Collection_addItem_transfer (Collection me, autoDaata item);
+	 *    extern void Collection_addItem_move (Collection me, autoDaata item);
 	 *    autoPitch pitch = Pitch_create (...);
-	 *    Collection_addItem_transfer (collection, pitch.move());   // compiler error if you don't call move()
+	 *    Collection_addItem_move (collection, pitch.move());   // compiler error if you don't call move()
 	 */
-	template <class Y> _Thing_auto<Y> static_cast_move () {
-		return _Thing_auto<Y> (static_cast<Y*> (our transfer()));
+	template <class Y> _Thing_auto<Y> static_cast_move () noexcept {
+		return _Thing_auto<Y> (static_cast<Y*> (our releaseToAmbiguousOwner()));
 	}
 };
 
+typedef _Thing_auto<structThing> autoThing;
+
+#define Thing_new(Klas)  Thing_newFromClass (class##Klas).static_cast_move<struct##Klas>()
+/*
+	Function:
+		return a new object of class 'klas'.
+	Postconditions:
+		result -> classInfo == class'klas';
+		other members are 0.
+*/
+
+autoThing Thing_newFromClass (ClassInfo klas);
+/*
+	Function:
+		return a new object of class 'klas'.
+	Postconditions:
+		result -> classInfo == 'klas';
+		other members are 0.
+*/
+
+autoThing Thing_newFromClassName (const char32 *className, int *p_formatVersion);
+/*
+	Function:
+		return a new object of class 'className', or null if the class name is not recognized.
+	Postconditions:
+		result -> classInfo == class'className';
+		other members are 0.
+	Side effect:
+		see Thing_classFromClassName.
+*/
+
 template <class T>
 class autoThingVector {
-	T* d_ptr;
+	_Thing_auto<T> *d_ptr;
 	long d_from, d_to;
 public:
 	autoThingVector<T> (long from, long to) : d_from (from), d_to (to) {
-		d_ptr = static_cast <T*> (NUMvector (sizeof (T), from, to));
+		d_ptr = static_cast <_Thing_auto<T>*> (NUMvector (sizeof (_Thing_auto<T>), from, to));
 	}
-	autoThingVector (T *ptr, long from, long to) : d_ptr (ptr), d_from (from), d_to (to) {
+	autoThingVector (_Thing_auto<T> *ptr, long from, long to) : d_ptr (ptr), d_from (from), d_to (to) {
 	}
 	autoThingVector () : d_ptr (nullptr), d_from (1), d_to (0) {
 	}
 	~autoThingVector<T> () {
 		if (d_ptr) {
 			for (long i = d_from; i <= d_to; i ++)
-				forget (d_ptr [i]);
-			NUMvector_free (sizeof (T), d_ptr, d_from);
+				d_ptr [i].reset();
+			NUMvector_free (sizeof (_Thing_auto<T>), d_ptr, d_from);
 		}
 	}
-	T& operator[] (long i) {
+	_Thing_auto<T>& operator[] (long i) {
 		return d_ptr [i];
 	}
-	T* peek () const {
+	_Thing_auto<T>* peek () const {
 		return d_ptr;
 	}
+	/*
 	T* transfer () {
 		T* temp = d_ptr;
 		d_ptr = nullptr;   // make the pointer non-automatic again
 		return temp;
 	}
+
 	void reset (long from, long to) {
 		if (d_ptr) {
 			for (long i = d_from; i <= d_to; i ++)
@@ -549,6 +562,7 @@ public:
 		d_to = to;
 		d_ptr = static_cast <T*> (NUMvector (sizeof (T), from, to));
 	}
+	*/
 };
 
 /* End of file Thing.h */
