@@ -1,20 +1,19 @@
 /* Ui.cpp
  *
- * Copyright (C) 1992-2012,2013,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2013,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <wctype.h>
@@ -50,12 +49,11 @@
 
 Thing_implement (UiField, Thing, 0);
 
-void structUiField :: v_destroy () {
+void structUiField :: v_destroy () noexcept {
 	Melder_free (formLabel);
 	Melder_free (stringValue);
 	Melder_free (stringValueA);
 	Melder_free (stringDefaultValue);
-	forget (options);
 	UiField_Parent :: v_destroy ();
 }
 
@@ -68,47 +66,38 @@ static UiField UiField_create (int type, const char32 *name) {
 	/*
 	 * Strip parentheses and colon off parameter name.
 	 */
-	if ((p = (char32 *) str32chr (shortName, U'(')) != NULL) {
+	if (!! (p = (char32 *) str32chr (shortName, U'('))) {
 		*p = U'\0';
 		if (p - shortName > 0 && p [-1] == U' ') p [-1] = U'\0';
 	}
 	p = shortName;
 	if (*p != U'\0' && p [str32len (p) - 1] == U':') p [str32len (p) - 1] = U'\0';
-	Thing_setName (me.peek(), shortName);
-	return me.transfer();
+	Thing_setName (me.get(), shortName);
+	return me.releaseToAmbiguousOwner();
 }
 
 /***** class UiOption: radio buttons and menu options *****/
 
-Thing_define (UiOption, Thing) {
-	GuiRadioButton radioButton;
-	GuiObject menuItem;
-};
-
 Thing_implement (UiOption, Thing, 0);
 
 static UiOption UiOption_create (const char32 *label) {
-	UiOption me = Thing_new (UiOption);
-	Thing_setName (me, label);
-	return me;
+	autoUiOption me = Thing_new (UiOption);
+	Thing_setName (me.get(), label);
+	return me.releaseToAmbiguousOwner();
 }
 
-Any UiRadio_addButton (I, const char32 *label) {
-	iam (UiField);
-	if (me == NULL) return NULL;
+UiOption UiRadio_addButton (UiField me, const char32 *label) {
+	if (! me) return nullptr;
 	Melder_assert (my type == UI_RADIO || my type == UI_OPTIONMENU);
-	UiOption thee = UiOption_create (label);
-	Collection_addItem (my options, thee);
-	return thee;
+	autoUiOption thee (UiOption_create (label));
+	return my options. addItem_move (thee.move());
 }
 
-Any UiOptionMenu_addButton (I, const char32 *label) {
-	iam (UiField);
-	if (me == NULL) return NULL;
+UiOption UiOptionMenu_addButton (UiField me, const char32 *label) {
+	if (! me) return nullptr;
 	Melder_assert (my type == UI_RADIO || my type == UI_OPTIONMENU);
-	UiOption thee = UiOption_create (label);
-	Collection_addItem (my options, thee);
-	return thee;
+	autoUiOption thee (UiOption_create (label));
+	return my options. addItem_move (thee.move());
 }
 
 /***** Things to do with UiField objects. *****/
@@ -122,9 +111,9 @@ static void UiField_setDefault (UiField me) {
 		} break; case UI_BOOLEAN: {
 			GuiCheckButton_setValue (my checkButton, my integerDefaultValue);
 		} break; case UI_RADIO: {
-			for (int i = 1; i <= my options -> size; i ++) {
+			for (int i = 1; i <= my options.size; i ++) {
 				if (i == my integerDefaultValue) {
-					UiOption b = static_cast <UiOption> (my options -> item [i]);
+					UiOption b = my options.at [i];
 					GuiRadioButton_set (b -> radioButton);
 				}
 			}
@@ -144,10 +133,10 @@ static int colourToValue (UiField me, char32 *string) {
 	if (first == U'{') {
 		my colourValue. red = Melder_atof (++ p);
 		p = str32chr (p, U',');
-		if (p == NULL) return 0;
+		if (! p) return 0;
 		my colourValue. green = Melder_atof (++ p);
 		p = str32chr (p, U',');
-		if (p == NULL) return 0;
+		if (! p) return 0;
 		my colourValue. blue = Melder_atof (++ p);
 	} else {
 		*p = (char32) tolower ((int) *p);
@@ -178,7 +167,7 @@ static void UiField_widgetToValue (UiField me) {
 	switch (my type) {
 		case UI_REAL: case UI_REAL_OR_UNDEFINED: case UI_POSITIVE: {
 			autostring32 dirty = GuiText_getString (my text);   // the text as typed by the user
-			Interpreter_numericExpression (NULL, dirty.peek(), & my realValue);
+			Interpreter_numericExpression (nullptr, dirty.peek(), & my realValue);
 			/*
 			 * Put a clean version of the new value in the form.
 			 * If the value is equal to the default value, make sure that any default comments are included.
@@ -210,7 +199,7 @@ static void UiField_widgetToValue (UiField me) {
 				my integerValue = 2;
 			} else {
 				double realValue;
-				Interpreter_numericExpression (NULL, dirty.peek(), & realValue);
+				Interpreter_numericExpression (nullptr, dirty.peek(), & realValue);
 				my integerValue = lround (realValue);
 			}
 			if (my integerValue == Melder_atoi (my stringDefaultValue)) {
@@ -218,8 +207,9 @@ static void UiField_widgetToValue (UiField me) {
 			} else {
 				GuiText_setString (my text, Melder_integer (my integerValue));
 			}
-			if (my type == UI_NATURAL && my integerValue < 1)
+			if ((my type == UI_NATURAL || my type == UI_CHANNEL) && my integerValue < 1) {
 				Melder_throw (U_LEFT_DOUBLE_QUOTE, my name, U_RIGHT_DOUBLE_QUOTE U" must be a positive whole number.");
+			}
 		} break; case UI_WORD: {
 			Melder_free (my stringValue);
 			my stringValue = GuiText_getString (my text);
@@ -233,8 +223,8 @@ static void UiField_widgetToValue (UiField me) {
 			my integerValue = GuiCheckButton_getValue (my checkButton);
 		} break; case UI_RADIO: {
 			my integerValue = 0;
-			for (int i = 1; i <= my options -> size; i ++) {
-				UiOption b = static_cast <UiOption> (my options -> item [i]);
+			for (int i = 1; i <= my options.size; i ++) {
+				UiOption b = my options.at [i];
 				if (GuiRadioButton_getValue (b -> radioButton))
 					my integerValue = i;
 			}
@@ -246,7 +236,7 @@ static void UiField_widgetToValue (UiField me) {
 				Melder_throw (U"No option chosen for " U_LEFT_DOUBLE_QUOTE, my name, U_RIGHT_DOUBLE_QUOTE U".");
 		} break; case UI_LIST: {
 			long numberOfSelected, *selected = GuiList_getSelectedPositions (my list, & numberOfSelected);   // BUG memory
-			if (selected == NULL) {
+			if (! selected) {
 				Melder_warning (U"No items selected.");
 				my integerValue = 1;
 			} else {
@@ -259,7 +249,7 @@ static void UiField_widgetToValue (UiField me) {
 			if (colourToValue (me, string.peek())) {
 				// do nothing
 			} else {
-				Interpreter_numericExpression (NULL, string.peek(), & my colourValue. red);
+				Interpreter_numericExpression (nullptr, string.peek(), & my colourValue. red);
 				my colourValue. green = my colourValue. blue = my colourValue. red;
 			}
 		}
@@ -302,8 +292,8 @@ static void UiField_stringToValue (UiField me, const char32 *string, Interpreter
 				string [0] == U't' || string [0] == U'T';
 		} break; case UI_RADIO: case UI_OPTIONMENU: {
 			my integerValue = 0;
-			for (int i = 1; i <= my options -> size; i ++) {
-				UiOption b = static_cast <UiOption> (my options -> item [i]);
+			for (int i = 1; i <= my options.size; i ++) {
+				UiOption b = my options.at [i];
 				if (str32equ (string, b -> name))
 					my integerValue = i;
 			}
@@ -311,8 +301,8 @@ static void UiField_stringToValue (UiField me, const char32 *string, Interpreter
 				/*
 				 * Retry with different case.
 				 */
-				for (int i = 1; i <= my options -> size; i ++) {
-					UiOption b = static_cast <UiOption> (my options -> item [i]);
+				for (int i = 1; i <= my options.size; i ++) {
+					UiOption b = my options.at [i];
 					char32 name2 [100];
 					str32cpy (name2, b -> name);
 					if (islower ((int) name2 [0])) name2 [0] = (char32) toupper ((int) name2 [0]);
@@ -352,16 +342,16 @@ static void UiField_stringToValue (UiField me, const char32 *string, Interpreter
 
 /***** History mechanism. *****/
 
-static MelderString theHistory = { 0 };
+static MelderString theHistory { 0 };
 void UiHistory_write (const char32 *string) { MelderString_append (& theHistory, string); }
 void UiHistory_write_expandQuotes (const char32 *string) {
-	if (string == NULL) return;
+	if (! string) return;
 	for (const char32 *p = & string [0]; *p != U'\0'; p ++) {
 		if (*p == U'\"') MelderString_append (& theHistory, U"\"\""); else MelderString_appendCharacter (& theHistory, *p);
 	}
 }
 void UiHistory_write_colonize (const char32 *string) {
-	if (string == NULL) return;
+	if (! string) return;
 	for (const char32 *p = & string [0]; *p != U'\0'; p ++) {
 		if (*p == U'.' && p [1] == U'.' && p [2] == U'.') {
 			MelderString_append (& theHistory, U":");
@@ -378,15 +368,15 @@ void UiHistory_clear () { MelderString_empty (& theHistory); }
 
 Thing_implement (UiForm, Thing, 0);
 
-bool (*theAllowExecutionHookHint) (void *closure) = NULL;
-void *theAllowExecutionClosureHint = NULL;
+bool (*theAllowExecutionHookHint) (void *closure) = nullptr;
+void *theAllowExecutionClosureHint = nullptr;
 
 void Ui_setAllowExecutionHook (bool (*allowExecutionHook) (void *closure), void *allowExecutionClosure) {
 	theAllowExecutionHookHint = allowExecutionHook;
 	theAllowExecutionClosureHint = allowExecutionClosure;
 }
 
-void structUiForm :: v_destroy () {
+void structUiForm :: v_destroy () noexcept {
 	for (int ifield = 1; ifield <= numberOfFields; ifield ++)
 		forget (field [ifield]);
 	if (d_dialogForm) {
@@ -398,20 +388,17 @@ void structUiForm :: v_destroy () {
 	UiForm_Parent :: v_destroy ();
 }
 
-static void gui_button_cb_revert (I, GuiButtonEvent /* event */) {
-	iam (UiForm);
+static void gui_button_cb_revert (UiForm me, GuiButtonEvent /* event */) {
 	for (int ifield = 1; ifield <= my numberOfFields; ifield ++)
 		UiField_setDefault (my field [ifield]);
 }
 
-static void gui_dialog_cb_close (I) {
-	iam (UiForm);
+static void gui_dialog_cb_close (UiForm me) {
 	if (my cancelCallback) my cancelCallback (me, my buttonClosure);
 	GuiThing_hide (my d_dialogForm);
 	if (my destroyWhenUnmanaged) forget (me);
 }
-static void gui_button_cb_cancel (I, GuiButtonEvent /* event */) {
-	iam (UiForm);
+static void gui_button_cb_cancel (UiForm me, GuiButtonEvent /* event */) {
 	if (my cancelCallback) my cancelCallback (me, my buttonClosure);
 	GuiThing_hide (my d_dialogForm);
 	if (my destroyWhenUnmanaged) forget (me);
@@ -445,8 +432,8 @@ static void UiForm_okOrApply (UiForm me, GuiButton button, int hide) {
 	if (my cancelButton) GuiThing_setSensitive (my cancelButton, false);
 	if (my revertButton) GuiThing_setSensitive (my revertButton, false);
 	if (my helpButton)   GuiThing_setSensitive (my helpButton,   false);
-	#if motif
-	XmUpdateDisplay (my d_dialogForm -> d_widget);
+	#if defined (_WIN32)
+		GdiFlush ();
 	#endif
 	if (my isPauseForm) {
 		for (int i = 1; i <= my numberOfContinueButtons; i ++) {
@@ -459,7 +446,7 @@ static void UiForm_okOrApply (UiForm me, GuiButton button, int hide) {
 	 * Keep the gate for error handling.
 	 */
 	try {
-		my okCallback (me, 0, NULL, NULL, NULL, NULL, false, my buttonClosure);
+		my okCallback (me, 0, nullptr, nullptr, nullptr, nullptr, false, my buttonClosure);
 		/*
 		 * Write everything to history. Before destruction!
 		 */
@@ -486,7 +473,7 @@ static void UiForm_okOrApply (UiForm me, GuiButton button, int hide) {
 					} break; case UI_BOOLEAN: {
 						UiHistory_write (field -> integerValue ? (next -- ? U", \"yes\"" : U" \"yes\"") : (next -- ? U", \"no\"" : U" \"no\""));
 					} break; case UI_RADIO: case UI_OPTIONMENU: {
-						UiOption b = static_cast <UiOption> (field -> options -> item [field -> integerValue]);
+						UiOption b = field -> options.at [field -> integerValue];
 						UiHistory_write (next -- ? U", \"" : U" \"");
 						UiHistory_write_expandQuotes (b -> name);
 						UiHistory_write (U"\"");
@@ -539,18 +526,15 @@ static void UiForm_okOrApply (UiForm me, GuiButton button, int hide) {
 	if (my helpButton)   GuiThing_setSensitive (my helpButton,   true);
 }
 
-static void gui_button_cb_ok (I, GuiButtonEvent event) {
-	iam (UiForm);
+static void gui_button_cb_ok (UiForm me, GuiButtonEvent event) {
 	UiForm_okOrApply (me, event -> button, true);
 }
 
-static void gui_button_cb_apply (I, GuiButtonEvent event) {
-	iam (UiForm);
+static void gui_button_cb_apply (UiForm me, GuiButtonEvent event) {
 	UiForm_okOrApply (me, event -> button, false);
 }
 
-static void gui_button_cb_help (I, GuiButtonEvent /* event */) {
-	iam (UiForm);
+static void gui_button_cb_help (UiForm me, GuiButtonEvent /* event */) {
 	Melder_help (my helpTitle);
 }
 
@@ -560,12 +544,12 @@ UiForm UiForm_create (GuiWindow parent, const char32 *title,
 {
 	autoUiForm me = Thing_new (UiForm);
 	my d_dialogParent = parent;
-	Thing_setName (me.peek(), title);
+	Thing_setName (me.get(), title);
 	my okCallback = okCallback;
 	my buttonClosure = buttonClosure;
 	my invokingButtonTitle = Melder_dup (invokingButtonTitle);
 	my helpTitle = Melder_dup (helpTitle);
-	return me.transfer();
+	return me.releaseToAmbiguousOwner();
 }
 
 void UiForm_setPauseForm (UiForm me,
@@ -574,7 +558,7 @@ void UiForm_setPauseForm (UiForm me,
 	const char32 *continue4, const char32 *continue5, const char32 *continue6,
 	const char32 *continue7, const char32 *continue8, const char32 *continue9,
 	const char32 *continue10,
-	void (*cancelCallback) (Any dia, void *closure))
+	void (*cancelCallback) (UiForm dia, void *closure))
 {
 	my isPauseForm = true;
 	my numberOfContinueButtons = numberOfContinueButtons;
@@ -597,114 +581,112 @@ static void commonOkCallback (UiForm /* dia */, int /* narg */, Stackel /* args
 	Interpreter interpreter, const char32 * /* invokingButtonTitle */, bool /* modified */, void *closure)
 {
 	EditorCommand cmd = (EditorCommand) closure;
-	cmd -> commandCallback (cmd -> d_editor, cmd, cmd -> d_uiform, 0, NULL, NULL, interpreter);
+	cmd -> commandCallback (cmd -> d_editor, cmd, cmd -> d_uiform.get(), 0, nullptr, nullptr, interpreter);
 }
 
-UiForm UiForm_createE (EditorCommand cmd, const char32 *title, const char32 *invokingButtonTitle, const char32 *helpTitle) {
+autoUiForm UiForm_createE (EditorCommand cmd, const char32 *title, const char32 *invokingButtonTitle, const char32 *helpTitle) {
 	Editor editor = cmd -> d_editor;
-	UiForm dia = UiForm_create (editor -> d_windowForm, title, commonOkCallback, cmd, invokingButtonTitle, helpTitle);
+	autoUiForm dia (UiForm_create (editor -> d_windowForm, title, commonOkCallback, cmd, invokingButtonTitle, helpTitle));
 	dia -> command = cmd;
 	return dia;
 }
 
 static UiField UiForm_addField (UiForm me, int type, const char32 *label) {
 	if (my numberOfFields == MAXIMUM_NUMBER_OF_FIELDS)
-		Melder_throw (U"Cannot have more than ", MAXIMUM_NUMBER_OF_FIELDS, U"in a form.");
+		Melder_throw (U"Cannot have more than ", MAXIMUM_NUMBER_OF_FIELDS, U"fields in a form.");
 	return my field [++ my numberOfFields] = UiField_create (type, label);
 }
 
-Any UiForm_addReal (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_REAL, label);
+UiField UiForm_addReal (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_REAL, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addRealOrUndefined (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_REAL_OR_UNDEFINED, label);
+UiField UiForm_addRealOrUndefined (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_REAL_OR_UNDEFINED, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addPositive (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_POSITIVE, label);
+UiField UiForm_addPositive (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_POSITIVE, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addInteger (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_INTEGER, label);
+UiField UiForm_addInteger (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_INTEGER, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addNatural (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_NATURAL, label);
+UiField UiForm_addNatural (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_NATURAL, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addWord (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_WORD, label);
+UiField UiForm_addWord (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_WORD, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addSentence (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_SENTENCE, label);
+UiField UiForm_addSentence (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_SENTENCE, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addLabel (UiForm me, const char32 *name, const char32 *label) {
-	autoUiField thee = UiForm_addField (me, UI_LABEL, name);
+UiField UiForm_addLabel (UiForm me, const char32 *name, const char32 *label) {
+	autoUiField thee (UiForm_addField (me, UI_LABEL, name));
 	thy stringValue = Melder_dup (label);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addBoolean (UiForm me, const char32 *label, int defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_BOOLEAN, label);
+UiField UiForm_addBoolean (UiForm me, const char32 *label, int defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_BOOLEAN, label));
 	thy integerDefaultValue = defaultValue;
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addText (UiForm me, const char32 *name, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_TEXT, name);
+UiField UiForm_addText (UiForm me, const char32 *name, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_TEXT, name));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addRadio (UiForm me, const char32 *label, int defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_RADIO, label);
+UiField UiForm_addRadio (UiForm me, const char32 *label, int defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_RADIO, label));
 	thy integerDefaultValue = defaultValue;
-	thy options = Ordered_create ();
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addOptionMenu (UiForm me, const char32 *label, int defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_OPTIONMENU, label);
+UiField UiForm_addOptionMenu (UiForm me, const char32 *label, int defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_OPTIONMENU, label));
 	thy integerDefaultValue = defaultValue;
-	thy options = Ordered_create ();
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addList (UiForm me, const char32 *label, long numberOfStrings, const char32 **strings, long defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_LIST, label);
+UiField UiForm_addList (UiForm me, const char32 *label, long numberOfStrings, const char32 **strings, long defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_LIST, label));
 	thy numberOfStrings = numberOfStrings;
 	thy strings = strings;
 	thy integerDefaultValue = defaultValue;
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addColour (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_COLOUR, label);
+UiField UiForm_addColour (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_COLOUR, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
-Any UiForm_addChannel (UiForm me, const char32 *label, const char32 *defaultValue) {
-	autoUiField thee = UiForm_addField (me, UI_CHANNEL, label);
+UiField UiForm_addChannel (UiForm me, const char32 *label, const char32 *defaultValue) {
+	autoUiField thee (UiForm_addField (me, UI_CHANNEL, label));
 	thy stringDefaultValue = Melder_dup (defaultValue);
-	return thee.transfer();
+	return thee.releaseToAmbiguousOwner();
 }
 
 #define DIALOG_X  150
@@ -716,7 +698,7 @@ Any UiForm_addChannel (UiForm me, const char32 *label, const char32 *defaultValu
 #define HELP_BUTTON_X  20
 #define LIST_HEIGHT  192
 
-static MelderString theFinishBuffer = { 0 };
+static MelderString theFinishBuffer { 0 };
 
 static void appendColon () {
 	long length = theFinishBuffer.length;
@@ -751,8 +733,8 @@ void UiForm_finish (UiForm me) {
 		thy y = dialogHeight;
 		dialogHeight +=
 			thy type == UI_BOOLEAN ? Gui_CHECKBUTTON_HEIGHT :
-			thy type == UI_RADIO ? thy options -> size * Gui_RADIOBUTTON_HEIGHT +
-				(thy options -> size - 1) * Gui_RADIOBUTTON_SPACING :
+			thy type == UI_RADIO ? thy options.size * Gui_RADIOBUTTON_HEIGHT +
+				(thy options.size - 1) * Gui_RADIOBUTTON_SPACING :
 			thy type == UI_OPTIONMENU ? Gui_OPTIONMENU_HEIGHT :
 			thy type == UI_LIST ? LIST_HEIGHT :
 			thy type == UI_LABEL && thy stringValue [0] != U'\0' && thy stringValue [str32len (thy stringValue) - 1] != U'.' &&
@@ -829,14 +811,14 @@ void UiForm_finish (UiForm me) {
 				field -> label = GuiLabel_createShown (form, x, x + labelWidth, ylabel, ylabel + Gui_RADIOBUTTON_HEIGHT,
 					theFinishBuffer.string, GuiLabel_RIGHT);
 				GuiRadioGroup_begin ();
-				for (long ibutton = 1; ibutton <= field -> options -> size; ibutton ++) {
-					UiOption button = static_cast <UiOption> (field -> options -> item [ibutton]);
+				for (long ibutton = 1; ibutton <= field -> options.size; ibutton ++) {
+					UiOption button = field -> options.at [ibutton];
 					MelderString_copy (& theFinishBuffer, button -> name);
 					button -> radioButton = GuiRadioButton_createShown (form,
 						fieldX, dialogWidth /* allow to extend into the margin */,
 						y + (ibutton - 1) * (Gui_RADIOBUTTON_HEIGHT + Gui_RADIOBUTTON_SPACING),
 						y + (ibutton - 1) * (Gui_RADIOBUTTON_HEIGHT + Gui_RADIOBUTTON_SPACING) + Gui_RADIOBUTTON_HEIGHT,
-						theFinishBuffer.string, NULL, NULL, 0);
+						theFinishBuffer.string, nullptr, nullptr, 0);
 				}
 				GuiRadioGroup_end ();
 			} break; 
@@ -851,8 +833,8 @@ void UiForm_finish (UiForm me) {
 				field -> label = GuiLabel_createShown (form, x, x + labelWidth, ylabel, ylabel + Gui_OPTIONMENU_HEIGHT,
 					theFinishBuffer.string, GuiLabel_RIGHT);
 				field -> optionMenu = GuiOptionMenu_createShown (form, fieldX, fieldX + fieldWidth, y, y + Gui_OPTIONMENU_HEIGHT, 0);
-				for (long ibutton = 1; ibutton <= field -> options -> size; ibutton ++) {
-					UiOption button = static_cast <UiOption> (field -> options -> item [ibutton]);
+				for (long ibutton = 1; ibutton <= field -> options.size; ibutton ++) {
+					UiOption button = field -> options.at [ibutton];
 					MelderString_copy (& theFinishBuffer, button -> name);
 					GuiOptionMenu_addOption (field -> optionMenu, theFinishBuffer.string);
 				}
@@ -864,7 +846,7 @@ void UiForm_finish (UiForm me) {
 					theFinishBuffer.string, GuiLabel_RIGHT); */
 				field -> checkButton = GuiCheckButton_createShown (form,
 					fieldX, dialogWidth /* allow to extend into the margin */, y, y + Gui_CHECKBUTTON_HEIGHT,
-					theFinishBuffer.string, NULL, NULL, 0);
+					theFinishBuffer.string, nullptr, nullptr, 0);
 			} break;
 			case UI_LIST:
 			{
@@ -883,7 +865,7 @@ void UiForm_finish (UiForm me) {
 	}
 	for (long ifield = 1; ifield <= my numberOfFields; ifield ++)
 		UiField_setDefault (my field [ifield]);
-	/*separator = XmCreateSeparatorGadget (column, "separator", NULL, 0);*/
+	/*separator = XmCreateSeparatorGadget (column, "separator", nullptr, 0);*/
 	y = dialogHeight - Gui_BOTTOM_DIALOG_SPACING - Gui_PUSHBUTTON_HEIGHT;
 	if (my helpTitle) {
 		my helpButton = GuiButton_createShown (form, HELP_BUTTON_X, HELP_BUTTON_X + HELP_BUTTON_WIDTH, y, y + Gui_PUSHBUTTON_HEIGHT,
@@ -953,7 +935,7 @@ void UiForm_do (UiForm me, bool modified) {
 	Melder_assert (my d_dialogForm);
 	GuiThing_show (my d_dialogForm);
 	if (modified)
-		UiForm_okOrApply (me, NULL, true);
+		UiForm_okOrApply (me, nullptr, true);
 }
 
 static void UiField_argToValue (UiField me, Stackel arg, Interpreter /* interpreter */) {
@@ -1013,8 +995,8 @@ static void UiField_argToValue (UiField me, Stackel arg, Interpreter /* interpre
 			if (arg -> which != Stackel_STRING)
 				Melder_throw (U"Option argument \"", my name, U"\" should be a string, not ", Stackel_whichText (arg), U".");
 			my integerValue = 0;
-			for (int i = 1; i <= my options -> size; i ++) {
-				UiOption b = static_cast <UiOption> (my options -> item [i]);
+			for (int i = 1; i <= my options.size; i ++) {
+				UiOption b = my options.at [i];
 				if (str32equ (arg -> string, b -> name))
 					my integerValue = i;
 			}
@@ -1022,8 +1004,8 @@ static void UiField_argToValue (UiField me, Stackel arg, Interpreter /* interpre
 				/*
 				 * Retry with different case.
 				 */
-				for (int i = 1; i <= my options -> size; i ++) {
-					UiOption b = static_cast <UiOption> (my options -> item [i]);
+				for (int i = 1; i <= my options.size; i ++) {
+					UiOption b = my options.at [i];
 					char32 name2 [100];
 					str32cpy (name2, b -> name);
 					if (iswlower ((int) name2 [0])) name2 [0] = (char32) towupper ((int) name2 [0]);
@@ -1075,7 +1057,7 @@ void UiForm_call (UiForm me, int narg, Stackel args, Interpreter interpreter) {
 	}
 	if (iarg < narg)
 		Melder_throw (U"Command requires only ", iarg, U" arguments, not the ", narg, U" given.");
-	my okCallback (me, 0, NULL, NULL, interpreter, NULL, false, my buttonClosure);
+	my okCallback (me, 0, nullptr, nullptr, interpreter, nullptr, false, my buttonClosure);
 }
 
 void UiForm_parseString (UiForm me, const char32 *arguments, Interpreter interpreter) {
@@ -1090,7 +1072,7 @@ void UiForm_parseString (UiForm me, const char32 *arguments, Interpreter interpr
 		/*
 		 * Skip spaces until next argument.
 		 */
-		while (*arguments == ' ' || *arguments == '\t') arguments ++;
+		while (*arguments == U' ' || *arguments == U'\t') arguments ++;
 		/*
 		 * The argument is everything up to the next space, or, if that starts with a double quote,
 		 * everything between this quote and the matching double quote;
@@ -1100,7 +1082,7 @@ void UiForm_parseString (UiForm me, const char32 *arguments, Interpreter interpr
 		 * will be passed to the dialog as a single argument containing the text
 		 *     I said "hello"
 		 */
-		if (*arguments == '\"') {
+		if (*arguments == U'\"') {
 			arguments ++;   // do not include leading double quote
 			for (;;) {
 				if (*arguments == U'\0')
@@ -1124,27 +1106,27 @@ void UiForm_parseString (UiForm me, const char32 *arguments, Interpreter interpr
 	 * Leading spaces are skipped, but trailing spaces are included.
 	 */
 	if (size > 0) {
-		while (*arguments == ' ' || *arguments == '\t') arguments ++;
+		while (*arguments == U' ' || *arguments == U'\t') arguments ++;
 		try {
 			UiField_stringToValue (my field [size], arguments, interpreter);
 		} catch (MelderError) {
 			Melder_throw (U"Don't understand contents of field \"", my field [size] -> name, U"\".");
 		}
 	}
-	my okCallback (me, 0, NULL, NULL, interpreter, NULL, false, my buttonClosure);
+	my okCallback (me, 0, nullptr, nullptr, interpreter, nullptr, false, my buttonClosure);
 }
 
 void UiForm_parseStringE (EditorCommand cmd, int narg, Stackel args, const char32 *arguments, Interpreter interpreter) {
 	if (args)
-		UiForm_call(cmd -> d_uiform, narg, args, interpreter);
+		UiForm_call (cmd -> d_uiform.get(), narg, args, interpreter);
 	else
-		UiForm_parseString (cmd -> d_uiform, arguments, interpreter);
+		UiForm_parseString (cmd -> d_uiform.get(), arguments, interpreter);
 }
 
 static UiField findField (UiForm me, const char32 *fieldName) {
 	for (int ifield = 1; ifield <= my numberOfFields; ifield ++)
 		if (str32equ (fieldName, my field [ifield] -> name)) return my field [ifield];
-	return NULL;
+	return nullptr;
 }
 
 static void fatalField (UiForm dia) {
@@ -1153,7 +1135,7 @@ static void fatalField (UiForm dia) {
 
 void UiForm_setReal (UiForm me, const char32 *fieldName, double value) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_setReal:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_setReal:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_REAL: case UI_REAL_OR_UNDEFINED: case UI_POSITIVE: {
 			if (value == Melder_atof (field -> stringDefaultValue)) {
@@ -1181,7 +1163,7 @@ void UiForm_setReal (UiForm me, const char32 *fieldName, double value) {
 
 void UiForm_setInteger (UiForm me, const char32 *fieldName, long value) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_setInteger:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_setInteger:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_INTEGER: case UI_NATURAL: case UI_CHANNEL: {
 			if (value == Melder_atoi (field -> stringDefaultValue)) {
@@ -1192,15 +1174,15 @@ void UiForm_setInteger (UiForm me, const char32 *fieldName, long value) {
 		} break; case UI_BOOLEAN: {
 			GuiCheckButton_setValue (field -> checkButton, value);
 		} break; case UI_RADIO: {
-			if (value < 1 || value > field -> options -> size) value = 1;   // guard against incorrect prefs file
-			for (int i = 1; i <= field -> options -> size; i ++) {
+			if (value < 1 || value > field -> options.size) value = 1;   // guard against incorrect prefs file
+			for (int i = 1; i <= field -> options.size; i ++) {
 				if (i == value) {
-					UiOption b = static_cast <UiOption> (field -> options -> item [i]);
+					UiOption b = field -> options.at [i];
 					GuiRadioButton_set (b -> radioButton);
 				}
 			}
 		} break; case UI_OPTIONMENU: {
-			if (value < 1 || value > field -> options -> size) value = 1;   // guard against incorrect prefs file
+			if (value < 1 || value > field -> options.size) value = 1;   // guard against incorrect prefs file
 			GuiOptionMenu_setValue (field -> optionMenu, value);
 		} break; case UI_LIST: {
 			if (value < 1 || value > field -> numberOfStrings) value = 1;   // guard against incorrect prefs file
@@ -1213,8 +1195,8 @@ void UiForm_setInteger (UiForm me, const char32 *fieldName, long value) {
 
 void UiForm_setString (UiForm me, const char32 *fieldName, const char32 *value /* cattable */) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_setString:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
-	if (value == NULL) value = U"";   /* Accept NULL strings. */
+	if (! field) Melder_fatal (U"(UiForm_setString:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! value) value = U"";   // accept null strings
 	switch (field -> type) {
 		case UI_REAL: case UI_REAL_OR_UNDEFINED: case UI_POSITIVE: case UI_INTEGER: case UI_NATURAL:
 			case UI_WORD: case UI_SENTENCE: case UI_COLOUR: case UI_CHANNEL: case UI_TEXT:
@@ -1223,8 +1205,8 @@ void UiForm_setString (UiForm me, const char32 *fieldName, const char32 *value /
 		} break; case UI_LABEL: {
 			GuiLabel_setText (field -> label, value);
 		} break; case UI_RADIO: {
-			for (int i = 1; i <= field -> options -> size; i ++) {
-				UiOption b = static_cast <UiOption> (field -> options -> item [i]);
+			for (int i = 1; i <= field -> options.size; i ++) {
+				UiOption b = field -> options.at [i];
 				if (str32equ (value, b -> name)) {
 					GuiRadioButton_set (b -> radioButton);
 				}
@@ -1232,8 +1214,8 @@ void UiForm_setString (UiForm me, const char32 *fieldName, const char32 *value /
 			/* If not found: do nothing (guard against incorrect prefs file). */
 		} break; case UI_OPTIONMENU: {
 			int integerValue = 0;
-			for (int i = 1; i <= field -> options -> size; i ++) {
-				UiOption b = static_cast <UiOption> (field -> options -> item [i]);
+			for (int i = 1; i <= field -> options.size; i ++) {
+				UiOption b = field -> options.at [i];
 				if (str32equ (value, b -> name)) {
 					integerValue = i;
 					break;
@@ -1255,7 +1237,7 @@ void UiForm_setString (UiForm me, const char32 *fieldName, const char32 *value /
 
 static UiField findField_check (UiForm me, const char32 *fieldName) {
 	UiField result = findField (me, fieldName);
-	if (result == NULL) {
+	if (! result) {
 		Melder_throw (U"Cannot find field \"", fieldName, U"\" in form.\n"
 			U"The script may have changed while the form was open.\n"
 			U"Please click Cancel in the form and try again.");
@@ -1265,7 +1247,7 @@ static UiField findField_check (UiForm me, const char32 *fieldName) {
 
 double UiForm_getReal (UiForm me, const char32 *fieldName) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_getReal:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_getReal:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_REAL: case UI_REAL_OR_UNDEFINED: case UI_POSITIVE: {
 			return field -> realValue;
@@ -1292,7 +1274,7 @@ double UiForm_getReal_check (UiForm me, const char32 *fieldName) {
 
 long UiForm_getInteger (UiForm me, const char32 *fieldName) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_getInteger:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_getInteger:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_INTEGER: case UI_NATURAL: case UI_CHANNEL: case UI_BOOLEAN: case UI_RADIO:
 			case UI_OPTIONMENU: case UI_LIST:
@@ -1323,12 +1305,12 @@ long UiForm_getInteger_check (UiForm me, const char32 *fieldName) {
 
 char32 * UiForm_getString (UiForm me, const char32 *fieldName) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_getString:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_getString:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_WORD: case UI_SENTENCE: case UI_TEXT: {
 			return field -> stringValue;
 		} break; case UI_RADIO: case UI_OPTIONMENU: {
-			UiOption b = static_cast <UiOption> (field -> options -> item [field -> integerValue]);
+			UiOption b = field -> options.at [field -> integerValue];
 			return b -> name;
 		} break; case UI_LIST: {
 			return (char32 *) field -> strings [field -> integerValue];
@@ -1336,7 +1318,7 @@ char32 * UiForm_getString (UiForm me, const char32 *fieldName) {
 			fatalField (me);
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 char32 * UiForm_getString_check (UiForm me, const char32 *fieldName) {
@@ -1345,7 +1327,7 @@ char32 * UiForm_getString_check (UiForm me, const char32 *fieldName) {
 		case UI_WORD: case UI_SENTENCE: case UI_TEXT: {
 			return field -> stringValue;
 		} break; case UI_RADIO: case UI_OPTIONMENU: {
-			UiOption b = static_cast <UiOption> (field -> options -> item [field -> integerValue]);
+			UiOption b = field -> options.at [field -> integerValue];
 			return b -> name;
 		} break; case UI_LIST: {
 			return (char32 *) field -> strings [field -> integerValue];
@@ -1355,12 +1337,12 @@ char32 * UiForm_getString_check (UiForm me, const char32 *fieldName) {
 				U"Please click Cancel in the form and try again.");
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 Graphics_Colour UiForm_getColour (UiForm me, const char32 *fieldName) {
 	UiField field = findField (me, fieldName);
-	if (field == NULL) Melder_fatal (U"(UiForm_getColour:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
+	if (! field) Melder_fatal (U"(UiForm_getColour:) No field \"", fieldName, U"\" in command window \"", my name, U"\".");
 	switch (field -> type) {
 		case UI_COLOUR: {
 			return field -> colourValue;
@@ -1410,7 +1392,7 @@ void UiForm_Interpreter_addVariables (UiForm me, Interpreter interpreter) {
 				MelderString_appendCharacter (& lowerCaseFieldName, U'$');
 				var = Interpreter_lookUpVariable (interpreter, lowerCaseFieldName.string);
 				Melder_free (var -> stringValue);
-				UiOption b = static_cast <UiOption> (field -> options -> item [field -> integerValue]);
+				UiOption b = field -> options.at [field -> integerValue];
 				var -> stringValue = Melder_dup (b -> name);
 			} break; case UI_LIST: {
 				InterpreterVariable var = Interpreter_lookUpVariable (interpreter, lowerCaseFieldName.string);
