@@ -2,35 +2,25 @@
  *
  * Copyright (C) 2009-2011,2013,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-/*
- * pb 2009/01/20 created
- * pb 2010/07/13 GTK
- * pb 2010/07/26 removed UiFile_hide
- * pb 2011/02/01 cancelContinueButton
- * pb 2011/05/15 C++
- * pb 2011/07/12 C++
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "UiPause.h"
 #include "praatP.h"
 
-static UiForm thePauseForm = NULL;
-static Any thePauseFormRadio = NULL;
+static UiForm thePauseForm = nullptr;
+static UiField thePauseFormRadio = nullptr;
 static int thePauseForm_clicked = 0;
 static int theCancelContinueButton = 0;
 static int theEventLoopDepth = 0;
@@ -46,9 +36,7 @@ static void thePauseFormOkCallback (UiForm /* sendingForm */, int /* narg */, St
 	if (thePauseForm_clicked != theCancelContinueButton)
 		UiForm_Interpreter_addVariables (thePauseForm, (Interpreter) closure);   // 'closure', not 'interpreter' or 'theInterpreter'!
 }
-static void thePauseFormCancelCallback (Any dia, void *closure) {
-	(void) dia;
-	(void) closure;
+static void thePauseFormCancelCallback (UiForm /* dia */, void * /* closure */) {
 	if (theCancelContinueButton != 0) {
 		thePauseForm_clicked = theCancelContinueButton;
 	} else {
@@ -61,69 +49,69 @@ void UiPause_begin (GuiWindow topShell, const char32 *title, Interpreter interpr
 	forget (thePauseForm);   // in case an earlier build-up of another pause window was interrupted
 	thePauseForm = UiForm_create (topShell, Melder_cat (U"Pause: ", title),
 		thePauseFormOkCallback, interpreter,   // pass interpreter as closure!
-		NULL, NULL);
+		nullptr, nullptr);
 }
 void UiPause_real (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"real\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addReal (thePauseForm, label, defaultValue);
 }
 void UiPause_positive (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"positive\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addPositive (thePauseForm, label, defaultValue);
 }
 void UiPause_integer (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"integer\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addInteger (thePauseForm, label, defaultValue);
 }
 void UiPause_natural (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"natural\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addNatural (thePauseForm, label, defaultValue);
 }
 void UiPause_word (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"word\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addWord (thePauseForm, label, defaultValue);
 }
 void UiPause_sentence (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"sentence\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addSentence (thePauseForm, label, defaultValue);
 }
 void UiPause_text (const char32 *label, const char32 *defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"text\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addText (thePauseForm, label, defaultValue);
 }
 void UiPause_boolean (const char32 *label, int defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"boolean\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addBoolean (thePauseForm, label, defaultValue);
 }
 void UiPause_choice (const char32 *label, int defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"choice\" has to be between a \"beginPause\" and an \"endPause\".");
 	thePauseFormRadio = UiForm_addRadio (thePauseForm, label, defaultValue);
 }
 void UiPause_optionMenu (const char32 *label, int defaultValue) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"optionMenu\" has to be between a \"beginPause\" and an \"endPause\".");
 	thePauseFormRadio = UiForm_addOptionMenu (thePauseForm, label, defaultValue);
 }
 void UiPause_option (const char32 *label) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"option\" has to be between a \"beginPause\" and an \"endPause\".");
-	if (thePauseFormRadio == NULL) {
+	if (! thePauseFormRadio) {
 		forget (thePauseForm);
 		Melder_throw (U"Found the function \"option\" without a preceding \"choice\" or \"optionMenu\".");
 	}
 	UiOptionMenu_addButton (thePauseFormRadio, label);
 }
 void UiPause_comment (const char32 *label) {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"The function \"comment\" has to be between a \"beginPause\" and an \"endPause\".");
 	UiForm_addLabel (thePauseForm, U"", label);
 }
@@ -133,7 +121,7 @@ int UiPause_end (int numberOfContinueButtons, int defaultContinueButton, int can
 	const char32 *continueText7, const char32 *continueText8, const char32 *continueText9,
 	const char32 *continueText10, Interpreter interpreter)
 {
-	if (thePauseForm == NULL)
+	if (! thePauseForm)
 		Melder_throw (U"Found the function \"endPause\" without a preceding \"beginPause\".");
 	UiForm_setPauseForm (thePauseForm, numberOfContinueButtons, defaultContinueButton, cancelContinueButton,
 		continueText1, continueText2, continueText3, continueText4, continueText5,
@@ -172,7 +160,7 @@ int UiPause_end (int numberOfContinueButtons, int defaultContinueButton, int can
 						inMode: NSDefaultRunLoopMode
 						dequeue: YES
 					];
-					Melder_assert (nsEvent != NULL);
+					Melder_assert (nsEvent);
 					[NSApp  sendEvent: nsEvent];
 					[NSApp  updateWindows];   // called automatically?
 					[pool release];
@@ -191,8 +179,8 @@ int UiPause_end (int numberOfContinueButtons, int defaultContinueButton, int can
 	}
 	if (wasBackgrounding) praat_background ();
 	/* BUG: should also restore praatP. editor. */
-	thePauseForm = NULL;   // undangle
-	thePauseFormRadio = NULL;   // undangle
+	thePauseForm = nullptr;   // undangle
+	thePauseFormRadio = nullptr;   // undangle
 	if (thePauseForm_clicked == -1) {
 		Interpreter_stop (interpreter);
 		Melder_throw (U"You interrupted the script.");
