@@ -1,20 +1,19 @@
 /* melder.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse, 2013 Tom Naughton
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma, 2008 Stefan de Konink, 2010 Franz Brausse, 2013 Tom Naughton
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <time.h>
@@ -36,11 +35,7 @@
 #include "machine.h"
 #ifdef macintosh
 	#include "macport_on.h"
-	#if useCarbon
-		#include <Carbon/Carbon.h>
-	#else
-		#include <AudioToolbox/AudioToolbox.h>
-	#endif
+	#include <AudioToolbox/AudioToolbox.h>
 	#include "macport_off.h"
 #endif
 #include "Gui.h"
@@ -81,11 +76,6 @@ static void defaultFatal (const char32 *message) {
 	Melder_writeToConsole (U"\n", true);
 }
 
-static int defaultPublish (void *anything) {
-	(void) anything;
-	return 0;   // nothing published
-}
-
 static int defaultRecord (double duration) {
 	(void) duration;
 	return 0;   // nothing recorded
@@ -111,7 +101,6 @@ static struct {
 	void (*search) ();
 	void (*warning) (const char32 *message);
 	void (*fatal) (const char32 *message);
-	int (*publish) (void *anything);
 	int (*record) (double duration);
 	int (*recordFromFile) (MelderFile file);
 	void (*play) ();
@@ -121,7 +110,6 @@ static struct {
 	theMelder = {
 		defaultHelp, defaultSearch,
 		defaultWarning, defaultFatal,
-		defaultPublish,
 		defaultRecord, defaultRecordFromFile, defaultPlay, defaultPlayReverse, defaultPublishPlayed
 	};
 
@@ -143,50 +131,17 @@ static bool waitWhileProgress (double progress, const char32 *message, GuiDialog
 			gtk_main_iteration ();
 		}
 	#elif defined (macintosh)
-		#if useCarbon
-			EventRecord event;
-			while (GetNextEvent (mDownMask, & event)) {
-				WindowPtr macWindow;
-				int part = FindWindow (event. where, & macWindow);
-				if (part == inContent) {
-					if (GetWindowKind (macWindow) == userKind) {
-						SetPortWindowPort (macWindow);
-						GlobalToLocal (& event. where);  //CGContextConvertPointToUserSpace
-						ControlPartCode controlPart;
-						ControlHandle macControl = FindControlUnderMouse (event. where, macWindow, & controlPart);
-						if (macControl) {
-							GuiObject control = (GuiObject) GetControlReference (macControl);
-							if (control == cancelButton -> d_widget) {
-								FlushEvents (everyEvent, 0);
-								XtUnmanageChild (dia -> d_widget);
-								return false;   // don't continue
-							} else {
-								break;
-							}
-						} else {
-							XtDispatchEvent ((XEvent *) & event);
-						}
-					} else {
-						XtDispatchEvent ((XEvent *) & event);
-					}
-				} else {
-					XtDispatchEvent ((XEvent *) & event);
-				}
-			}
-			do { XtNextEvent ((XEvent *) & event); XtDispatchEvent ((XEvent *) & event); } while (event.what);
-		#else
-			NSEvent *nsEvent = [NSApp
-				nextEventMatchingMask: NSAnyEventMask
-				untilDate: [NSDate distantPast]
-				inMode: NSDefaultRunLoopMode
-				dequeue: YES
-				];
-			if (nsEvent) {
-				NSUInteger nsEventType = [nsEvent type];
-				if (nsEventType == NSKeyDown) NSBeep ();
-				[[nsEvent window]  sendEvent: nsEvent];
-			}
-		#endif
+		NSEvent *nsEvent = [NSApp
+			nextEventMatchingMask: NSAnyEventMask
+			untilDate: [NSDate distantPast]
+			inMode: NSDefaultRunLoopMode
+			dequeue: YES
+			];
+		if (nsEvent) {
+			NSUInteger nsEventType = [nsEvent type];
+			if (nsEventType == NSKeyDown) NSBeep ();
+			[[nsEvent window]  sendEvent: nsEvent];
+		}
 	#elif defined (_WIN32)
 		XEvent event;
 		while (PeekMessage (& event, 0, 0, 0, PM_REMOVE)) {
@@ -253,29 +208,26 @@ static bool waitWhileProgress (double progress, const char32 *message, GuiDialog
 			}
 		#elif motif
 			GuiProgressBar_setValue (scale, progress);
-			XmUpdateDisplay (dia -> d_widget);
+			GdiFlush ();
 		#endif
 	}
 	trace (U"continue");
 	return true;
 }
 
+static GuiButton theProgressCancelButton = nullptr;
+
 #if gtk || macintosh
-static void progress_dia_close (void *cancelButton) {
+static void progress_dia_close (Thing /* boss */) {
 	theProgressCancelled = true;
 	#if gtk
-		g_object_set_data (G_OBJECT ((* (GuiButton *) cancelButton) -> d_widget), "pressed", (gpointer) 1);
-	#else
-		(void) cancelButton;
+		g_object_set_data (G_OBJECT (theProgressCancelButton -> d_widget), "pressed", (gpointer) 1);
 	#endif
 }
-static void progress_cancel_btn_press (void *cancelButton, GuiButtonEvent event) {
-	(void) event;
+static void progress_cancel_btn_press (Thing /* boss */, GuiButtonEvent /* event */) {
 	theProgressCancelled = true;
 	#if gtk
-		g_object_set_data (G_OBJECT ((* (GuiButton *) cancelButton) -> d_widget), "pressed", (gpointer) 1);
-	#else
-		(void) cancelButton;
+		g_object_set_data (G_OBJECT (theProgressCancelButton -> d_widget), "pressed", (gpointer) 1);
 	#endif
 }
 #endif
@@ -284,7 +236,7 @@ static void _Melder_dia_init (GuiDialog *dia, GuiProgressBar *scale, GuiLabel *l
 	trace (U"creating the dialog");
 	*dia = GuiDialog_create (Melder_topShell, 200, 100, 400, hasMonitor ? 430 : 200, U"Work in progress",
 		#if gtk || macintosh
-			progress_dia_close, cancelButton,
+			progress_dia_close, nullptr,
 		#else
 			nullptr, nullptr,
 		#endif
@@ -300,8 +252,10 @@ static void _Melder_dia_init (GuiDialog *dia, GuiProgressBar *scale, GuiLabel *l
 	trace (U"creating the cancel button");
 	*cancelButton = GuiButton_createShown (*dia, 0, 400, 170, 170 + Gui_PUSHBUTTON_HEIGHT,
 		U"Interrupt",
-		#if gtk || macintosh
-			progress_cancel_btn_press, cancelButton,
+		#if gtk
+			progress_cancel_btn_press, nullptr,
+		#elif macintosh
+			progress_cancel_btn_press, nullptr,
 		#else
 			nullptr, nullptr,
 		#endif
@@ -314,15 +268,14 @@ static void _Melder_progress (double progress, const char32 *message) {
 		static clock_t lastTime;
 		static GuiDialog dia = nullptr;
 		static GuiProgressBar scale = nullptr;
-		static GuiButton cancelButton = nullptr;
 		static GuiLabel label1 = nullptr, label2 = nullptr;
 		clock_t now = clock ();
 		if (progress <= 0.0 || progress >= 1.0 ||
 			now - lastTime > CLOCKS_PER_SEC / 4)   // this time step must be much longer than the null-event waiting time
 		{
 			if (! dia)
-				_Melder_dia_init (& dia, & scale, & label1, & label2, & cancelButton, false);
-			if (! waitWhileProgress (progress, message, dia, scale, label1, label2, cancelButton))
+				_Melder_dia_init (& dia, & scale, & label1, & label2, & theProgressCancelButton, false);
+			if (! waitWhileProgress (progress, message, dia, scale, label1, label2, theProgressCancelButton))
 				Melder_throw (U"Interrupted!");
 			lastTime = now;
 		}
@@ -391,7 +344,14 @@ void Melder_progress (double progress, Melder_19_ARGS) {
 	_Melder_progress (progress, theProgressBuffer.string);
 }
 
-static void * _Melder_monitor (double progress, const char32 *message) {
+static autoGraphics graphics;
+
+static void gui_drawingarea_cb_expose (Thing /* boss */, GuiDrawingArea_ExposeEvent /* event */) {
+	if (! graphics) return;
+	Graphics_play (graphics.get(), graphics.get());
+}
+
+static Graphics _Melder_monitor (double progress, const char32 *message) {
 	if (! Melder_batch && theProgressDepth >= 0) {
 		static clock_t lastTime;
 		static GuiDialog dia = nullptr;
@@ -400,24 +360,25 @@ static void * _Melder_monitor (double progress, const char32 *message) {
 		static GuiButton cancelButton = nullptr;
 		static GuiLabel label1 = nullptr, label2 = nullptr;
 		clock_t now = clock ();
-		static Any graphics = nullptr;
 		if (progress <= 0.0 || progress >= 1.0 ||
 			now - lastTime > CLOCKS_PER_SEC / 4)   // this time step must be much longer than the null-event waiting time
 		{
 			if (! dia) {
 				_Melder_dia_init (& dia, & scale, & label1, & label2, & cancelButton, true);
-				drawingArea = GuiDrawingArea_createShown (dia, 0, 400, 230, 430, nullptr, nullptr, nullptr, nullptr, nullptr, 0);
+				drawingArea = GuiDrawingArea_createShown (dia, 0, 400, 230, 430, gui_drawingarea_cb_expose, nullptr, nullptr, nullptr, nullptr, 0);
 				GuiThing_show (dia);
 				graphics = Graphics_create_xmdrawingarea (drawingArea);
 			}
+			if (graphics)
+				Graphics_flushWs (graphics.get());
 			if (! waitWhileProgress (progress, message, dia, scale, label1, label2, cancelButton))
 				Melder_throw (U"Interrupted!");
 			lastTime = now;
 			if (progress == 0.0)
-				return graphics;
+				return graphics.get();
 		}
 	}
-	return progress <= 0.0 ? nullptr /* no Graphics */ : (void *) -1 /* any non-null pointer */;
+	return progress <= 0.0 ? nullptr /* no Graphics */ : (Graphics) -1 /* any non-null pointer */;
 }
 
 void * Melder_monitor (double progress) {
@@ -616,11 +577,7 @@ void Melder_warning (Melder_19_ARGS) {
 
 void Melder_beep () {
 	#ifdef macintosh
-		#if useCarbon
-			SysBeep (0);
-		#else
-            AudioServicesPlayAlertSound (kSystemSoundID_UserPreferredAlert);
-		#endif
+		AudioServicesPlayAlertSound (kSystemSoundID_UserPreferredAlert);
 	#else
 		fprintf (stderr, "\a");
 	#endif
@@ -1053,42 +1010,34 @@ static void mac_message (NSAlertStyle macAlertType, const char32 *message32) {
 			}
 		}
 	}
-	#if useCarbon
-        DialogRef dialog;
-		CFStringRef messageCF = CFStringCreateWithCharacters (nullptr, messageU, j);
-		CreateStandardAlert (macAlertType, messageCF, nullptr, nullptr, & dialog);
-		CFRelease (messageCF);
-		RunStandardAlert (dialog, nullptr, nullptr);
-	#else
-		/*
-		 * Create an alert dialog with an icon that is appropriate for the level.
-		 */
-		NSAlert *alert = [[NSAlert alloc] init];
-		[alert setAlertStyle: macAlertType];
-		/*
-		 * Add the header in bold.
-		 */
-		NSString *header = [[NSString alloc] initWithCharacters: messageU   length: lineBreak - messageU];   // note: init can change the object pointer!
-		if (header) {   // make this very safe, because we can be at error time or at fatal time
-			[alert setMessageText: header];
-			[header release];
+	/*
+	 * Create an alert dialog with an icon that is appropriate for the level.
+	 */
+	NSAlert *alert = [[NSAlert alloc] init];
+	[alert setAlertStyle: macAlertType];
+	/*
+	 * Add the header in bold.
+	 */
+	NSString *header = [[NSString alloc] initWithCharacters: messageU   length: lineBreak - messageU];   // note: init can change the object pointer!
+	if (header) {   // make this very safe, because we can be at error time or at fatal time
+		[alert setMessageText: header];
+		[header release];
+	}
+	/*
+	 * Add the rest of the message in small type.
+	 */
+	if (lineBreak - messageU < j) {
+		NSString *rest = [[NSString alloc] initWithCharacters: lineBreak + 1   length: j - 1 - (lineBreak - messageU)];
+		if (rest) {   // make this very safe, because we can be at error time or at fatal time
+			[alert setInformativeText: rest];
+			[rest release];
 		}
-		/*
-		 * Add the rest of the message in small type.
-		 */
-		if (lineBreak - messageU < j) {
-			NSString *rest = [[NSString alloc] initWithCharacters: lineBreak + 1   length: j - 1 - (lineBreak - messageU)];
-			if (rest) {   // make this very safe, because we can be at error time or at fatal time
-				[alert setInformativeText: rest];
-				[rest release];
-			}
-		}
-		/*
-		 * Display the alert dialog and synchronously wait for the user to click OK.
-		 */
-		[alert runModal];
-		[alert release];
-	#endif
+	}
+	/*
+	 * Display the alert dialog and synchronously wait for the user to click OK.
+	 */
+	[alert runModal];
+	[alert release];
 }
 #endif
 
@@ -1103,13 +1052,8 @@ static void gui_fatal (const char32 *message) {
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	#elif defined (macintosh)
-		#if useCarbon
-			mac_message (NSCriticalAlertStyle, message);
-			SysError (11);
-		#else
-			mac_message (NSCriticalAlertStyle, message);
-			SysError (11);
-		#endif
+		mac_message (NSCriticalAlertStyle, message);
+		SysError (11);
 	#elif defined (_WIN32)
 		MessageBox (nullptr, Melder_peek32toW (message), L"Fatal error", MB_OK | MB_TOPMOST | MB_ICONSTOP);
 	#endif
@@ -1129,12 +1073,7 @@ static void gui_error (const char32 *message) {
 		trace (U"destroy dialog");
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	#elif defined (macintosh)
-		#if useCarbon
-			mac_message (NSWarningAlertStyle, message);
-			XmUpdateDisplay (0);
-		#else
-			mac_message (NSWarningAlertStyle, message);
-		#endif
+		mac_message (NSWarningAlertStyle, message);
 	#elif defined (_WIN32)
 		MessageBox (nullptr, Melder_peek32toW (message), L"Message", MB_OK | MB_TOPMOST | MB_ICONWARNING);   // or (HWND) XtWindow ((GuiObject) Melder_topShell)
 	#endif
@@ -1147,12 +1086,7 @@ static void gui_error (const char32 *message) {
 				gtk_dialog_run (GTK_DIALOG (dialog));
 				gtk_widget_destroy (GTK_WIDGET (dialog));
 			#elif defined (macintosh)
-				#if useCarbon
-					mac_message (NSCriticalAlertStyle, U"Praat is very low on memory.\nSave your work and quit Praat.\nIf you don't do that, Praat may crash.");
-					XmUpdateDisplay (0);
-				#else
-					mac_message (NSCriticalAlertStyle, U"Praat is very low on memory.\nSave your work and quit Praat.\nIf you don't do that, Praat may crash.");
-				#endif
+				mac_message (NSCriticalAlertStyle, U"Praat is very low on memory.\nSave your work and quit Praat.\nIf you don't do that, Praat may crash.");
 			#elif defined (_WIN32)
 				MessageBox (nullptr, L"Praat is very low on memory.\nSave your work and quit Praat.\nIf you don't do that, Praat may crash.", L"Message", MB_OK);
 			#endif
@@ -1167,12 +1101,7 @@ static void gui_warning (const char32 *message) {
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	#elif defined (macintosh)
-		#if useCarbon
-			mac_message (NSInformationalAlertStyle, message);
-			XmUpdateDisplay (0);
-		#else
-			mac_message (NSInformationalAlertStyle, message);
-		#endif
+		mac_message (NSInformationalAlertStyle, message);
 	#elif defined (_WIN32)
 		MessageBox (nullptr, Melder_peek32toW (message), L"Warning", MB_OK | MB_TOPMOST | MB_ICONINFORMATION);
 	#endif
@@ -1189,10 +1118,6 @@ void MelderGui_create (void *parent) {
 	Melder_setWarningProc (gui_warning);
 }
 
-int Melder_publish (void *anything) {
-	return theMelder. publish (anything);
-}
-
 int Melder_record (double duration) {
 	return theMelder. record (duration);
 }
@@ -1227,9 +1152,6 @@ void Melder_setWarningProc (void (*warning) (const char32 *))
 void Melder_setFatalProc (void (*fatal) (const char32 *))
 	{ theMelder. fatal = fatal ? fatal : defaultFatal; }
 
-void Melder_setPublishProc (int (*publish) (void *))
-	{ theMelder. publish = publish ? publish : defaultPublish; }
-
 void Melder_setRecordProc (int (*record) (double))
 	{ theMelder. record = record ? record : defaultRecord; }
 
