@@ -2,21 +2,20 @@
 #define _melder_h_
 /* melder.h
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -34,6 +33,7 @@
 	#include <sys/types.h>   // for off_t
 #endif
 #include <stdbool.h>
+#include <functional>
 /*
  * The following two lines are for obsolete (i.e. C99) versions of stdint.h
  */
@@ -166,7 +166,7 @@ inline static int64 str32spn (const char32 *string1, const char32 *string2) {
 	char32 kar1, kar2;
 cont:
 	kar1 = * p ++;
-	for (const char32 * q = string2; (kar2 = * q ++) != 0;)
+	for (const char32 * q = string2; (kar2 = * q ++) != U'\0';)
 		if (kar2 == kar1)
 			goto cont;
 	return p - 1 - string1;
@@ -407,18 +407,14 @@ struct structMelderDir {
 };
 typedef struct structMelderDir *MelderDir;
 
-#if defined (macintosh) && useCarbon
-	void Melder_machToFile (void *void_fsref, MelderFile file);
-#endif
-
 const char32 * MelderFile_name (MelderFile file);
-char32 * MelderDir_name (MelderDir dir);
+const char32 * MelderDir_name (MelderDir dir);
 void Melder_pathToDir (const char32 *path, MelderDir dir);
 void Melder_pathToFile (const char32 *path, MelderFile file);
 void Melder_relativePathToFile (const char32 *path, MelderFile file);
-char32 * Melder_dirToPath (MelderDir dir);
+const char32 * Melder_dirToPath (MelderDir dir);
 	/* Returns a pointer internal to 'dir', like "/u/paul/praats" or "D:\Paul\Praats" */
-char32 * Melder_fileToPath (MelderFile file);
+const char32 * Melder_fileToPath (MelderFile file);
 void MelderFile_copy (MelderFile file, MelderFile copy);
 void MelderDir_copy (MelderDir dir, MelderDir copy);
 bool MelderFile_equal (MelderFile file1, MelderFile file2);
@@ -434,7 +430,7 @@ void MelderDir_getParentDir (MelderDir file, MelderDir parent);
 bool MelderDir_isDesktop (MelderDir dir);
 void MelderDir_getSubdir (MelderDir parent, const char32 *subdirName, MelderDir subdir);
 void Melder_rememberShellDirectory ();
-char32 * Melder_getShellDirectory ();
+const char32 * Melder_getShellDirectory ();
 void Melder_getHomeDir (MelderDir homeDir);
 void Melder_getPrefDir (MelderDir prefDir);
 void Melder_getTempDir (MelderDir tempDir);
@@ -1100,7 +1096,7 @@ void * Melder_monitor (double progress, Melder_16_TO_19_ARGS);
 	Arguments:
 		Any of 'arg1' through 'arg19' may be null.
 	Batch behaviour:
-		Does nothing, returns nullptr if 'progress' <= 0.0 and a non-nullptr pointer otherwise.
+		Does nothing, returns null if 'progress' <= 0.0 and a non-null pointer otherwise.
 	Interactive behaviour:
 		Shows the progress of a time-consuming process:
 		- if 'progress' <= 0.0, show a window with text and a Cancel button and
@@ -1129,7 +1125,7 @@ void * Melder_monitor (double progress, Melder_16_TO_19_ARGS);
 		      }
 		- after the process has finished, call with 'progress' = 1.0:
 		      (void) Melder_monitor (1.0, nullptr);
-		- the first and third steps can be automated by autoMelderMonitor:
+		- showing and hiding can be automated by autoMelderMonitor:
 		      autoMelderMonitor monitor ("Starting work...");
 		      if (monitor.graphics()) {   // always check; might be batch
 		          Graphics_clearWs (monitor.graphics());   // only if you redraw all every time
@@ -1152,8 +1148,6 @@ public:
 
 /********** RECORD AND PLAY ROUTINES **********/
 
-int Melder_publish (void *anything);
-
 int Melder_record (double duration);
 int Melder_recordFromFile (MelderFile file);
 void Melder_play ();
@@ -1200,7 +1194,6 @@ void Melder_setSearchProc (void (*search) ());
 void Melder_setWarningProc (void (*warningProc) (const char32 *message));
 void Melder_setErrorProc (void (*errorProc) (const char32 *message));
 void Melder_setFatalProc (void (*fatalProc) (const char32 *message));
-void Melder_setPublishProc (int (*publish) (void *));
 void Melder_setRecordProc (int (*record) (double));
 void Melder_setRecordFromFileProc (int (*recordFromFile) (MelderFile));
 void Melder_setPlayProc (void (*play) ());
@@ -1208,6 +1201,7 @@ void Melder_setPlayReverseProc (void (*playReverse) ());
 void Melder_setPublishPlayedProc (int (*publishPlayed) ());
 
 double Melder_stopwatch ();
+void Melder_sleep (double duration);
 
 /********** AUDIO **********/
 
@@ -1388,23 +1382,23 @@ public:
 };
 
 class autoMelderFile {
-	MelderFile file;
+	MelderFile _file;
 public:
-	autoMelderFile (MelderFile a_file) : file (a_file) {
+	autoMelderFile (MelderFile file) : _file (file) {
 	}
 	~autoMelderFile () {
-		if (file) MelderFile_close_nothrow (file);
+		if (_file) MelderFile_close_nothrow (_file);
 	}
 	void close () {
-		if (file && file -> filePointer) {
-			MelderFile tmp = file;
-			file = nullptr;
+		if (_file && _file -> filePointer) {
+			MelderFile tmp = _file;
+			_file = nullptr;
 			MelderFile_close (tmp);
 		}
 	}
 	MelderFile transfer () {
-		MelderFile tmp = file;
-		file = nullptr;
+		MelderFile tmp = _file;
+		_file = nullptr;
 		return tmp;
 	}
 };
@@ -1446,15 +1440,20 @@ public:
 
 class autoMelderTokens {
 	char32 **tokens;
+	long numberOfTokens;
 public:
 	autoMelderTokens () {
 		tokens = nullptr;
 	}
-	autoMelderTokens (const char32 *string, long *n) {
-		tokens = Melder_getTokens (string, n);
+	autoMelderTokens (const char32 *string) {
+		tokens = Melder_getTokens (string, & numberOfTokens);
 	}
 	~autoMelderTokens () {
-		if (tokens) Melder_freeTokens (& tokens);
+		if (tokens) {
+			for (long itoken = 1; itoken <= numberOfTokens; itoken ++)
+				Melder_free (tokens [itoken]);
+			Melder_freeTokens (& tokens);
+		}
 	}
 	char32*& operator[] (long i) {
 		return tokens [i];
@@ -1462,9 +1461,16 @@ public:
 	char32 ** peek () const {
 		return tokens;
 	}
-	void reset (const char32 *string, long *n) {
-		if (tokens) Melder_freeTokens (& tokens);
-		tokens = Melder_getTokens (string, n);
+	long count () const {
+		return numberOfTokens;
+	}
+	void reset (const char32 *string) {
+		if (tokens) {
+			for (long itoken = 1; itoken <= numberOfTokens; itoken ++)
+				Melder_free (tokens [itoken]);
+			Melder_freeTokens (& tokens);
+		}
+		tokens = Melder_getTokens (string, & numberOfTokens);
 	}
 };
 
@@ -1541,5 +1547,58 @@ struct autoMelderAsynchronous {
 	~autoMelderAsynchronous () { Melder_asynchronous = false; }
 };
 
+#define Melder_ENABLE_IF_ISA(A,B)  , class = typename std::enable_if<std::is_base_of<B,A>::value>::type
+
+template <typename Ret, typename T, typename... Args>
+class MelderCallback {
+	public:
+		typedef Ret* (*FunctionType) (T*, Args...);
+		MelderCallback (FunctionType f = nullptr) : _f (f) { }
+		template <typename T2  Melder_ENABLE_IF_ISA(T2,T), typename Ret2  Melder_ENABLE_IF_ISA(Ret2,Ret)>
+			MelderCallback (Ret2* (*f) (T2*, Args...)) : _f (reinterpret_cast<FunctionType> (f)) { };
+		Ret* operator () (T* data, Args ... args) { return _f (data, args...); }
+		explicit operator bool () const { return !! _f; }
+	private:
+		FunctionType _f;
+};
+template <typename T, typename... Args>
+class MelderCallback <void, T, Args...> {   // specialization
+	public:
+		typedef void (*FunctionType) (T*, Args...);
+		MelderCallback (FunctionType f = nullptr) : _f (f) { }
+		template <typename T2  Melder_ENABLE_IF_ISA(T2,T)>
+			MelderCallback (void (*f) (T2*, Args...)) : _f (reinterpret_cast<FunctionType> (f)) { };
+		void operator () (T* data, Args ... args) { _f (data, args...); }
+		explicit operator bool () const { return !! _f; }
+	private:
+		FunctionType _f;
+};
+template <typename T, typename... Args>
+class MelderCallback <int, T, Args...> {   // specialization
+	public:
+		typedef int (*FunctionType) (T*, Args...);
+		MelderCallback (FunctionType f = nullptr) : _f (f) { }
+		template <typename T2  Melder_ENABLE_IF_ISA(T2,T)>
+			MelderCallback (int (*f) (T2*, Args...)) : _f (reinterpret_cast<FunctionType> (f)) { };
+		int operator () (T* data, Args ... args) { return _f (data, args...); }
+		explicit operator bool () const { return !! _f; }
+	private:
+		FunctionType _f;
+};
+
+template <typename T>
+class MelderCompareHook {
+	public:
+		typedef int (*FunctionType) (T*, T*);
+		MelderCompareHook (FunctionType f = nullptr) : _f (f) { }
+		template <typename T2  Melder_ENABLE_IF_ISA(T2,T)>
+			MelderCompareHook (int (*f) (T2*, T2*)) : _f (reinterpret_cast<FunctionType> (f)) { };
+		int operator () (T* data1, T* data2) noexcept { return _f (data1, data2); }
+		explicit operator bool () const { return !! _f; }
+		FunctionType get () { return _f; }
+	private:
+		FunctionType _f;
+};
+
 /* End of file melder.h */
 #endif
