@@ -1,46 +1,19 @@
 /* melder_audio.cpp
  *
- * Copyright (C) 1992-2011,2012,2013,2014,2015 Paul Boersma, David Weenink
+ * Copyright (C) 1992-2011,2012,2013,2014,2015,2016 Paul Boersma, David Weenink
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-/*
- * pb 2003/08/22 used getenv ("AUDIODEV") on Sun (thanks to Michel Scheffers)
- * pb 2003/10/22 fake mono for Linux drivers that do not support mono
- * pb 2003/12/06 use sys/soundcard.h instead of linux/soundcard.h for FreeBSD compatibility
- * pb 2004/05/07 removed motif_mac_setNullEventWaitingTime (we nowadays use 1 clock tick everywhere anyway)
- * pb 2004/08/10 fake mono for Linux drivers etc, also if not asynchronous
- * pb 2005/02/13 added O_NDELAY when opening /dev/dsp on Linux (suggestion by Rafael Laboissiere)
- * pb 2005/03/31 undid previous change (four complaints that sound stopped playing)
- * pb 2005/05/19 redid previous change (with fctrl fix suggested by Rafael Laboissiere)
- * pb 2005/10/13 edition for OpenBSD
- * pb 2006/10/28 erased MacOS 9 stuff
- * pb 2006/12/16 Macintosh uses CoreAudio (via PortAudio)
- * pb 2007/01/03 best sample rate can be over 64 kHz
- * pb 2007/05/13 null pointer test for deviceInfo (thanks to Stefan de Konink)
- * pb 2007/08/12 wchar
- * Stefan de Konink 2007/12/02 big-endian Linux
- * pb 2007/12/04 enums
- * pb 2008/06/01 removed SPEXLAB audio server
- * pb 2008/06/10 made PortAudio and foreground playing optional
- * pb 2008/07/03 DirectSound
- * pb 2010/05/09 GTK
- * pb 2011/02/11 better message
- * pb 2011/04/05 C++
- * pb 2015/06/08 char32
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if defined (macintosh)
@@ -77,11 +50,16 @@
 #include "../external/portaudio/portaudio.h"
 
 #ifdef HAVE_PULSEAUDIO
+	void pulseAudio_initialize ();
 	void pulseAudio_cleanup ();
+	void pulseAudio_serverReport ();
+	void stream_state_cb (pa_stream *stream, void *userdata);
 	void stream_drain_complete_cb (pa_stream *stream, int success, void *userdata);
 	void context_state_cb (pa_context *context, void *userdata);
 	void context_drain_complete_cb (pa_context *context, void *userdata);
 	void stream_write_cb (pa_stream *stream, size_t length, void *userdata);
+	void stream_write_cb2 (pa_stream *stream, size_t length, void *userdata);
+	void pulseAudio_server_info_cb (pa_context *context, const pa_server_info *info, void *userdata);
 #endif
 
 static struct {
@@ -95,7 +73,7 @@ static struct {
 void Melder_audio_prefs () {
 	Preferences_addEnum (U"Audio.maximumAsynchronicity", & preferences. maximumAsynchronicity, kMelder_asynchronicityLevel, kMelder_asynchronicityLevel_DEFAULT);
 	Preferences_addEnum (U"Audio.inputSoundSystem", & preferences. inputSoundSystem, kMelder_inputSoundSystem, kMelder_inputSoundSystem_DEFAULT);
-	Preferences_addEnum (U"Audio.inputSoundSystem", & preferences. outputSoundSystem, kMelder_outputSoundSystem, kMelder_outputSoundSystem_DEFAULT);
+	Preferences_addEnum (U"Audio.outputSoundSystem", & preferences. outputSoundSystem, kMelder_outputSoundSystem, kMelder_outputSoundSystem_DEFAULT);
 	Preferences_addBool (U"Audio.useInternalSpeaker", & preferences. useInternalSpeaker, true);
 	Preferences_addDouble (U"Audio.silenceBefore2", & preferences. silenceBefore, kMelderAudio_outputSilenceBefore_DEFAULT);
 	Preferences_addDouble (U"Audio.silenceAfter2", & preferences. silenceAfter, kMelderAudio_outputSilenceAfter_DEFAULT);
@@ -159,10 +137,12 @@ bool MelderAudio_isPlaying;
 
 static double theStartingTime = 0.0;
 
-#define PA_INFO_GETTING 1
-#define PA_INFO_FOUND 2
+#define PA_GETTINGINFO 1
+#define PA_GETTINGINFO_DONE 2
 #define PA_WRITING 4
 #define PA_WRITING_DONE 8
+#define PA_RECORDING 16
+#define PA_RECORDING_DONE 32
 
 #ifdef HAVE_PULSEAUDIO
 typedef struct pulseAudio {
@@ -673,7 +653,7 @@ void pulseAudio_server_info_cb (pa_context *context, const pa_server_info *info,
 	}
 	MelderInfo_close ();
 	trace (U"before signal");
-	my pulseAudio.occupation |= PA_INFO_FOUND;
+	my pulseAudio.occupation |= PA_GETTINGINFO_DONE;
 	 // We are done, signal it to pulseAudio_serverReport
 	pa_threaded_mainloop_signal (my pulseAudio.mainloop, 0);
 }
@@ -683,24 +663,24 @@ void pulseAudio_serverReport () {
 	struct MelderPlay *me = & thePlay;
 	if (my pulseAudio.mainloop) {
 		pa_threaded_mainloop_lock (my pulseAudio.mainloop);
-		my pulseAudio.occupation |= PA_INFO_GETTING;
+		my pulseAudio.occupation |= PA_GETTINGINFO;
 		if (my pulseAudio.context) {
 			pa_operation *operation = pa_context_get_server_info (my pulseAudio.context, pulseAudio_server_info_cb, me);
 			trace (U"operation started");
 			if (! operation) {
 				Melder_throw (U"pulseAudioServer report: ", Melder_peek8to32 (pa_strerror (pa_context_errno (my pulseAudio.context))));
 			}
-			while ((my pulseAudio.occupation & PA_INFO_FOUND) != PA_INFO_FOUND) {
+			while ((my pulseAudio.occupation & PA_GETTINGINFO_DONE) != PA_GETTINGINFO_DONE) {
 				pa_threaded_mainloop_wait (my pulseAudio.mainloop);
 			}
 			// Now it is save to unref because the server info operation has completed
 			pa_operation_unref (operation);
-			my pulseAudio.occupation ^= ~PA_INFO_FOUND;
+			my pulseAudio.occupation ^= ~PA_GETTINGINFO_DONE;
 		}
-		my pulseAudio.occupation ^= ~PA_INFO_GETTING;
+		my pulseAudio.occupation ^= ~PA_GETTINGINFO;
 		pa_threaded_mainloop_unlock (my pulseAudio.mainloop);
 	} else {
-		my pulseAudio.occupation |= PA_INFO_GETTING;
+		my pulseAudio.occupation |= PA_GETTINGINFO;
 		pulseAudio_initialize ();
 		/*
 		 * First acquire a lock because the operation to get server info (in context_state_cb) may not have started yet.
@@ -708,14 +688,14 @@ void pulseAudio_serverReport () {
 		 */
 		pa_threaded_mainloop_lock (my pulseAudio.mainloop);
 
-		while ((my pulseAudio.occupation & PA_INFO_FOUND) != PA_INFO_FOUND) {
+		while ((my pulseAudio.occupation & PA_GETTINGINFO_DONE) != PA_GETTINGINFO_DONE) {
 			pa_threaded_mainloop_wait (my pulseAudio.mainloop);
 		}
 		// Now we know that the operation to get server info has succeeded!
 		pa_operation_unref (my pulseAudio.operation_info);
 		my pulseAudio.operation_info = nullptr;
-		my pulseAudio.occupation ^= ~PA_INFO_GETTING;
-		my pulseAudio.occupation ^= ~PA_INFO_FOUND;
+		my pulseAudio.occupation ^= ~PA_GETTINGINFO;
+		my pulseAudio.occupation ^= ~PA_GETTINGINFO_DONE;
 		pa_threaded_mainloop_unlock (my pulseAudio.mainloop);
 		if (! MelderAudio_isPlaying) {
 			my pulseAudio.occupation = 0;
@@ -762,9 +742,9 @@ void stream_drain_complete_cb (pa_stream *stream, int success, void *userdata) {
 	}
 }
 
-static void free_cb(void *p) { // to prevent copying of data
-	(void) p;
+static void free_cb(void * /* p */) { // to prevent copying of data
 }
+
 // asynchronous version
 void stream_write_cb2 (pa_stream *stream, size_t length, void *userdata) {
 	struct MelderPlay *me = (struct MelderPlay *) userdata;
@@ -839,6 +819,7 @@ void stream_write_cb (pa_stream *stream, size_t length, void *userdata) {
 				// returns the address of an internal buffer and the number of bytes that can maximally be written to it.
 				if (pa_stream_begin_write (stream, (void **) & pa_buffer, & nbytes) < 0) {
 					trace (U"error: no memory");
+					Melder_throw (U"Pulseaudio has no internal memory left.");
 				}
 				trace (U"buffer size = ", nbytes);
 				// do we need the full buffer space ?
@@ -948,7 +929,7 @@ void context_state_cb (pa_context *context, void *userdata) {
 			break;
 		case PA_CONTEXT_READY: {
 			trace (U"PA_CONTEXT_READY");
-			if ((my pulseAudio.occupation & PA_INFO_GETTING) == PA_INFO_GETTING) {
+			if ((my pulseAudio.occupation & PA_GETTINGINFO) == PA_GETTINGINFO) {
 				my pulseAudio.operation_info = pa_context_get_server_info (my pulseAudio.context, pulseAudio_server_info_cb, me);
 				trace (U"operation started");
 				if (! my pulseAudio.operation_info) {
@@ -994,6 +975,8 @@ void context_state_cb (pa_context *context, void *userdata) {
 					Melder_throw (U"pa_stream_connect_playback() failed: ", Melder_peek8to32 (pa_strerror (pa_context_errno (my pulseAudio.context))));
 				}
 				trace (U"tlength = ", my pulseAudio.buffer_attr.tlength, U", channels = ", my numberOfChannels);
+			} else if ((my pulseAudio.occupation & PA_RECORDING) == PA_RECORDING) {
+				
 			}
 			break;
 		}
