@@ -2,19 +2,18 @@
  *
  * Copyright (C) 2000-2012,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "melder.h"
@@ -44,8 +43,7 @@ the behaviour of that program changes in the following way:
 8: Windows: don't reset waveIn, in SoundRecorder.cpp.
 9: flush Error in FunctionEditor_Sound_draw
 10: geometric pens
-11: clicked item in option menu in Ui.cpp.
-12: no forced update event in XmUpdateDisplay on Mac
+11: clicked item in option menu in Ui.cpp
 14: switches off the progress window in melder.cpp
 15: don't use TrueType IPA fonts, but always bitmaps instead
 16: Linux: open /dev/dsp without O_NDELAY
@@ -183,6 +181,7 @@ static const char16 * peek32to16 (const char32 *string) {
 
 bool Melder_consoleIsAnsi = false;
 
+//extern FILE *winstdout;
 void Melder_writeToConsole (const char32 *message, bool useStderr) {
 	if (! message) return;
 	#if defined (_WIN32)
@@ -190,6 +189,7 @@ void Melder_writeToConsole (const char32 *message, bool useStderr) {
 		static HANDLE console = nullptr;
 		if (! console) {
 			console = CreateFileW (L"CONOUT$", GENERIC_WRITE, FILE_SHARE_WRITE, nullptr, OPEN_EXISTING, 0, 0);
+			//freopen ("CONOUT$", "w", stdout);
 		}
 		if (Melder_consoleIsAnsi) {
 			size_t n = str32len (message);
@@ -197,6 +197,8 @@ void Melder_writeToConsole (const char32 *message, bool useStderr) {
 				unsigned int kar = (unsigned short) message [i];
 				fputc (kar, stdout);
 			}
+			//CHAR* messageA = (CHAR*) peek32to8 (message);
+			//WriteConsoleA (console, messageA, strlen (messageA), nullptr, nullptr);
 		//} else if (Melder_consoleIsUtf8) {
 			//char *messageA = Melder_peek32to8 (message);
 			//fprintf (stdout, "%s", messageA);
