@@ -1,20 +1,19 @@
 /* oo_READ_TEXT.h
  *
- * Copyright (C) 1994-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1994-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "oo_undef.h"
@@ -139,26 +138,19 @@
 		} \
 	}
 
-#define oo_OBJECT(Class,formatVersion,x)  \
-	if (texgetex (a_text) == 1) { \
-		our x = Thing_new (Class); \
-		our x -> v_readText (a_text, formatVersion); \
-	}
-
 #define oo_AUTO_OBJECT(Class,formatVersion,x)  \
 	if (texgetex (a_text) == 1) { \
 		our x = Thing_new (Class); \
 		our x -> v_readText (a_text, formatVersion); \
 	}
 
-#define oo_COLLECTION(Class,x,ItemClass,formatVersion)  \
+#define oo_COLLECTION_OF(Class,x,ItemClass,formatVersion)  \
 	{ \
 		long n = texgeti4 (a_text); \
-		our x = Class##_create (); \
 		for (long i = 1; i <= n; i ++) { \
-			auto##ItemClass item = (ItemClass) Thing_new (ItemClass); \
-			item.peek() -> v_readText (a_text, formatVersion); \
-			Collection_addItem (our x, item.transfer()); \
+			auto##ItemClass item = Thing_new (ItemClass); \
+			item -> v_readText (a_text, formatVersion); \
+			our x.addItem_move (item.move()); \
 		} \
 	}
 
@@ -167,9 +159,9 @@
 		long n = texgeti4 (a_text); \
 		our x = Class##_create (); \
 		for (long i = 1; i <= n; i ++) { \
-			auto##ItemClass item = (ItemClass) Thing_new (ItemClass); \
-			item.peek() -> v_readText (a_text, formatVersion); \
-			Collection_addItem (our x.get(), item.transfer()); \
+			auto##ItemClass item = Thing_new (ItemClass); \
+			item -> v_readText (a_text, formatVersion); \
+			our x -> addItem_move (item.move()); \
 		} \
 	}
 
