@@ -1,20 +1,19 @@
 /* praat.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "melder.h"
@@ -27,13 +26,6 @@
 	#include <signal.h>
 #endif
 #include <locale.h>
-#ifdef macintosh
-	#include "macport_on.h"
-    #if useCarbon
-        #include <Carbon/Carbon.h>
-    #endif
-	#include "macport_off.h"
-#endif
 #if defined (UNIX)
 	#include <unistd.h>
 #endif
@@ -56,6 +48,8 @@
 	#include <gdk/gdkx.h>
 #endif
 
+Thing_implement (Praat_Command, Thing, 0);
+
 #define EDITOR  theCurrentPraatObjects -> list [IOBJECT]. editors
 
 #define WINDOW_WIDTH 520
@@ -69,7 +63,7 @@ structPraatPicture theForegroundPraatPicture;
 PraatPicture theCurrentPraatPicture = & theForegroundPraatPicture;
 struct PraatP praatP;
 static char32 programName [64];
-static structMelderDir homeDir = { { 0 } };
+static structMelderDir homeDir { { 0 } };
 /*
  * praatDirectory: preferences file, buttons file, message files, tracing file, plugins.
  *    Unix:   /u/miep/.myProg-dir   (without slash)
@@ -78,7 +72,7 @@ static structMelderDir homeDir = { { 0 } };
  *    Mac X:   /Users/Miep/Library/Preferences/MyProg Prefs
  */
 extern structMelderDir praatDir;
-structMelderDir praatDir = { { 0 } };
+structMelderDir praatDir { { 0 } };
 /*
  * prefsFile: preferences file.
  *    Unix:   /u/miep/.myProg-dir/prefs5
@@ -86,7 +80,7 @@ structMelderDir praatDir = { { 0 } };
  *                       or:   C:\Users\Miep\MyProg\Preferences5.ini
  *    Mac X:   /Users/Miep/Library/Preferences/MyProg Prefs/Prefs5
  */
-static structMelderFile prefsFile = { 0 };
+static structMelderFile prefsFile { 0 };
 /*
  * buttonsFile: buttons file.
  *    Unix:   /u/miep/.myProg-dir/buttons
@@ -94,12 +88,12 @@ static structMelderFile prefsFile = { 0 };
  *                    or:   C:\Users\Miep\MyProg\Buttons5.ini
  *    Mac X:   /Users/Miep/Library/Preferences/MyProg Prefs/Buttons5
  */
-static structMelderFile buttonsFile = { 0 };
+static structMelderFile buttonsFile { 0 };
 #if defined (UNIX)
-	static structMelderFile pidFile = { 0 };   // like /u/miep/.myProg-dir/pid
-	static structMelderFile messageFile = { 0 };   // like /u/miep/.myProg-dir/message
+	static structMelderFile pidFile { 0 };   // like /u/miep/.myProg-dir/pid
+	static structMelderFile messageFile { 0 };   // like /u/miep/.myProg-dir/message
 #elif defined (_WIN32)
-	static structMelderFile messageFile = { 0 };   // like C:\Users\Miep\myProg\Message.txt
+	static structMelderFile messageFile { 0 };   // like C:\Users\Miep\myProg\Message.txt
 #endif
 /*
  * tracingFile: tracing file.
@@ -108,7 +102,7 @@ static structMelderFile buttonsFile = { 0 };
  *                    or:   C:\Users\Miep\MyProg\Tracing.txt
  *    Mac X:   /Users/Miep/Library/Preferences/MyProg Prefs/Tracing.txt
  */
-static structMelderFile tracingFile = { 0 };
+static structMelderFile tracingFile { 0 };
 
 static GuiList praatList_objects;
 
@@ -253,15 +247,14 @@ Daata praat_firstObject_any () {
 	return nullptr;   // this is often OK
 }
 
-Collection praat_getSelectedObjects () {
-	autoCollection thee = Collection_create (nullptr, 10);
-	Collection_dontOwnItems (thee.peek());
+autoCollection praat_getSelectedObjects () {
+	autoCollection thee = Collection_create ();
 	int IOBJECT;
 	LOOP {
 		iam_LOOP (Daata);
-		Collection_addItem (thee.peek(), me);
+		thy addItem_ref (me);
 	}
-	return thee.transfer();
+	return thee;
 }
 
 char32 *praat_name (int IOBJECT) { return str32chr (FULL_NAME, U' ') + 1; }
@@ -283,7 +276,7 @@ void praat_write_do (UiForm dia, const char32 *extension) {
 	UiOutfile_do (dia, defaultFileName.string);
 }
 
-static void removeAllReferencesToEditor (Any editor) {
+static void removeAllReferencesToMoribundEditor (Editor editor) {
 	/*
 	 * Remove all references to this editor.
 	 * It may be editing multiple objects.
@@ -296,9 +289,11 @@ static void removeAllReferencesToEditor (Any editor) {
 		praatP. editor = nullptr;
 }
 
+/**
+	Remove the "object" from the list,
+	killing everything that has to do with the selection.
+*/
 static void praat_remove (int iobject, bool removeVisibly) {
-/* Remove the "object" from the list. */
-/* Kill everything to do with selection. */
 
 	Melder_assert (iobject >= 1 && iobject <= theCurrentPraatObjects -> n);
 	if (theCurrentPraatObjects -> list [iobject]. isBeingCreated) {
@@ -317,10 +312,10 @@ static void praat_remove (int iobject, bool removeVisibly) {
 		Editor editor = theCurrentPraatObjects -> list [iobject]. editors [ieditor];   // save this one reference
 		if (editor) {
 			trace (U"remove references to editor ", ieditor);
-			removeAllReferencesToEditor (editor);
+			removeAllReferencesToMoribundEditor (editor);
 			trace (U"forget editor ", ieditor);
 			if (removeVisibly)
-				forget (editor);
+				forget (editor);   // TODO: doesn't this call removeAllReferencesToMoribundEditor() again?
 			trace (U"forgeotten editor ", ieditor);
 		}
 	}
@@ -328,7 +323,7 @@ static void praat_remove (int iobject, bool removeVisibly) {
 	trace (U"free name");
 	Melder_free (theCurrentPraatObjects -> list [iobject]. name);
 	trace (U"forget object");
-	forget (theCurrentPraatObjects -> list [iobject]. object);
+	forget (theCurrentPraatObjects -> list [iobject]. object);   // note: this might save a file-based object to file
 	trace (U"forgotten object");
 }
 
@@ -345,8 +340,8 @@ void praat_cleanUpName (char32 *name) {
 
 static void praat_new_unpackCollection (autoCollection me, const char32* myName) {
 	for (long idata = 1; idata <= my size; idata ++) {
-		autoDaata object = (Daata) my item [idata];
-		my item [idata] = nullptr;   // disown; once the elements are autoThings, the move will handle this
+		autoDaata object = autoDaata ((Daata) my at [idata]);
+		my at [idata] = nullptr;   // disown; once the elements are autoThings, the move will handle this
 		const char32 *name = object -> name ? object -> name : myName;
 		Melder_assert (name);
 		praat_new (object.move(), name);   // recurse
@@ -392,7 +387,7 @@ void praat_newWithFile (autoDaata me, MelderFile file, const char32 *myName) {
 			theCurrentPraatObjects -> n);
 	}
 	CLASS = my classInfo;
-	OBJECT = me.transfer();
+	OBJECT = me.releaseToAmbiguousOwner();   // FIXME: should be move()
 	SELECTED = false;
 	for (int ieditor = 0; ieditor < praat_MAXNUM_EDITORS; ieditor ++)
 		EDITOR [ieditor] = nullptr;
@@ -460,8 +455,8 @@ void praat_updateSelection () {
 	}
 }
 
-static void gui_cb_list (void *void_me, GuiListEvent event) {
-	(void) event; (void) void_me;
+static void gui_cb_list_selectionChanged (Thing /* boss */, GuiList_SelectionChangedEvent event) {
+	Melder_assert (event -> list == praatList_objects);
 	int IOBJECT;
 	bool first = true;
 	WHERE (SELECTED) {
@@ -607,11 +602,11 @@ static void praat_exit (int exit_code) {
 	exit (exit_code);
 }
 
-static void cb_Editor_destruction (Editor me, void * /*closure*/) {
-	removeAllReferencesToEditor (me);   // remove reference(s) to moribund Editor
+static void cb_Editor_destruction (Editor me) {
+	removeAllReferencesToMoribundEditor (me);
 }
 
-static void cb_Editor_dataChanged (Editor me, void * /*closure*/) {
+static void cb_Editor_dataChanged (Editor me) {
 	for (int iobject = 1; iobject <= theCurrentPraatObjects -> n; iobject ++) {
 		bool editingThisObject = false;
 		/*
@@ -636,13 +631,13 @@ static void cb_Editor_dataChanged (Editor me, void * /*closure*/) {
 	}
 }
 
-static void cb_Editor_publication (Editor /*me*/, void * /*closure*/, Daata publication) {
+static void cb_Editor_publication (Editor /* me */, autoDaata publication) {
 /*
    The default publish callback.
    Works nicely if the publisher invents a name.
 */
 	try {
-		praat_new (publication, U"");
+		praat_new (publication.move(), U"");
 	} catch (MelderError) {
 		Melder_flushError ();
 	}
@@ -654,14 +649,14 @@ int praat_installEditor (Editor editor, int IOBJECT) {
 	for (int ieditor = 0; ieditor < praat_MAXNUM_EDITORS; ieditor ++) {
 		if (! EDITOR [ieditor]) {
 			EDITOR [ieditor] = editor;
-			Editor_setDestructionCallback (editor, cb_Editor_destruction, nullptr);
-			Editor_setDataChangedCallback (editor, cb_Editor_dataChanged, nullptr);
+			Editor_setDestructionCallback (editor, cb_Editor_destruction);
+			Editor_setDataChangedCallback (editor, cb_Editor_dataChanged);
 			if (! editor -> d_publicationCallback)
-				Editor_setPublicationCallback (editor, cb_Editor_publication, nullptr);
+				Editor_setPublicationCallback (editor, cb_Editor_publication);
 			return 1;
 		}
 	}
-	forget (editor);
+	//forget (editor);
 	Melder_throw (U"(praat_installEditor:) Cannot have more than ", praat_MAXNUM_EDITORS, U" editors with one object.");
 }
 
@@ -677,12 +672,12 @@ int praat_installEditor2 (Editor editor, int i1, int i2) {
 			break;
 	if (ieditor1 < praat_MAXNUM_EDITORS && ieditor2 < praat_MAXNUM_EDITORS) {
 		theCurrentPraatObjects -> list [i1]. editors [ieditor1] = theCurrentPraatObjects -> list [i2]. editors [ieditor2] = editor;
-		Editor_setDestructionCallback (editor, cb_Editor_destruction, nullptr);
-		Editor_setDataChangedCallback (editor, cb_Editor_dataChanged, nullptr);
+		Editor_setDestructionCallback (editor, cb_Editor_destruction);
+		Editor_setDataChangedCallback (editor, cb_Editor_dataChanged);
 		if (! editor -> d_publicationCallback)
-			Editor_setPublicationCallback (editor, cb_Editor_publication, nullptr);
+			Editor_setPublicationCallback (editor, cb_Editor_publication);
 	} else {
-		forget (editor);
+		//forget (editor);
 		Melder_throw (U"(praat_installEditor2:) Cannot have more than ", praat_MAXNUM_EDITORS, U" editors with one object.");
 	}
 	return 1;
@@ -704,25 +699,25 @@ int praat_installEditor3 (Editor editor, int i1, int i2, int i3) {
 			break;
 	if (ieditor1 < praat_MAXNUM_EDITORS && ieditor2 < praat_MAXNUM_EDITORS && ieditor3 < praat_MAXNUM_EDITORS) {
 		theCurrentPraatObjects -> list [i1]. editors [ieditor1] = theCurrentPraatObjects -> list [i2]. editors [ieditor2] = theCurrentPraatObjects -> list [i3]. editors [ieditor3] = editor;
-		Editor_setDestructionCallback (editor, cb_Editor_destruction, nullptr);
-		Editor_setDataChangedCallback (editor, cb_Editor_dataChanged, nullptr);
+		Editor_setDestructionCallback (editor, cb_Editor_destruction);
+		Editor_setDataChangedCallback (editor, cb_Editor_dataChanged);
 		if (! editor -> d_publicationCallback)
-			Editor_setPublicationCallback (editor, cb_Editor_publication, nullptr);
+			Editor_setPublicationCallback (editor, cb_Editor_publication);
 	} else {
-		forget (editor);
+		//forget (editor);
 		Melder_throw (U"(praat_installEditor3:) Cannot have more than ", praat_MAXNUM_EDITORS, U" editors with one object.");
 	}
 	return 1;
 }
 
-int praat_installEditorN (Editor editor, Ordered objects) {
+int praat_installEditorN (Editor editor, DaataList objects) {
 	if (! editor) return 0;
 	/*
 	 * First check whether all objects in the Ordered are also in the List of Objects (Praat crashes if not),
 	 * and check whether there is room to add an editor for each.
 	 */
-	for (long iOrderedObject = 1; iOrderedObject <= objects -> size; iOrderedObject ++) {
-		Daata object = (Daata) objects -> item [iOrderedObject];
+	for (long iOrderedObject = 1; iOrderedObject <= objects->size; iOrderedObject ++) {
+		Daata object = objects->at [iOrderedObject];
 		long iPraatObject = 1;
 		for (; iPraatObject <= theCurrentPraatObjects -> n; iPraatObject ++) {
 			if (object == theCurrentPraatObjects -> list [iPraatObject]. object) {
@@ -733,7 +728,7 @@ int praat_installEditorN (Editor editor, Ordered objects) {
 					}
 				}
 				if (ieditor >= praat_MAXNUM_EDITORS) {
-					forget (editor);
+					//forget (editor);
 					Melder_throw (U"Cannot view the same object in more than ", praat_MAXNUM_EDITORS, U" windows.");
 				}
 				break;
@@ -744,8 +739,8 @@ int praat_installEditorN (Editor editor, Ordered objects) {
 	/*
 	 * There appears to be room for all elements of the Ordered. The editor window can appear. Install the editor in all objects.
 	 */
-	for (long iOrderedObject = 1; iOrderedObject <= objects -> size; iOrderedObject ++) {
-		Daata object = (Daata) objects -> item [iOrderedObject];
+	for (long iOrderedObject = 1; iOrderedObject <= objects->size; iOrderedObject ++) {
+		Daata object = objects->at [iOrderedObject];
 		long iPraatObject = 1;
 		for (; iPraatObject <= theCurrentPraatObjects -> n; iPraatObject ++) {
 			if (object == theCurrentPraatObjects -> list [iPraatObject]. object) {
@@ -753,10 +748,10 @@ int praat_installEditorN (Editor editor, Ordered objects) {
 				for (; ieditor < praat_MAXNUM_EDITORS; ieditor ++) {
 					if (! theCurrentPraatObjects -> list [iPraatObject]. editors [ieditor]) {
 						theCurrentPraatObjects -> list [iPraatObject]. editors [ieditor] = editor;
-						Editor_setDestructionCallback (editor, cb_Editor_destruction, nullptr);
-						Editor_setDataChangedCallback (editor, cb_Editor_dataChanged, nullptr);
+						Editor_setDestructionCallback (editor, cb_Editor_destruction);
+						Editor_setDataChangedCallback (editor, cb_Editor_dataChanged);
 						if (! editor -> d_publicationCallback)
-							Editor_setPublicationCallback (editor, cb_Editor_publication, nullptr);
+							Editor_setPublicationCallback (editor, cb_Editor_publication);
 						break;
 					}
 				}
@@ -769,7 +764,7 @@ int praat_installEditorN (Editor editor, Ordered objects) {
 	return 1;
 }
 
-void praat_dataChanged (Any object) {
+void praat_dataChanged (Daata object) {
 	/*
 	 * This function can be called at error time, which is weird.
 	 */
@@ -800,15 +795,16 @@ static void helpProc (const char32 *query) {
 		return;
 	}
 	try {
-		Manual_create (query, theCurrentPraatApplication -> manPages, false);
+		autoManual manual = Manual_create (query, theCurrentPraatApplication -> manPages, false);
+		manual.releaseToUser();
 	} catch (MelderError) {
 		Melder_flushError (U"help: no help on \"", query, U"\".");
 	}
 }
 
-static int publishProc (void *anything) {
+static int publishProc (autoDaata me) {
 	try {
-		praat_new ((Daata) anything, U"");
+		praat_new (me.move(), U"");
 		praat_updateSelection ();
 		return 1;
 	} catch (MelderError) {
@@ -840,7 +836,7 @@ DO
 	praat_exit (0);
 END
 
-static void gui_cb_quit (void *) {
+static void gui_cb_quit (Thing /* me */) {
 	DO_Quit (nullptr, 0, nullptr, nullptr, nullptr, nullptr, false, nullptr);
 }
 
@@ -871,9 +867,7 @@ void praat_dontUsePictureWindow () { praatP.dontUsePictureWindow = true; }
 
 #if defined (UNIX)
 	#if ALLOW_GDK_DRAWING && ! defined (NO_GRAPHICS)
-		static gboolean cb_userMessage (GtkWidget widget, GdkEventClient *event, gpointer user_data) {
-			(void) widget;
-			(void) user_data;
+		static gboolean cb_userMessage (GtkWidget /* widget */, GdkEventClient * /* event */, gpointer /* userData */) {
 			//Melder_casual (U"client event called");
 			autofile f;
 			try {
@@ -926,10 +920,8 @@ void praat_dontUsePictureWindow () { praatP.dontUsePictureWindow = true; }
 	static void mac_setUserMessageCallback (int (*userMessageCallback) (char32 *message)) {
 		theUserMessageCallback = userMessageCallback;
 	}
-	static pascal OSErr mac_processSignal8 (const AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefCon) {
+	static pascal OSErr mac_processSignal8 (const AppleEvent *theAppleEvent, AppleEvent * /* reply */, long /* handlerRefCon */) {
 		static bool duringAppleEvent = false;   // FIXME: may have to be atomic?
-		(void) reply;
-		(void) handlerRefCon;
 		if (! duringAppleEvent) {
 			char *buffer;
 			Size actualSize;
@@ -950,10 +942,8 @@ void praat_dontUsePictureWindow () { praatP.dontUsePictureWindow = true; }
 		}
 		return noErr;
 	}
-	static pascal OSErr mac_processSignal16 (const AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefCon) {
+	static pascal OSErr mac_processSignal16 (const AppleEvent *theAppleEvent, AppleEvent * /* reply */, long /* handlerRefCon */) {
 		static bool duringAppleEvent = false;   // FIXME: may have to be atomic?
-		(void) reply;
-		(void) handlerRefCon;
 		if (! duringAppleEvent) {
 			char16 *buffer;
 			Size actualSize;
@@ -1019,64 +1009,25 @@ static bool tryToAttachToTheCommandLine ()
 		 * has to be handled explicitly, as here.
 		 */
 		if (AttachConsole (ATTACH_PARENT_PROCESS)) {   // was Praat called from either a console window or a "system" command?
-			/*
-			 * Redirect stdout to the console (note: no UTF-8!).
-			 */
-			HANDLE handle = GetStdHandle (STD_OUTPUT_HANDLE);
-			int fileHandle = _open_osfhandle ((intptr_t) handle, _O_TEXT);
-			FILE* f = _fdopen (fileHandle, "w");
-			*stdout = *f;
-			setvbuf (stdout, nullptr, _IONBF, 0);
-			/*
-			 * Redirect stderr to the console (note: no UTF-8!).
-			 */
-			handle = GetStdHandle (STD_ERROR_HANDLE);
-			fileHandle = _open_osfhandle ((intptr_t) handle, _O_TEXT);
-			f = _fdopen (fileHandle, "w");
-			*stderr = *f;
-			setvbuf (stderr, nullptr, _IONBF, 0);
-			/*
-			 * Redirect stdin from the console (note: no UTF-8!).
-			 */
-			handle = GetStdHandle (STD_INPUT_HANDLE);
-			fileHandle = _open_osfhandle ((intptr_t) handle, _O_TEXT);
-			f = _fdopen (fileHandle, "r");
-			*stdin = *f;
-			setvbuf (stdin, nullptr, _IONBF, 0);
-			/*
-			 */
 			weHaveSucceeded = true;
-		} else {   // Praat was called from Windows Explorer, typically by double-clicking or dropping a file
-			DWORD err = GetLastError ();
-			if (err == ERROR_ACCESS_DENIED) {
-				printf ("Apparently Praat has been called as a console application.\n"
-					"Did you compile it without the -mwindows flag?\n");
-				Melder_fatal (U"Apparently Praat has been called as a console application.\n"
-					"Did you compile it without the -mwindows flag?");
-			} else if (err == ERROR_INVALID_HANDLE) {
-				(void) 0;   // a normal case: the parent process is Windows Explorer or so, which doesn't have a console (Windows XP, 10?)
-			} else if (err == ERROR_GEN_FAILURE) {
-				(void) 0;   // another normal case: there is no parent process (Windows 7?)
-			} else {
-				printf ("AttachConsole() returned unknown error %d\n", (int) err);
-				Melder_fatal (U"AttachConsole() returned unknown error ", err);
-			}
 		}
 	#else
-		weHaveSucceeded = isatty (fileno (stdout));
-			// this is true if Praat was called from a terminal window or a system() command or Xcode,
-			// and false if Praat was called from the Finder by double-clicking or dropping a file.
+		weHaveSucceeded = isatty (fileno (stdin)) || isatty (fileno (stdout)) || isatty (fileno (stderr));
+		/*
+			The result is `true` if Praat was called from a terminal window or some system() commands or Xcode,
+			and `false` if Praat was called from the Finder by double-clicking or dropping a file.
+			
+			FIXME:
+			The result is incorrectly `false` if the output is redirected to a file or pipe.
+			A proposed improvement is therefore:
+				isatty (fileno (stdin)) || isatty (fileno (stdout)) || isatty (fileno (stderr))
+			This might be incorrectly false only if all three streams are redirected, but this hasn't been tested yet.
+		*/
 	#endif
 	return weHaveSucceeded;
 }
 
-void praat_init (const char32 *title, unsigned int argc, char **argv)
-{
-	bool weWereStartedFromTheCommandLine = tryToAttachToTheCommandLine ();
-
-	for (unsigned int iarg = 0; iarg < argc; iarg ++) {
-		//Melder_casual (U"arg ", iarg, U": <<", Melder_peek8to32 (argv [iarg]), U">>");
-	}
+static void setThePraatLocale () {
 	#if defined (UNIX)
 		setlocale (LC_ALL, "C");
 		//setenv ("PULSE_LATENCY_MSEC", "1", 0);   // Rafael Laboissiere, August 2014
@@ -1085,6 +1036,9 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 	#elif defined (macintosh)
 		setlocale (LC_ALL, "en_US");   // required to make swprintf work correctly; the default "C" locale does not do that!
 	#endif
+}
+
+static void getSystemVersion () {
 	#ifdef macintosh
 		SInt32 sys1, sys2, sys3;
 		Gestalt ('sys1', & sys1);
@@ -1092,21 +1046,16 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		Gestalt ('sys3', & sys3);
 		Melder_systemVersion = sys1 * 10000 + sys2 * 100 + sys3;
 	#endif
-	/*
-		Initialize numerical libraries.
-	*/
+}
+
+static void initializeNumericalLibraries () {
 	NUMmachar ();
 	NUMinit ();
 	Melder_alloc_init ();
 	Melder_message_init ();
-	/*
-		Remember the current directory. Useful only for scripts run from batch.
-	*/
-	Melder_rememberShellDirectory ();
+}
 
-	/*
-	 * Install the preferences of the Praat shell, and set the defaults.
-	 */
+static void installPraatShellPreferences () {
 	praat_statistics_prefs ();   // number of sessions, memory used...
 	praat_picture_prefs ();   // font...
 	Graphics_prefs ();
@@ -1117,6 +1066,49 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 	Melder_textEncoding_prefs ();
 	Printer_prefs ();   // paper size, printer command...
 	structTextEditor :: f_preferences ();   // font size...
+}
+
+extern "C" void praatlib_init () {
+	setThePraatLocale ();   // FIXME: don't use the global locale
+	getSystemVersion ();
+	initializeNumericalLibraries ();
+	Melder_rememberShellDirectory ();
+	installPraatShellPreferences ();   // needed in the library, because this sets the defaults
+	praatP.argc = 0;
+	praatP.argv = nullptr;
+	praatP.argumentNumber = 1;
+	Melder_batch = true;
+	praatP.userWantsToOpen = false;
+	praatP.title = Melder_dup (U"Praatlib");
+	theCurrentPraatApplication -> batch = true;
+	Melder_getHomeDir (& homeDir);
+	praat_actions_init ();
+	praat_menuCommands_init ();
+	Thing_recognizeClassesByName (classCollection, classStrings, classManPages, classStringSet, nullptr);
+	Thing_recognizeClassByOtherName (classStringSet, U"SortedSetOfString");
+	Melder_backgrounding = true;
+	praat_addMenus (nullptr);
+	praat_addFixedButtons (nullptr);
+	praat_addMenus2 ();
+}
+
+void praat_init (const char32 *title, int argc, char **argv)
+{
+	bool weWereStartedFromTheCommandLine = tryToAttachToTheCommandLine ();
+
+	for (int iarg = 0; iarg < argc; iarg ++) {
+		//Melder_casual (U"arg ", iarg, U": <<", Melder_peek8to32 (argv [iarg]), U">>");
+	}
+	setThePraatLocale ();
+	getSystemVersion ();
+	initializeNumericalLibraries ();
+
+	/*
+		Remember the current directory. Useful only for scripts run from batch.
+	*/
+	Melder_rememberShellDirectory ();
+
+	installPraatShellPreferences ();
 
 	praatP.argc = argc;
 	praatP.argv = argv;
@@ -1148,20 +1140,23 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		} else if (strequ (argv [praatP.argumentNumber], "--version")) {
 			#define xstr(s) str(s)
 			#define str(s) #s
-			printf ("%s %s (%s %d, %d)\n", Melder_peek32to8 (title), xstr (PRAAT_VERSION_STR), xstr (PRAAT_MONTH), PRAAT_DAY, PRAAT_YEAR);
+			Melder_information (title, U" " xstr (PRAAT_VERSION_STR) " (" xstr (PRAAT_MONTH) " ", PRAAT_DAY, U" ", PRAAT_YEAR, U")");
 			exit (0);
 		} else if (strequ (argv [praatP.argumentNumber], "--help")) {
-			printf ("Usage: praat [options] script-file-name [script-arguments]\n");
-			printf ("Options:\n");
-			printf ("  --open           interpret the command line arguments as files to be opened in the GUI\n");
-			printf ("  --run            interpret the command line arguments as a script file name and its arguments\n");
-			printf ("                   (--run is superfluous when you use a Console or Terminal window)\n");
-			printf ("  --no-pref-files  don't read or write the preferences file and the buttons file\n");
-			printf ("  --no-plugins     don't activate the plugins\n");
-			printf ("  --pref-dir=DIR   set the preferences directory to DIR\n");
-			printf ("  --version        print the Praat version\n");
-			printf ("  --help           print this list of command line options\n");
-			printf ("  -a, --ansi       on Windows: use ISO Latin-1 encoding instead of UTF-16LE (not recommended)\n");
+			MelderInfo_open ();
+			MelderInfo_writeLine (U"Usage: praat [options] script-file-name [script-arguments]");
+			MelderInfo_writeLine (U"Options:");
+			MelderInfo_writeLine (U"  --open           regard the command line as files to be opened in the GUI");
+			MelderInfo_writeLine (U"  --run            regard the command line as a script to run, with its arguments");
+			MelderInfo_writeLine (U"                   (--run is superfluous when you use a Console or Terminal)");
+			MelderInfo_writeLine (U"  --no-pref-files  don't read or write the preferences file and the buttons file");
+			MelderInfo_writeLine (U"  --no-plugins     don't activate the plugins");
+			MelderInfo_writeLine (U"  --pref-dir=DIR   set the preferences directory to DIR");
+			MelderInfo_writeLine (U"  --version        print the Praat version");
+			MelderInfo_writeLine (U"  --help           print this list of command line options");
+			MelderInfo_writeLine (U"  -a, --ansi       Windows only: use ISO Latin-1 encoding instead of UTF-16LE");
+			MelderInfo_writeLine (U"                   (this option is needed when you redirect to a pipe or file");
+			MelderInfo_close ();
 			exit (0);
 		} else if (strequ (argv [praatP.argumentNumber], "-a") || strequ (argv [praatP.argumentNumber], "--ansi")) {
 			Melder_consoleIsAnsi = true;
@@ -1176,7 +1171,7 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		#endif
 		} else {
 			unknownCommandLineOption = Melder_8to32 (argv [praatP.argumentNumber]);
-			praatP.argumentNumber = UINT32_MAX;   // ignore all other command line options
+			praatP.argumentNumber = INT32_MAX;   // ignore all other command line options
 			break;
 		}
 	}
@@ -1314,17 +1309,10 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		if (! Melder_batch)
 			motif_win_setUserMessageCallback (cb_userMessage);
 	#elif defined (macintosh)
-		#if useCarbon
-			if (! Melder_batch) {
-				motif_mac_setUserMessageCallback (cb_userMessage);
-				Gui_setQuitApplicationCallback (cb_quitApplication);
-			}
-		#else
-			if (! Melder_batch) {
-				mac_setUserMessageCallback (cb_userMessage);
-				Gui_setQuitApplicationCallback (cb_quitApplication);
-			}
-		#endif
+		if (! Melder_batch) {
+			mac_setUserMessageCallback (cb_userMessage);
+			Gui_setQuitApplicationCallback (cb_quitApplication);
+		}
 	#endif
 
 	/*
@@ -1338,7 +1326,7 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 	GuiWindow raam = nullptr;
 	if (Melder_batch) {
 		MelderString_empty (& theCurrentPraatApplication -> batchName);
-		for (unsigned int i = praatP.argumentNumber - 1; i < argc; i ++) {
+		for (int i = praatP.argumentNumber - 1; i < argc; i ++) {
 			if (i >= praatP.argumentNumber) MelderString_append (& theCurrentPraatApplication -> batchName, U" ");
 			bool needsQuoting = !! strchr (argv [i], ' ') && (i == praatP.argumentNumber - 1 || i < argc - 1);
 			if (needsQuoting) MelderString_append (& theCurrentPraatApplication -> batchName, U"\"");
@@ -1384,7 +1372,8 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		GuiThing_show (raam);
 		trace (U"after objects window shows locale ", Melder_peek8to32 (setlocale (LC_ALL, nullptr)));
 	}
-	Thing_recognizeClassesByName (classCollection, classStrings, classManPages, classSortedSetOfString, nullptr);
+	Thing_recognizeClassesByName (classCollection, classStrings, classManPages, classStringSet, nullptr);
+	Thing_recognizeClassByOtherName (classStringSet, U"SortedSetOfString");
 	if (Melder_batch) {
 		Melder_backgrounding = true;
 		trace (U"adding menus without GUI");
@@ -1394,16 +1383,10 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 	} else {
 
 		#ifdef macintosh
-			#if ! useCarbon
-				AEInstallEventHandler (758934755, 0, (AEEventHandlerProcPtr) (mac_processSignal8), 0, false);   // for receiving sendpraat
-				AEInstallEventHandler (758934756, 0, (AEEventHandlerProcPtr) (mac_processSignal16), 0, false);   // for receiving sendpraatW
-			#endif
+			AEInstallEventHandler (758934755, 0, (AEEventHandlerProcPtr) (mac_processSignal8), 0, false);   // for receiving sendpraat
+			AEInstallEventHandler (758934756, 0, (AEEventHandlerProcPtr) (mac_processSignal16), 0, false);   // for receiving sendpraatW
 			MelderGui_create (raam);   // BUG: default Melder_assert would call printf recursively!!!
 		#endif
-		#if defined (macintosh) && useCarbon
-			trace (U"creating the menu bar along the top of the screen (Mac only)");
-			GuiWindow_addMenuBar (raam);   // yes, on the Mac we create a menu bar twice: once at the top of the screen, once in the Objects window
-		#endif
 		trace (U"creating the menu bar in the Objects window");
 		GuiWindow_addMenuBar (raam);
 		praatP.menuBar = raam;
@@ -1412,7 +1395,7 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		trace (U"creating the object list in the Objects window");
 		GuiLabel_createShown (raam, 3, -250, Machine_getMainWindowMenuBarHeight () + 5, Machine_getMainWindowMenuBarHeight () + 5 + Gui_LABEL_HEIGHT, U"Objects:", 0);
 		praatList_objects = GuiList_create (raam, 0, -250, Machine_getMainWindowMenuBarHeight () + 26, -100, true, U" Objects ");
-		GuiList_setSelectionChangedCallback (praatList_objects, gui_cb_list, 0);
+		GuiList_setSelectionChangedCallback (praatList_objects, gui_cb_list_selectionChanged, nullptr);
 		GuiThing_show (praatList_objects);
 		praat_addFixedButtons (raam);
 
@@ -1445,8 +1428,8 @@ void praat_init (const char32 *title, unsigned int argc, char **argv)
 		#endif
 		Melder_setHelpProc (helpProc);
 	}
-	Melder_setPublishProc (publishProc);
-	theCurrentPraatApplication -> manPages = ManPages_create ();
+	Data_setPublishProc (publishProc);
+	theCurrentPraatApplication -> manPages = ManPages_create ().releaseToAmbiguousOwner();
 
 	trace (U"creating the Picture window");
 	trace (U"before picture window shows: locale is ", Melder_peek8to32 (setlocale (LC_ALL, nullptr)));
@@ -1481,7 +1464,11 @@ static void executeStartUpFile (MelderDir startUpDirectory, const char32 *fileNa
 			trace (U"keyval ", event -> keyval, U", type ", event -> type);
 			if ((event -> keyval == GDK_Tab || event -> keyval == GDK_ISO_Left_Tab) && event -> type == GDK_KEY_PRESS) {
 				trace (U"tab key pressed in window ", Melder_pointer (widget));
-				if ((event -> state & GDK_MODIFIER_MASK) == 0) {
+				constexpr bool theTabKeyShouldWorkEvenIfNumLockIsOn = true;
+				constexpr uint32 theProbableNumLockModifierMask = GDK_MOD2_MASK;
+				constexpr uint32 modifiersToIgnore = ( theTabKeyShouldWorkEvenIfNumLockIsOn ? theProbableNumLockModifierMask : 0 );
+				constexpr uint32 modifiersNotToIgnore = GDK_MODIFIER_MASK & ~ modifiersToIgnore;
+				if ((event -> state & modifiersNotToIgnore) == 0) {
 					if (GTK_IS_WINDOW (widget)) {
 						GtkWidget *shell = gtk_widget_get_toplevel (GTK_WIDGET (widget));
 						trace (U"tab pressed in GTK window ", Melder_pointer (shell));
@@ -1493,7 +1480,7 @@ static void executeStartUpFile (MelderDir startUpDirectory, const char32 *fileNa
 							return true;
 						}
 					}
-				} else if ((event -> state & GDK_MODIFIER_MASK) == GDK_SHIFT_MASK) {
+				} else if ((event -> state & modifiersNotToIgnore) == GDK_SHIFT_MASK) {
 					if (GTK_IS_WINDOW (widget)) {
 						GtkWidget *shell = gtk_widget_get_toplevel (GTK_WIDGET (widget));
 						trace (U"shift-tab pressed in GTK window ", Melder_pointer (shell));
@@ -1519,12 +1506,8 @@ void praat_run () {
 	trace (U"locale is ", Melder_peek8to32 (setlocale (LC_ALL, nullptr)));
 
 	trace (U"adding the Quit command");
-	#if defined (macintosh) && useCarbon
-		praat_addMenuCommand (U"Objects", U"Praat", U"Quit", nullptr, praat_HIDDEN, DO_Quit);   // the Quit command is needed for scripts, not for the GUI
-	#else
-		praat_addMenuCommand (U"Objects", U"Praat", U"-- quit --", nullptr, 0, nullptr);
-		praat_addMenuCommand (U"Objects", U"Praat", U"Quit", nullptr, praat_UNHIDABLE + 'Q', DO_Quit);
-	#endif
+	praat_addMenuCommand (U"Objects", U"Praat", U"-- quit --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Quit", nullptr, praat_UNHIDABLE + 'Q', DO_Quit);
 
 	trace (U"read the preferences file, and notify those who want to be notified of this");
 	/* ...namely, those who already have a window (namely, the Picture window),
@@ -1567,8 +1550,8 @@ void praat_run () {
 			autoStrings directoryNames = Strings_createAsDirectoryList (Melder_fileToPath (& searchPattern));
 			if (directoryNames -> numberOfStrings > 0) {
 				for (long i = 1; i <= directoryNames -> numberOfStrings; i ++) {
-					structMelderDir pluginDir = { { 0 } };
-					structMelderFile plugin = { 0 };
+					structMelderDir pluginDir { { 0 } };
+					structMelderFile plugin { 0 };
 					MelderDir_getSubdir (& praatDir, directoryNames -> strings [i], & pluginDir);
 					MelderDir_getFile (& pluginDir, U"setup.praat", & plugin);
 					if (MelderFile_readable (& plugin)) {
