@@ -2,29 +2,28 @@
  *
  * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "praat.h"
 
 void praat_addActionScript (const char32 *className1, int n1, const char32 *className2, int n2, const char32 *className3, int n3,
 	const char32 *title, const char32 *after, int depth, const char32 *script);
-/* No strings may be NULL; unspecify them by giving an empty string. 'title' and 'script' are deep-copied. */
+/* No strings may be null; unspecify them by giving an empty string. 'title' and 'script' are deep-copied. */
 void praat_removeAction_classNames (const char32 *className1, const char32 *className2,
 	const char32 *className3, const char32 *title);
-/* No arguments may be NULL; unspecify them by giving an empty string. */
+/* No arguments may be null; unspecify them by giving an empty string. */
 /* 'title' is deep-copied. */
 void praat_hideAction (ClassInfo class1, ClassInfo class2, ClassInfo class3, const char32 *title);
 void praat_hideAction_classNames (const char32 *className1, const char32 *className2,
@@ -36,7 +35,7 @@ void praat_sortActions ();
 
 void praat_addMenuCommandScript (const char32 *window, const char32 *menu, const char32 *title,
 	const char32 *after, int depth, const char32 *script);
-/* All strings are deep-copied and may not be NULL; unspecify them by giving an empty string. */
+/* All strings are deep-copied and may not be null; unspecify them by giving an empty string. */
 /*
 	For the Praat objects window:
 	'window' is "Objects", 'menu' is "Praat", "New", "Open", "Help", "Preferences", "Goodies", or "Technical".
@@ -53,12 +52,12 @@ void praat_sortMenuCommands ();
 #define praat_MAXNUM_MENUS 20   /* Maximum number of added New, Open, Save, or Help menus. */
 #define praat_MAXNUM_FILE_TYPE_RECOGNIZERS 50   /* File types recognizable by 'Read from file...'. */
 
-typedef struct structPraat_Command {
+Thing_define (Praat_Command, Thing) {
 	ClassInfo class1, class2, class3, class4;   // selected classes
 	int32 n1, n2, n3, n4;   // number of selected objects of each class; 0 means "any number"
 	const char32 *title;   // button text = command text
 	UiCallback callback;   // multi-purpose
-		/* If both UiCallback::sendingForm and sendingString are NULL, this routine is an activate callback;
+		/* If both UiCallback::sendingForm and sendingString are null, this routine is an activate callback;
 			you should directly execute the command, or call UiForm_do(dialog) if you need arguments;
 			UiForm_do will call this routine again with sendingForm = dialog. */
 		/* If sendingForm exists, this routine is an ok callback,
@@ -78,10 +77,10 @@ typedef struct structPraat_Command {
 	GuiThing button;
 	const char32 *window, *menu;
 	const char32 *script;   // if 'callback' equals DO_RunTheScriptFromAnyAddedMenuCommand
-	const char32 *after;   // title of previous command, often NULL
+	const char32 *after;   // title of previous command, often null
 	int32 uniqueID;   // for sorting the added commands
 	int32 sortingTail;
-} *praat_Command;
+};
 
 #define praat_STARTING_UP  1
 #define praat_READING_BUTTONS  2
@@ -129,7 +128,7 @@ void praat_menuCommands_exit ();
 int praat_doMenuCommand (const char32 *command, const char32 *arguments, Interpreter interpreter);   // 0 = not found
 int praat_doMenuCommand (const char32 *command, int narg, Stackel args, Interpreter interpreter);   // 0 = not found
 long praat_getNumberOfMenuCommands ();
-praat_Command praat_getMenuCommand (long i);
+Praat_Command praat_getMenuCommand (long i);
 
 /* Communication with praat_actions.cpp: */
 void praat_actions_show ();
@@ -140,7 +139,7 @@ void praat_saveAddedActions (MelderString *buffer);
 int praat_doAction (const char32 *command, const char32 *arguments, Interpreter interpreter);   // 0 = not found
 int praat_doAction (const char32 *command, int narg, Stackel args, Interpreter interpreter);   // 0 = not found
 long praat_getNumberOfActions ();   // for ButtonEditor
-praat_Command praat_getAction (long i);   // for ButtonEditor
+Praat_Command praat_getAction (long i);   // for ButtonEditor
 
 /* Communication with praat_statistics.cpp: */
 void praat_statistics_prefs ();   // at init time
@@ -162,9 +161,9 @@ void praat_cleanUpName (char32 *name);
 void praat_list_renameAndSelect (int position, const char32 *name);
 
 extern struct PraatP {
-	unsigned int argc;
+	int argc;
 	char **argv;
-	unsigned int argumentNumber;
+	int argumentNumber;
 	bool userWantsToOpen;
 	bool dontUsePictureWindow;   // see praat_dontUsePictureWindow ()
 	bool ignorePreferenceFiles, ignorePlugins;
