@@ -1,20 +1,19 @@
 /* praat_objectMenus.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1992-2012,2013,2014,2015,2016 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <ctype.h>
@@ -44,7 +43,7 @@ FORM (Rename, U"Rename object", U"Rename...") {
 	LABEL (U"rename object", U"New name:")
 	TEXTFIELD (U"newName", U"")
 	OK2
-{ int IOBJECT; WHERE (SELECTED) SET_STRING (U"newName", NAME) }
+int IOBJECT; WHERE (SELECTED) SET_STRING (U"newName", NAME)
 DO
 	char32 *string = GET_STRING (U"newName");
 	if (theCurrentPraatObjects -> totalSelection == 0)
@@ -108,7 +107,9 @@ DIRECT2 (Inspect) {
 		Melder_throw (U"Cannot inspect data from batch.");
 	} else {
 		WHERE (SELECTED) {
-			praat_installEditor (DataEditor_create (ID_AND_FULL_NAME, OBJECT), IOBJECT);
+			autoDataEditor editor = DataEditor_create (ID_AND_FULL_NAME, OBJECT);
+			praat_installEditor (editor.get(), IOBJECT);
+			editor.releaseToUser();
 		}
 	}
 END2 }
@@ -135,7 +136,7 @@ GuiMenu praat_objects_resolveMenu (const char32 *menu) {
 		#else
 			str32equ (menu, U"ApplicationHelp") ? helpMenu :
 		#endif
-		newMenu;   /* Default. */
+		newMenu;   // default
 }
 
 /********** Callbacks of the Praat menu. **********/
@@ -145,30 +146,30 @@ DIRECT2 (About) {
 END2 }
 
 DIRECT2 (praat_newScript) {
-	autoScriptEditor editor = ScriptEditor_createFromText (NULL, NULL);
-	editor.transfer();   // the user becomes the owner
+	autoScriptEditor editor = ScriptEditor_createFromText (nullptr, nullptr);
+	editor.releaseToUser();
 END2 }
 
 DIRECT2 (praat_openScript) {
-	autoScriptEditor editor = ScriptEditor_createFromText (NULL, NULL);
-	TextEditor_showOpen (editor.peek());
-	editor.transfer();   // the user becomes the owner
+	autoScriptEditor editor = ScriptEditor_createFromText (nullptr, nullptr);
+	TextEditor_showOpen (editor.get());
+	editor.releaseToUser();
 END2 }
 
-static ButtonEditor theButtonEditor;
+static ButtonEditor theReferenceToTheOnlyButtonEditor;
 
-static void cb_ButtonEditor_destruction (Editor editor, void *closure) {
-	(void) editor;
-	(void) closure;
-	theButtonEditor = NULL;
+static void cb_ButtonEditor_destruction (Editor /* editor */) {
+	theReferenceToTheOnlyButtonEditor = nullptr;
 }
 
 DIRECT2 (praat_editButtons) {
-	if (theButtonEditor) {
-		Editor_raise (theButtonEditor);
+	if (theReferenceToTheOnlyButtonEditor) {
+		Editor_raise (theReferenceToTheOnlyButtonEditor);
 	} else {
-		theButtonEditor = ButtonEditor_create ();
-		Editor_setDestructionCallback (theButtonEditor, cb_ButtonEditor_destruction, NULL);
+		autoButtonEditor editor = ButtonEditor_create ();
+		Editor_setDestructionCallback (editor.get(), cb_ButtonEditor_destruction);
+		theReferenceToTheOnlyButtonEditor = editor.get();
+		editor.releaseToUser();
 	}
 END2 }
 
@@ -263,7 +264,7 @@ DO
 	Melder_setOutputEncoding (GET_ENUM (kMelder_textOutputEncoding, U"Output encoding"));
 END2 }
 
-FORM (GraphicsCjkFontStyleSettings, U"CJK font style preferences", 0) {
+FORM (GraphicsCjkFontStyleSettings, U"CJK font style preferences", nullptr) {
 	OPTIONMENU_ENUM (U"CJK font style", kGraphics_cjkFontStyle, DEFAULT)
 	OK2
 SET_ENUM (U"CJK font style", kGraphics_cjkFontStyle, theGraphicsCjkFontStyle)
@@ -282,15 +283,9 @@ FORM (praat_calculator, U"Calculator", U"Calculator") {
 	OK2
 DO
 	struct Formula_Result result;
-	if (interpreter == NULL) {
-		interpreter = Interpreter_create (NULL, NULL);
-		try {
-			Interpreter_anyExpression (interpreter, GET_STRING (U"expression"), & result);
-			forget (interpreter);
-		} catch (MelderError) {
-			forget (interpreter);
-			throw;
-		}
+	if (! interpreter) {
+		autoInterpreter tempInterpreter = Interpreter_create (nullptr, nullptr);
+		Interpreter_anyExpression (tempInterpreter.get(), GET_STRING (U"expression"), & result);
 	} else {
 		Interpreter_anyExpression (interpreter, GET_STRING (U"expression"), & result);
 	}
@@ -302,7 +297,9 @@ DO
 			Melder_information (result. result.stringResult);
 			Melder_free (result. result.stringResult);
 		} break;
-		case kFormula_EXPRESSION_TYPE_NUMERIC_ARRAY: {
+		case kFormula_EXPRESSION_TYPE_NUMERIC_VECTOR: {
+		} break;
+		case kFormula_EXPRESSION_TYPE_NUMERIC_MATRIX: {
 		}
 	}
 END2 }
@@ -358,7 +355,7 @@ END2 }
 
 /********** Callbacks of the Technical menu. **********/
 
-FORM (praat_debug, U"Set debugging options", 0) {
+FORM (praat_debug, U"Set debugging options", nullptr) {
 	LABEL (U"", U"If you switch Tracing on, Praat will write lots of detailed ")
 	LABEL (U"", U"information about what goes on in Praat")
 	structMelderDir dir;
@@ -370,7 +367,7 @@ FORM (praat_debug, U"Set debugging options", 0) {
 		MelderDir_getFile (& dir, U"Tracing.txt", & file);
 	#endif
 	LABEL (U"", Melder_cat (U"to ", Melder_fileToPath (& file), U"."))
-	BOOLEAN (U"Tracing", 0)
+	BOOLEAN (U"Tracing", false)
 	LABEL (U"", U"Setting the following to anything other than zero")
 	LABEL (U"", U"will alter the behaviour of Praat")
 	LABEL (U"", U"in unpredictable ways.")
@@ -410,22 +407,28 @@ END2 }
 /********** Callbacks of the Open menu. **********/
 
 static void readFromFile (MelderFile file) {
-	autoDaata object = (Daata) Data_readFromFile (file);
-	if (object.peek() == NULL) return;
-	if (Thing_isa (object.peek(), classManPages) && ! Melder_batch) {
-		ManPages pages = (ManPages) object.peek();
-		ManPage firstPage = static_cast<ManPage> (pages -> pages -> item [1]);
-		Manual_create (firstPage -> title, object.transfer(), true);
-		if (pages -> executable)
+	autoDaata object = Data_readFromFile (file);
+	if (! object.get()) return;
+	if (Thing_isa (object.get(), classManPages) && ! Melder_batch) {
+		ManPages manPages = (ManPages) object.get();
+		ManPage firstPage = manPages -> pages.at [1];
+		autoManual manual = Manual_create (firstPage -> title, object.releaseToAmbiguousOwner(), true);
+		if (manPages -> executable)
 			Melder_warning (U"These manual pages contain links to executable scripts.\n"
 				"Only navigate these pages if you trust their author!");
+		manual.releaseToUser();
 		return;
 	}
-	if (Thing_isa (object.peek(), classScript) && ! Melder_batch) {
-		ScriptEditor_createFromScript (NULL, (Script) object.peek());
+	if (Thing_isa (object.get(), classScript) && ! Melder_batch) {
+		autoScriptEditor editor = ScriptEditor_createFromScript_canBeNull (nullptr, (Script) object.get());
+		if (! editor) {
+			(void) 0;   // the script was already open, and the user has been notified of that
+		} else {
+			editor.releaseToUser();
+		}
 		return;
 	}
-	praat_newWithFile (object.transfer(), file, MelderFile_name (file));
+	praat_newWithFile (object.move(), file, MelderFile_name (file));
 	praat_updateSelection ();
 }
 
@@ -435,7 +438,7 @@ END2 }
 
 /********** Callbacks of the Save menu. **********/
 
-FORM_WRITE2 (Data_writeToTextFile, U"Save Object(s) as one text file", 0, 0) {
+FORM_WRITE2 (Data_writeToTextFile, U"Save Object(s) as one text file", nullptr, nullptr) {
 	if (theCurrentPraatObjects -> totalSelection == 1) {
 		LOOP {
 			iam (Daata);
@@ -443,11 +446,11 @@ FORM_WRITE2 (Data_writeToTextFile, U"Save Object(s) as one text file", 0, 0) {
 		}
 	} else {
 		autoCollection set = praat_getSelectedObjects ();
-		Data_writeToTextFile (set.peek(), file);
+		Data_writeToTextFile (set.get(), file);
 	}
 END2 }
 
-FORM_WRITE2 (Data_writeToShortTextFile, U"Save Object(s) as one short text file", 0, 0) {
+FORM_WRITE2 (Data_writeToShortTextFile, U"Save Object(s) as one short text file", nullptr, nullptr) {
 	if (theCurrentPraatObjects -> totalSelection == 1) {
 		LOOP {
 			iam (Daata);
@@ -455,11 +458,11 @@ FORM_WRITE2 (Data_writeToShortTextFile, U"Save Object(s) as one short text file"
 		}
 	} else {
 		autoCollection set = praat_getSelectedObjects ();
-		Data_writeToShortTextFile (set.peek(), file);
+		Data_writeToShortTextFile (set.get(), file);
 	}
 END2 }
 
-FORM_WRITE2 (Data_writeToBinaryFile, U"Save Object(s) as one binary file", 0, 0) {
+FORM_WRITE2 (Data_writeToBinaryFile, U"Save Object(s) as one binary file", nullptr, nullptr) {
 	if (theCurrentPraatObjects -> totalSelection == 1) {
 		LOOP {
 			iam (Daata);
@@ -467,15 +470,15 @@ FORM_WRITE2 (Data_writeToBinaryFile, U"Save Object(s) as one binary file", 0, 0)
 		}
 	} else {
 		autoCollection set = praat_getSelectedObjects ();
-		Data_writeToBinaryFile (set.peek(), file);
+		Data_writeToBinaryFile (set.get(), file);
 	}
 END2 }
 
-FORM (ManPages_saveToHtmlDirectory, U"Save all pages as HTML files", 0) {
+FORM (ManPages_saveToHtmlDirectory, U"Save all pages as HTML files", nullptr) {
 	LABEL (U"", U"Type a directory name:")
 	TEXTFIELD (U"directory", U"")
 	OK2
-structMelderDir currentDirectory = { { 0 } };
+structMelderDir currentDirectory { { 0 } };
 Melder_getDefaultDir (& currentDirectory);
 SET_STRING (U"directory", Melder_dirToPath (& currentDirectory))
 DO
@@ -489,12 +492,13 @@ END2 }
 DIRECT2 (ManPages_view) {
 	LOOP {
 		iam (ManPages);
-		ManPage firstPage = static_cast<ManPage> (my pages -> item [1]);
+		ManPage firstPage = my pages.at [1];
 		autoManual manual = Manual_create (firstPage -> title, me, false);
 		if (my executable)
 			Melder_warning (U"These manual pages contain links to executable scripts.\n"
 				"Only navigate these pages if you trust their author!");
-		praat_installEditor (manual.transfer(), IOBJECT);
+		praat_installEditor (manual.get(), IOBJECT);
+		manual.releaseToUser();
 	}
 END2 }
 
@@ -507,11 +511,12 @@ FORM (SearchManual, U"Search manual", U"Manual") {
 DO
 	if (theCurrentPraatApplication -> batch)
 		Melder_throw (U"Cannot view a manual from batch.");
-	Manual manPage = Manual_create (U"Intro", theCurrentPraatApplication -> manPages, false);
-	Manual_search (manPage, GET_STRING (U"query"));
+	autoManual manual = Manual_create (U"Intro", theCurrentPraatApplication -> manPages, false);
+	Manual_search (manual.get(), GET_STRING (U"query"));
+	manual.releaseToUser();
 END2 }
 
-FORM (GoToManualPage, U"Go to manual page", 0) {
+FORM (GoToManualPage, U"Go to manual page", nullptr) {
 	{long numberOfPages;
 	const char32 **pages = ManPages_getTitles (theCurrentPraatApplication -> manPages, & numberOfPages);
 	LIST (U"Page", numberOfPages, pages, 1)}
@@ -519,15 +524,16 @@ FORM (GoToManualPage, U"Go to manual page", 0) {
 DO
 	if (theCurrentPraatApplication -> batch)
 		Melder_throw (U"Cannot view a manual from batch.");
-	Manual manPage = Manual_create (U"Intro", theCurrentPraatApplication -> manPages, false);
-	HyperPage_goToPage_i (manPage, GET_INTEGER (U"Page"));
+	autoManual manual = Manual_create (U"Intro", theCurrentPraatApplication -> manPages, false);
+	HyperPage_goToPage_i (manual.get(), GET_INTEGER (U"Page"));
+	manual.releaseToUser();
 END2 }
 
-FORM (WriteManualToHtmlDirectory, U"Save all pages as HTML files", 0) {
+FORM (WriteManualToHtmlDirectory, U"Save all pages as HTML files", nullptr) {
 	LABEL (U"", U"Type a directory name:")
 	TEXTFIELD (U"directory", U"")
 	OK2
-structMelderDir currentDirectory = { { 0 } };
+structMelderDir currentDirectory { { 0 } };
 Melder_getDefaultDir (& currentDirectory);
 SET_STRING (U"directory", Melder_dirToPath (& currentDirectory))
 DO
@@ -547,8 +553,8 @@ void praat_show () {
 	praat_sensitivizeFixedButtonCommand (U"Info", theCurrentPraatObjects -> totalSelection == 1);
 	praat_sensitivizeFixedButtonCommand (U"Inspect", theCurrentPraatObjects -> totalSelection != 0);
 	praat_actions_show ();
-	if (theCurrentPraatApplication == & theForegroundPraatApplication && theButtonEditor)
-		Editor_dataChanged (theButtonEditor);
+	if (theCurrentPraatApplication == & theForegroundPraatApplication && theReferenceToTheOnlyButtonEditor)
+		Editor_dataChanged (theReferenceToTheOnlyButtonEditor);
 }
 
 /********** Menu descriptions. **********/
@@ -562,20 +568,20 @@ void praat_addFixedButtons (GuiWindow window) {
 }
 
 static void searchProc () {
-	DO_SearchManual (NULL, 0, NULL, NULL, NULL, NULL, false, NULL);
+	DO_SearchManual (nullptr, 0, nullptr, nullptr, nullptr, nullptr, false, nullptr);
 }
 
 static MelderString itemTitle_about { 0 };
 
-static Any scriptRecognizer (int nread, const char *header, MelderFile file) {
+static autoDaata scriptRecognizer (int nread, const char *header, MelderFile file) {
 	const char32 *name = MelderFile_name (file);
-	if (nread < 2) return NULL;
+	if (nread < 2) return autoDaata ();
 	if ((header [0] == '#' && header [1] == '!') || str32str (name, U".praat") == name + str32len (name) - 6
 	    || str32str (name, U".html") == name + str32len (name) - 5)
 	{
 		return Script_createFromFile (file);
 	}
-	return NULL;
+	return autoDaata ();
 }
 
 static void cb_openDocument (MelderFile file) {
@@ -617,10 +623,10 @@ void praat_addMenus (GuiWindow window) {
 	 */
 	if (! theCurrentPraatApplication -> batch) {
 		#ifdef macintosh
-			praatMenu = GuiMenu_createInWindow (NULL, U"\024", 0);
+			praatMenu = GuiMenu_createInWindow (nullptr, U"\024", 0);
 			#if cocoa
-				editMenu = GuiMenu_createInWindow (NULL, U"Edit", 0);
-				windowMenu = GuiMenu_createInWindow (NULL, U"Window", 0);
+				editMenu = GuiMenu_createInWindow (nullptr, U"Edit", 0);
+				windowMenu = GuiMenu_createInWindow (nullptr, U"Window", 0);
 			#endif
 		#else
 			praatMenu = GuiMenu_createInWindow (window, U"Praat", 0);
@@ -629,88 +635,88 @@ void praat_addMenus (GuiWindow window) {
 		readMenu = GuiMenu_createInWindow (window, U"Open", 0);
 		praat_actions_createWriteMenu (window);
 		#ifdef macintosh
-			applicationHelpMenu = GuiMenu_createInWindow (NULL, U"Help", 0);
+			applicationHelpMenu = GuiMenu_createInWindow (nullptr, U"Help", 0);
 		#endif
 		helpMenu = GuiMenu_createInWindow (window, U"Help", 0);
 	}
 	
 	MelderString_append (& itemTitle_about, U"About ", praatP.title, U"...");
 	#ifdef macintosh
-		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, 0, praat_UNHIDABLE, DO_About);
+		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, nullptr, praat_UNHIDABLE, DO_About);
 		#if cocoa
 			/*
 			 * HACK: give the following command weird names,
 			 * because otherwise they may be called from a script.
 			 * (we add three alt-spaces)
 			 */
-			praat_addMenuCommand (U"Objects", U"Edit", U"Cut   ", 0, praat_UNHIDABLE + 'X', DO_praat_cut);
-			praat_addMenuCommand (U"Objects", U"Edit", U"Copy   ", 0, praat_UNHIDABLE + 'C', DO_praat_copy);
-			praat_addMenuCommand (U"Objects", U"Edit", U"Paste   ", 0, praat_UNHIDABLE + 'V', DO_praat_paste);
-			praat_addMenuCommand (U"Objects", U"Window", U"Minimize   ", 0, praat_UNHIDABLE, DO_praat_minimize);
-			praat_addMenuCommand (U"Objects", U"Window", U"Zoom   ", 0, praat_UNHIDABLE, DO_praat_zoom);
-			praat_addMenuCommand (U"Objects", U"Window", U"Close   ", 0, 'W', DO_praat_close);
+			praat_addMenuCommand (U"Objects", U"Edit", U"Cut   ", nullptr, praat_UNHIDABLE + 'X', DO_praat_cut);
+			praat_addMenuCommand (U"Objects", U"Edit", U"Copy   ", nullptr, praat_UNHIDABLE + 'C', DO_praat_copy);
+			praat_addMenuCommand (U"Objects", U"Edit", U"Paste   ", nullptr, praat_UNHIDABLE + 'V', DO_praat_paste);
+			praat_addMenuCommand (U"Objects", U"Window", U"Minimize   ", nullptr, praat_UNHIDABLE, DO_praat_minimize);
+			praat_addMenuCommand (U"Objects", U"Window", U"Zoom   ", nullptr, praat_UNHIDABLE, DO_praat_zoom);
+			praat_addMenuCommand (U"Objects", U"Window", U"Close   ", nullptr, 'W', DO_praat_close);
 		#endif
 	#endif
 	#ifdef UNIX
-		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, 0, praat_UNHIDABLE, DO_About);
+		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, nullptr, praat_UNHIDABLE, DO_About);
 	#endif
-	praat_addMenuCommand (U"Objects", U"Praat", U"-- script --", 0, 0, 0);
-	praat_addMenuCommand (U"Objects", U"Praat", U"New Praat script", 0, 0, DO_praat_newScript);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Open Praat script...", 0, 0, DO_praat_openScript);
-	praat_addMenuCommand (U"Objects", U"Praat", U"-- buttons --", 0, 0, 0);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Add menu command...", 0, praat_HIDDEN, DO_praat_addMenuCommand);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Hide menu command...", 0, praat_HIDDEN, DO_praat_hideMenuCommand);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Show menu command...", 0, praat_HIDDEN, DO_praat_showMenuCommand);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Add action command...", 0, praat_HIDDEN, DO_praat_addAction);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Hide action command...", 0, praat_HIDDEN, DO_praat_hideAction);
-	praat_addMenuCommand (U"Objects", U"Praat", U"Show action command...", 0, praat_HIDDEN, DO_praat_showAction);
+	praat_addMenuCommand (U"Objects", U"Praat", U"-- script --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"Praat", U"New Praat script", nullptr, 0, DO_praat_newScript);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Open Praat script...", nullptr, 0, DO_praat_openScript);
+	praat_addMenuCommand (U"Objects", U"Praat", U"-- buttons --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Add menu command...", nullptr, praat_HIDDEN, DO_praat_addMenuCommand);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Hide menu command...", nullptr, praat_HIDDEN, DO_praat_hideMenuCommand);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Show menu command...", nullptr, praat_HIDDEN, DO_praat_showMenuCommand);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Add action command...", nullptr, praat_HIDDEN, DO_praat_addAction);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Hide action command...", nullptr, praat_HIDDEN, DO_praat_hideAction);
+	praat_addMenuCommand (U"Objects", U"Praat", U"Show action command...", nullptr, praat_HIDDEN, DO_praat_showAction);
 
-	GuiMenuItem menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Goodies", 0, praat_UNHIDABLE, 0);
-	goodiesMenu = menuItem ? menuItem -> d_menu : NULL;
-	praat_addMenuCommand (U"Objects", U"Goodies", U"Calculator...", 0, 'U', DO_praat_calculator);
-	praat_addMenuCommand (U"Objects", U"Goodies", U"Report difference of two proportions...", 0, 0, DO_praat_reportDifferenceOfTwoProportions);
+	GuiMenuItem menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Goodies", nullptr, praat_UNHIDABLE, nullptr);
+	goodiesMenu = menuItem ? menuItem -> d_menu : nullptr;
+	praat_addMenuCommand (U"Objects", U"Goodies", U"Calculator...", nullptr, 'U', DO_praat_calculator);
+	praat_addMenuCommand (U"Objects", U"Goodies", U"Report difference of two proportions...", nullptr, 0, DO_praat_reportDifferenceOfTwoProportions);
 
-	menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Preferences", 0, praat_UNHIDABLE, 0);
-	preferencesMenu = menuItem ? menuItem -> d_menu : NULL;
-	praat_addMenuCommand (U"Objects", U"Preferences", U"Buttons...", 0, praat_UNHIDABLE, DO_praat_editButtons);   /* Cannot be hidden. */
-	praat_addMenuCommand (U"Objects", U"Preferences", U"-- encoding prefs --", 0, 0, 0);
-	praat_addMenuCommand (U"Objects", U"Preferences", U"Text reading preferences...", 0, 0, DO_TextInputEncodingSettings);
-	praat_addMenuCommand (U"Objects", U"Preferences", U"Text writing preferences...", 0, 0, DO_TextOutputEncodingSettings);
-	praat_addMenuCommand (U"Objects", U"Preferences", U"CJK font style preferences...", 0, 0, DO_GraphicsCjkFontStyleSettings);
+	menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Preferences", nullptr, praat_UNHIDABLE, nullptr);
+	preferencesMenu = menuItem ? menuItem -> d_menu : nullptr;
+	praat_addMenuCommand (U"Objects", U"Preferences", U"Buttons...", nullptr, praat_UNHIDABLE, DO_praat_editButtons);   /* Cannot be hidden. */
+	praat_addMenuCommand (U"Objects", U"Preferences", U"-- encoding prefs --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"Preferences", U"Text reading preferences...", nullptr, 0, DO_TextInputEncodingSettings);
+	praat_addMenuCommand (U"Objects", U"Preferences", U"Text writing preferences...", nullptr, 0, DO_TextOutputEncodingSettings);
+	praat_addMenuCommand (U"Objects", U"Preferences", U"CJK font style preferences...", nullptr, 0, DO_GraphicsCjkFontStyleSettings);
 
-	menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Technical", 0, praat_UNHIDABLE, 0);
-	technicalMenu = menuItem ? menuItem -> d_menu : NULL;
-	praat_addMenuCommand (U"Objects", U"Technical", U"List readable types of objects", 0, 0, DO_praat_listReadableTypesOfObjects);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Report memory use", 0, 0, DO_praat_reportMemoryUse);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Report integer properties", 0, 0, DO_praat_reportIntegerProperties);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Report text properties", 0, 0, DO_praat_reportTextProperties);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Report system properties", 0, 0, DO_praat_reportSystemProperties);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Report graphical properties", 0, 0, DO_praat_reportGraphicalProperties);
-	praat_addMenuCommand (U"Objects", U"Technical", U"Debug...", 0, 0, DO_praat_debug);
+	menuItem = praat_addMenuCommand (U"Objects", U"Praat", U"Technical", nullptr, praat_UNHIDABLE, nullptr);
+	technicalMenu = menuItem ? menuItem -> d_menu : nullptr;
+	praat_addMenuCommand (U"Objects", U"Technical", U"List readable types of objects", nullptr, 0, DO_praat_listReadableTypesOfObjects);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Report memory use", nullptr, 0, DO_praat_reportMemoryUse);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Report integer properties", nullptr, 0, DO_praat_reportIntegerProperties);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Report text properties", nullptr, 0, DO_praat_reportTextProperties);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Report system properties", nullptr, 0, DO_praat_reportSystemProperties);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Report graphical properties", nullptr, 0, DO_praat_reportGraphicalProperties);
+	praat_addMenuCommand (U"Objects", U"Technical", U"Debug...", nullptr, 0, DO_praat_debug);
 
-	praat_addMenuCommand (U"Objects", U"Open", U"Read from file...", 0, praat_ATTRACTIVE + 'O', DO_Data_readFromFile);
+	praat_addMenuCommand (U"Objects", U"Open", U"Read from file...", nullptr, praat_ATTRACTIVE + 'O', DO_Data_readFromFile);
 
-	praat_addAction1 (classDaata, 0, U"Save as text file...", 0, 0, DO_Data_writeToTextFile);
-	praat_addAction1 (classDaata, 0, U"Write to text file...", 0, praat_HIDDEN, DO_Data_writeToTextFile);
-	praat_addAction1 (classDaata, 0, U"Save as short text file...", 0, 0, DO_Data_writeToShortTextFile);
-	praat_addAction1 (classDaata, 0, U"Write to short text file...", 0, praat_HIDDEN, DO_Data_writeToShortTextFile);
-	praat_addAction1 (classDaata, 0, U"Save as binary file...", 0, 0, DO_Data_writeToBinaryFile);
-	praat_addAction1 (classDaata, 0, U"Write to binary file...", 0, praat_HIDDEN, DO_Data_writeToBinaryFile);
+	praat_addAction1 (classDaata, 0, U"Save as text file...", nullptr, 0, DO_Data_writeToTextFile);
+	praat_addAction1 (classDaata, 0, U"Write to text file...", nullptr, praat_HIDDEN, DO_Data_writeToTextFile);
+	praat_addAction1 (classDaata, 0, U"Save as short text file...", nullptr, 0, DO_Data_writeToShortTextFile);
+	praat_addAction1 (classDaata, 0, U"Write to short text file...", nullptr, praat_HIDDEN, DO_Data_writeToShortTextFile);
+	praat_addAction1 (classDaata, 0, U"Save as binary file...", nullptr, 0, DO_Data_writeToBinaryFile);
+	praat_addAction1 (classDaata, 0, U"Write to binary file...", nullptr, praat_HIDDEN, DO_Data_writeToBinaryFile);
 
-	praat_addAction1 (classManPages, 1, U"Save to HTML directory...", 0, 0, DO_ManPages_saveToHtmlDirectory);
-	praat_addAction1 (classManPages, 1, U"View", 0, 0, DO_ManPages_view);
+	praat_addAction1 (classManPages, 1, U"Save to HTML directory...", nullptr, 0, DO_ManPages_saveToHtmlDirectory);
+	praat_addAction1 (classManPages, 1, U"View", nullptr, 0, DO_ManPages_view);
 }
 
 void praat_addMenus2 () {
-	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"-- manual --", 0, 0, 0);
-	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Go to manual page...", 0, 0, DO_GoToManualPage);
-	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Write manual to HTML directory...", 0, praat_HIDDEN, DO_WriteManualToHtmlDirectory);
+	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"-- manual --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Go to manual page...", nullptr, 0, DO_GoToManualPage);
+	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Write manual to HTML directory...", nullptr, praat_HIDDEN, DO_WriteManualToHtmlDirectory);
 	praat_addMenuCommand (U"Objects", U"ApplicationHelp",
 		Melder_cat (U"Search ", praatP.title, U" manual..."),
-		0, 'M', DO_SearchManual);
+		nullptr, 'M', DO_SearchManual);
 	#ifdef _WIN32
-		praat_addMenuCommand (U"Objects", U"Help", U"-- about --", 0, 0, 0);
-		praat_addMenuCommand (U"Objects", U"Help", itemTitle_about.string, 0, praat_UNHIDABLE, DO_About);
+		praat_addMenuCommand (U"Objects", U"Help", U"-- about --", nullptr, 0, nullptr);
+		praat_addMenuCommand (U"Objects", U"Help", itemTitle_about.string, nullptr, praat_UNHIDABLE, DO_About);
 	#endif
 
 	#if defined (macintosh) || defined (_WIN32)
