@@ -1,20 +1,19 @@
 /* praat_script.cpp
  *
- * Copyright (C) 1993-2012,2013,2014,2015 Paul Boersma
+ * Copyright (C) 1993-2012,2013,2014,2015,2016 Paul Boersma
  * 
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <ctype.h>
@@ -36,7 +35,7 @@ static int praat_findObjectFromString (Interpreter interpreter, const char32 *st
 			static MelderString buffer { 0 };
 			MelderString_copy (& buffer, string);
 			char32 *space = str32chr (buffer.string, U' ');
-			if (space == NULL)
+			if (! space)
 				Melder_throw (U"Missing space in name.");
 			*space = U'\0';
 			char32 *className = & buffer.string [0], *givenName = space + 1;
@@ -78,9 +77,13 @@ Editor praat_findEditorFromString (const char32 *string) {
 		WHERE_DOWN (1) {
 			for (int ieditor = 0; ieditor < praat_MAXNUM_EDITORS; ieditor ++) {
 				Editor editor = theCurrentPraatObjects -> list [IOBJECT]. editors [ieditor];
-				if (editor != NULL) {
-					const char32 *name = str32chr (editor -> name, U' ') + 1;
-					if (str32equ (name, string)) return editor;
+				if (editor) {
+					Melder_assert (editor -> name);
+					const char32 *space = str32chr (editor -> name, U' ');   // editors tend to be called like "3. Sound kanweg"
+					if (space) {   // but not all
+						const char32 *name = space + 1;
+						if (str32equ (name, string)) return editor;
+					}
 				}
 			}
 		}
@@ -427,7 +430,7 @@ int praat_executeCommand (Interpreter interpreter, char32 *command) {
 			colon [0] = colon [1] = colon [2] = U'.';
 			colon [3] = U'\0';
 		}
-		if (theCurrentPraatObjects == & theForegroundPraatObjects && praatP. editor != NULL) {
+		if (theCurrentPraatObjects == & theForegroundPraatObjects && praatP. editor) {
 			if (hasColon) {
 				Editor_doMenuCommand (praatP. editor, command2, narg, args, NULL, interpreter);
 			} else {
@@ -513,7 +516,7 @@ void praat_executeCommandFromStandardInput (const char32 *programName) {
 		if (newLine) *newLine = '\0';
 		autostring32 command32 = Melder_8to32 (command8);
 		try {
-			praat_executeCommand (NULL, command32.peek());
+			praat_executeCommand (nullptr, command32.peek());
 		} catch (MelderError) {
 			Melder_flushError (programName, U": command \"", Melder_peek8to32 (command8), U"\" not executed.");
 		}
@@ -540,7 +543,7 @@ void praat_executeScriptFromFileName (const char32 *fileName, int narg, Stackel
 	/*
 	 * The argument 'fileName' is unsafe. Duplicate its contents.
 	 */
-	structMelderFile file = { 0 };
+	structMelderFile file { 0 };
 	Melder_relativePathToFile (fileName, & file);
 	try {
 		autostring32 text = MelderFile_readText (& file);
@@ -558,7 +561,7 @@ void praat_executeScriptFromFileName (const char32 *fileName, int narg, Stackel
 void praat_executeScriptFromFileNameWithArguments (const char32 *nameAndArguments) {
 	char32 path [256];
 	const char32 *p, *arguments;
-	structMelderFile file = { 0 };
+	structMelderFile file { 0 };
 	/*
 	 * Split into file name and arguments.
 	 */
@@ -583,11 +586,21 @@ void praat_executeScriptFromFileNameWithArguments (const char32 *nameAndArgument
 	praat_executeScriptFromFile (& file, arguments);
 }
 
+extern "C" void praatlib_executeScript (const char *text8) {
+	try {
+		autoInterpreter interpreter = Interpreter_create (nullptr, nullptr);
+		autostring32 string = Melder_8to32 (text8);
+		Interpreter_run (interpreter.get(), string.peek());
+	} catch (MelderError) {
+		Melder_throw (U"Script not completed.");
+	}
+}
+
 void praat_executeScriptFromText (const char32 *text) {
 	try {
-		autoInterpreter interpreter = Interpreter_create (NULL, NULL);
+		autoInterpreter interpreter = Interpreter_create (nullptr, nullptr);
 		autostring32 string = Melder_dup (text);   // copy, because Interpreter will change it (UGLY)
-		Interpreter_run (interpreter.peek(), string.peek());
+		Interpreter_run (interpreter.get(), string.peek());
 	} catch (MelderError) {
 		Melder_throw (U"Script not completed.");
 	}
@@ -595,16 +608,16 @@ void praat_executeScriptFromText (const char32 *text) {
 
 void praat_executeScriptFromDialog (UiForm dia) {
 	char32 *path = UiForm_getString (dia, U"$file");
-	structMelderFile file = { 0 };
+	structMelderFile file { 0 };
 	Melder_pathToFile (path, & file);
 	autostring32 text = MelderFile_readText (& file);
 	autoMelderFileSetDefaultDir dir (& file);
 	Melder_includeIncludeFiles (& text);
 	autoInterpreter interpreter = Interpreter_createFromEnvironment (praatP.editor);
-	Interpreter_readParameters (interpreter.peek(), text.peek());
-	Interpreter_getArgumentsFromDialog (interpreter.peek(), dia);
+	Interpreter_readParameters (interpreter.get(), text.peek());
+	Interpreter_getArgumentsFromDialog (interpreter.get(), dia);
 	autoPraatBackground background;
-	Interpreter_run (interpreter.peek(), text.peek());
+	Interpreter_run (interpreter.get(), text.peek());
 }
 
 static void secondPassThroughScript (UiForm sendingForm, int /* narg */, Stackel /* args */,
@@ -623,14 +636,14 @@ static void firstPassThroughScript (MelderFile file) {
 		}
 		autoInterpreter interpreter = Interpreter_createFromEnvironment (praatP.editor);
 		if (Interpreter_readParameters (interpreter.get(), text.peek()) > 0) {
-			UiForm form = Interpreter_createForm (interpreter.peek(),
+			UiForm form = Interpreter_createForm (interpreter.get(),
 				praatP.editor ? praatP.editor -> d_windowForm : theCurrentPraatApplication -> topShell,
 				Melder_fileToPath (file), secondPassThroughScript, NULL, false);
 			UiForm_destroyWhenUnmanaged (form);
 			UiForm_do (form, false);
 		} else {
 			autoPraatBackground background;
-			praat_executeScriptFromFile (file, NULL);
+			praat_executeScriptFromFile (file, nullptr);
 		}
 	} catch (MelderError) {
 		Melder_throw (U"Script ", file, U" not completed.");
@@ -655,8 +668,8 @@ void DO_RunTheScriptFromAnyAddedMenuCommand (UiForm /* sendingForm_dummy */, int
 
 void DO_RunTheScriptFromAnyAddedEditorCommand (Editor editor, const char32 *script) {
 	praatP.editor = editor;
-	DO_RunTheScriptFromAnyAddedMenuCommand (NULL, 0, NULL, script, NULL, NULL, false, NULL);
-	/*praatP.editor = NULL;*/
+	DO_RunTheScriptFromAnyAddedMenuCommand (nullptr, 0, nullptr, script, nullptr, nullptr, false, nullptr);
+	/*praatP.editor = nullptr;*/
 }
 
 /* End of file praat_script.cpp */
