@@ -2,19 +2,18 @@
  *
  * Copyright (C) 1992-2012,2014,2015 Paul Boersma
  *
- * This program is free software; you can redistribute it and/or modify
+ * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This code is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <time.h>
@@ -140,7 +139,7 @@ void praat_reportMemoryUse () {
 	MelderInfo_writeLine (U"Currently in use:\n"
 		U"   Strings: ", MelderString_allocationCount () - MelderString_deallocationCount ());
 	MelderInfo_writeLine (U"   Arrays: ", NUM_getTotalNumberOfArrays ());
-	MelderInfo_writeLine (U"   Things: ", Thing_getTotalNumberOfThings (),
+	MelderInfo_writeLine (U"   Things: ", theTotalNumberOfThings,
 		U" (objects in list: ", theCurrentPraatObjects -> n, U")");
 	long numberOfMotifWidgets =
 	#if motif
@@ -151,7 +150,7 @@ void praat_reportMemoryUse () {
 	#endif
 	MelderInfo_writeLine (U"   Other: ",
 		Melder_allocationCount () - Melder_deallocationCount ()
-		- Thing_getTotalNumberOfThings () - NUM_getTotalNumberOfArrays ()
+		- theTotalNumberOfThings - NUM_getTotalNumberOfArrays ()
 		- (MelderString_allocationCount () - MelderString_deallocationCount ())
 		- numberOfMotifWidgets);
 	MelderInfo_writeLine (
